;**************************************************************
;
;   _____     
;  |  ___|
;  | |
;  | |  _  M E T R I C A
;  | |_| |
;  |_____|
;
; GMetrica - Copyright (c) 2010 by Josef Sniatecki
;   2D Jump&Run (platformer)
;
;**************************************************************
;   Copyright (c) 2010 Josef Sniatecki
;
; This program is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License
; As published by the Free Software Foundation; either version
; 3 of the License, Or (at your option) any later version.
;
; This program is distributed in the hope that it will be
; useful, but WITHOUT ANY WARRANTY; without even the implied
; warranty of MERCHANTABILITY Or FITNESS For A PARTICULAR
; PURPOSE. See the GNU General Public License For more details.
;
; You should have received a copy of the GNU General Public
; License along With this program;
; If Not, see "http://www.gnu.org/licenses/".
;**************************************************************



EnableExplicit


;******************************************
; Includes
;******************************************

; Folgende Quelldateien gehren zur eigenen PBSTL:

IncludePath "..\..\Includes"
IncludeFile "Standard.pbi"
IncludeFile "Window.pbi"
IncludeFile "Keyboard.pbi"
IncludeFile "Queue.pbi"
IncludeFile "Collision.pbi"
IncludeFile "EditorGadget.pbi"
IncludeFile "Angle.pbi"

; Die Quelldatei "Queue.pbi" ersetzt die nativen LinkedLists aus PureBasic.
; Der Grund dafr ist, dass ich dadurch eine Liste mehrfach gleichzeitig
; Durchlaufen kann, ohne stndig "ChangeCurrentElement" anzuwenden, um
; Komplikationen mit dem aktuellen Listen-Element zu bekommen.
; Desweiteren habe ich dieses Projekt vor der Einfhrung von Listen, Arrays
; und Maps in Strukturen gestartet, weshalb die Aktualisierung auf PB 4.51
; zu viel Aufwand bentigt.



;******************************************
; Constants
;******************************************

;{ Compiler
Enumeration
  #Mode_Game
  #Mode_Editor
  #Mode_Menu
EndEnumeration
;}

;{ Scene & Tiles
#FieldSize = 24

#SceneWidth   = 27
#SceneHeight  = 13
#ScreenWidth  = #SceneWidth * #FieldSize
#ScreenHeight = #SceneHeight * #FieldSize

; Editor:
#SceneMap_Width            = 250
#SceneMap_Height           = 100

Enumeration 0
  #BackgroundTile1
  #BackgroundTile2
  #BackgroundTile3
  #Tile
  #Ground
  #ForegroundTile
  #BackgroundObject1
  #BackgroundObject2
  #Object
  #ForegroundObject1
  #ForegroundObject2
EndEnumeration

#FirstLayer = #BackgroundTile1
#LastLayer  = #ForegroundObject2

#FirstTile = #BackgroundTile1
#LastTile  = #ForegroundTile

#FirstObject = #BackgroundObject1
#LastObject  = #ForegroundObject2
;}

;{ Player
#ClimbingLeft  = -1
#ClimbingRight =  1

Enumeration
  #Player_Animation_StandingRight
  #Player_Animation_StandingLeft
  #Player_Animation_WalkingRight
  #Player_Animation_WalkingLeft
  #Player_Animation_ClimbingRight
  #Player_Animation_ClimbingLeft
  #Player_Animation_FlyingRight
  #Player_Animation_FlyingLeft
  #Player_Animation_FallingRight
  #Player_Animation_FallingLeft
  #Player_Animation_GhostRight
  #Player_Animation_GhostLeft
  #Player_Animation_Clambering
EndEnumeration

Enumeration
  #Player_Key_Right
  #Player_Key_Left
  #Player_Key_Down
  #Player_Key_Up
  #Player_Key_Shoot
  #Player_Key_Jump
  #Player_Key_Fly
  #Player_Key_ItemA
  #Player_Key_ItemB
  #Player_Key_ItemC
  #Player_Key_Inventory
  #Player_KeyCount
EndEnumeration

Enumeration
  #Player_Action_Standing
  #Player_Action_Falling
  #Player_Action_Jump
  #Player_Action_DoubleJump
  #Player_Action_Climbing
  #Player_Action_Clambering
  #Player_Action_JumpedOff
  #Player_Action_Flying
  #Player_Action_FlyingTruncated
  #Player_Action_LumiFlight
EndEnumeration

Enumeration
  #Player_Shot_White
  #Player_Shot_Blue
  #Player_Shot_Green
  #Player_ShotCount
EndEnumeration

#Player_MultiplierDuration    = 190.0
#Player_MultiplierChangedTime = 50

#Gravity    = 0.205
#MaxGravity = 18.0

#Player_JumpEnergy               = 2.1
#Player_JumpAddition             = 0.27
#Player_JumpTime                 = 14
#Player_DoubleJumpEnergy         = 2.9
#Player_DoubleJumpAddition       = 0.3
#Player_DoubleJumpTime           = 8
#Player_ClimbSpeed               = 0.75
#Player_SlideSpeed               = 0.1
#Player_MaxFallSpeed             = 6.0
#Player_FlightSpeedFactor        = 0.08
#Player_FlightFriction           = 0.982
#Player_MaxFlightSpeedFactor     = 0.72
#Player_MaxFlyUpSpeed            = 1
#Player_MaxLumiFlightSpeedFactor = 2.5
#Player_LumiFlightSpeedFactor    = 0.06
#Player_LumiFlightFriction       = 0.982

#Player_SpeedForBlur = 6.0

#Player_ShotSpeed           = 4.0
#Player_MaximalShotPower    = 3.0
#Player_AdditionalShotPower = 0.02

#Player_ClimbSide_Left  = -1
#Player_ClimbSide_Right = 1

#Player_ClimbSpeedFactor = 0.6

#FlightGravity = 0.55

#Player_AnimationDuration_Walking  = 1
#Player_AnimationDuration_Standing = 16
#Player_AnimationDuration_Flying   = 9
#Player_AnimationDuration_Climbing = 1

#ShotSize = 7.0

#Player_HitTime = 75

#Player_LostLiveAnimationDuration = 5
#Player_LostLiveTime              = 75
;}

;{ Control
  Enumeration
    #ControlMode_Keyboard
    #ControlMode_Joystick
  EndEnumeration
;}

;{ Camera
Enumeration
  #Camera_FollowPlayer
  #Camera_FollowObject
  #Camera_SetOnField
EndEnumeration
;}

;{ Entry
Enumeration
  #Entry_File
  #Entry_Folder
EndEnumeration
;}

;{ Editor Screen
#EditorScreenWidth  = #FieldSize * #SceneWidth
#EditorScreenHeight = #FieldSize * #SceneHeight + 2*#FieldSize + 8*#FieldSize
;}

;{ Object
Enumeration 1
  #Object_Display
  #Object_Frame
  #Object_Collision
  #Object_Delete
  #Object_PlayerEnteredScene
  #Object_PlayerLeftScene
  #Object_AllowClambering
  #Object_ForbidClimbing
  #Object_Hit
  #Object_Hide
  #Object_Unhide
  
  #Object_Load
  #Object_Save
  #Object_GlobalLoad
  #Object_GlobalSave
  #Object_Initialize
  #Object_GlobalInitialize
  
  #Object_GlobalEditorSave
  #Object_EditorSave
  #Object_GlobalEditorLoad
  #Object_EditorLoad
  #Object_EditorInitialize
  #Object_EditorConfigure
  #Object_EditorCopy
  #Object_EditorFree
  
  #Object_CallAction
  #Object_LoadAction
  #Object_InitializeAction
  #Object_EditorSaveAction
  #Object_EditorLoadAction
  #Object_EditorInitializeAction
  #Object_EditorFreeAction
  #Object_ActionName
  #Object_ActionDescription
  #Object_ActionCount
  
  #Object_PreferedLayer
  #Object_CheckLayer
  #Object_ViewSprite
  #Object_ViewRegion
  #Object_Name
  #Object_Description
EndEnumeration
Enumeration 1
  #ObjectDo_Delete
  #ObjectDo_Collision
EndEnumeration
;}

;{ Background
Enumeration 1
  #Background_Format_Gradient
  #Background_Format_VerticalLine
  #Background_Format_Field
  #Background_Format_Full
EndEnumeration
;}

;{ Acts, Events & Effects
Enumeration 1
  #Event_Triggered
  #Event_Initialize
  #Event_GlobalLoad
  #Event_Load
  #Event_Start
  #Event_EditorInitialize
  #Event_EditorLoad
  #Event_EditorSave
  #Event_GlobalEditorLoad
  #Event_GlobalEditorSave
  #Event_Edit
  #Event_EditorDelete
  #Event_Name
  #Event_Description
  #Event_Text
  #Event_Check
EndEnumeration
Enumeration 0
  #SpecialEvent_Start
  #SpecialEvent_LostLive
  #SpecialEvent_GameOver
  #SpecialEvent_ScoreAdded
  #SpecialEvent_CoinCollected
  #SpecialEventCount
EndEnumeration
Enumeration 1
  #Effect_Call
  #Effect_Initialize
  #Effect_GlobalLoad
  #Effect_Load
  #Effect_EditorInitialize
  #Effect_EditorLoad
  #Effect_EditorSave
  #Effect_GlobalEditorLoad
  #Effect_GlobalEditorSave
  #Effect_Edit
  #Effect_EditorDelete
  #Effect_Name
  #Effect_Description
  #Effect_Text
  #Effect_Check
EndEnumeration
;}

;{ Item
Enumeration 1
  #Item_IsUseable
  #Item_Use
  #Item_NamePlural
  #Item_NameSingular
EndEnumeration
;}

;{ Particle
Enumeration
  #Particle_Frame
  #Particle_NextFrame
EndEnumeration
;}

;{ Interface
Enumeration
  #ScreenWindow
  #EditorWindow
  #SPWindow
  #EditorCPWindow
  
  #AEWindow
  #TEWindow
  #TESWindow
  #TVWindow
  #OVWindow
  #SVWindow
  #BVWindow
  #CVWindow
EndEnumeration
Enumeration
  ;{ Editor
    #SP_HLoop
    #SP_VLoop
    #SP_OK
    #SP_Panel
    #SP_WidthText
    #SP_Width
    #SP_HeightText
    #SP_Height
    #SP_BordersFrame
    #SP_Cancel
    #SP_PowersFrame
    #SP_Fly3
    #SP_LumiFlight
    #SP_UTop
    #SP_Act
    #SP_SelectAct
    #SP_ParallelAct
    #SP_MainAct
    
    #EditorCP_OK
    #EditorCP_Panel
    #EditorCP_NameText
    #EditorCP_Name
    #EditorCP_AuthorText
    #EditorCP_Author
    #EditorCP_PasswordText
    #EditorCP_Password
    #EditorCP_CommentFrame
    #EditorCP_Comment
    #EditorCP_AuthorsHighscoreText
    #EditorCP_AuthorsHighscore
    #EditorCP_Play
    #EditorCP_Locked
    #EditorCP_PowersFrame
    #EditorCP_CanDoubleJump
    #EditorCP_CanFly1
    #EditorCP_CanFly2
    #EditorCP_CanFly3
    #EditorCP_CanShoot1
    #EditorCP_CanShoot2
    #EditorCP_CanShoot3
    #EditorCP_CanClimb
    #EditorCP_LivesText
    #EditorCP_Lives
    #EditorCP_DifficultyText
    #EditorCP_GenreText
    #EditorCP_Genre
    #EditorCP_Difficulty
    #EditorCP_SizeText
    #EditorCP_Size
    #EditorCP_FolderText
    #EditorCP_Folder
    #EditorCP_MaxEnergyText
    #EditorCP_MaxEnergy
    #EditorCP_EnergyText
    #EditorCP_Energy
    #EditorCP_CanLumiFlight
    #EditorCP_CanLoadShot
  ;}
  
  ;{ Acts, Events & Effects
    #AE_Add
    #AE_Delete
    #AE_Edit
    #AE_Acts
    #AE_NameText
    #AE_Name
    #AE_DescriptionFrame
    #AE_Description
    #AE_OK
    #AE_Apply
    
    #EE_Add
    #EE_Delete
    #EE_Edit
    #EE_Up
    #EE_Down
    #EE_OK
    #EE_Events
    #EE_Comment
    #EE_EffectFrame
    #EE_Effects
    #EE_AddEffect
    #EE_DeleteEffect
    #EE_EditEffect
    #EE_EffectUp
    #EE_EffectDown
    #EE_EffectComment
    
    #TES_Select
    #TES_Tree
    #TES_Cancel
    #TES_InfoFrame
    #TES_Info
  ;}
  
  ;{ Viewers
    #TV_Image
    #TV_Frame
    #TV_Tilesets
    #TV_OK
    #TV_Cancel
    
    #OV_Frame
    #OV_Objects
    #OV_OK
    #OV_Info
    #OV_Image
    #OV_Cancel
    
    #SV_Image
    #SV_Frame
    #SV_OK
    #SV_Cancel
    #SV_Properties
    #SV_Background
    
    #BV_Frame
    #BV_Image
    #BV_Backgrounds
    #BV_OK
    #BV_Cancel
    #BV_ChangeAll
    #BV_Preview
    
    #CV_Challanges
    #CV_Author
    #CV_Comment
    #CV_CommentFrame
    #CV_Select
    #CV_Cancel
  ;}
EndEnumeration
;}

;{ Screen
Enumeration
  #ScreenMode_Fullscreen
  #ScreenMode_Windowed
EndEnumeration
;}

#MaxNamespaces = 2

Enumeration
  #Challange_ChildType_Folder
  #Challange_ChildType_Bundle
EndEnumeration

;{ Selection
Enumeration
  #Selection_Frame
  #Selection_CheckTile
  #Selection_CheckObject
  #Selection_CheckField
  #Selection_Mode
EndEnumeration

Enumeration
  #Selection_Mode_AllLayers
  #Selection_Mode_Tiles
  #Selection_Mode_Walls
  #Selection_Mode_Objects
  #Selection_Mode_None
EndEnumeration
;}

;{ CharSet
#CharSet_HSpace = 1
#CharSet_VSpace = 1

#TinyMessage_Duration = 28
#TinyMessage_Halt     = 20
#TinyMessage_Speed    = 2
;}

#CameraSpeed = 0.02



;******************************************
; Structures
;******************************************

Structure Account
  Name.s
  Password.s
EndStructure

Structure Sprite
  Sprite.i
  Sprite3D.i
EndStructure
Structure Entry
  *Parent.Entry
  *Item.QueueItem
  
  Type.i
  Name.s
  
  StructureUnion
    *Entries.Queue
    *Memory
  EndStructureUnion
EndStructure

Structure Player
  Width.i
  Height.i
  
  X.f
  Y.f
  MoveX.f
  DoMoveX.f
  MoveY.f
  PreviousX.f
  PreviousY.f
  PreviousMoveX.f
  PreviousMoveY.f
  MoveInfluence.f
  MoveFriction.f
  MaxMoveX.f
  HeightOfFall.f
  
  FieldX.i
  FieldY.i
  *Scene.Scene
  *PreviousScene.Scene
  
  *SceneToLocate.Scene
  FieldXToLocate.i
  FieldYToLocate.i
  
  Speed.f
  Mass.f
  
  KeyPressed.i[#Player_KeyCount]
  KeyPressing.i[#Player_KeyCount]
  KeyReleased.i[#Player_KeyCount]
  
  Action.i
  ActionTime.i
  *Animation.Animation
  ClimbSide.i
  *BottomObject.Object
  
  *StickedObject.Object
  StickedX.f
  StickedY.f
  
  CanFly.i
  CanDoubleJump.i
  CanClimb.i
  CanShoot.i
  CanLoadShot.i
  CanLumiFlight.i
  
  Lives.i
  MaxEnergy.i
  Energy.i
  Coins.i
  
  Score.i
  NewScore.i
  ScoreReason.s
  ScoreReasonDuration.i
  AdditiveScore.i
  
  Multiplier.i
  MultiplierDuration.i
  MultiplierCounter.i
  MultiplierChangedTime.i
  
  Comment.s
  CommentDuration.i
  
  *LastCheckpointScene.Scene
  LastCheckpointX.i
  LastCheckpointY.i
  LoseGame.i
  
  HitTime.i
  LostLiveTime.i
  *LostLiveAnimation.Animation
  
  ShootMode.i
  ShootByMouse.i
  ShotPower.f
  ShotDuration.i
  *ShotFrameset.Frameset[#Player_ShotCount]
  *Shots.Queue
  
  *UsingItem.Item[3]
  
  LumiLight.f
  Shield.i
  
  EnergySprite.i
  EnergySprite3D.i
  NoEnergySprite.i
  NoEnergySprite3D.i
  
  SignSprite.i
  SignSprite3D.i
  
  *Blur.Blur
EndStructure
Structure Shot
  X.f
  Y.f
  MoveX.f
  MoveY.f
  *Animation.Animation
  
  Power.f
  
  *Item.QueueItem
EndStructure
Structure Camera
  Mode.i
  *TargetObject.Object
  TargetFieldX.i
  TargetFieldY.i
  
  XF.f
  YF.f
  X.i
  Y.i
EndStructure

Structure Challange
  Type.i
  *Entry.Entry
  
  Name.s
  Author.s
  Password.s
  Comment.s
  Locked.i
  
  HeaderChecksum.s
  ScoresChecksum.s
  SettingsChecksum.s
  
  *Highscores.Queue
  *BestHighscore.Highscore
EndStructure
Structure Highscore
  Player.s
  Score.i
  Medal.i
  SuperCoins.i
EndStructure
Structure ChallangeScript
  Library.i
  
  *Frame
  
  *Start
  *GameOver
  *Won
  
  *Score
  *MultiplierEnhanced
  
  *LostLive
  *Hit
  *SceneChanged
EndStructure
Structure CurrentChallange
  *Header.Challange
  
  *Scenes
  Width.i
  Height.i
  
  Lives.i
  MaxEnergy.i
  Energy.i
  
  CanDoubleJump.i
  CanClimb.i
  CanLumiFlight.i
  CanFly.i
  CanShoot.i
  CanLoadShot.i
  
  *StartScene.Scene
  StartX.i
  StartY.i
  
  Script.ChallangeScript
EndStructure

Structure CharSet
  Width.i
  Height.i
  CharacterCount.i
  *Characters
EndStructure
Structure TinyMessage
  X.i
  Y.i
  Text.s
  Time.i
EndStructure

Structure RegisteredSprite
  Name.s
  Sprite.i
  Sprite3D.i
EndStructure

CompilerSelect #Mode
  CompilerCase #Mode_Game ;{

Structure Field
  *Tile.Tile
  *Ground.Tile
  *Object.Object
EndStructure
Structure Scene
  *BackgroundTiles1
  *BackgroundTiles2
  *BackgroundTiles3
  *Tiles
  *Grounds
  *ForegroundTiles
  *BackgroundObjects1.Queue
  *BackgroundObjects2.Queue
  *Objects.Queue
  *ForegroundObjects1.Queue
  *ForegroundObjects2.Queue
  
  Background.i
  
  AllowFly3.i
  AllowLumiFlight.i
  
  HorinzontalLoop.i
  VerticalLoop.i
  UnpassableTop.i
  
  *Act.Act
  
  *Left.Scene
  *Right.Scene
  *Top.Scene
  *Bottom.Scene
  
  X.i
  Y.i
  Width.i
  Height.i
  
  Initialized.i
EndStructure

Structure Tileset
  Name.s
  Image.i
  Width.i
  Height.i
  *Tiles
EndStructure
Structure Tile
  Sprite.i
  X.i
  Y.i
EndStructure

Structure Bank
  *Callbacks.Queue
  Name.s
  Library.i
EndStructure
Structure Object
  X.f
  Y.f
  Width.i
  Height.i
  Hidden.i
  Transparency.f
  
  *Callback
  
  StructureUnion
    Sprite3D.i
    *Animation.Animation
  EndStructureUnion
  *Memory
  
  References.i
  *Scene.Scene
  *Queue.Queue
  *Item.QueueItem
EndStructure
Structure ObjectAction
  *Selection.Selection
  *Callback
  Action.i
  *Memory
EndStructure

Structure Background
  Name.s
  Sprite.i
EndStructure

Structure EventBank
  Name.s
  Library.i
  *Callbacks.Queue
EndStructure
Structure Event
  *Act.Act
  *Callback
  Triggered.i
  *Effects.Queue
  
  *LastEffectItem.QueueItem
  IdleTime.i
  
  *Memory
EndStructure
Structure EffectBank
  Name.s
  Library.i
  *Callbacks.Queue
EndStructure
Structure Effect
  *Event.Event
  *Callback
  *Memory
EndStructure
Structure Act
  Name.s
  Description.s
  *Events.Queue
  Remaining.i
  *SpecialEvent.Event[#SpecialEventCount]
  *ParallelItem.QueueItem
EndStructure

Structure Selection
  Size.i
  *Selected
EndStructure
Structure SelectedTile
  *Scene.Scene
  Layer.i
  X.i
  Y.i
EndStructure
Structure SelectedObject
  *Object.Object
EndStructure
Structure Selected
  *Scene.Scene
  X.i
  Y.i
EndStructure

Structure BlurFrame
  X.f
  Y.f
  Sprite.i
  Transparency.i
EndStructure
Structure Blur
  *Frames
  Frame.i
  Length.i
  Time.i
  DeltaTime.i
EndStructure
;}
  CompilerCase #Mode_Editor ;{

Structure Field
  X.i
  Y.i
  StructureUnion
    *Memory
    *RegisteredTile.RegisteredTile
    *RegisteredObject.RegisteredObject
  EndStructureUnion
EndStructure

Structure TilesetEntry
  *Entry.Entry
  *Registered.RegisteredTileset
  Number.i
EndStructure
Structure RegisteredTileset
  *Entry.TilesetEntry
  Width.i
  Height.i
  Image.i
  *Tiles
  *RegisteredTiles.Queue
EndStructure
Structure RegisteredTile
  *Tileset.RegisteredTileset
  *Item.QueueItem
  Index.i
  Sprite.i
  Sprite3D.i
  LastLayer.i
  References.i
  Number.i
EndStructure
Structure TileEntry
  *Registered.RegisteredTile
EndStructure
Structure Tile
  X.i
  Y.i
  *Registered.RegisteredTile
EndStructure

Structure Bank
  *Entries.Queue
  *RegisteredObjects.Queue
  Name.s
  Library.i
  Number.i
EndStructure
Structure ObjectEntry
  *Bank.Bank
  *Callback
  *Entry.Entry
  *Registered.RegisteredObject
  CallbackName.s
EndStructure
Structure RegisteredObject
  *Entry.ObjectEntry
  *BankItem.QueueItem
  Sprite.i
  Sprite3D.i
  References.i
  Number.i
EndStructure
Structure Object
  X.i
  Y.i
  *Registered.RegisteredObject
  
  *Scene.Scene
  Layer.i
  
  *Memory
  Number.i
EndStructure
Structure ObjectAction
  *Selection.Selection
  *Registered.RegisteredObject
  Action.i
  *Memory
EndStructure

Structure BackgroundEntry
  *Entry.Entry
  *Registered.RegisteredBackground
  Format.i
EndStructure
Structure RegisteredBackground
  *Entry.BackgroundEntry
  Sprite.i
  Number.i
  References.i
EndStructure

Structure Act
  *Item.QueueItem
  Name.s
  Description.s
  *Events.Queue
  References.i
  Number.i
EndStructure

Structure EventBank
  Name.s
  Library.i
  *Entries.Queue
  UsedEvents.i
  Number.i
EndStructure
Structure EventEntry
  *Entry.Entry
  *Bank.EventBank
  CallbackName.s
  *Callback
  References.i
  Number.i
EndStructure
Structure Event
  *Entry.EventEntry
  *Act.Act
  *Item.QueueItem
  *Effects.Queue
  *Memory
  Comment.s
  References.i
EndStructure

Structure EffectBank
  Name.s
  Library.i
  *Entries.Queue
  UsedEffects.i
  Number.i
EndStructure
Structure EffectEntry
  *Entry.Entry
  *Bank.EffectBank
  CallbackName.s
  *Callback
  References.i
  Number.i
EndStructure
Structure Effect
  *Entry.EffectEntry
  *Event.Event
  *Item.QueueItem
  *Memory
  Comment.s
EndStructure

Structure Scene
  ; Im Editor wird fr jedes Layer eine dynamisch verkettete
  ; Liste verwendet, die die erstellten Felder im jeweiligen
  ; Layer spiechert.
  ; Wird die Challange gespeichert, so werden diese Listen in
  ; Arrays umgewanldet, die in der Gre optimiert sind (und
  ; in der Geschwindigkeit ebenfalls).
  
  *BackgroundTiles1
  *BackgroundTiles2
  *BackgroundTiles3
  *Tiles
  *Grounds
  *ForegroundTiles
  *BackgroundObjects1
  *BackgroundObjects2
  *Objects
  *ForegroundObjects1
  *ForegroundObjects2
  
  *Background.RegisteredBackground
  
  AllowFly3.i
  AllowLumiFlight.i
  
  HorinzontalLoop.i
  VerticalLoop.i
  UnpassableTop.i
  
  *Act.Act
  
  X.i
  Y.i
  Width.i
  Height.i
  *Item.QueueItem
  
  *Main.Scene
  
  Number.i
EndStructure

Structure EditorChallange
  Folder.s
  Name.s
  Author.s
  Password.s
  Comment.s
  
  AuthorsHighscore.i
  
  Lives.i
  MaxEnergy.i
  Energy.i
  
  CanDoubleJump.i
  CanClimb.i
  CanFly.i
  CanShoot.i
  CanLoadShot.i
  CanLumiFlight.i
  
  Genre.i
  Difficulty.i
  Size.i
EndStructure
Structure Editor
  *SceneMap
  *Tilesets
  *Backgrounds
  
  Challange.EditorChallange
  
  *CurrentScene.Scene
  SceneX.i
  SceneY.i
  
  *OverScene.Scene
  OverSceneX.i
  OverSceneY.i
  
  Event.i
  Window.i
  Gadget.i
  wParam.i
  lParam.i
  
  OverX.i
  OverY.i
  TileX.i
  TileY.i
  LeftClickAllowed.i
  
  CameraX.i
  CameraY.i
  PushedX.i
  PushedY.i
  
  *CurrentTile.RegisteredTile
  *CurrentObject.RegisteredObject
  *Selection.Selection
  Layer.i
  
  Overwrite.i
  AutoLayerChange.i
  DisplaySingleLayer.i
  DisplayRuler.i
  
  UsingRectangle.i
  RectangleX.i
  RectangleY.i
  
  CursorSprite.i
  CursorSprite3D.i
  
  ButtonSprite_1x1.i
  ButtonSprite_1x1Green.i
  ButtonSprite_1x1Yellow.i
  ButtonSprite_1x1Red.i
  ButtonSprite_2x2.i
  ButtonSprite_2x1.i
  
  IconSprite_Tileset.i
  IconSprite_Object.i
  IconSprite_Scene.i
  IconSprite_Background.i
  IconSprite_B1.i
  IconSprite_B2.i
  IconSprite_B3.i
  IconSprite_W.i
  IconSprite_G.i
  IconSprite_F.i
  IconSprite_O1.i
  IconSprite_O2.i
  IconSprite_O.i
  IconSprite_O3.i
  IconSprite_O4.i
  IconSprite_SingleLayer.i
  IconSprite_Adapt.i
  IconSprite_Adjust.i
  IconSprite_Clear.i
  IconSprite_Grass.i
  IconSprite_StoryEditor.i
  IconSprite_Properties.i
  IconSprite_Challange.i
  IconSprite_Save.i
  IconSprite_Load.i
  IconSprite_Help.i
  
  Menu.i
  Menu_Tileset.i
  Menu_Object.i
  Menu_Scene.i
  Menu_Background.i
  Menu_B1.i
  Menu_B2.i
  Menu_B3.i
  Menu_W.i
  Menu_G.i
  Menu_F.i
  Menu_O1.i
  Menu_O2.i
  Menu_O.i
  Menu_O3.i
  Menu_O4.i
  Menu_SingleLayer.i
  Menu_Adapt.i
  Menu_Adjust.i
  Menu_Clear.i
  Menu_Grass.i
  Menu_StoryEditor.i
  Menu_Properties.i
  Menu_Challange.i
  Menu_Save.i
  Menu_Load.i
  Menu_Help.i
  
  BarSprite.i
  BarSprite3D.i
  DisabledBarSprite.i
  DisabledBarSprite3D.i
  
  StartSceneSprite.i
  
  TilesetSprite.i
  TilesetSprite3D.i
  
  LayerSprite.i
  LayerSprite3D.i
  
  *StartScene.Scene
  StartX.i
  StartY.i
  
  *UsedScenes.Queue
  *UsedObjects.Queue
EndStructure

Structure Selection
  *Queue.Queue
  MinX.i
  MinY.i
  MaxX.i
  MaxY.i
EndStructure
Structure Selected
  SceneX.i
  SceneY.i
  Layer.i
  X.i
  Y.i
EndStructure
;}
  CompilerCase #Mode_Menu ;{
Structure Layer
  *Buttons.Queue
  *Texts.Queue
EndStructure
Structure ButtonSprite
  Normal.i
  Over.i
  Press.i
EndStructure
Structure Button
  X.i
  Y.i
  Width.i
  Height.i
  *ButtonSprite.ButtonSprite
  IconSprite.i
  Transparency.i
  Hidden.i
  Text.s
EndStructure

Structure Text
  X.i
  Y.i
  *CharSet.CharSet
  Text.s
  Hidden.i
EndStructure
;}
CompilerEndSelect

Structure Animation
  *Frameset.AnimationFrameset
  Frame.i
  Set.i
  Duration.i
EndStructure
Structure AnimationFrameset
  *Frames
  FramesPerSet.i
  Sets.i
  FrameWidth.i
  FrameHeight.i
  *Registered.RegisteredAnimationFrameset
EndStructure
Structure RegisteredAnimationFrameset
  *Frameset.AnimationFrameset
  Name.s
  
  *Item.QueueItem
EndStructure

Structure Particle
  *Animation.Animation
  X.f
  Y.f
  AnimationSpeed.f
  Transparency.i
  *Callback
  
  *Item.QueueItem
EndStructure
Structure ParticleFramesets
  *ShotHit.AnimationFrameset
  *KilledEnemy.AnimationFrameset
  *Landed.AnimationFrameset
  *DoubleJump.AnimationFrameset
  *AirExplosion.AnimationFrameset
EndStructure

Structure Item
  Name.s
  *Callback
  Sprite3D.i
  Count.i
  MaxCount.i
EndStructure

Structure ViewRegion
  X.i
  Y.i
  Width.i
  Height.i
EndStructure

Structure ScreenMode
  Width.i
  Height.i
  BPP.i ;Bits per Pel
  Frequency.i
EndStructure
Structure Screen
  X.i
  Y.i
  Width.i
  Height.i
EndStructure



;******************************************
; Declarations
;******************************************

IncludePath "Sources"

  IncludeFile "Declarations.pb"



;******************************************
; Variables
;******************************************

Global Dim Namespace.s(#MaxNamespaces)

CompilerSelect #Mode
  CompilerCase #Mode_Game
    ; Player:
    Global Player.Player
    Global Camera.Camera
    
    ; Particles:
    Global *Particles.Queue = CreateQueue(@FreeParticle())
    Global Particle.ParticleFramesets
    
    ; Sprites:
    Global ShieldSprite.Sprite
    Global GreenBottom.Sprite, RedBottom.Sprite
    
    ; Acts, Events & Effects:
    Global *EventBanks.Queue   = CreateQueue()
    Global *EffectBanks.Queue  = CreateQueue()
    Global *Acts.Queue         = CreateQueue()
    Global *CurrentAct.Act
    Global *ParallelActs.Queue = CreateQueue(#Queue_SaveMemory)
    Global Dim *SpecialEventCallback(#SpecialEventCount - 1)
    
    ; Save & Load:
    Global Dim Tile(0), Dim *Object(0), Dim Background(0), Dim *Event(0), Dim *Effect(0)
    Global Dim *IO_SceneObject.Object(0), IO_SceneObjectCount.i, IO_SceneObjectNumber.i
    Global Dim *IO_Act.Act(0)
    
    ; Items:
    Global *Items.Queue = CreateQueue()
    Global *CoinItem.Item
    Global *SuperCoinItem.Item
    
    ; Tiny Messages:
    Global *TinyMessages.Queue = CreateQueue(@FreeTinyMessage())
    
    ; Charsets:
    Global *StandardCharSet.CharSet, *TinyMessageCharSet.CharSet
    
    ; Stats:
    Global Quit.i = #False
    Global ControlMode.i
  CompilerCase #Mode_Editor
    Global Editor.Editor
    
    ; Acts, Events & Effects:
    Global *EventBanks.Queue  = CreateQueue()
    Global *EffectBanks.Queue = CreateQueue()
    Global *Acts.Queue        = CreateQueue(@FreeAct())
    
    ; Save & Load:
    Global IO_OutputFile.i
    Global IO_SceneMap_Left.i, IO_SceneMap_Right.i, IO_SceneMap_Top.i, IO_SceneMap_Bottom.i
    Global IO_SceneMap_Width.i, IO_SceneMap_Height.i
    Global IO_TilesetNumber.i, IO_TileNumber.i, IO_BankNumber.i, IO_ObjectNumber.i, IO_BackgroundNumber.i
    Global IO_SceneNumber.i, IO_SceneObjectNumber.i, IO_SceneObjectCount.i
    Global IO_EventNumber.i, IO_EventBankNumber.i, IO_EffectNumber.i, IO_EffectBankNumber.i, IO_ActNumber.i
    Global Dim *IO_Tile.RegisteredTile(0)
    Global Dim *IO_Object.RegisteredObject(0)
    Global Dim *IO_SceneObject.Object(0)
    Global Dim *IO_Background.RegisteredBackground(0)
    Global Dim *IO_Event.EventEntry(0)
    Global Dim *IO_Effect.EffectEntry(0)
    Global Dim *IO_Act.Act(0)
  CompilerCase #Mode_Menu
    ; Mouse:
    Global Mouse_X.i, Mouse_Y.i, Mouse_FieldX.i, Mouse_FieldY.i
    
    ; Gadgets:
    Global *CurrentLayer.Layer, *PreviousLayer.Layer
    Global *ActiveButton.Button, *PressedButton.Button
    Global ButtonTip.s
    
    ; Charsets:
    Global *StandardCharSet.CharSet
CompilerEndSelect

Global Account.Account

Global *Tilesets.Queue    = CreateQueue()
Global *Banks.Queue       = CreateQueue()
Global *Backgrounds.Queue = CreateQueue()

Global *ChallangeFolder.Entry  = CreateEntryFolder("Challanges")
Global *TilesetFolder.Entry    = CreateEntryFolder("Tilesets")
Global *ObjectFolder.Entry     = CreateEntryFolder("Objects")
Global *BackgroundFolder.Entry = CreateEntryFolder("Backgrounds")
Global *EventFolder.Entry      = CreateEntryFolder("Events")
Global *EffectFolder.Entry     = CreateEntryFolder("Effects")

Global ResultString.s
Global ResultInteger.i

Global IO_InputFile.i, IO_OutputFile.i
Global IO_SceneX.i, IO_SceneY.i
Global IO_X.i, IO_Y.i

Global Replay.i, ReplayFile.i, RecordFile.i, RecordFilename.s

Global *ChallangeFolders.Queue = CreateQueue()
Global Challange.CurrentChallange

Global Screen.ScreenMode, InnerScreen.Screen, PreviousScreen.ScreenMode
Global ScreenMode.i = #ScreenMode_Fullscreen

Global BarSprite.i

Global WhiteLight.Sprite
Global YellowLight.Sprite
Global CyanLight.Sprite
Global SmallWhiteLight.Sprite

Global RandomSeed.i



;******************************************
; Procedures
;******************************************

Procedure.i LoadPreferences()
  If OpenPreferences("Data\Settings.txt")
    PreferenceGroup("Screen")
      ScreenMode = ReadPreferenceInteger("Windowed", #True)
    ClosePreferences()
  ElseIf CreatePreferences("Data\Settings.txt")
    GetScreenSettings(@PreviousScreen\Width, @PreviousScreen\Height, @PreviousScreen\BPP, @PreviousScreen\Frequency)
    
    PreferenceGroup("Screen")
      WritePreferenceInteger("Windowed" , #False)
    ClosePreferences()
  EndIf
EndProcedure

Procedure.i LoadSprite3D(*Sprite.Sprite, Filename.s)
  *Sprite\Sprite   = LoadSprite(#PB_Any, Filename + ".png", #PB_Sprite_Texture|#PB_Sprite_Memory|#PB_Sprite_AlphaBlending)
  *Sprite\Sprite3D = CreateSprite3D(#PB_Any, *Sprite\Sprite)
EndProcedure
Procedure.i LoadClippedSprite(Filename.s, X.i, Y.i, Width.i, Height.i)
  Protected Image.i, ClippedImage.i
  
  Image        = LoadImage(#PB_Any, Filename, 32)
  ClippedImage = GrabImage(Image, #PB_Any, X, Y, Width, Height)
  SaveImage(ClippedImage, "Graphics\Temp.png", #PB_ImagePlugin_PNG, 0, 32)
  
  FreeImage(Image)
  FreeImage(ClippedImage)
  
  ProcedureReturn LoadSprite(#PB_Any, "Graphics\Temp.png", #PB_Sprite_Texture|#PB_Sprite_Memory|#PB_Sprite_AlphaBlending)
EndProcedure

Macro LayerQueue(Scene, Layer)
  PeekI(Scene + Layer * SizeOf(Integer))
EndMacro

Procedure.i FileExists(Filename.s)
  If FileSize(Filename) = -1
    ProcedureReturn #False
  Else
    ProcedureReturn #True
  EndIf
EndProcedure
Procedure.s GetFile(Filename.s)
  Protected Index.i
  
  For Index = 0 To #MaxNamespaces
    If FileExists(Namespace(Index) + Filename)
      ProcedureReturn Namespace(Index) + Filename
    EndIf
  Next
  
  ProcedureReturn ""
EndProcedure

Procedure.i DrawShadowText(X.i, Y.i, Text.s, FrontColor.i = #White, BackColor.i = #Black)
  DrawingMode(#PB_2DDrawing_Transparent)
  DrawText(X + 1, Y + 1, Text, BackColor)
  DrawText(X, Y, Text, FrontColor)
EndProcedure

Procedure.i OpenPreviewScreen()
  Protected Window.i, Image.i, Event.i, Start.i
  
  Image  = LoadImage(#PB_Any, "Graphics\GMetrica.png")
  Window = OpenWindow(#PB_Any, #PB_Ignore, #PB_Ignore, ImageWidth(Image), ImageHeight(Image), "GMetrica", #PB_Window_ScreenCentered)
  
  SetWindowBackground(Window, Image)
  
  Start = ElapsedMilliseconds()
  Repeat
    Event = WindowEvent()
    Select Event
      Case #WM_LBUTTONDOWN
        MoveWindowByMouse(Window)
    EndSelect
    If ElapsedMilliseconds() - Start > 2500
      Break
    EndIf
    Delay(1)
  ForEver
  
  CloseWindow(Window)
  FreeImage(Image)
EndProcedure

CompilerIf #PB_Compiler_Debugger = #False
  Procedure.i Error()
    CloseGMetricaScreen()
    MessageRequester("Error", ErrorMessage())
    End
  EndProcedure
  OnErrorCall(@Error())
CompilerEndIf



;******************************************
; Sources
;******************************************

IncludePath "Sources"
  
  CompilerSelect #Mode
    CompilerCase #Mode_Game
      IncludeFile "CharSet.pb"
      IncludeFile "Item.pb"
      IncludeFile "TinyMessage.pb"
      IncludeFile "Cheat.pb"
      IncludeFile "Entry.pb"
      IncludeFile "Tile.pb"
      IncludeFile "Object.pb"
      IncludeFile "ObjectAction.pb"
      IncludeFile "Scene.pb"
      IncludeFile "Background.pb"
      IncludeFile "Animation.pb"
      IncludeFile "GameOver.pb"
      IncludeFile "Replay.pb"
      IncludeFile "GameAct.pb"
      IncludeFile "Player.pb"
      IncludeFile "Collision.pb"
      IncludeFile "Particle.pb"
      IncludeFile "Challange.pb"
      IncludeFile "NewGame.pb"
      IncludeFile "Highscore.pb"
      IncludeFile "Camera.pb"
      IncludeFile "Interface.pb"
      IncludeFile "Selection.pb"
      IncludeFile "Blur.pb"
      
      IncludeFile "Exports.pb"
      IncludeFile "ExportTable.pb"
      
      IncludeFile "Game.pb"
    CompilerCase #Mode_Editor
      IncludeFile "CharSet.pb"
      IncludeFile "Menu\Gadget.pb"
      IncludeFile "Patcher.pb"
      IncludeFile "Entry.pb"
      IncludeFile "Background.pb"
      IncludeFile "Tile.pb"
      IncludeFile "Object.pb"
      IncludeFile "Challange.pb"
      IncludeFile "Highscore.pb"
      IncludeFile "Scene.pb"
      IncludeFile "EditorAct.pb"
      IncludeFile "Interface.pb"
      IncludeFile "ObjectAction.pb"
      IncludeFile "Editor.pb"
      
      IncludeFile "Exports.pb"
      IncludeFile "ExportTable.pb"
    CompilerCase #Mode_Menu
      IncludeFile "Entry.pb"
      IncludeFile "Highscore.pb"
      IncludeFile "Challange.pb"
      IncludeFile "CharSet.pb"
      IncludeFile "Menu\Main.pb"
  CompilerEndSelect
  
  IncludeFile "Account.pb"
  IncludeFile "Sprite.pb"
  IncludeFile "Screen.pb"
  IncludeFile "Screens.pb"



;******************************************
; Main
;******************************************

Define String.s, Index.i, Integer.i, ParameterCount.i

UsePNGImageDecoder()
UsePNGImageEncoder()

InitSprite()
InitSprite3D()

LoadPreferences()

CompilerSelect #Mode
  CompilerCase #Mode_Editor
    Editor()
  CompilerCase #Mode_Game
    ; Namensrume festlegen (damit auch lokale Daten der Herausforderung geladen werden knnen):
    Namespace(0) = GetCurrentDirectory()
    Namespace(1) = GetCurrentDirectory()
    Namespace(2) = GetCurrentDirectory()
    
    ; Screen ffnen:
    OpenGMetricaScreen(#SceneWidth*#FieldSize, #SceneHeight*#FieldSize + #FieldSize*2)
    
    InitializeControl()
    InitializePlayer()
    LoadChallanges("Challanges\", *ChallangeFolder)
    
    RandomSeed = #None
    
    ; Parameter durchlaufen:
    ParameterCount = CountProgramParameters()
    If ParameterCount
      For Index = 0 To ParameterCount - 1
        Select ProgramParameter(Index)
          Case "load"
            ; Eine angegebene Herausforderung laden:
            If ProgramParameter(Index + 1)
              If Not NewGame(GetChallange(ProgramParameter(Index + 1)))
                CloseGMetricaScreen()
                End
              EndIf
              LoadPlayerSettings()
              Index + 1
            Else
              MessageRequester("Error", "No challange specified by program parameters.", #MB_ICONERROR)
              CloseGMetricaScreen()
              End
            EndIf
          Case "replay"
            ; Ein Replay aus der jeweiligen Herausforderung abspielen:
            If ProgramParameter(Index + 1)
              Replay = 2
              If Not UnpackReplay(ProgramParameter(Index + 1))
                End
              EndIf
              Index + 1
            Else
              MessageRequester("Error", "No record specified by program parameters.", #MB_ICONERROR)
              CloseGMetricaScreen()
              End
            EndIf
          Case "record"
            ; Ein Replay in der jeweiligen Herausforderung aufnehmen:
            If ProgramParameter(Index + 1)
              Replay = 1
              RecordFile = CreateFile(#PB_Any, "Unpacked.dat")
              If Not RecordFile
                MessageRequester("Error", "Can not create record buffer: " + #Q + "Unpacked.dat" + #Q, #MB_ICONERROR)
                CloseGMetricaScreen()
                End
              EndIf
              RecordFilename = ProgramParameter(Index + 1)
              RandomSeed = Random(2147483647)
              WriteInteger(RecordFile, RandomSeed)
              Index + 1
            Else
              MessageRequester("Error", "No record filename specified by program parameters.", #MB_ICONERROR)
              CloseGMetricaScreen()
              End
            EndIf
        EndSelect
      Next
    Else
      ; Keine Parameter gegeben:
      MessageRequester("Warning", "You should specify parameters.", #MB_ICONERROR)
      CloseGMetricaScreen()
      End
    EndIf
    
    ; Zeichenstze laden:
    *StandardCharSet    = LoadCharSet("Fonts\Standard", 10, 22)
    *TinyMessageCharSet = LoadCharSet("Fonts\Mini", 5, 7)
    
    ; Partikel laden:
    With Particle
      \ShotHit      = LoadAnimationFrameset("Particles\ShotHit", 18, 18)
      \KilledEnemy  = LoadAnimationFrameset("Particles\KilledEnemy", 37, 36)
      \Landed       = LoadAnimationFrameset("Particles\Landed", 18, 6)
      \DoubleJump   = LoadAnimationFrameset("Particles\DoubleJump", 20, 15)
      \AirExplosion = LoadAnimationFrameset("Particles\AirExplosion", 30, 30)
    EndWith
    
    ; Leuchteffekte fr [BILD RUNTER] laden:
    LoadSprite3D(@GreenBottom, "Graphics\Player\GreenBottom")
    LoadSprite3D(@RedBottom  , "Graphics\Player\RedBottom")
    
    ; Lichteffekte laden:
    LoadSprite3D(@WhiteLight     , "Graphics\Particles\WhiteLight")
    LoadSprite3D(@YellowLight    , "Graphics\Particles\YellowLight")
    LoadSprite3D(@CyanLight      , "Graphics\Particles\CyanLight")
    LoadSprite3D(@SmallWhiteLight, "Graphics\Particles\SmallWhiteLight")
    
    ; Sonstiges laden:
    LoadSprite3D(@ShieldSprite, "Graphics\Player\Shield")
    
    ; Hintergrund des HUD laden:
    BarSprite = LoadSprite(#PB_Any, "Graphics\Menu\Bar.bmp")
    
    CloseLoadingScreen()
    
    Play()
    
    If Replay = 1
      PackReplay()
    ElseIf Replay = 2
      CloseFile(ReplayFile)
    EndIf
    
    CloseGMetricaScreen()
  CompilerCase #Mode_Menu
    OpenPreviewScreen()
    OpenGMetricaScreen(#SceneWidth * #FieldSize, #SceneHeight * #FieldSize)
    
    *StandardCharSet = LoadCharSet("Fonts\Standard", 10, 22)
    
    Menu()
    
    CloseGMetricaScreen()
CompilerEndSelect



End
; IDE Options = PureBasic 4.51 (Windows - x86)
; CursorPosition = 733
; FirstLine = 311
; Folding = EqACEEAAAAAAAAMAs
; EnableXP
; Executable = ..\Test.exe