CompilerSelect #Mode
  CompilerCase #Mode_Game ;{

Procedure.i LoadBackground(Name.s)
  Protected *Background.Background, *Item.QueueItem
  Protected Image.i, IndexX.i, IndexY.i
  
  *Item = *Backgrounds\First
  While *Item
    *Background = *Item\Memory
    If *Background\Name = Name
      ProcedureReturn *Background\Sprite
    EndIf
    _Move(*Item, N)
  Wend
  
  *Background = _New(Background)
  AppendQueueItem(*Backgrounds, *Background)
  
  *Background\Name = Name
  
  Image  = LoadImage(#PB_Any, GetFile("Graphics\Backgrounds\" + Name + ".bmp"))
  If Not Image
    MessageRequester("Error", "Background does not exist: " + #Q + Name + ".bmp" + #Q, #MB_ICONERROR)
    ProcedureReturn #False
  EndIf
  
  *Background\Sprite = CreateSprite(#PB_Any, #SceneWidth*#FieldSize, #SceneHeight*#FieldSize)
  StartDrawing(SpriteOutput(*Background\Sprite))
  
    If ImageHeight(Image) = #SceneHeight * #FieldSize
      If ImageWidth(Image) = 1
        DrawImage(ImageID(Image), 0, 0, #SceneWidth*#FieldSize, #SceneHeight*#FieldSize)
      ElseIf ImageWidth(Image) = #FieldSize
        For IndexX = 0 To #SceneWidth - 1
          DrawImage(ImageID(Image), IndexX * #FieldSize, 0)
        Next
      ElseIf ImageWidth(Image) = #SceneWidth * #FieldSize
        DrawImage(ImageID(Image), 0, 0)
      Else
        MessageRequester("Error", "Invalid background size: " + Str(ImageWidth(Image)) + "x" + Str(ImageHeight(Image)), #MB_ICONERROR)
        ProcedureReturn #False
      EndIf
    ElseIf ImageHeight(Image) = #FieldSize
      If ImageWidth(Image) = #FieldSize
        For IndexY = 0 To #SceneHeight - 1
          For IndexX = 0 To #SceneWidth - 1
            DrawImage(ImageID(Image), IndexX * #FieldSize, IndexY * #FieldSize)
          Next
        Next
      Else
        MessageRequester("Error", "Invalid background size: " + Str(ImageWidth(Image)) + "x" + Str(ImageHeight(Image)), #MB_ICONERROR)
        ProcedureReturn #False
      EndIf
    Else
      MessageRequester("Error", "Invalid background size: " + Str(ImageWidth(Image)) + "x" + Str(ImageHeight(Image)), #MB_ICONERROR)
      ProcedureReturn #False
    EndIf
  
  StopDrawing()
  FreeImage(Image)
  
  ProcedureReturn *Background\Sprite
EndProcedure
Procedure.i SetSceneBackground(*Scene.Scene, Sprite.i)
  *Scene\Background = Sprite
EndProcedure
Procedure.i DisplayBackground(Sprite.i)
  DisplaySprite(Sprite, 0, 0)
EndProcedure

Procedure.i GetBackground(Name.s)
  Protected *Item.QueueItem, *Background.Background
  
  *Item = *Backgrounds\First
  While *Item
    *Background = *Item\Memory
    If *Background\Name = Name
      ProcedureReturn *Background
    EndIf
    _Move(*Item, N)
  Wend
  
  ProcedureReturn #Null
EndProcedure

;}
  CompilerCase #Mode_Editor ;{

Procedure.i LoadBackgrounds(Path.s, *Parent.Entry)
  Protected Directory.i, *Background.BackgroundEntry
  Protected Name.s, Image.i
  
  Directory = ExamineDirectory(#PB_Any, Path, "*.*")
  If Directory
    While NextDirectoryEntry(Directory)
      Name = DirectoryEntryName(Directory)
      Select DirectoryEntryType(Directory)
        Case #PB_DirectoryEntry_File
          Name = StringField(Name, 1, ".")
          If Not GetEntry(*Parent, Name)
            *Background = _New(BackgroundEntry)
            AppendQueueItem(*Backgrounds, *Background)
            
            *Background\Entry = CreateEntryFile(GetFilePart(Name), *Background, *Parent)
          EndIf
        Case #PB_DirectoryEntry_Directory
          If Name <> "." And Name <> ".."
            If Not LoadBackgrounds(Path + Name + "\", CreateEntryFolder(Name, *Parent))
              ProcedureReturn #False
            EndIf
          EndIf
      EndSelect
    Wend
    FinishDirectory(Directory)
    ProcedureReturn #True
  Else
    MessageRequester("Error", "Can not find folder: " + #Q + Path + #Q, #MB_ICONERROR)
    End
  EndIf
EndProcedure

Procedure.s BackgroundPath(*Background.BackgroundEntry)
  ProcedureReturn SStr(EntryPath(*Background\Entry), -Len("Backgrounds\"))
EndProcedure
Procedure.s BackgroundFilename(*Background.BackgroundEntry)
  ProcedureReturn "Graphics\" + EntryPath(*Background\Entry) + ".bmp"
EndProcedure
Procedure.i GetBackground(Name.s)
  Protected *Entry.Entry
  
  *Entry = GetEntryByPath(*BackgroundFolder, Name)
  ProcedureReturn *Entry\Memory
EndProcedure

Procedure.i GetBackgroundFormat(*Background.BackgroundEntry, Image.i)
  If Not *Background\Format
    If ImageHeight(Image) = #SceneHeight * #FieldSize
      If ImageWidth(Image) = 1
        *Background\Format = #Background_Format_Gradient
      ElseIf ImageWidth(Image) = #FieldSize
        *Background\Format = #Background_Format_VerticalLine
      ElseIf ImageWidth(Image) = #SceneWidth * #FieldSize
        *Background\Format = #Background_Format_Full
      EndIf
    ElseIf ImageHeight(Image) = #FieldSize
      If ImageWidth(Image) = #FieldSize
        *Background\Format = #Background_Format_Field
      EndIf
    EndIf
    
    If Not *Background\Format
      MessageRequester("Error", "Invalid background size: " + Str(ImageWidth(Image)) + "x" + Str(ImageHeight(Image)), #MB_ICONERROR)
      ProcedureReturn #False
    EndIf
  EndIf
  
  ProcedureReturn *Background\Format
EndProcedure
Procedure.i DrawBackground(*Background.BackgroundEntry, Image.i)
  Protected IndexX.i, IndexY.i
  
  Select *Background\Format
    Case #Background_Format_Gradient
      DrawImage(ImageID(Image), 0, 0, #SceneWidth*#FieldSize, #SceneHeight*#FieldSize)
    Case #Background_Format_Full
      DrawImage(ImageID(Image), 0, 0)
    Case #Background_Format_VerticalLine
      For IndexX = 0 To #SceneWidth - 1
        DrawImage(ImageID(Image), IndexX * #FieldSize, 0)
      Next
    Case #Background_Format_Field
      For IndexY = 0 To #SceneHeight - 1
        For IndexX = 0 To #SceneWidth - 1
          DrawImage(ImageID(Image), IndexX * #FieldSize, IndexY * #FieldSize)
        Next
      Next
  EndSelect
EndProcedure
Procedure.i LoadBackgroundSprite(*Background.BackgroundEntry)
  Protected Sprite.i, IndexX.i, IndexY.i, Image.i
  
  If *Background\Registered And *Background\Registered\Sprite
    ProcedureReturn CopySprite(*Background\Registered\Sprite, #PB_Any)
  EndIf
  
  Image = LoadImage(#PB_Any, GetFile(BackgroundFilename(*Background)))
  If Not Image
    MessageRequester("Error", "Can not load background image:" + #Q + BackgroundFilename(*Background) + #Q)
    ProcedureReturn #False
  EndIf
  
  Sprite = CreateSprite(#PB_Any, #SceneWidth * #FieldSize, #SceneHeight * #FieldSize)
  
  StartDrawing(SpriteOutput(Sprite))
  
  GetBackgroundFormat(*Background, Image)
  DrawBackground(*Background, Image)
  
  StopDrawing()
  
  FreeImage(Image)
  ProcedureReturn Sprite
EndProcedure
Procedure.i FreeBackgroundSprite(Sprite, *Background.BackgroundEntry)
  If *Background\Registered And *Background\Registered\Sprite = Sprite
    ProcedureReturn #False
  Else
    ProcedureReturn FreeSprite(Sprite)
  EndIf
EndProcedure

Procedure.i LoadRegBackground(*Entry.BackgroundEntry)
  Protected *Background.RegisteredBackground, *Item.QueueItem
  
  If *Entry\Registered
    ProcedureReturn *Entry\Registered
  EndIf
  
  *Background = _New(RegisteredBackground)
  
  *Background\Entry  = *Entry
  *Background\Sprite = LoadBackgroundSprite(*Entry)
  
  *Entry\Registered = *Background
  
  ProcedureReturn *Background
EndProcedure
Procedure.i FreeRegBackground(*Background.RegisteredBackground)
  FreeSprite(*Background\Sprite)
  *Background\Entry\Registered = #Null
  
  _Free(*Background)
  ProcedureReturn #True
EndProcedure

Procedure.i UseRegBackground(*Background.RegisteredBackground)
  *Background\References + 1
  ProcedureReturn *Background\References
EndProcedure
Procedure.i ReleaseRegBackground(*Background.RegisteredBackground)
  If *Background\References <= 1
    ProcedureReturn FreeRegBackground(*Background)
  Else
    *Background\References - 1
    ProcedureReturn #False
  EndIf
EndProcedure
Procedure.i CheckRegBackground(*Background.RegisteredBackground)
  If *Background\References = 0
    ProcedureReturn FreeRegBackground(*Background)
  Else
    ProcedureReturn #False
  EndIf
EndProcedure

Procedure.i ChangeBackground(*Scene.Scene, *Background.RegisteredBackground, IncludeChildren.i = #True)
  Protected X.i, Y.i, *Main.Scene
  
  If Not *Scene
    ProcedureReturn #False
  EndIf
  
  *Main = *Scene\Main
  
  If IncludeChildren And *Main\Width * *Main\Height > 1
    For X = *Main\X To *Main\X + *Main\Width - 1
      For Y = *Main\Y To *Main\Y + *Main\Height - 1
        ChangeBackground(GetScene(X, Y), *Background, #False)
      Next
    Next
  Else
    If *Scene\Background <> *Background
      If *Scene\Background
        ReleaseRegBackground(*Scene\Background)
      EndIf
      *Scene\Background = *Background
      If *Background
        UseRegBackground(*Background)
      EndIf
    EndIf
  EndIf
  
  ProcedureReturn #True
EndProcedure
Procedure.i ChangeAllBackgrounds(*Background.RegisteredBackground)
  Protected *Item.QueueItem
  
  *Item = Editor\UsedScenes\First
  While *Item
    ChangeBackground(*Item\Memory, *Background, #False)
    _Move(*Item, N)
  Wend
EndProcedure

Procedure.i DrawTilesetBackground()
  If Editor\CurrentScene\Background
    DisplaySprite(Editor\CurrentScene\Background\Sprite, 0, #EditorScreenHeight - 192)
  EndIf
EndProcedure

;}
CompilerEndSelect

; IDE Options = PureBasic 4.50 (Windows - x86)
; CursorPosition = 274
; FirstLine = 94
; Folding = -AAy
; EnableXP