;**************************************************************
;     Challange
;**************************************************************
;   Copyright (c) 2010 Josef Sniatecki
;
; This program is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License
; As published by the Free Software Foundation; either version
; 3 of the License, Or (at your option) any later version.
;
; This program is distributed in the hope that it will be
; useful, but WITHOUT ANY WARRANTY; without even the implied
; warranty of MERCHANTABILITY Or FITNESS For A PARTICULAR
; PURPOSE. See the GNU General Public License For more details.
;
; You should have received a copy of the GNU General Public
; License along With this program;
; If Not, see "http://www.gnu.org/licenses/".
;**************************************************************



Procedure.i CreateChallange(Path.s, Name.s)
  Protected *Challange.Challange, *Entry.Entry
  
  If Not CreateDirectory("Challanges\" + Path + Name) And FileSize("Challanges\" + Path + Name) <> -1
    MessageRequester("Error", "Can not create challange: " + #Q + "Challanges\" + Path + Name + #Q, #MB_ICONERROR)
    ProcedureReturn #False
  EndIf
  
  *Entry = GetEntryByPath(*ChallangeFolder, SStr(Path, 1))
  If Not *Entry
    MessageRequester("Error", "Challange folder not found: " + #Q + "Challanges\" + Path + #Q, #MB_ICONERROR)
    ProcedureReturn #False
  EndIf
  
  *Challange       = _New(Challange)
  *Challange\Entry = CreateEntryFile(Name, *Challange, *Entry)
  
  *Challange\Name = Name
  
  ProcedureReturn *Challange
EndProcedure
Procedure.i FreeChallange(*Challange.Challange)
  ClearStructure(*Challange, Challange)
  _Free(*Challange)
  
  ProcedureReturn #True
EndProcedure
Procedure.i RemoveChallange(*Challange.Challange)
  DeleteDirectory(ChallangePath(*Challange), "", #PB_FileSystem_Recursive)
  FreeEntry(*Challange\Entry)
  ProcedureReturn FreeChallange(*Challange)
EndProcedure
Procedure.i RenameChallange(*Challange.Challange, NewName.s)
  NewName = SStr(EntryPath(*Challange\Entry), Len(*Challange\Entry\Name)) + NewName + "\"
  
  If RenameFile(ChallangePath(*Challange), NewName)
    ProcedureReturn #True
  Else
    MessageRequester("Error", "Can not rename challange: " + #Q + ChallangePath(*Challange) + #Q + " to" + #N + #Q + NewName + "\" + #Q, #MB_ICONERROR)
    ProcedureReturn #False
  EndIf
EndProcedure

Procedure.s ChallangePath(*Challange.Challange)
  ProcedureReturn EntryPath(*Challange\Entry) + "\"
EndProcedure
Procedure.s ChallangeFilename(*Challange.Challange)
  ProcedureReturn SStr(EntryPath(*Challange\Entry), -Len("Challanges\"))
EndProcedure
Procedure.i GetChallange(Path.s)
  Protected *Entry.Entry
  
  *Entry = GetEntryByPath(*ChallangeFolder, Path)
  If *Entry
    ProcedureReturn *Entry\Memory
  Else
    ProcedureReturn #Null
  EndIf
EndProcedure
Procedure.i IsChallangeUnlocked(*Challange.Challange)
  If (Not *Challange\Locked) Or GetHighscore(*Challange, Account\Name)
    ProcedureReturn #True
  Else
    ProcedureReturn #False
  EndIf
EndProcedure
Procedure.i UnlockChallange(*Challange.Challange)
  If *Challange\Locked And Not GetHighscore(*Challange, Account\Name)
    NewHighscore(*Challange, Account\Name, 0, 0, 0)
    ProcedureReturn #True
  Else
    ProcedureReturn #False
  EndIf
EndProcedure

Procedure.i LoadChallangeChecksums(*Challange.Challange)
  Protected File.i, Path.s = ChallangePath(*Challange)
  
  UnpackMap(*Challange)
  
  File = ReadFile(#PB_Any, Path + "UnpackedMap.bin")
  If Not File
    MessageRequester("Error", "Challange does not have a map: " + #Q + Path + #Q, #MB_ICONERROR)
    ProcedureReturn #False
  EndIf
  
  *Challange\HeaderChecksum   = ReadNTString(File)
  *Challange\ScoresChecksum   = ReadNTString(File)
  *Challange\SettingsChecksum = ReadNTString(File)
  
  CloseFile(File)
  DeleteFile(Path + "UnpackedMap.bin")
  
  ProcedureReturn #True
EndProcedure
Procedure.i LoadChallangeHeader(*Challange.Challange)
  Protected Path.s = ChallangePath(*Challange), File.i
  
  File = ReadFile(#PB_Any, Path + "Header.bin")
  If Not File
    MessageRequester("Error", "Header file not found for challange in " + #Q + Path + #Q + ".", #MB_ICONERROR)
    ProcedureReturn #False
  EndIf
  
  *Challange\Name     = ReadNTString(File)
  *Challange\Author   = ReadNTString(File)
  *Challange\Comment  = ReadNTString(File)
  *Challange\Password = ReadNTString(File)
  *Challange\Locked   = ReadByte(File)
  
  ;TODO: Kategorien auslesen
  
  CloseFile(File)
  
  If MD5FileFingerprint(Path + "Header.bin") <> *Challange\HeaderChecksum
    MessageRequester("Error", "The header file of " + #Q + Path + #Q + " is modified by a third person!", #MB_ICONERROR)
    ProcedureReturn #False
  EndIf
  
  ProcedureReturn #True
EndProcedure
Procedure.i LoadChallangeHighscores(*Challange.Challange)
  Protected Path.s = ChallangePath(*Challange), File.i
  Protected Count.i, *Highscore.Highscore
  
  If *Challange\Highscores
    ClearQueue(*Challange\Highscores)
  Else
    *Challange\Highscores = CreateQueue(@FreeHighscore())
  EndIf
  
  File = ReadFile(#PB_Any, Path + "Scores.bin")
  If File
    Count = ReadInteger(File)
    While Count
      *Highscore = _New(Highscore)
      AppendQueueItem(*Challange\Highscores, *Highscore)
      
      *Highscore\Player     = ReadNTString(File)
      *Highscore\Score      = ReadInteger(File)
      *Highscore\Medal      = ReadInteger(File)
      *Highscore\SuperCoins = ReadInteger(File)
      
      If (Not *Challange\BestHighscore) Or (*Challange\BestHighscore\Score < *Highscore\Score)
        *Challange\BestHighscore = *Highscore
      EndIf
      Count - 1
    Wend
    
    CloseFile(File)
    
    If MD5FileFingerprint(Path + "Scores.bin") <> *Challange\ScoresChecksum
      MessageRequester("Errro", "The highscores of " + #Q + Path + #Q + " are modified by a thrid person!", #MB_ICONERROR)
      ProcedureReturn #False
    EndIf
  EndIf
  
  ProcedureReturn #True
EndProcedure

CompilerIf #Mode = #Mode_Game
  Procedure.i LoadChallangeSettings(*Challange.Challange)
    Protected File.i, Path.s = ChallangePath(*Challange)
    
    File = ReadFile(#PB_Any, Path + "Settings.bin")
    If Not File
      MessageRequester("Error", "Settings not found: " + #Q + Path + #Q, #MB_ICONERROR)
      ProcedureReturn #False
    EndIf
    
    With Challange
      
      \Lives     = ReadInteger(File)
      \MaxEnergy = ReadInteger(File)
      \Energy    = ReadInteger(File)
      
      \CanDoubleJump = ReadInteger(File)
      \CanClimb      = ReadInteger(File)
      \CanLumiFlight = ReadInteger(File)
      \CanFly        = ReadInteger(File)
      \CanShoot      = ReadInteger(File)
      \CanLoadShot   = ReadInteger(File)
      
    EndWith
    
    CloseFile(File)
    
    If MD5FileFingerprint(Path + "Settings.bin") <> *Challange\SettingsChecksum
      MessageRequester("Errro", "The settings of " + #Q + Path + #Q + " are modified by a thrid person!", #MB_ICONERROR)
      ProcedureReturn #False
    EndIf
    
    ProcedureReturn #True
  EndProcedure
CompilerEndIf

Procedure.i LoadChallange(Path.s, *Entry.Entry)
  Protected *Challange.Challange
  
  *Challange       = _New(Challange)
  *Challange\Entry = *Entry
  *Entry\Memory    = *Challange
  
  *Challange\Type = 1 ;TODO
  
  If LoadChallangeChecksums(*Challange) And LoadChallangeHeader(*Challange) And LoadChallangeHighscores(*Challange)
    ProcedureReturn *Challange
  Else
    ProcedureReturn #False
  EndIf
EndProcedure
Procedure.i LoadChallanges(Path.s, *Parent.Entry)
  Protected Directory.i, Name.s
  
  Directory = ExamineDirectory(#PB_Any, Path, "*.*")
  If Directory
    While NextDirectoryEntry(Directory)
      Select DirectoryEntryType(Directory)
        Case #PB_DirectoryEntry_Directory
          Name = DirectoryEntryName(Directory)
          If Name <> "." And Name <> ".."
            If FileExists(Path + Name + "\" + "Map.bin")
              ; Eine einzelne Herausforderung:
              If Not LoadChallange(Path + Name + "\", CreateEntryFile(Name, #Null, *Parent))
                Break
              EndIf
            ElseIf FileExists(Path + Name + "\" + "Bundle.bin")
              ;TODO
              ; Ein Bndel von Herausforderungen:
  ;             If Not LoadChallangeBundle(Path + Name + "\", CreateEntryFile(Name, #Null, *Parent))
  ;               Break
  ;             EndIf
            Else
              ; Ordner von Herausforderungen und Bndeln:
              If Not LoadChallanges(Path + Name + "\", CreateEntryFolder(Name, *Parent))
                Break
              EndIf
            EndIf
          EndIf
      EndSelect
    Wend
    
    FinishDirectory(Directory)
    ProcedureReturn *Parent
  Else
    MessageRequester("Error", "Can not find challange folder: " + #Q + Path + #Q, #MB_ICONERROR)
    ProcedureReturn #False
  EndIf
EndProcedure

Procedure.i UpdateChallange(*Challange.Challange)
  If LoadChallangeChecksums(*Challange) And LoadChallangeHeader(*Challange) And LoadChallangeHighscores(*Challange)
    ProcedureReturn #True
  Else
    ProcedureReturn #False
  EndIf
EndProcedure
Procedure.i UpdateChallanges(*Parent.Entry)
  Protected *Item.QueueItem, *Entry.Entry
  
  *Item = *Parent\Entries\First
  While *Item
    *Entry = *Item\Memory
    
    Select *Entry\Type
      Case #Entry_File
        If Not UpdateChallange(*Entry\Memory)
          ProcedureReturn #False
        EndIf
      Case #Entry_Folder
        If Not UpdateChallanges(*Entry)
          ProcedureReturn #False
        EndIf
    EndSelect
    
    _Move(*Item, N)
  Wend
  
  ProcedureReturn #True
EndProcedure
Procedure.i RenameHighscoreAccount(*Parent.Entry, NewName.s)
  Protected *Item.QueueItem, *Entry.Entry, *HighscoreItem.QueueItem, *Highscore.Highscore
  Protected *Challange.Challange
  
  *Item = *Parent\Entries\First
  While *Item
    *Entry = *Item\Memory
    
    Select *Entry\Type
      Case #Entry_File
        *Challange = *Entry\Memory
        *HighscoreItem = *Challange\Highscores
        While *HighscoreItem
          *Highscore = *HighscoreItem\Memory
          If *Highscore\Player = Account\Name
            *Highscore\Player = NewName
          EndIf
          _Move(*HighscoreItem, N)
        Wend
      Case #Entry_Folder
        RenameHighscoreAccount(*Entry, NewName)
    EndSelect
    
    _Move(*Item, N)
  Wend
  
  ProcedureReturn #True
EndProcedure

Procedure.i FreeChallangeScreen(*Screen)
  FreeSprite(*Screen)
  ProcedureReturn #True
EndProcedure
Procedure.i PackChallangeScreens(Folder.s, *Challange.Challange)
  Protected Directory.i, Path.s = ChallangePath(*Challange)
  
  Directory = ExamineDirectory(#PB_Any, Folder, "*.bmp")
  If Directory
    If CreatePack(Path + "Screens.dat")
      While NextDirectoryEntry(Directory)
        AddPackFile(Folder + "\" + DirectoryEntryName(Directory), 9)
      Wend
      ClosePack()
    Else
      MessageRequester("Error", "Can not pack screens: " + #Q + Path + "Screens.dat" + #Q, #MB_ICONERROR)
      ProcedureReturn #False
    EndIf
    
    FinishDirectory(Directory)
    ProcedureReturn #True
  Else
    MessageRequester("Error", "Can not find folder to pack: " + #Q + Folder + #Q, #MB_ICONERROR)
    ProcedureReturn #False
  EndIf
EndProcedure
Procedure.i LoadChallangeScreens(*Challange.Challange)
  Protected *Memory, *Screens.Queue
  
  *Screens = CreateQueue(@FreeChallangeScreen())
  
  If OpenPack(ChallangePath(*Challange) + "Screens.dat")
    Repeat
      *Memory = NextPackFile()
      If *Memory
        AppendQueueItem(*Screens, CatchSprite(#PB_Any, *Memory))
      Else
        Break
      EndIf
    ForEver
    
    ProcedureReturn *Screens
  Else
    ProcedureReturn #False
  EndIf
EndProcedure
Procedure.i FreeChallangeScreens(*Screens.Queue)
  FreeQueue(*Screens)
EndProcedure

Procedure.i ExamineChecksums(*Challange.Challange)
  Protected Path.s = ChallangePath(*Challange)
  
  If FileSize(Path + "Header.bin")   <> -1 : *Challange\HeaderChecksum   = MD5FileFingerprint(Path + "Header.bin")   : EndIf
  If FileSize(Path + "Scores.bin")   <> -1 : *Challange\ScoresChecksum   = MD5FileFingerprint(Path + "Scores.bin")   : EndIf
  If FileSize(Path + "Settings.bin") <> -1 : *Challange\SettingsChecksum = MD5FileFingerprint(Path + "Settings.bin") : EndIf
EndProcedure
Procedure.i WriteChecksums(File.i, *Challange.Challange)
  WriteNTString(File, *Challange\HeaderChecksum)
  WriteNTString(File, *Challange\ScoresChecksum)
  WriteNTString(File, *Challange\SettingsChecksum)
  
  ProcedureReturn #True
EndProcedure
Procedure.i PassChecksums(File.i)
  Protected Index.i
  
  For Index = 1 To 3
    ReadNTString(File)
  Next
EndProcedure
Procedure.i UnpackMap(*Challange.Challange)
  Protected Path.s = ChallangePath(*Challange), *Memory, File.i
  
  If Not OpenPack(Path + "Map.bin")
    MessageRequester("Errro", "Challange does not have a map: " + #Q + Path + #Q, #MB_ICONERROR)
    ProcedureReturn #False
  EndIf
  
  File = CreateFile(#PB_Any, Path + "UnpackedMap.bin")
  If Not File
    MessageRequester("Error", "Can not unpack map: " + #Q + Path + #Q, #MB_ICONERROR)
    ProcedureReturn #False
  EndIf
  
  *Memory = NextPackFile()
  WriteData(File, *Memory, PackFileSize())
  CloseFile(File)
  
  ClosePack()
  
  ProcedureReturn #True
EndProcedure
Procedure.i RewriteChecksums(*Challange.Challange)
  Protected Path.s = ChallangePath(*Challange), File.i
  
  UnpackMap(*Challange)
  
  File = OpenFile(#PB_Any, Path + "UnpackedMap.bin")
  If Not File
    MessageRequester("Error", "Can not load map: " + #Q + Path + #Q, #MB_ICONERROR)
    ProcedureReturn #False
  EndIf
  
  ExamineChecksums(*Challange)
  WriteChecksums(File, *Challange)
  
  CloseFile(File)
  
  PackMap(*Challange)
  
  ProcedureReturn #True
EndProcedure
Procedure.i PackMap(*Challange.Challange)
  Protected Path.s = ChallangePath(*Challange)
  
  If Not CreatePack(Path + "Map.bin")
    MessageRequester("Error", "Can not pack map: " + #Q + Path + #Q, #MB_ICONERROR)
    ProcedureReturn #False
  EndIf
  
  AddPackFile(Path + "UnpackedMap.bin", 9)
  ClosePack()
  
  DeleteFile(Path + "UnpackedMap.bin")
  
  ProcedureReturn #True
EndProcedure

Procedure.i SaveChallangeHighscores(*Challange.Challange)
  Protected Path.s = ChallangePath(*Challange), File.i
  Protected *Item.QueueItem, *Highscore.Highscore
  
  File = CreateFile(#PB_Any, Path + "Scores.bin")
  If Not File
    MessageRequester("Errro", "Can not save highscores: " + #Q + Path + #Q, #MB_ICONERROR)
    ProcedureReturn #False
  EndIf
  
  WriteInteger(File, *Challange\Highscores\Count)
  
  *Item = *Challange\Highscores\First
  While *Item
    *Highscore = *Item\Memory
    
    WriteNTString(File, *Highscore\Player)
    WriteInteger(File , *Highscore\Score)
    WriteInteger(File , *Highscore\Medal)
    WriteInteger(File , *Highscore\SuperCoins)
    
    _Move(*Item, N)
  Wend
  
  CloseFile(File)
  
  ProcedureReturn RewriteChecksums(*Challange)
EndProcedure
; IDE Options = PureBasic 4.51 (Windows - x86)
; CursorPosition = 94
; FirstLine = 6
; Folding = AGAAw
; EnableXP