;**************************************************************
;     Char Set
;**************************************************************
;   Copyright (c) 2010 Josef Sniatecki
;
; This program is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License
; As published by the Free Software Foundation; either version
; 3 of the License, Or (at your option) any later version.
;
; This program is distributed in the hope that it will be
; useful, but WITHOUT ANY WARRANTY; without even the implied
; warranty of MERCHANTABILITY Or FITNESS For A PARTICULAR
; PURPOSE. See the GNU General Public License For more details.
;
; You should have received a copy of the GNU General Public
; License along With this program;
; If Not, see "http://www.gnu.org/licenses/".
;**************************************************************



Procedure.i LoadCharSet(Name.s, Width.i, Height.i)
  Protected *CharSet.CharSet, Image.i, GrabbedImage.i
  Protected Index.i, Sprite.i
  
  *CharSet = _New(CharSet)
  
  *CharSet\Width  = Width
  *CharSet\Height = Height
  
  ; Vollstndiges CharSet als Image laden:
  Image = LoadImage(#PB_Any, GetFile("Graphics\" + Name + ".bmp"))
  If Not Image
    MessageRequester("Error", "Can not load character set: " + #Q + Name + #Q, #MB_ICONERROR)
    ProcedureReturn #False
  EndIf
  
  ; berprfen, ob die Dimensionen des Images
  ; durch die angegebenen Dimensionen der Buchstaben
  ; teilbar sind:
  If ImageWidth(Image) % Width Or ImageHeight(Image) <> Height
    MessageRequester("Error", "Invalid size of character set: " + #Q + Name + #Q, #MB_ICONERROR)
    ProcedureReturn #False
  EndIf
  
  ; Jeder Buchstabe bekommt ein eigenes Sprite, das in einem
  ; Array "Characters" angelegt wird:
  *CharSet\CharacterCount = ImageWidth(Image) / Width
  *CharSet\Characters = _Dim(*CharSet\CharacterCount * _Sz(Integer))
  For Index = 0 To *CharSet\CharacterCount - 1
    GrabbedImage = GrabImage(Image, #PB_Any, Index * Width, 0, Width, Height)
    SaveImage(GrabbedImage, "Graphics\Temp.bmp", #PB_ImagePlugin_BMP, 0, 24)
    FreeImage(GrabbedImage)
    
    Sprite = LoadSprite(#PB_Any, "Graphics\Temp.bmp")
    TransparentSpriteColor(Sprite, $FF00FF) ;Pures Lila ist transparent
    PokeI(*CharSet\Characters + Index * _Sz(Integer), Sprite)
  Next
  
  ProcedureReturn *CharSet
EndProcedure

Procedure.i DisplayText(X.i, Y.i, Text.s, *CharSet.CharSet)
  Protected StartX.i = X, *Char.Character, Index.i
  
  *Char = @Text
  While *Char\c
    Select *Char\c
      Case 13
        ; Neue Zeile:
        Y + *CharSet\Height + #CharSet_VSpace
        X = StartX
        *Char + _Sz(Character)
        Continue
      Case 32
        ; Leerzeichen:
        X + *CharSet\Width + #CharSet_HSpace
        *Char + _Sz(Character)
        Continue
      Default
        ; Das CharSet beginnt nach dem Leerzeichen:
        Index = *Char\c - '!'
    EndSelect
    
    DisplayTransparentSprite(PeekI(*CharSet\Characters + Index * _Sz(Integer)), X, Y)
    X + *CharSet\Width + #CharSet_HSpace
    *Char + _Sz(Character)
  Wend
  
  ; Position der Zeile unter der zuletzt gerenderten Zeile zurckgeben:
  ProcedureReturn Y + *CharSet\Height + #CharSet_VSpace
EndProcedure
Procedure.i DisplayNumber(X.i, Y.i, Number.s, *CharSet.CharSet)
  Protected *Char.Character, Index.i
  
  *Char = @Number
  While *Char\c
    Select *Char\c
      Case '0' To '9'
        Index = *Char\c - '0'
      Case '+'
        Index = 10
      Case '-'
        Index = 11
      Case '*'
        Index = 12
      Case '/'
        Index = 13
    EndSelect
    
    DisplayTransparentSprite(PeekI(*CharSet\Characters + Index * _Sz(Integer)), X, Y)
    X + *CharSet\Width + #CharSet_HSpace
    *Char + _Sz(Character)
  Wend
  
  ; X-Position nach dem zuletzt gerenderten Zeichen zurckgeben:
  ProcedureReturn X
EndProcedure

; IDE Options = PureBasic 4.50 (Windows - x86)
; CursorPosition = 108
; FirstLine = 66
; Folding = -
; EnableXP