;**************************************************************
;     Collision
;**************************************************************
;   Copyright (c) 2009 Josef Sniatecki
;
; This program is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License
; As published by the Free Software Foundation; either version
; 3 of the License, Or (at your option) any later version.
;
; This program is distributed in the hope that it will be
; useful, but WITHOUT ANY WARRANTY; without even the implied
; warranty of MERCHANTABILITY Or FITNESS For A PARTICULAR
; PURPOSE. See the GNU General Public License For more details.
;
; You should have received a copy of the GNU General Public
; License along With this program;
; If Not, see "http://www.gnu.org/licenses/".
;**************************************************************



Procedure.i Collision(AX.f, AY.f, AWidth.f, AHeight.f, ADX.f, ADY.f, BX.f, BY.f, BWidth.f, BHeight.f)
  Protected M.f, Y.f
  
  If AX < BX + BWidth And AX + AWidth > BX And AY < BY + BHeight And AY + AHeight > BY
    If ADX > 0
      If ADY > 0
        M = ADY/ADX
        Y = AY - ((AX + AWidth) - BX)*M
        If Y + AHeight > BY
          ProcedureReturn #Collision_Left
        Else
          ProcedureReturn #Collision_Top
        EndIf
      ElseIf ADY < 0
        M = ADY/ADX
        Y = AY - ((AX + AWidth) - BX)*M
        If Y < BY + BHeight
          ProcedureReturn #Collision_Left
        Else
          ProcedureReturn #Collision_Bottom
        EndIf
      Else
        ProcedureReturn #Collision_Left
      EndIf
    ElseIf ADX < 0
      If ADY < 0
        M = ADY/ADX
        Y = AY + ((BX + BWidth) - AX)*M
        If Y > BY + BHeight - 1
          ProcedureReturn #Collision_Bottom
        Else
          ProcedureReturn #Collision_Right
        EndIf
      ElseIf ADY > 0
        M = ADY/ADX
        Y = AY + ((BX + BWidth) - AX)*M
        If Y + AHeight - 1 < BY
          ProcedureReturn #Collision_Top
        Else
          ProcedureReturn #Collision_Right
        EndIf
      Else
        ProcedureReturn #Collision_Right
      EndIf
    ElseIf ADY > 0
      ProcedureReturn #Collision_Top
    ElseIf ADY < 0
      ProcedureReturn #Collision_Bottom
    EndIf
  EndIf
  
  ProcedureReturn #False
EndProcedure

Procedure.i CheckFieldCollision(X.i, Y.i)
  ; Dies ist der eigentliche Kollisions-Algorithmus fr
  ; die Kollision zwischen Spieler und einem Feld.
  
  ; Dieser Algorithmus darf in anderen Projekten verwendet werden. Jedoch
  ; soll die Verwendung mit meinem Namen "Josef Sniatecki" vermerkt sein.
  
  Protected FieldX.i, FieldY.i
  Protected M.f
  
  FieldX = X * #FieldSize
  FieldY = Y * #FieldSize
  
  If Player\X < FieldX + #FieldSize And Player\X + Player\Width > FieldX And Player\Y < FieldY + #FieldSize And Player\Y + Player\Height > FieldY ;berlappung?
    ; Wenn eine berlappung besteht, dann wird anhand der Bewegung des Spielers <MoveX, MoveY>
    ; ermittelt, welche Seiten fr eine Kollision in Frage kommen.
    If Player\MoveX > 0 ;Bewegt sich nach rechts?
      ; Also ist schon mal die linke Seite relevant fr eine Abfrage.
      ; Und nun eine waagerechte Seite ermitteln...
      If Player\MoveY > 0 ;Bewegt sich nach unten?
        ; Nun kommt die Kernidee, die das Ganze berhaupt zum Laufen bringt:
        ; "M" ist die Steigung des Vektors <MoveX, MoveY>:
        M = Player\MoveY / Player\MoveX
        ; Nun berechnet man mithilfe der Steigung, welche relative Y-Position der Spieler
        ; zum Zeitpunkt hatte (existiert jedoch nicht als Frame), als der Spieler
        ; die linke Seite nur "berhrte":
        Y = Player\Y - ((Player\X + Player\Width) - FieldX)*M
        ; Nun kann man prfen, ob der Spieler die Linke, oder die obere Seite
        ; berhrt hat:
        If Y + Player\Height > FieldY
          ProcedureReturn #Collision_Left
        Else
          ProcedureReturn #Collision_Top
        EndIf
        ; Und das war's auch schon ;)
      ElseIf Player\MoveY < 0 ;Bewegt sich nach oben?
        M = Player\MoveY / Player\MoveX
        Y = Player\Y - ((Player\X + Player\Width) - FieldX)*M
        If Y < FieldY + #FieldSize
          ProcedureReturn #Collision_Left
        Else
          ProcedureReturn #Collision_Bottom
        EndIf
      Else
        ProcedureReturn #Collision_Left
      EndIf
    ElseIf Player\MoveX < 0 ;Bewegt sich nach links?
      If Player\MoveY < 0
        M = Player\MoveY / Player\MoveX
        Y = Player\Y + ((FieldX + #FieldSize) - Player\X)*M
        If Y > FieldY + #FieldSize - 1
          ProcedureReturn #Collision_Bottom
        Else
          ProcedureReturn #Collision_Right
        EndIf
      ElseIf Player\MoveY > 0 ;Bewegt sich nach unten?
        M = Player\MoveY / Player\MoveX
        Y = Player\Y + ((FieldX + #FieldSize) - Player\X)*M
        If Y + Player\Height - 1 < FieldY
          ProcedureReturn #Collision_Top
        Else
          ProcedureReturn #Collision_Right
        EndIf
      Else ;Nur eine waagerechte Bewegung nach links:
        ProcedureReturn #Collision_Right
      EndIf
    ElseIf Player\MoveY > 0 ;Nur eine senkrechte Bewegung nach unten?
      ProcedureReturn #Collision_Top
    ElseIf Player\MoveY < 0 ;Nur eine senkrechte Bewegung nach oben?
      ProcedureReturn #Collision_Bottom
    EndIf
    ; Der Fall "MoveX = 0 AND MoveY = 0" soll nicht auftreten, da eine
    ; berlappung immer mit einer Bewegung verbunden sein soll.
  EndIf
  
  ProcedureReturn #False
EndProcedure
Procedure.i FieldCollision(DeltaX.i, DeltaY.i)
  Protected *Field.Field, X.i, Y.i, Collision.i, *Object.Object, Result.i
  
  X = Player\FieldX + DeltaX
  Y = Player\FieldY + DeltaY
  
  If X => 0 And X < Player\Scene\Width*#SceneWidth And Y => 0 And Y < Player\Scene\Height*#SceneHeight
    *Object = GetObject(X, Y)
    If *Object
      Collision = Collision(Player\X, Player\Y, Player\Width, Player\Height, Player\MoveX, Player\MoveY, *Object\X, *Object\Y, *Object\Width, *Object\Height)
      If Collision
        Result = _Call(*Object\Callback, *Object, #Object_Collision, Collision)
        If Result
          Select Result
            Case #ObjectDo_Delete
              ReleaseObject(*Object)
              ;TODO: Remove from table
            Case #ObjectDo_Collision
              ProcedureReturn Collision
          EndSelect
        EndIf
      EndIf
    EndIf
    
    Collision = #False
    
    If GetWall(X, Y)
      Collision = CheckFieldCollision(X, Y)
      If Collision
        Select Collision
          Case #Collision_Left
            Player\X             = (Player\FieldX + 1) * #FieldSize - Player\Width
            Player\MoveX         = 0
            Player\PreviousMoveX = 0
          Case #Collision_Right
            Player\X             = Player\FieldX * #FieldSize
            Player\MoveX         = 0
            Player\PreviousMoveX = 0
          Case #Collision_Bottom
            Player\Y             = Player\FieldY * #FieldSize
            Player\MoveY         = 0
            Player\PreviousMoveY = 0
          Case #Collision_Top
            Player\Y     = (Player\FieldY + 1) * #FieldSize - Player\Height
            ChangePlayerAction(#Player_Action_Standing)
        EndSelect
        ExaminePlayerField()
      EndIf
    ElseIf GetGround(X, Y)
      Collision = CheckFieldCollision(X, Y)
      If Collision
        Select Collision
          Case #Collision_Top
            If Player\MoveY => 0 And (Player\Y - Player\MoveY + Player\Height <= Y * #FieldSize)
              Player\Y = Y * #FieldSize - Player\Height
              ChangePlayerAction(#Player_Action_Standing)
              ExaminePlayerField()
            Else
              ProcedureReturn #False
            EndIf
          Default
            ProcedureReturn #False
        EndSelect
      EndIf
    EndIf
    
    ProcedureReturn Collision
  Else
    ProcedureReturn #False
  EndIf
EndProcedure
Procedure.i PlayerAndObjectCollision(*Object.Object, Direction.i)
  Select Direction
    Case #Collision_Left
      Player\X = *Object\X - #FieldSize
    Case #Collision_Right
      Player\X = *Object\X + *Object\Width
    Case #Collision_Top
      Player\Y = *Object\Y - #FieldSize
      ChangePlayerAction(#Player_Action_Standing)
      Player\BottomObject = *Object
    Case #Collision_Bottom
      Player\Y     = *Object\Y + *Object\Height
      Player\MoveY = 0
  EndSelect
  Player\FieldX = Player\X / #FieldSize
  Player\FieldY = Player\Y / #FieldSize
EndProcedure

Procedure.i DetectNextBarrier(X.i, Y.i, DX.i, DY.i, IncludeGrounds.i = #True)
  Protected Distance.i
  
  While X => 0 And X < Table\Scene\Width * #SceneWidth And Y => 0 And Y < Table\Scene\Height * #SceneHeight
    If GetWall(X, Y) Or (IncludeGrounds And DY > 0 And GetGround(X, Y))
      ProcedureReturn Distance
    EndIf
    
    Distance + 1
    X        + DX
    Y        + DY
  Wend
  
  ProcedureReturn -1
EndProcedure
; IDE Options = PureBasic 4.51 (Windows - x86)
; CursorPosition = 243
; FirstLine = 211
; Folding = -
; EnableXP