;**************************************************************
;     Control
;**************************************************************
;   Copyright (c) 2010 Josef Sniatecki
;
; This program is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License
; As published by the Free Software Foundation; either version
; 3 of the License, Or (at your option) any later version.
;
; This program is distributed in the hope that it will be
; useful, but WITHOUT ANY WARRANTY; without even the implied
; warranty of MERCHANTABILITY Or FITNESS For A PARTICULAR
; PURPOSE. See the GNU General Public License For more details.
;
; You should have received a copy of the GNU General Public
; License along With this program;
; If Not, see "http://www.gnu.org/licenses/".
;**************************************************************



; Fr die jeweilige Aktion des Spielers wird ein Timer
; pro Frame hochgezhlt. Dadurch haben die Aktionen die
; Mglichkeit zu erfahren, wie lange die jeweilige Aktion
; schon aktiv ist. Somit wird z.B. ein hherer Sprung
; ausgefhrt, wenn noch immer die Sprungtaste gedrckt wird
; und "ActionTime" nur wenige Frames gezhlt hat.
; Bei einem Wechsel einer Aktion wird "ActionTime" wieder
; auf Null gesetzt.
\ActionTime + 1

; Hier wird zwischen den Aktionen gewhlt:
Select \Action



;******************************************
; Standing - Auf einem Grund stehend
;******************************************
  Case #Player_Action_Standing ;{
    ; Horinzontale Bewegung:
    If \KeyPressing[#Player_Key_Right]
      ; Nach rechts laufen:
      \DoMoveX = \Speed
      If \Animation\Set <> #Player_Animation_WalkingRight
        \Animation\Set = #Player_Animation_WalkingRight
        \Animation\Duration = #Player_AnimationDuration_Walking
      EndIf
    ElseIf \KeyPressing[#Player_Key_Left]
      ; Nach links laufen:
      \DoMoveX = -\Speed
      If \Animation\Set <> #Player_Animation_WalkingLeft
        \Animation\Set = #Player_Animation_WalkingLeft
        \Animation\Duration = #Player_AnimationDuration_Walking
      EndIf
    Else
      ; Spieler bleibt stehen:
      If \ActionTime = 1
        ; Falls die Animation zum stehen bleiben noch nicht
        ; gewhlt wurde (siehe obere expr.), dann wird auf
        ; diese, je nach vorherige Laufrichtung, passend gesetzt:
        If \Animation\Set % 2 = 0
          \Animation\Set = #Player_Animation_StandingRight
        Else
          \Animation\Set = #Player_Animation_StandingLeft
        EndIf
      Else
        If \DoMoveX > 0
          ; Ist nach rechts gelaufen -> nach rechts ausrichten:
          \Animation\Set = #Player_Animation_StandingRight
        ElseIf \DoMoveX < 0
          ; Ist nach links gelaufen -> nach links ausrichten:
          \Animation\Set = #Player_Animation_StandingLeft
        EndIf
      EndIf
      \DoMoveX = 0
    EndIf
    
    If \Scene\AllowLumiFlight And \CanLumiFlight And \ShotPower = #Player_MaximalShotPower And \KeyPressing[#Player_Key_Up]
      \ShotPower = 0
      \MoveY     = -2
      \Y         - 0.5
      ChangePlayerAction(#Player_Action_LumiFlight)
    EndIf
    
    If \DoMoveX
      ; Wenn sich der Spieler seitlich bewegt, dann wird
      ; die Animation abgespielt:
      If ExamineAnimation(\Animation)
        \Animation\Duration = #Player_AnimationDuration_Walking
      EndIf
    Else
      ; Der Spieler bleibt stehen:
      If ExamineAnimation(\Animation)
        \Animation\Duration = #Player_AnimationDuration_Standing
      EndIf
    EndIf
    
    \MoveY = #Gravity * \Mass
    
    If \KeyPressed[#Player_Key_Jump]
      If \KeyPressing[#Player_Key_Down] And GetGround(\FieldX, \FieldY + 1)
        ; Durch einen Grund hindurchfallen:
        ChangePlayerAction(#Player_Action_Falling)
        \Y + 1
      Else
        ; Einen normalen Sprung ausfhren:
        ChangePlayerAction(#Player_Action_Jump)
        \MoveY = -#Player_JumpEnergy
      EndIf
    ElseIf \KeyPressing[#Player_Key_Up] And ObjectQualifiedForClambering()
      ; Wenn ein Kletter-Objekt hinter dem Spieler plaziert ist, kann er an dieser
      ; hoch klettern:
      \Y - (\Speed * #Player_ClimbSpeedFactor)
      ChangePlayerAction(#Player_Action_Clambering)
    EndIf
    
    If \CanShoot
      If \Animation\Set % 2 = 0
        Shoot( 1)
      Else
        Shoot(-1)
      EndIf
    EndIf
    
    If \StickedObject
      ; Der Spieler "klebt" an einem Objekt und wird mit diesem verschoben:
      \DoMoveX  + (\StickedObject\X - \StickedX)
      \MoveY    + (\StickedObject\Y - \StickedY)
      \StickedX = \StickedObject\X
      \StickedY = \StickedObject\Y
    EndIf
  ;}

;******************************************
; Falling - Fallend
;******************************************
  Case #Player_Action_Falling ;{
    If \KeyPressing[#Player_Key_Right]
      ; Nach rechts fallen:
      \DoMoveX       = \Speed
      \Animation\Set = #Player_Animation_WalkingRight
    ElseIf \KeyPressing[#Player_Key_Left]
      ; Nach links fallen:
      \DoMoveX       = -\Speed
      \Animation\Set = #Player_Animation_WalkingLeft
    Else
      If \ActionTime = 1
        ; Falls die Animation zum stehen bleiben noch nicht
        ; gewhlt wurde (siehe obere expr.), dann wird auf
        ; diese, je nach vorherige Laufrichtung, passend gesetzt:
        If \Animation\Set % 2 = 0
          \Animation\Set = #Player_Animation_StandingRight
        Else
          \Animation\Set = #Player_Animation_StandingLeft
        EndIf
      EndIf
      \DoMoveX = 0
    EndIf
    
    If \MoveY > 0
      If \Animation\Set % 2 = 0
        \Animation\Set = #Player_Animation_FallingRight
      Else
        \Animation\Set = #Player_Animation_FallingLeft
      EndIf
    EndIf
    
    If \KeyPressing[#Player_Key_Up]
      If ObjectQualifiedForClambering()
        ChangePlayerAction(#Player_Action_Clambering)
      EndIf
    ElseIf \KeyPressed[#Player_Key_Fly] And \CanFly
      ChangePlayerAction(#Player_Action_Flying)
    EndIf
    
    If \CanShoot
      If \Animation\Set % 2 = 0
        Shoot( 1)
      Else
        Shoot(-1)
      EndIf
    EndIf
    
    UseGravity()
  ;}

;******************************************
; Jump - Sprung ausgefhrt
;******************************************
  Case #Player_Action_Jump ;{
    If \KeyPressing[#Player_Key_Right]
      ; Nach rechts fallen/springen:
      \DoMoveX       = \Speed
      \Animation\Set = #Player_Animation_WalkingRight
    ElseIf \KeyPressing[#Player_Key_Left]
      ; Nach links fallen/springen:
      \DoMoveX       = -\Speed
      \Animation\Set = #Player_Animation_WalkingLeft
    Else
      If \ActionTime = 1
        If \DoMoveX > 0
          \Animation\Set = #Player_Animation_StandingRight
        ElseIf \DoMoveX < 0
          \Animation\Set = #Player_Animation_StandingLeft
        Else
          If \Animation\Set % 2 = 0
            \Animation\Set = #Player_Animation_StandingRight
          Else
            \Animation\Set = #Player_Animation_StandingLeft
          EndIf
        EndIf
      EndIf
      \DoMoveX = 0
    EndIf
    
    If \MoveY > 0
      If \Animation\Set % 2 = 0
        \Animation\Set = #Player_Animation_FallingRight
      Else
        \Animation\Set = #Player_Animation_FallingLeft
      EndIf
    EndIf
    
    If \KeyPressed[#Player_Key_Jump] And \CanDoubleJump
      CreateParticle(Particle\DoubleJump, \X + \Width/2 - 20/2, \Y + \Height/2 - 5, 4)
      ChangePlayerAction(#Player_Action_DoubleJump)
      If \MoveY > -#Player_DoubleJumpEnergy
        \MoveY = -#Player_DoubleJumpEnergy
      EndIf
    ElseIf \KeyPressing[#Player_Key_Jump] And \ActionTime < #Player_JumpTime
      \MoveY - #Player_JumpAddition
    EndIf
    If \KeyPressed[#Player_Key_Fly] And \CanFly
      ChangePlayerAction(#Player_Action_Flying)
    ElseIf \KeyPressing[#Player_Key_Up] And ObjectQualifiedForClambering()
      ChangePlayerAction(#Player_Action_Clambering)
    EndIf
    
    If \CanShoot
      If \Animation\Set % 2 = 0
        Shoot( 1)
      Else
        Shoot(-1)
      EndIf
    EndIf
    
    UseGravity()
  ;}

;******************************************
; Double Jump - Doppel-Sprung ausgefhrt
;******************************************
  Case #Player_Action_DoubleJump ;{
    If \KeyPressing[#Player_Key_Right]
      ; Nach rechts fallen/springen:
      \DoMoveX       = \Speed
      \Animation\Set = #Player_Animation_WalkingRight
    ElseIf \KeyPressing[#Player_Key_Left]
      ; Nach links fallen/springen:
      \DoMoveX       = -\Speed
      \Animation\Set = #Player_Animation_WalkingLeft
    Else
      If \ActionTime = 1
        If \Animation\Set % 2 = 0
          \Animation\Set = #Player_Animation_StandingRight
        Else
          \Animation\Set = #Player_Animation_StandingLeft
        EndIf
      EndIf
      \DoMoveX = 0
    EndIf
    
    If \MoveY > 0
      If \Animation\Set % 2 = 0
        \Animation\Set = #Player_Animation_FallingRight
      Else
        \Animation\Set = #Player_Animation_FallingLeft
      EndIf
    EndIf
    
    If \KeyPressing[#Player_Key_Jump] And \ActionTime < #Player_DoubleJumpTime
      If \DoMoveX > -#Player_DoubleJumpEnergy - #Player_DoubleJumpTime*#Player_DoubleJumpAddition
        \MoveY - #Player_DoubleJumpAddition
      EndIf
    EndIf
    If \KeyPressed[#Player_Key_Fly] And \CanFly
      ChangePlayerAction(#Player_Action_Flying)
    ElseIf \KeyPressing[#Player_Key_Up] And ObjectQualifiedForClambering()
      ChangePlayerAction(#Player_Action_Clambering)
    EndIf
    
    If \CanShoot
      If \Animation\Set % 2 = 0
        Shoot( 1)
      Else
        Shoot(-1)
      EndIf
    EndIf
    
    UseGravity()
  ;}
  
;******************************************
; Climbing - An Wnden klettern
;******************************************
  Case #Player_Action_Climbing ;{
    If \ActionTime = 1
      Select \ClimbSide
        Case #Player_ClimbSide_Left
          \Animation\Set = #Player_Animation_ClimbingLeft
        Case #Player_ClimbSide_Right
          \Animation\Set = #Player_Animation_ClimbingRight
      EndSelect
    EndIf
    
    ; Damit sich der Spieler an der jeweiligen Wand
    ; festhlt, muss die horinzontale Bewegung gegen
    ; die Wand gesetzt werden, damit bei der
    ; Kollisionsberprfung eine Berhrung mit der
    ; Kletterwand auftritt:
    
    \DoMoveX = 0
    ; Steuert der Spieler in die entgegengesetzte
    ; Klettterrichtung, so stt sich dieser von
    ; der Kletterwand ab:
    If \KeyPressing[#Player_Key_Left]
      \DoMoveX = -\Speed
      If \ClimbSide = #Player_ClimbSide_Right
        If ObjectQualifiedForClambering()
          ChangePlayerAction(#Player_Action_Clambering)
        Else
          ChangePlayerAction(#Player_Action_JumpedOff)
        EndIf
      EndIf
    ElseIf \KeyPressing[#Player_Key_Right]
      \DoMoveX = \Speed
      If \ClimbSide = #Player_ClimbSide_Left
        If ObjectQualifiedForClambering()
          ChangePlayerAction(#Player_Action_Clambering)
        Else
          ChangePlayerAction(#Player_Action_JumpedOff)
        EndIf
      EndIf
    EndIf
    
    ; Hoch/Runter klettern:
    If \KeyPressing[#Player_Key_Up]
      \MoveY = -\Speed * #Player_ClimbSpeedFactor
      If ExamineAnimation(\Animation)
        \Animation\Duration = #Player_AnimationDuration_Climbing
      EndIf
    ElseIf \KeyPressing[#Player_Key_Down]
      \MoveY = \Speed * #Player_ClimbSpeedFactor + #Gravity
      \Animation\Frame = 0
    Else
      ; Falls der Spieler in keine vertikale Richtung
      ; beabsichtigt steuert, wird dieser durch die
      ; Gravitation nach unten gezogen (Rutscheffekt):
      If \MoveY < 0
        \MoveY = 0
      Else
        \MoveY * 0.8
      EndIf
      \Animation\Frame = 0
    EndIf
    
    ; Der Spieler kann durch die Sprungtaste abspringen:
    If \KeyPressed[#Player_Key_Jump]
      If Sign(\DoMoveX) <> \ClimbSide
        ChangePlayerAction(#Player_Action_Jump)
        ; Bewegung in die entgegengesetzte Kletterrichtung
        ; festlegen:
        \DoMoveX = \Speed * -\ClimbSide
        \MoveY = -#Player_JumpEnergy
      EndIf
    EndIf
    
    If Not \DoMoveX
      \DoMoveX = \ClimbSide * 7
    EndIf
  ;}
  
;******************************************
; Clambering - An Objekten klettern
;******************************************
  Case #Player_Action_Clambering ;{
    If \ActionTime = 1
      \Animation\Set = #Player_Animation_Clambering
    EndIf
    
    \DoMoveX = 0
    \MoveY = 0
    
    If \KeyPressing[#Player_Key_Right]
      \DoMoveX = \Speed * #Player_ClimbSpeedFactor
    ElseIf \KeyPressing[#Player_Key_Left]
      \DoMoveX = -\Speed * #Player_ClimbSpeedFactor
    EndIf
    If \KeyPressing[#Player_Key_Down]
      \MoveY = \Speed * #Player_ClimbSpeedFactor
    ElseIf \KeyPressing[#Player_Key_Up]
      \MoveY = -\Speed * #Player_ClimbSpeedFactor
    EndIf
    
    If \DoMoveX Or \MoveY
      If ExamineAnimation(\Animation)
        \Animation\Duration = #Player_AnimationDuration_Climbing
      EndIf
    Else
      \Animation\Frame = 0
    EndIf
    
    If \KeyPressed[#Player_Key_Jump] And (\DoMoveX Or \MoveY = 0)
      ChangePlayerAction(#Player_Action_Jump)
      \MoveY = -#Player_JumpEnergy
    EndIf
  ;}

;******************************************
; Jumped Off - Von einer Wand abgesprungen
;******************************************
  Case #Player_Action_JumpedOff ;{
    If \KeyPressing[#Player_Key_Right]
      ; Nach rechts fallen:
      \DoMoveX         = \Speed
      \Animation\Set = #Player_Animation_WalkingRight
    ElseIf \KeyPressing[#Player_Key_Left]
      ; Nach links fallen:
      \DoMoveX         = -\Speed
      \Animation\Set = #Player_Animation_WalkingLeft
    Else
      If \ActionTime = 1
        ; Falls die Animation zum stehen bleiben noch nicht
        ; gewhlt wurde (siehe obere expr.), dann wird auf
        ; diese, je nach vorherige Laufrichtung, passend gesetzt:
        If \DoMoveX > 0
          ; Ist nach rechts gelaufen -> nach rechts ausrichten:
          \Animation\Set = #Player_Animation_StandingRight
        ElseIf \DoMoveX < 0
          ; Ist nach links gelaufen -> nach links ausrichten:
          \Animation\Set = #Player_Animation_StandingLeft
        EndIf
      EndIf
      \DoMoveX = 0
    EndIf
    
    If \MoveY > 0
      If \Animation\Set % 2 = 0
        \Animation\Set = #Player_Animation_FallingRight
      Else
        \Animation\Set = #Player_Animation_FallingLeft
      EndIf
    EndIf
    
    If \KeyPressing[#Player_Key_Jump] And \ActionTime < 4
      ChangePlayerAction(#Player_Action_Jump)
      \MoveY = -#Player_JumpEnergy
    ElseIf \KeyPressing[#Player_Key_Up] And ObjectQualifiedForClambering()
      ChangePlayerAction(#Player_Action_Clambering)
    ElseIf \KeyPressed[#Player_Key_Fly] And \CanFly
      ChangePlayerAction(#Player_Action_Flying)
    EndIf
    
    If \CanShoot
      If \Animation\Set % 2 = 0
        Shoot( 1)
      Else
        Shoot(-1)
      EndIf
    EndIf
    
    UseGravity()
  ;}

;******************************************
; Flying - Flug durch den Headcopter
;******************************************
  Case #Player_Action_Flying ;{
    If \KeyPressing[#Player_Key_Right]
      ; Nach rechts fallen/springen:
      \DoMoveX         + \Speed * #Player_FlightSpeedFactor
      \Animation\Set = #Player_Animation_FlyingRight
    ElseIf \KeyPressing[#Player_Key_Left]
      ; Nach links fallen/springen:
      \DoMoveX         - \Speed * #Player_FlightSpeedFactor
      \Animation\Set = #Player_Animation_FlyingLeft
    Else
      \DoMoveX * #Player_FlightFriction
      If \ActionTime = 1
        If \Animation\Set % 2 = 0
          \Animation\Set = #Player_Animation_FlyingRight
        Else
          \Animation\Set = #Player_Animation_FlyingLeft
        EndIf
      EndIf
    EndIf
    
    _Lim(\DoMoveX, -\Speed * #Player_MaxFlightSpeedFactor, \Speed * #Player_MaxFlightSpeedFactor)
    
    If ExamineAnimation(\Animation)
      \Animation\Duration = #Player_AnimationDuration_Flying
    EndIf
    
    If \Scene\AllowFly3 And \CanFly = 3
      If \KeyPressing[#Player_Key_Fly]
        \MoveY - #Player_FlightSpeedFactor
      EndIf
    ElseIf \KeyPressed[#Player_Key_Fly]
      ChangePlayerAction(#Player_Action_FlyingTruncated)
    EndIf
    
    If \KeyPressing[#Player_Key_Up] And ObjectQualifiedForClambering()
      ChangePlayerAction(#Player_Action_Clambering)
    EndIf
    If \KeyPressed[#Player_Key_Down]
      ChangePlayerAction(#Player_Action_FlyingTruncated)
    EndIf
    
    If \CanShoot
      If \Animation\Set % 2 = 0
        Shoot( 1)
      Else
        Shoot(-1)
      EndIf
    EndIf
    
    \MoveY + (#FlightGravity * \Mass * #Player_FlightSpeedFactor)
    
    If \MoveY > #FlightGravity * \Mass
      \MoveY = #FlightGravity * \Mass
    ElseIf \MoveY < -#Player_MaxFlyUpSpeed
      \MoveY = -#Player_MaxFlyUpSpeed
    EndIf
    
    If \CanFly = 1 And \ActionTime => 80
      ChangePlayerAction(#Player_Action_FlyingTruncated)
    EndIf
  ;}

;******************************************
; Flying Truncated - Fliegen abgebrochen
;******************************************
  Case #Player_Action_FlyingTruncated ;{
    If \KeyPressing[#Player_Key_Right]
      ; Nach rechts fallen:
      \DoMoveX         = \Speed
      \Animation\Set = #Player_Animation_WalkingRight
    ElseIf \KeyPressing[#Player_Key_Left]
      ; Nach links fallen:
      \DoMoveX         = -\Speed
      \Animation\Set = #Player_Animation_WalkingLeft
    Else
      If \ActionTime = 1
        ; Falls die Animation zum stehen bleiben noch nicht
        ; gewhlt wurde (siehe obere expr.), dann wird auf
        ; diese, je nach vorherige Laufrichtung, passend gesetzt:
        If \Animation\Set % 2 = 0
          \Animation\Set = #Player_Animation_StandingRight
        Else
          \Animation\Set = #Player_Animation_StandingLeft
        EndIf
      EndIf
      \DoMoveX = 0
    EndIf
    
    If \KeyPressing[#Player_Key_Up]
      If ObjectQualifiedForClambering()
        ChangePlayerAction(#Player_Action_Clambering)
      EndIf
    EndIf
    
    If \KeyPressed[#Player_Key_Fly] And \CanFly > 1
      ChangePlayerAction(#Player_Action_Flying)
    EndIf
    
    If \CanShoot
      If \Animation\Set % 2 = 0
        Shoot( 1)
      Else
        Shoot(-1)
      EndIf
    EndIf
    
    UseGravity()
  ;}

;******************************************
; Lumi Flight - Als Lichtball fliegen
;******************************************
  Case #Player_Action_LumiFlight ;{
    If \ActionTime = 1
      If \Animation\Set % 2 = 0
        \Animation\Set = #Player_Animation_GhostRight
      Else
        \Animation\Set = #Player_Animation_GhostLeft
      EndIf
    EndIf
    
    If \KeyPressing[#Player_Key_Right]
      ; Nach rechts steuern:
      \DoMoveX         + \Speed * #Player_LumiFlightSpeedFactor
      \Animation\Set = #Player_Animation_GhostRight
    ElseIf \KeyPressing[#Player_Key_Left]
      ; Nach links steuern:
      \DoMoveX         - \Speed * #Player_LumiFlightSpeedFactor
      \Animation\Set = #Player_Animation_GhostLeft
    Else
      \DoMoveX * #Player_LumiFlightFriction
    EndIf
    
    If \KeyPressing[#Player_Key_Down]
      ; Nach unten steuern:
      \MoveY + \Speed * #Player_LumiFlightSpeedFactor
    ElseIf \KeyPressing[#Player_Key_Up]
      ; Nach oben steuern:
      \MoveY - \Speed * #Player_LumiFlightSpeedFactor
    Else
      \MoveY * #Player_LumiFlightFriction
    EndIf
    
    ; Grenzen bei der Geschwindigkeit festlegen:
    _Lim(\DoMoveX, -\Speed * #Player_MaxLumiFlightSpeedFactor, \Speed * #Player_MaxLumiFlightSpeedFactor)
    _Lim(\MoveY, -\Speed * #Player_MaxLumiFlightSpeedFactor, \Speed * #Player_MaxLumiFlightSpeedFactor)
    
    ; Die Lichtstrke auf "voll" setzen:
    \LumiLight = 255
    
    ; Glitzer emittieren:
    If Random(3) = 0
      EmitParticle_Glitter(Player\X + Player\Width / 2 + (4 - Random(8)), Player\Y + Player\Height / 2 + (4 - Random(8)))
      If Random(3) = 0
        EmitParticle_Glitter(Player\X + Player\Width / 2 + (4 - Random(8)), Player\Y + Player\Height / 2 + (4 - Random(8)))
      EndIf
    EndIf
    
    If \KeyPressed[#Player_Key_Shoot]
      ; LumiFlight abbrechen:
      ChangePlayerAction(#Player_Action_Falling)
    EndIf
  ;}



EndSelect
; IDE Options = PureBasic 4.51 (Windows - x86)
; CursorPosition = 286
; FirstLine = 123
; Folding = Mw
; EnableXP