;{ Account
  Declare.i LoadAccounts(List Account.s())
  Declare.i LoadAccount(Name.s)
  Declare.i CheckAccountPassword(Password.s, Name.s)
  
  Declare.i RenameAccount(NewName.s)
  Declare.i SaveAccount()
;}
;{ Entry
  Declare.i CreateEntryFile(Name.s, *Memory, *Parent.Entry = #Null)
  Declare.i CreateEntryFolder(Name.s, *Parent.Entry = #Null)
  Declare.i FreeEntry(*Entry.Entry)
  Declare.i RemoveEntry(*Entry.Entry)
  
  Declare.s EntryPath(*Entry.Entry)
  
  Declare.i GetEntry(*Parent.Entry, Name.s)
  Declare.i GetEntryByPath(*Parent.Entry, Path.s)
;}
;{ Animation
  CompilerIf #Mode = #Mode_Game
    Declare.i CreateAnimation(*Frameset.AnimationFrameset)
    Declare.i LoadAnimationFrameset(Name.s, FrameWidth.i, FrameHeight.i)
    Declare.i LoadRegisteredAnimationFrameset(Name.s, FrameWidth.i, FrameHeight.i)
    Declare.i FreeAnimation(*Animation.Animation)
    
    Declare.i ExamineAnimation(*Animation.Animation)
    Declare.i ExamineAnimationBack(*Animation.Animation)
    Declare.i DisplayAnimation(*Animation.Animation, X.i, Y.i, Transparency.i = 255)
    Declare.i DisplayFramesetFrame(*Frameset.AnimationFrameset, Frame.i, Set.i, X.i, Y.i, Transparency.i = 255)
  CompilerEndIf
;}
;{ Background
  CompilerSelect #Mode
    CompilerCase #Mode_Game
      Declare.i LoadBackground(Name.s)
      Declare.i SetSceneBackground(*Scene.Scene, Sprite.i)
      Declare.i DisplayBackground(Sprite.i)
      
      Declare.i GetBackground(Name.s)
    CompilerCase #Mode_Editor
      Declare.i LoadBackgrounds(Path.s, *Parent.Entry)
      Declare.s BackgroundPath(*Background.BackgroundEntry)
      Declare.i GetBackground(Name.s)
      
      Declare.i LoadBackgroundSprite(*Background.BackgroundEntry)
      Declare.i FreeBackgroundSprite(Sprite, *Background.BackgroundEntry)
      
      Declare.i LoadRegBackground(*Entry.BackgroundEntry)
      Declare.i FreeRegBackground(*Background.RegisteredBackground)
      
      Declare.i UseRegBackground(*Background.RegisteredBackground)
      Declare.i ReleaseRegBackground(*Background.RegisteredBackground)
      Declare.i CheckRegBackground(*Background.RegisteredBackground)
      
      Declare.i ChangeBackground(*Scene.Scene, *Background.RegisteredBackground, IncludeChildren.i = #True)
      Declare.i ChangeAllBackgrounds(*Background.RegisteredBackground)
      
      Declare.i DrawTilesetBackground()
  CompilerEndSelect
;}
;{ Camera
  Declare.i UpdateCamera()
  
  Declare.i ResetCameraX()
  Declare.i ResetCameraY()
  Declare.i ResetCamera()
  
  Declare.i ExamineCamera()
;}
;{ Challange
  Declare.i CreateChallange(Path.s, Name.s)
  Declare.i FreeChallange(*Challange.Challange)
  Declare.i RemoveChallange(*Challange.Challange)
  Declare.i RenameChallange(*Challange.Challange, NewName.s)
  
  Declare.s ChallangePath(*Challange.Challange)
  Declare.i GetChallange(Path.s)
  
  Declare.i LoadChallangeChecksums(*Challange.Challange)
  Declare.i LoadChallangeHeader(*Challange.Challange)
  Declare.i LoadChallangeHighscores(*Challange.Challange)
  
  CompilerIf #Mode = #Mode_Game
    Declare.i LoadChallangeSettings(*Challange.Challange)
  CompilerEndIf
  
  Declare.i LoadChallange(Path.s, *Entry.Entry)
  Declare.i LoadChallanges(Path.s, *Parent.Entry)
  
  Declare.i UpdateChallange(*Challange.Challange)
  Declare.i UpdateChallanges(*Parent.Entry)
  
  Declare.i ExamineChecksums(*Challange.Challange)
  Declare.i WriteChecksums(File.i, *Challange.Challange)
  Declare.i PassChecksums(File.i)
  Declare.i UnpackMap(*Challange.Challange)
  Declare.i RewriteChecksums(*Challange.Challange)
  Declare.i PackMap(*Challange.Challange)
  
  Declare.i LockChallange(*Challange.Challange, State.i = #True)
  Declare.i SaveChallangeHighscores(*Challange.Challange)
;}
;{ CharSet
  Declare.i LoadCharSet(Name.s, Width.i, Height.i)
  
  Declare.i DisplayText(X.i, Y.i, Text.s, *CharSet.CharSet)
  Declare.i DisplayNumber(X.i, Y.i, Number.s, *CharSet.CharSet)
;}
;{ TinyMessage
  Declare.i CreateTinyMessage(X.f, Y.f, Text.s)
  Declare.i FreeTinyMessage(*Message.TinyMessage)
  Declare.i ExamineTinyMessages()
  Declare.i DisplayTinyMessages(Move.i = #True)
;}
;{ NewGame
  CompilerIf #Mode = #Mode_Game
    Declare.i NewGame(*Challange.Challange)
    
    Declare.i ReadTilesets()
    Declare.i ReadBanks()
    Declare.i ReadBackgrounds()
    Declare.i ReadEvents()
    Declare.i ReadEffects()
    Declare.i ReadActs()
    Declare.i ReadScene(X.i, Y.i, *Main.Scene)
    Declare.i ReadTileGroup(*Scene.Scene, Group.i)
    Declare.i ReadObjectGroup(*Scene.Scene, Group.i)
    Declare.i ReadFixObjectGroup(*Scene.Scene)
    Declare.i ReadScenes()
    
    Declare.i InitializeObjects()
    Declare.i InitializeActs()
    
    Declare.i SaveGame()
    Declare.i LoadGame()
  CompilerEndIf
;}
;{ Collision
  CompilerIf #Mode = #Mode_Game
    Declare.i Collision(AX.f, AY.f, AWidth.f, AHeight.f, ADX.f, ADY.f, BX.f, BY.f, BWidth.f, BHeight.f)
    
    Declare.i CheckFieldCollision(X.i, Y.i)
    Declare.i FieldCollision(DeltaX.i, DeltaY.i)
    Declare.i PlayerAndObjectCollision(*Object.Object, Direction.i)
  CompilerEndIf
;}
;{ GameOver
  CompilerIf #Mode = #Mode_Game
    Declare.i LoseGame()
  CompilerEndIf
;}
;{ Highscore
  Declare.i NewHighscore(*Challange.Challange, Player.s, Score.i, Medal.i, SuperCoins.i)
  Declare.i FreeHighscore(*Highscore.Highscore)
  
  Declare.i GetHighscore(*Challange.Challange, Player.s)
  Declare.i HighscoreScore(*Highscore.Highscore)
  Declare.s HighscorePlayer(*Highscore.Highscore)
;}
;{ Interface
  Declare.i ShowMessage(Title.s, Message.s)
  Declare.s EditMessage(Title.s, Message.s = "")
  
  CompilerIf #Mode = #Mode_Game
    Declare.i FadeIn(Color.i = #Black, DisplayPlayer.i = #True)
    Declare.i FadeOff(DisplayPlayer.i = #True)
  CompilerEndIf
;}
;{ Object
  CompilerSelect #Mode
    CompilerCase #Mode_Game
      Declare.i LoadBank(Name.s)
      Declare.i CallBankLoadFunction(*Bank.Bank)
      
      Declare.i LoadObject(*Bank.Bank, Callback.s)
      Declare.i GlobalLoadObject(*Callback)
      Declare.i GlobalInitObject(*Callback)
      Declare.i GetObject(*Bank.Bank, Name.s)
      
      Declare.i CreateObject(*Callback, X.f, Y.f, *Scene.Scene, Layer.i, *Parameter = #Null)
      Declare.i FreeObject(*Object.Object)
      
      Declare.i ReleaseObject(*Object.Object)
    CompilerCase #Mode_Editor
      Declare.i InitializeBank(*Bank.Bank)
      Declare.i LoadBanks(Path.s)
      Declare.i ExtractObjectEntries(*Bank.Bank, *Memory.All, *Parent.Entry)
      
      Declare.i LoadRegObject(*Entry.ObjectEntry)
      Declare.i FreeRegObject(*Object.RegisteredObject)
      
      Declare.i GetObjectByCallbackName(*Bank.Bank, Name.s)
      Declare.s ObjectName(*Object.ObjectEntry)
      Declare.s ObjectViewSpriteFilename(*Object.ObjectEntry)
      Declare.s ObjectDescription(*Object.ObjectEntry)
      
      Declare.i UseRegObject(*Object.RegisteredObject)
      Declare.i ReleaseRegObject(*Object.RegisteredObject)
      Declare.i CheckRegObject(*Object.RegisteredObject)
      
      Declare.i FreeObject(*Object.Object)
      Declare.i RemoveObject(*Scene.Scene, Layer.i, X.i, Y.i)
      
      Declare.i GetObject(*Scene.Scene, Layer.i, X.i, Y.i)
      Declare.i PutObject(Layer.i, *Registered.RegisteredObject, X.i, Y.i)
      Declare.i CopyObject(*Object.Object, Layer.i, X.i, Y.i)
      Declare.i GetObjectsOf(*Callback, *Scene.Scene, Layer.i, List *Object.Object())
      
      Declare.i SelectObject(*Entry.ObjectEntry)
      Declare.i ChangeObject(*Object.RegisteredObject = #Null)
      Declare.i LoadObjectViewSprite(*Object.ObjectEntry)
      
      Declare.i GetBank(Name.s)
  CompilerEndSelect
  
  Declare.i LoadBank(Name.s)
  Declare.i GetBank(Name.s)
;}
;{ ObjectAction
  CompilerSelect #Mode
    CompilerCase #Mode_Game
      Declare.i ReadObjectAction()
      Declare.i InitializeObjectAction(*Action.ObjectAction)
      
      Declare.i CallObjectAction(*Action.ObjectAction)
    CompilerCase #Mode_Editor
      Declare.i ObjectActionCount(*Registered.RegisteredObject)
      Declare.s ObjectActionName(*Registered.RegisteredObject, ID.i)
      Declare.s ObjectActionDescription(*Registered.RegisteredObject, ID.i)
      
      Declare.i CreateObjectAction()
      Declare.i EditObjectAction(*Action.ObjectAction)
      Declare.i FreeObjectAction(*Action.ObjectAction)
      
      Declare.i WriteObjectAction(*Action.ObjectAction)
      Declare.i ReadObjectAction()
      Declare.i InitializeObjectAction(*Action.ObjectAction)
  CompilerEndSelect
;}
;{ Particle
  CompilerIf #Mode = #Mode_Game
    Declare.i CreateParticle(*Frameset.AnimationFrameset, X.i, Y.i, AnimationSpeed.f, *Callback = #Null)
    Declare.i FreeParticle(*Particle.Particle)
    Declare.i ExamineParticles()
    
    Declare.i EmitParticle_TinyExplosion(X.i, Y.i)
    Declare.i EmitParticle_KilledEnemy(X.i, Y.i)
    Declare.i EmitParticle_Glitter(X.i, Y.i)
    
    Declare.i DisplayWhiteLight(X.i, Y.i, Transparency.i = 255)
    Declare.i DisplayYellowLight(X.i, Y.i, Transparency.i = 255)
    Declare.i DisplayCyanLight(X.i, Y.i, Transparency.i = 255)
    Declare.i DisplaySmallWhiteLight(X.i, Y.i, Transparency.i = 255)
  CompilerEndIf
;}
;{ Player
  CompilerIf #Mode = #Mode_Game
    Declare.i ExaminePlayerField()
    Declare.i ChangePlayerAction(Action.i)
    
    Declare.i LocatePlayer(X.i, Y.i, *Scene.Scene = #Null)
    
    Declare.i EnhanceMultiplierCounter()
    Declare.i SetMultiplier(Multiplier.i)
    Declare.i AddScore(Score.i, Reason.s = "")
    Declare.i ShowComment(Comment.s, Duration.i = 100)
    
    Declare.i LoseLife()
    Declare.i HitPlayer(Energy.i)
    Declare.i PlayerLands()
    
    Declare.i PlayerInScene()
    Declare.i IsWall(DX.i, DY.i)
    Declare.i IsGround(DX.i, DY.i)
    
    Declare.i NeighbourObjectQualifiedForClambering(DX.i, DY.i)
    Declare.i ObjectQualifiedForClambering()
    Declare.i NeighbourWallQualifiedForClimbing(Direction.i, DY.i)
    Declare.i WallQualifiedForClimbing(Direction.i)
    
    Declare.i UseGravity()
    
    Declare.i DrawHUD(DisplayItems.i = #True)
    
    Declare.i InitializeControl()
    Declare.i ExamineControl()
    
    Declare.i ExaminePlayer()
    Declare.i DisplayPlayer()
    
    Declare.i InitializePlayer()
    Declare.i LoadPlayerSettings()
  CompilerEndIf
;}
;{ Shot
  CompilerIf #Mode = #Mode_Game
    Declare.i EmitShot(Direction.i, Speed.f = #Player_ShotSpeed)
    Declare.i FreeShot(*Shot.Shot)
    
    Declare.i Shoot(Direction.i)
    Declare.i ShotCollision(*Shot.Shot, DeltaX.i, DeltaY.i)
    Declare.i EmitShotParticle(*Shot.Shot)
  CompilerEndIf
;}
;{ Act
  CompilerSelect #Mode
    CompilerCase #Mode_Game
      Declare.i LoadEventBank(Name.s)
      Declare.i LoadEvent(*Bank.EventBank, Callback.s, GlobalLoad.i = #False)
      
      Declare.i CreateEvent(*Act.Act, *Callback, *Parameter = #Null)
      Declare.i CreateLoadEvent(*Act.Act, *Callback)
      Declare.i ExamineEvent(*Event.Event)
      
      Declare.i LoadEffectBank(Name.s)
      Declare.i LoadEffect(*Bank.EffectBank, Callback.s, GlobalLoad.i = #False)
      
      Declare.i CheckSpecialEvent(*Act.Act, *Event.Event)
      Declare.i SpecialEventID(Name.s)
      Declare.i CallSpecialEventEffects(*Act.Act, ID.i)
      
      Declare.i CreateLoadEffect(*Event.Event, *Callback)
      Declare.i CallEffect(*Effect.Effect)
      Declare.i CallEffects(*Event.Event)
      Declare.i ContinueCallingEffects(*Event.Event)
      
      Declare.i CreateAct(Name.s)
      Declare.i ExamineAct(*Act.Act)
      Declare.i ExamineParallelActs()
      Declare.i GetAct(Name.s)
      
      Declare.i CallStartEvents()
      
      Declare.i ChangeAct(*Act.Act)
      Declare.i StartAct(*Act.Act)
      Declare.i AddParallelAct(*Act.Act)
      Declare.i RemoveParallelAct(*Act.Act)
    CompilerCase #Mode_Editor
      Declare.i InitializeEventBank(*Bank.EventBank)
      Declare.i LoadEventBanks(Path.s)
      Declare.i ExtractEventEntries(*Bank.EventBank, *Memory.All, *Parent.Entry)
      Declare.i GetEventBank(Name.s)
      Declare.i LoadEventBank(Name.s)
      
      Declare.i GetEventByCallbackName(*Bank.EventBank, Callback.s)
      Declare.s EventName(*Entry.EventEntry)
      Declare.s EventDescription(*Entry.EventEntry)
      
      Declare.i CreateEvent(*Act.Act, *Entry.EventEntry, Initialize.i = #True)
      Declare.i GetEvent(*Act.Act, Name.s)
      
      Declare.s EventText(*Event.Event)
      Declare.i EditEvent(*Event.Event)
      
      Declare.i UseEvent(*Event.Event)
      Declare.i ReleaseEvent(*Event.Event)
      Declare.i FreeEvent(*Event.Event)
      Declare.i RemoveEvent(*Event.Event)
      
      
      Declare.i InitializeEffectBank(*Bank.EffectBank)
      Declare.i LoadEffectBanks(Path.s)
      Declare.i ExtractEffectEntries(*Bank.EffectBank, *Memory.All, *Parent.Entry)
      Declare.i GetEffectBank(Name.s)
      Declare.i LoadEffectBank(Name.s)
      
      Declare.i GetEffectByCallbackName(*Bank.EffectBank, Callback.s)
      Declare.s EffectName(*Entry.EffectEntry)
      Declare.s EffectDescription(*Entry.EffectEntry)
      
      Declare.i CreateEffect(*Event.Event, *Entry.EffectEntry, Initialize.i = #True)
      
      Declare.s EffectText(*Effect.Effect)
      Declare.i EditEffect(*Effect.Effect)
      
      Declare.i FreeEffect(*Effect.Effect)
      Declare.i RemoveEffect(*Effect.Effect)
      Declare.i MoveEffectUp(*Effect.Effect)
      Declare.i MoveEffectDown(*Effect.Effect)
      
      
      Declare.i CreateAct(Name.s = "Unknow", Description.s = "")
      Declare.i GetAct(Name.s)
      Declare.s RenameAct(*Act.Act, Name.s)
      
      Declare.i UseAct(*Act.Act)
      Declare.i ReleaseAct(*Act.Act)
      
      Declare.i FreeAct(*Act.Act)
      Declare.i RemoveAct(*Act.Act)
  CompilerEndSelect
;}
;{ PlayerCore
  CompilerIf #Mode = #Mode_Game
    Declare.i ExaminePlayer()
    Declare.i DisplayPlayer()
    
    Declare.i InitializePlayer()
    Declare.i LoadPlayerSettings()
  CompilerEndIf
;}
;{ Scene
  CompilerSelect #Mode
    CompilerCase #Mode_Game
      Declare.i CreateScene(Width.i = 1, Height.i = 1)
      Declare.i DisplayScene(DisplayPlayer.i = #True)
      
      Declare.i GetScene(X.i, Y.i)
      Declare.i PlayerNeighbourScene(DX.i, DY.i)
      
      Declare.i PlayerRelativeSceneX()
      Declare.i PlayerRelativeSceneY()
      Declare.i PlayerSceneX()
      Declare.i PlayerSceneY()
      
      Declare.i SceneDeferralX(*Scene.Scene, *OtherScene.Scene)
      Declare.i SceneDeferralY(*Scene.Scene, *OtherScene.Scene)
      
      Declare.i FreeTable()
      Declare.i CreateTable(*Scene.Scene)
      
      Declare.i WakeUpScene(*Scene.Scene)
      Declare.i FreezeScene(*Scene.Scene)
      Declare.i ChangeScene(*PreviousScene.Scene, *NextScene.Scene)
    CompilerCase #Mode_Editor
      Declare.i FreeTable()
      Declare.i CreateTable()
      Declare.i SaveTable()
      
      Declare.i TableArray(Layer.i)
      Declare.i TableCount(*Scene.Scene, Layer.i)
      
      Declare.i GetField(*Scene.Scene, Layer.i, X.i, Y.i)
      Declare.i SetField(*Scene.Scene, Layer.i, X.i, Y.i, *Field.Field)
      
      Declare.i LayerArray(*Scene.Scene, Layer.i)
      Declare.i LayerSize(*Scene.Scene, Layer.i)
      
      Declare.i CreateScene(X.i, Y.i, *Background.RegisteredBackground = #Null)
      Declare.i GetScene(X.i, Y.i)
      Declare.i GetMapBorders(*MinX.Integer, *MinY.Integer, *MaxX.Integer, *MaxY.Integer)
      
      Declare.i FreeScene(*Scene.Scene)
      Declare.i RemoveScene(X.i, Y.i)
      
      Declare.i ClearScene()
      Declare.i AdaptScene()
      Declare.i ResizeScene(Width.i, Height.i)
      
      Declare.i ChangeScene(X.i, Y.i, ApplyBackground.i = #False, FreeScene.i = #True)
      Declare.i IsSceneEmpty(*Scene.Scene)
      Declare.i RemoveEmptyScene(X.i, Y.i)
      Declare.i IsMainSceneEqual(X.i, Y.i, *Main.Scene)
  CompilerEndSelect
;}
;{ Screen
  Declare.i SetScreenSettings(Width.i ,Height.i, BitsPerPel.i ,Frequency.i ,TestMode.i = #Disabled)
  Declare.i GetScreenSettings(*Width.Integer, *Height.Integer, *BitsPerPel.Integer, *Frequency.Integer)
  Declare.i OpenGMetricaScreen(Width.i, Height.i)
  Declare.i CloseGMetricaScreen()
  
  Declare.i GetMouseX()
  Declare.i GetMouseY()
;}
;{ Screens
  Declare.i OpenLoadingScreen(ClearScreen.i = #False)
  Declare.i CloseLoadingScreen()
  Declare.i OpenSavingScreen(ClearScreen.i = #False)
  Declare.i CloseSavingScreen()
;}
;{ Tiles
  CompilerSelect #Mode
    CompilerCase #Mode_Game
      Declare.i LoadTileset(Name.s)
      Declare.i GetTileset(Name.s)
      
      Declare.i LoadTileByIndex(*Tileset.Tileset, Index.i)
      Declare.i LoadTile(*Tileset.Tileset, X.i, Y.i)
      Declare.i CreateTile(Sprite.i, X.i, Y.i)
    CompilerCase #Mode_Editor
      Declare.i LoadTilesets(Path.s, *Parent.Entry)
      Declare.i GetTileset(Name.s)
      
      Declare.s TilesetPath(*Tileset.TilesetEntry)
      Declare.s TilesetFilename(*Tileset.TilesetEntry)
      Declare.s TilesetName(*Tileset.TilesetEntry)
      Declare.i LoadTilesetSprite(*Tileset.TilesetEntry)
      
      Declare.i LoadRegTileset(*Entry.TilesetEntry)
      Declare.i FreeRegTileset(*Tileset.RegisteredTileset)
      
      Declare.i LoadRegTile(X.i, Y.i, *Tileset.RegisteredTileset)
      Declare.i FreeRegTile(*Tile.RegisteredTile)
      
      Declare.i GetRegTile(X.i, Y.i, *Tileset.RegisteredTileset)
      Declare.i RegTileX(*Tile.RegisteredTile)
      Declare.i RegTileY(*Tile.RegisteredTile)
      
      Declare.i UseRegTile(*Tile.RegisteredTile)
      Declare.i ReleaseRegTile(*Tile.RegisteredTile)
      
      Declare.i GetTile(*Scene.Scene, Layer.i, X.i, Y.i)
      Declare.i PutTile(Layer.i, *Registered.RegisteredTile, X.i, Y.i)
      Declare.i FreeTile(*Tile.Tile)
      Declare.i RemoveTile(*Scene.Scene, Layer.i, X.i, Y.i)
      Declare.i CopyTile(*Tile.Tile, Layer.i, X.i, Y.i)
      
      Declare.i SelectTile(*Tileset.RegisteredTileset, X.i, Y.i)
      Declare.i ChangeTile(*Tile.RegisteredTile = #Null)
  CompilerEndSelect
;}
;{ Selection
  CompilerIf #Mode = #Mode_Game
    Declare.i CreateSelection(Size.i)
    Declare.i ReadTileSelection()
    Declare.i ReadObjectSelection()
    Declare.i ReadSelection()
    
    Declare.i InitializeSelection(*Selection.Selection)
    Declare.i InitializeObjectSelection(*Selection.Selection)
    
    Declare.i HideSelectedObjects(*Selection.Selection, State)
    Declare.i ForEachSelectedObject(*Selection.Selection, *Callback, *Parameter = #Null)
    
    Declare.i IsPlayerInSelection(*Selection.Selection)
  CompilerEndIf
;}
;{ Blur
  CompilerIf #Mode = #Mode_Game
    Declare.i CreateBlur(Length.i, DeltaTime.i)
    
    Declare.i UpdateBlur(*Blur.Blur)
    Declare.i AddBlurFrame(*Blur.Blur, Sprite.i, X.f, Y.f)
    
    Declare.i DisplayBlur(*Blur.Blur)
    Declare.i ResetBlur(*Blur.Blur)
  CompilerEndIf
;}
;{ Replay
  CompilerIf #Mode = #Mode_Game
    Declare.i ReplayKeys()
    Declare.i RecordKeys()
    
    Declare.i PackReplay()
    Declare.i UnpackReplay(Filename.s)
  CompilerEndIf
;}
;{ Game
  CompilerIf #Mode = #Mode_Game
    Declare.i DisplayBottomSignal()
    
    Declare.i Play()
  CompilerEndIf
;}

CompilerIf #Mode = #Mode_Editor
  ;{ Main
    Declare.i Editor()
  ;}
  ;{ Interface
    Declare.i IsCursorInScene()
    Declare.i IsCursorInBar()
    Declare.i IsCursorInTileset()
    
    Declare.i ClearAll()
    Declare.i ExamineEvents()
    Declare.s Tip()
    Declare.i ExamineSceneEvents()
    Declare.i ExamineInterfaceEvents()
    Declare.i ExamineTilesetEvents()
    
    Declare.i AbsoluteX(*Scene.Scene, X.i)
    Declare.i AbsoluteY(*Scene.Scene, Y.i)
  ;}
  ;{ Selection
    Declare.i CreateSelection()
    Declare.i FreeSelection(*Selection.Selection)
    
    Declare.i SelectField(*Selection.Selection, *Scene.Scene, Layer.i, X.i, Y.i)
    Declare.i GetSelected(*Selection.Selection, *Scene.Scene, Layer.i, X.i, Y.i)
    Declare.i RemoveSelected(*Selection.Selection, *Scene.Scene, Layer.i, X.i, Y.i)
    Declare.i GetSelectionBorders(*Selection.Selection)
    Declare.i InsertSelection(*Selection.Selection, *Scene.Scene, X.i, Y.i)
    Declare.i GetSelectedField(*Selected.Selected)
    
    Declare.i SelectedMainScene(*Selected.Selected)
    Declare.i IsSelectionInSingleScene(*Selection.Selection)
    
    Declare.i WriteSelection(*Selection.Selection)
    Declare.i WriteObjectSelection(*Selection.Selection)
    Declare.i WriteLayerlessSelection(*Selection.Selection)
    Declare.i ReadSelection(*Selection.Selection = #Null)
    Declare.i ReadObjectSelection(*Selection.Selection = #Null)
    Declare.i ReadLayerlessSelection(*Selection.Selection = #Null)
    
    Declare.i InitializeObjectSelection(*Selection.Selection)
    
    Declare.i SelectionMode(*Selection.Selection, *Callback, Info.s)
    
    Declare.i Selection_Callback_AllLayers(*Selection.Selection, Event.i, *Parameter)
    Declare.i Selection_Callback_None(*Selection.Selection, Event.i, *Parameter)
    Declare.i Selection_Callback_Objects(*Selection.Selection, Event.i, *Parameter)
    Declare.i Selection_Callback_SingleObject(*Selection.Selection, Event.i, *Parameter)
    Declare.i Selection_Callback_SingleField(*Selection.Selection, Event.i, *Parameter)
    Declare.i Selection_Callback_SameObjects(*Selection.Selection, Event.i, *Parameter)
  ;}
  ;{ SceneViewer
    Declare.i LoadSceneViewerGraphics()
    Declare.i DrawSceneViewerMap()
    Declare.i UpdateSceneViewerImage()
    Declare.i ExamineSceneViewerCursor()
    Declare.i SelectSceneViewerScene(X.i, Y.i)
    Declare.i CloseSceneViewer()
    Declare.i OpenSceneViewer()
    
    Declare.i SelectedSceneViewerX()
    Declare.i SelectedSceneViewerY()
  ;}
  ;{ TilesetViewer
    Declare.i UpdateTilesetViewerTilesets(*Parent.Entry, Level.i = 0)
    Declare.i DrawTilesetViewerBackground()
    Declare.i UpdateTilesetViewerImage()
    Declare.i SelectTilesetViewerTileset(*Tileset.TilesetEntry)
    Declare.i OpenTilesetViewer()
  ;}
  ;{ ObjectViewer
    Declare.i UpdateObjectViewerObjects(*Parent.Entry, Level.i = 0)
    Declare.i UpdateObjectViewerImage()
    Declare.i SelectObjectViewerObject(*Object.ObjectEntry)
    Declare.i OpenObjectViewer()
    Declare.i CloseObjectViewer()
  ;}
  ;{ BackgroundViewer
    Declare.i UpdateBackgroundViewerBackgrounds(*Parent.Entry, Level.i = 0)
    Declare.i BackgroundViewerPreview()
    Declare.i SelectBackgroundViewerBackground(*Background.BackgroundEntry)
    Declare.i OpenBackgroundViewer()
  ;}
  ;{ ChallangeViewer
    Declare.i UpdateChallangeViewerChallanges(*Folder.Entry, Level.i = 0)
    Declare.i SelectChallangeViewerChallange(*Challange.Challange)
    Declare.i CloseChallangeViewer()
    Declare.i OpenChallangeViewer()
  ;}
  ;{ Display
    Declare.i RenderScene()
    Declare.i RenderEditor()
    
    Declare.i DisplayCursor()
    Declare.i DisplaySelection(*Selection.Selection, Transparency.i = 100)
    Declare.i DisplaySelected(*Selection.Selection, X.i, Y.i, Transparency.i = 100)
    Declare.i DisplayBar()
    
    Declare.i DisplayBackground()
    Declare.i DisplayTilesetBackground()
    Declare.i DisplayScene()
    Declare.i DisplayTileLayer(Layer.i)
    Declare.i DisplayObjectLayer(Layer.i)
    Declare.i DisplayStartPosition()
    
    Declare.i DrawTip(Tip.s)
    Declare.i DrawComment(Comment.s, FrontColor.i = $C0C0C0, BackColor.i = #Black)
    
    Declare.i DrawEntry(*Item.QueueItem, Index.i = 0)
    Declare.i DrawEntries(*Item.QueueItem, Max.i, Index.i = 0)
  ;}
  ;{ Camera
    Declare.i SetCamera(X.i, Y.i)
    Declare.i UpdateCamera()
  ;}
  ;{ Load&Save
    Declare.i PassTilesets()
    Declare.i PassObjects()
    Declare.i PassBackgrounds()
    Declare.i PassSceneObjects(*Scene.Scene)
    Declare.i PassScene(*Scene.Scene)
    Declare.i PassScenes()
    Declare.i PassEvents()
    Declare.i PassEffects()
    Declare.i PassActs()
    
    Declare.i WriteTilesets()
    Declare.i WriteBanks()
    Declare.i WriteBackgrounds()
    Declare.i WriteScene(*Scene.Scene, LocalSettings.i = #True)
    Declare.i WriteTileGroup(*Cell.Integer)
    Declare.i WriteObjectGroup(*Cell.Integer)
    Declare.i WriteFixObjectGroup(*Cell.Integer)
    Declare.i WriteScenes()
    Declare.i WriteEvents()
    Declare.i WriteEffects()
    Declare.i WriteActs()
    
    Declare.i SaveEditorChallange()
    
    Declare.i ReadTilesets()
    Declare.i ReadBanks()
    Declare.i ReadBackgrounds()
    Declare.i ReadScene(X.i, Y.i, *Main.Scene = #Null)
    Declare.i ReadTileGroup(*Scene.Scene, Layer.i)
    Declare.i ReadObjectGroup(*Scene.Scene, Layer.i)
    Declare.i ReadFixObjectGroup(*Scene.Scene, Layer.i)
    Declare.i ReadScenes()
    Declare.i ReadActs()
    
    Declare.i InitializeObjects()
    Declare.i InitializeActs()
    
    Declare.i LoadEditorChallange(Filename.s)
  ;}
CompilerEndIf

;{ Menu
  ;{ Main
    Declare.i OpenLoadingScreen(ClearScreen.i = #False)
    Declare.i CloseLoadingScreen()
    Declare.i OpenSavingScreen(ClearScreen.i = #False)
    Declare.i CloseSavingScreen()
    
    Declare.i Menu()
  ;}
;}
; IDE Options = PureBasic 4.51 (Windows - x86)
; CursorPosition = 153
; FirstLine = 6
; Folding = DiAA1Bh
; EnableXP