;**************************************************************
;     Display
;**************************************************************
;   Copyright (c) 2010 Josef Sniatecki
;
; This program is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License
; As published by the Free Software Foundation; either version
; 3 of the License, Or (at your option) any later version.
;
; This program is distributed in the hope that it will be
; useful, but WITHOUT ANY WARRANTY; without even the implied
; warranty of MERCHANTABILITY Or FITNESS For A PARTICULAR
; PURPOSE. See the GNU General Public License For more details.
;
; You should have received a copy of the GNU General Public
; License along With this program;
; If Not, see "http://www.gnu.org/licenses/".
;**************************************************************



Global Display_MinX.i, Display_MinY.i, Display_MaxX.i, Display_MaxY.i

Procedure.i DisplayCursor()
  With Editor
  
  If \OverY < #SceneHeight
    DisplaySprite3D(\CursorSprite3D, \TileX * #FieldSize - \CameraX, \TileY * #FieldSize - \CameraY)
  Else
    DisplaySprite3D(\CursorSprite3D, \OverX * #FieldSize, \OverY * #FieldSize)
  EndIf
  
  EndWith
EndProcedure
Procedure.i DisplaySelection(*Selection.Selection, Transparency.i = 100)
  Protected *Selected.Selected, *Item.QueueItem, *Main.Scene
  Protected *Scene.Scene
  Protected DX.i, DY.i
  
  With Editor
  
  *Main = \CurrentScene\Main
  
  *Item = *Selection\Queue\First
  While *Item
    *Selected = *Item\Memory
    *Scene    = GetScene(*Selected\SceneX, *Selected\SceneY)
    If *Scene\Main = *Main
      DX = (*Scene\X - *Main\X) * #SceneWidth * #FieldSize  - \CameraX
      DY = (*Scene\Y - *Main\Y) * #SceneHeight * #FieldSize - \CameraY
      If \Layer = *Selected\Layer
        DisplaySprite3D(\CursorSprite3D, *Selected\X * #FieldSize + DX, *Selected\Y * #FieldSize + DY, Transparency)
      Else
        DisplaySprite3D(\CursorSprite3D, *Selected\X * #FieldSize + DX, *Selected\Y * #FieldSize + DY, 50)
      EndIf
    EndIf
    _Move(*Item, N)
  Wend
  
  EndWith
EndProcedure
Procedure.i DisplaySelected(*Selection.Selection, X.i, Y.i, Transparency.i = 100)
  Protected *Selected.Selected, *Item.QueueItem, *Tile.Tile, *Object.Object
  Protected *Scene.Scene
  
  *Item = *Selection\Queue\First
  While *Item
    *Selected = *Item\Memory
    
    *Scene = GetScene(*Selected\SceneX, *Selected\SceneY)
    If *Selected\Layer < #BackgroundObject1
      *Tile = GetTile(*Scene, *Selected\Layer, *Selected\X, *Selected\Y)
      If *Tile
        DisplaySprite3D(*Tile\Registered\Sprite3D, (AbsoluteX(*Scene, *Selected\X) - *Selection\MinX) * #FieldSize + X, (AbsoluteY(*Scene, *Selected\Y) - *Selection\MinY) * #FieldSize + Y, Transparency)
      EndIf
    Else
      *Object = GetObject(*Scene, *Selected\Layer, *Selected\X, *Selected\Y)
      If *Object
        DisplaySprite3D(*Object\Registered\Sprite3D, (AbsoluteX(*Scene, *Selected\X) - *Selection\MinX) * #FieldSize + X, (AbsoluteY(*Scene, *Selected\Y) - *Selection\MinY) * #FieldSize + Y, Transparency)
      EndIf
    EndIf
    
    _Move(*Item, N)
  Wend
EndProcedure
Procedure.i DisplayBar()
  With Editor
  
  DisplaySprite3D(\BarSprite3D, 0, #FieldSize * #SceneHeight)
  
  DisplayScreenGadgets3D()
  
  DisplaySprite3D(\LayerSprite3D, 9 * #FieldSize + (\Layer % 6) * #FieldSize, (\Layer / 6) * #FieldSize + #SceneHeight * #FieldSize)
  If \DisplaySingleLayer
    DisplaySprite3D(\LayerSprite3D, 9 * #FieldSize + 5 * #FieldSize, (#SceneHeight + 1) * #FieldSize)
  EndIf
  
  EndWith
EndProcedure

Procedure.i DisplayBackground()
  If Editor\CurrentScene\Background 
    DisplaySprite(Editor\CurrentScene\Background\Sprite, 0, 0)
  Else
    ClearScreen(#White)
  EndIf
EndProcedure
Procedure.i DisplayTilesetBackground()
  If Editor\CurrentScene\Background
    DisplaySprite(Editor\CurrentScene\Background\Sprite, 0, #EditorScreenHeight - 192)
  EndIf
EndProcedure
Procedure.i DisplayScene()
  Display_MinX = Editor\CameraX / #FieldSize - 1                           : _Min(Display_MinX, 0)
  Display_MinY = Editor\CameraY / #FieldSize - 1                           : _Min(Display_MinY, 0)
  Display_MaxX = (Editor\CameraX + #SceneWidth * #FieldSize)  / #FieldSize : _Max(Display_MaxX, Editor\CurrentScene\Width * #SceneWidth - 1)
  Display_MaxY = (Editor\CameraY + #SceneHeight * #FieldSize) / #FieldSize : _Max(Display_MaxY, Editor\CurrentScene\Height * #SceneHeight - 1)
  
  DisplayTileLayer(#BackgroundTile1)
  DisplayTileLayer(#BackgroundTile2)
  DisplayObjectLayer(#BackgroundObject1)
  DisplayTileLayer(#BackgroundTile3)
  DisplayObjectLayer(#BackgroundObject2)
  DisplayTileLayer(#Ground)
  DisplayTileLayer(#Tile)
  DisplayObjectLayer(#Object)
  DisplayStartPosition()
  DisplayObjectLayer(#ForegroundObject1)
  DisplayTileLayer(#ForegroundTile)
  DisplayObjectLayer(#ForegroundObject2)
EndProcedure
Procedure.i DisplayTileLayer(Layer.i)
  Static X.i, Y.i, *Tile.Tile, Transparency.i
  
  If Editor\DisplaySingleLayer And Editor\Layer <> Layer
    Transparency = 50
  Else
    Transparency = 255
  EndIf
  
  For Y = Display_MinY To Display_MaxY
    For X = Display_MinX To Display_MaxX
      *Tile = GetField(Editor\CurrentScene, Layer, X, Y)
      If *Tile
        DisplaySprite3D(*Tile\Registered\Sprite3D, X * #FieldSize - Editor\CameraX, Y * #FieldSize - Editor\CameraY, Transparency)
      EndIf
    Next
  Next
EndProcedure
Procedure.i DisplayObjectLayer(Layer.i)
  Static X.i, Y.i, *Object.Object, Transparency.i
  
  If Editor\DisplaySingleLayer And Editor\Layer <> Layer
    Transparency = 50
  Else
    Transparency = 255
  EndIf
  
  For Y = Display_MaxY To Display_MinY Step -1
    For X = Display_MaxX To Display_MinX Step -1
      *Object = GetField(Editor\CurrentScene, Layer, X, Y)
      If *Object
        DisplaySprite3D(*Object\Registered\Sprite3D, X * #FieldSize - Editor\CameraX, Y * #FieldSize - Editor\CameraY, Transparency)
      EndIf
    Next
  Next
EndProcedure
Procedure.i DisplayStartPosition()
  If Editor\StartScene = Editor\CurrentScene
    If Editor\StartX => Display_MinX And Editor\StartX <= Display_MaxX And Editor\StartY => Display_MinY And Editor\StartY <= Display_MaxY
      DisplayTransparentSprite(Editor\StartSceneSprite, Editor\StartX * #FieldSize - Editor\CameraX, Editor\StartY * #FieldSize - Editor\CameraY)
    EndIf
  EndIf
EndProcedure

Procedure.i DrawTip(Tip.s)
  DrawingMode(#PB_2DDrawing_Transparent)
  DrawShadowText(5, (#SceneHeight - 1) * #FieldSize, Tip, $C0C0C0, #Black)
EndProcedure
Procedure.i DrawComment(Comment.s, FrontColor.i = $C0C0C0, BackColor.i = #Black)
  DrawingMode(#PB_2DDrawing_Transparent)
  DrawShadowText(5, #EditorScreenHeight - 192 + 5, Comment, FrontColor, BackColor)
EndProcedure

; IDE Options = PureBasic 4.50 (Windows - x86)
; CursorPosition = 91
; FirstLine = 11
; Folding = IP-
; EnableXP