Procedure.i IsCursorInScene()
  If Editor\OverY < #SceneHeight
    ProcedureReturn #True
  Else
    ProcedureReturn #False
  EndIf
EndProcedure
Procedure.i IsCursorInBar()
  If Editor\OverY => #SceneHeight And Editor\OverY <= #SceneHeight + 1
    ProcedureReturn #True
  Else
    ProcedureReturn #False
  EndIf
EndProcedure
Procedure.i IsCursorInTileset()
  If Editor\OverY > #SceneHeight + 1
    ProcedureReturn #True
  Else
    ProcedureReturn #False
  EndIf
EndProcedure

Procedure.i ClearAll()
  SaveTable()
  FreeTable()
  
  ClearQueue(Editor\UsedScenes)
  ClearQueue(*Acts)
EndProcedure
Procedure.i ExamineEvents()
  Protected X.i, Y.i
  
  With Editor
  
  \Event  = WindowEvent()
  \Window = EventWindow()
  \Gadget = EventGadget()
  \wParam = EventwParam()
  \lParam = EventlParam()
  
  \OverX = GetMouseX() / #FieldSize
  \OverY = GetMouseY() / #FieldSize
  If \OverX < 0
    \OverX = 0
  ElseIf \OverX => #SceneWidth
    \OverX = #SceneWidth - 1
  EndIf
  If \OverY < 0
    \OverY = 0
  ElseIf \OverY => #SceneHeight + 2 + 8
    \OverY = #SceneHeight + 2 + 8 - 1
  EndIf
  
  If \OverX => 0 And \OverX < #SceneWidth And \OverY => 0 And \OverY < #SceneHeight
    X = (GetMouseX() + \CameraX) / #FieldSize
    Y = (GetMouseY() + \CameraY) / #FieldSize
    If X < 0
      X = 0
    ElseIf X => \CurrentScene\Width * #SceneWidth
      X = \CurrentScene\Width * #SceneWidth - 1
    EndIf
    If Y < 0
      Y = 0
    ElseIf Y => \CurrentScene\Height * #SceneHeight
      Y = \CurrentScene\Height * #SceneHeight - 1
    EndIf
  Else
    X = (#ScreenWidth  / 2 + \CameraX) / #FieldSize
    Y = (#ScreenHeight / 2 + \CameraY) / #FieldSize
  EndIf
  
  \OverSceneX = \CurrentScene\X + (X / #SceneWidth)
  \OverSceneY = \CurrentScene\Y + (Y / #SceneHeight)
  \OverScene  = GetScene(\OverSceneX, \OverSceneY)
  
  \TileX = X
  \TileY = Y
  
  EndWith
EndProcedure
Procedure.s Tip()
  Protected Tip.s, Layer.i
  
  With Editor
  
  If \OverY => #SceneHeight And \OverY < #SceneHeight + 2
    Select \OverX
      Case 0, 1
        Tip = "Select tileset"
      Case 2, 3
        Tip = "Select object"
      Case 4, 5
        Tip = "Select a (new) scene"
      Case 6, 7
        If \CurrentScene\Background
          Tip = "Change background"
        Else
          Tip = "Select background"
        EndIf
      Case 9 To 14
        Layer = (\OverX - 9) + (\OverY - #SceneHeight)*6
        Select Layer
          Case #BackgroundTile1
            Tip = "Background layer 1 (deepest)"
          Case #BackgroundTile2, #BackgroundTile3
            Tip = "Background layer " + Str(Layer + 1)
          Case #Tile
            Tip = "Wall (unpassable)"
          Case #Ground
            Tip = "Ground (passable form below)"
          Case #ForegroundTile
            Tip = "Foreground layer (in fornt of player)"
          Case #BackgroundObject1, #BackgroundObject2
            Tip = "Background object layer " + Str(Layer - #BackgroundObject1 + 1)
          Case #Object
            Tip = "Static object layer"
          Case #ForegroundObject1, #ForegroundObject2
            Tip = "Foreground object layer " + Str(Layer - #ForegroundObject1 + 1)
          Default
            Tip = "hilight current layer (on/off)"
        EndSelect
      Case 16
        If \OverY = #SceneHeight
          Tip = "Adapt scene to neighbour scenes"
        Else
          Tip = "Clear scene"
        EndIf
      Case 17
        If \OverY = #SceneHeight
          Tip = "Adjust tiles"
        Else
          Tip = ""
        EndIf
      Case 21
        If \OverY = #SceneHeight
          Tip = "Story editor"
        EndIf
      Case 22
        If \OverY = #SceneHeight
          Tip = "Scene properties"
        Else
          Tip = "Challange properties"
        EndIf
      Case 23, 24
        If \OverY = #SceneHeight
          Tip = "Save challange"
        Else
          Tip = "Load challange"
        EndIf
      Case 26, 25
        Tip = "Help"
      Default
        Tip = ""
    EndSelect
  Else
    Tip = ""
  EndIf
  
  ProcedureReturn Tip
  
  EndWith
EndProcedure
Procedure.i ExamineSceneEvents()
  Protected *Queue.Queue, *Item.QueueItem, *Tile.Tile, *Object.Object
  Protected X.i, Y.i
  
  With Editor
  
  If \Event = #WM_LBUTTONDOWN
    Debug \wParam
  EndIf
  
  If \LeftClickAllowed And (\Event = #WM_LBUTTONDOWN And \wParam & #MK_CONTROL)
    ; Durch [STRG + MOUSE LEFT] kann man die Pipette auf
    ; ein Tile, oder Objekt anwenden:
    
    If \Layer < #BackgroundObject1
      *Tile = GetTile(\CurrentScene, \Layer, \TileX, \TileY)
      If *Tile
        ChangeTile(*Tile\Registered)
      Else
        ChangeTile(#Null)
      EndIf
    Else
      *Object = GetObject(\CurrentScene, \Layer, \TileX, \TileY)
      If *Object
        ChangeObject(*Object\Registered)
      Else
        ChangeObject(#Null)
      EndIf
    EndIf
  ElseIf \LeftClickAllowed And (\Event = #WM_LBUTTONDOWN And \wParam & #MK_SHIFT)
    ; Ein Rechteck wird gezogen:
    \UsingRectangle = #True
    \RectangleX     = \TileX
    \RectangleY     = \TileY
  ElseIf \Event = #WM_LBUTTONUP And \wParam & #MK_SHIFT And \UsingRectangle
    ; Erstellt das gewhlte Rechteck:
    For Y = \RectangleY To \TileY
      For X = \RectangleX To \TileX
        PutTile(\Layer, \CurrentTile, X, Y)
      Next
    Next
    \UsingRectangle = #False
  ElseIf \LeftClickAllowed And (Not \UsingRectangle) And (\Event = #WM_LBUTTONDOWN Or (\Event = #WM_MOUSEMOVE And \wParam & #MK_LBUTTON))
    ; Wird die linke Maustaste bettigt, so knnen
    ; Tiles/Objekte gesetzt werden:
    
    If \Layer < #BackgroundObject1
      If \CurrentTile
        PutTile(\Layer, \CurrentTile, \TileX, \TileY)
      EndIf
    Else
      If \CurrentObject
        PutObject(\Layer, \CurrentObject, \TileX, \TileY)
      EndIf
    EndIf
  ElseIf \Event = #WM_RBUTTONDOWN Or (\LeftClickAllowed And \Event = #WM_MOUSEMOVE And \wParam & #MK_RBUTTON)
    If \Layer < #BackgroundObject1
      RemoveTile(\CurrentScene, \Layer, \TileX, \TileY)
    Else
      RemoveObject(\CurrentScene, \Layer, \TileX, \TileY)
    EndIf
  ElseIf \Event = #WM_MBUTTONDOWN
    ; Wenn die mittlere Maustaste zum Kameraschwenken aktiviert wird:
    \PushedX = GetMouseX()
    \PushedY = GetMouseY()
  ElseIf \Event = #WM_MOUSEMOVE And \wParam & #MK_MBUTTON
    ; Weiterschieben:
    SetCamera(\CameraX + \PushedX - GetMouseX(), \CameraY + \PushedY - GetMouseY())
    \PushedX = GetMouseX()
    \PushedY = GetMouseY()
  ElseIf \Event = #WM_KEYDOWN
    Select \wParam
      Case #VK_INSERT
        \Overwrite = #True - \Overwrite
      Case #VK_R
        ; Fadenkreuz aktivieren:
        \DisplayRuler = #True - \DisplayRuler
      Case #VK_F1
        BitmapGenerator()
    EndSelect
  ElseIf \Event = #WM_KEYUP
    Select \wParam
      Case #VK_SHIFT
        \UsingRectangle = #False
    EndSelect
  EndIf
  
  EndWith
EndProcedure
Procedure.i ExamineInterfaceEvents()
  Protected Layer.i, Text.s
  Protected *Background.BackgroundEntry, *Challange.Challange
  
  With Editor
  
  If *PressedScreenGadget
    Select *PressedScreenGadget
      Case \Menu_Tileset
        ; Select tile/tileset:
        OpenTilesetViewer()
      Case \Menu_Object
        ; Select object:
        OpenObjectViewer()
      Case \Menu_Scene
        ; View scene map:
        If OpenSceneViewer()
          ChangeScene(SelectedSceneViewerX(), SelectedSceneViewerY())
        EndIf
      Case \Menu_Background
        ; Change current background:
        *Background = OpenBackgroundViewer()
        If *Background
          ChangeBackground(\CurrentScene, LoadRegBackground(*Background))
        EndIf
      Case \Menu_B1, \Menu_B2, \Menu_B3, \Menu_W, \Menu_G, \Menu_F, \Menu_O1, \Menu_O2, \Menu_O, \Menu_O3, \Menu_O4, \Menu_SingleLayer
        ; Change current layer:
        Layer = (\OverX - 9) + (\OverY - #SceneHeight) * 6
        If Layer <= #ForegroundObject2
          \Layer = Layer
        Else
          \DisplaySingleLayer = #True - \DisplaySingleLayer
        EndIf
      Case \Menu_Adapt
        AdaptScene()
      Case \Menu_Clear
        If MessageRequester("Question", "Do you really want to clear this scene?", #PB_MessageRequester_YesNo) = #PB_MessageRequester_Yes
          ClearScene()
        EndIf
      Case \Menu_Adjust
        Adjuster(\Layer, \CurrentTile)
      Case \Menu_Grass
        If \CurrentTile
          GrassGenerator(\Layer, \CurrentTile\Tileset, RegTileY(\CurrentTile))
        EndIf
      Case \Menu_StoryEditor
        OpenActEditor()
      Case \Menu_Properties
        OpenSceneProperties(\CurrentScene\Main)
      Case \Menu_Challange
        OpenChallangeProperties()
      Case \Menu_Save
        SaveEditorChallange()
      Case \Menu_Load
        *Challange = OpenChallangeViewer()
        If *Challange
          LoadEditorChallange(ChallangeFilename(*Challange) + "\")
        EndIf
      Case \Menu_Help
        Text = "[ARROW KEYS]"         + #T + #T      + "- Change scene"          + #N
        Text + "[NUM 4/5/6/8]"        + #T + #T      + "- Change tile"           + #N
        Text + "[MOUSE LEFT]"         + #T + #T      + "- Place tile"            + #N
        Text + "[MOUSE RIGHT]"        + #T + #T      + "- Remove tile"           + #N
        Text + "[CTRL + MOUSE LEFT]"  + #T           + "- Pipette"               + #N
        Text + "[CTRL + MOUSE RIGHT]" + #T           + "- Configure object"      + #N
        Text + #N
        Text + "[A]"                  + #T + #T + #T + "- Locate start position" + #N
        Text + "[C]"                  + #T + #T + #T + "- Open/Close copy mode"  + #N
        Text + "[SHIFT]"              + #T + #T + #T + "- Use paste mode"
        MessageRequester("Help", Text)
    EndSelect
  EndIf
  
  EndWith
EndProcedure
Procedure.i ExamineTilesetEvents()
  With Editor
  
  If \Event = #WM_LBUTTONDOWN
    If \CurrentTile ;Tileset vorhanden?
      If \OverX < \CurrentTile\Tileset\Width And (\OverY - #SceneHeight - 2) < \CurrentTile\Tileset\Height
        SelectTile(\CurrentTile\Tileset, \OverX, \OverY - #SceneHeight - 2)
        If \Layer => #BackgroundObject1
          \Layer = #Tile
        EndIf
      EndIf
    EndIf
  EndIf
  
  EndWith
EndProcedure

Procedure.i AbsoluteX(*Scene.Scene, X.i)
  ProcedureReturn (*Scene\X - *Scene\Main\X) * #SceneWidth + X
EndProcedure
Procedure.i AbsoluteY(*Scene.Scene, Y.i)
  ProcedureReturn (*Scene\Y - *Scene\Main\Y) * #SceneHeight + Y
EndProcedure
; IDE Options = PureBasic 4.51 (Windows - x86)
; CursorPosition = 240
; FirstLine = 218
; Folding = --
; EnableXP