;**************************************************************
;     Editor
;**************************************************************
;   Copyright (c) 2010 Josef Sniatecki
;
; This program is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License
; As published by the Free Software Foundation; either version
; 3 of the License, Or (at your option) any later version.
;
; This program is distributed in the hope that it will be
; useful, but WITHOUT ANY WARRANTY; without even the implied
; warranty of MERCHANTABILITY Or FITNESS For A PARTICULAR
; PURPOSE. See the GNU General Public License For more details.
;
; You should have received a copy of the GNU General Public
; License along With this program;
; If Not, see "http://www.gnu.org/licenses/".
;**************************************************************



IncludeFile "Editor\Toolbars.pb"



Procedure.i Selection_Callback_Copy(*Selection.Selection, Event.i, *Parameter)
  Select Event
    Case #Selection_Frame
      If KeyPressedDown(#VK_C)
        ProcedureReturn #True
      Else
        ProcedureReturn #False
      EndIf
    Case #Selection_Mode
      ProcedureReturn #Selection_Mode_AllLayers
    Case #Selection_CheckTile, #Selection_CheckObject
      ProcedureReturn #True
  EndSelect
EndProcedure



Procedure.i RenderScene()
  StartSpecialFX()
    DisplayBackground()
    
    Start3D()
      DisplayScene()
    Stop3D()
  StopSpecialFX()
EndProcedure
Procedure.i RenderEditor()
  Static Tip.s, X.i, Y.i
  
  With Editor
  
  StartSpecialFX()
    DisplayBackground()
    
    Start3D()
      DisplayScene()
      
      ;DisplaySelected(\Selection, (\TileX + (\SceneX - \CurrentScene\Main\X)*#SceneWidth)*#FieldSize - \CameraX, (\TileY + (\SceneY - \CurrentScene\Main\Y)*#SceneHeight)*#FieldSize - \CameraY)
    Stop3D()
    
    DisplayTilesetBackground()
    
    Start3D()
      ;{ Selected tile
        If \UsingRectangle
          For Y = \RectangleY To \TileY
            For X = \RectangleX To \TileX
              DisplaySprite3D(\CurrentTile\Sprite3D, (X + (\SceneX - \CurrentScene\Main\X)*#SceneWidth)*#FieldSize - \CameraX, (Y + (\SceneY - \CurrentScene\Main\Y)*#SceneHeight)*#FieldSize - \CameraY)
            Next
          Next
        Else
          If \OverY < #SceneHeight
            If \Layer < #BackgroundObject1
              If \CurrentTile
                DisplaySprite3D(\CurrentTile\Sprite3D,  (\TileX + (\SceneX - \CurrentScene\Main\X)*#SceneWidth)*#FieldSize - \CameraX, (\TileY + (\SceneY - \CurrentScene\Main\Y)*#SceneHeight)*#FieldSize - \CameraY)
              EndIf
            Else
              If \CurrentObject
                DisplaySprite3D(\CurrentObject\Sprite3D, (\TileX + (\SceneX - \CurrentScene\Main\X)*#SceneWidth)*#FieldSize - \CameraX, (\TileY + (\SceneY - \CurrentScene\Main\Y)*#SceneHeight)*#FieldSize - \CameraY)
              EndIf
            EndIf
          EndIf
        EndIf
      ;}
      ;{ Interface
        If \CurrentTile
          ; Tileset anzeigen:
          DisplaySprite3D(\TilesetSprite3D, 0, #EditorScreenHeight - 192)
          ; Ausgewhltes Tile markieren:
          DisplaySprite3D(\LayerSprite3D, (\CurrentTile\Index % \CurrentTile\Tileset\Width)*#FieldSize, #EditorScreenHeight - 192 + (\CurrentTile\Index / \CurrentTile\Tileset\Width)*#FieldSize)
        EndIf
        
        DisplayStartPosition()
        DisplayCursor()
        DisplayBar()
      ;}
    Stop3D()
  StopSpecialFX()
  
  Tip = Tip()
  
  StartDrawing(ScreenOutput())
    If \DisplayRuler
      ;Horizontalen und vertikalen Lineal zeigen:
      DrawingMode(#PB_2DDrawing_Default)
      Line((\TileX + (\SceneX - \CurrentScene\Main\X) * #SceneWidth) * #FieldSize - \CameraX                     , 0, 1, #SceneHeight * #FieldSize, #Blue)
      Line((\TileX + (\SceneX - \CurrentScene\Main\X) * #SceneWidth) * #FieldSize - \CameraX + #FieldSize - 1    , 0, 1, #SceneHeight * #FieldSize, #Blue)
      Line(0, (\TileY + (\SceneY - \CurrentScene\Main\Y) * #SceneHeight) * #FieldSize - \CameraY                 , #SceneWidth * #FieldSize, 1, #Blue)
      Line(0, (\TileY + (\SceneY - \CurrentScene\Main\Y) * #SceneHeight) * #FieldSize - \CameraY + #FieldSize - 1, #SceneWidth * #FieldSize, 1, #Blue)
    EndIf
    
    DrawingMode(#PB_2DDrawing_Transparent)
    
    ; Tooltip zeigen:
    DrawShadowText(5, (#SceneHeight - 1)*#FieldSize, Tip, $C0C0C0, #Black)
    ; Falls kein Tileset gewhlt wurde, wird ein Hinweis angezeigt:
    If Not \TilesetSprite
      DrawShadowText(5, #EditorScreenHeight - 192 + 5, "No tileset selected", #Red, #Black)
    EndIf
  StopDrawing()
  
  FlipBuffers()
  
  EndWith
EndProcedure
Procedure.i Editor()
  Protected *Tile.Tile, *Object.Object, *Tileset.RegisteredTileset
  Protected X.i, Y.i, Number.i, Text.s
  Protected Tip.s
  Protected *Queue.Queue, *Item.QueueItem
  
  With Editor
  
  OpenGMetricaScreen(#EditorScreenWidth, #EditorScreenHeight)
  
  ;{ Initializations
    Namespace(0) = GetCurrentDirectory()
    Namespace(1) = GetCurrentDirectory()
    Namespace(2) = GetCurrentDirectory()
    
    
    LoadChallanges("Challanges\", *ChallangeFolder)
    
    
    \SceneMap = _Dim(#SceneMap_Width * #SceneMap_Height * _Sz(Integer))
    LoadSceneViewerGraphics()
    
    ; Load tilesets:
    LoadTilesets("Graphics\Tilesets\", *TilesetFolder)
    \CurrentTile = #Null
    
    ; Load objects:
    LoadBanks("Objects\")
    \CurrentObject = #Null
    
    ; Load backgrounds:
    LoadBackgrounds("Graphics\Backgrounds\", *BackgroundFolder)
    
    ; Load targets and effects:
    LoadEventBanks("Events\")
    LoadEffectBanks("Effects\")
    
    ; Layer:
    \Layer              = 3
    \DisplaySingleLayer = #False
    \AutoLayerChange    = #True
    
    ;TODO: In "Sprite"-Strukturen umwandeln:
    
    ; Load cursors and other graphics:
    \CursorSprite            = LoadSprite(#PB_Any, "Graphics\Editor\Cursor.png", #PB_Sprite_Texture|#PB_Sprite_AlphaBlending)
    \CursorSprite3D          = CreateSprite3D(#PB_Any, \CursorSprite)
    
    \LayerSprite   = LoadSprite(#PB_Any, "Graphics\Editor\Layer.png", #PB_Sprite_Texture|#PB_Sprite_AlphaBlending)
    \LayerSprite3D = CreateSprite3D(#PB_Any, \LayerSprite)
    
    \ButtonSprite_1x1       = LoadScreenButtonSprite("Editor\Buttons\1x1")
    \ButtonSprite_1x1Green  = LoadScreenButtonSprite("Editor\Buttons\1x1Green")
    \ButtonSprite_1x1Yellow = LoadScreenButtonSprite("Editor\Buttons\1x1Yellow")
    \ButtonSprite_1x1Red    = LoadScreenButtonSprite("Editor\Buttons\1x1Red")
    \ButtonSprite_2x1       = LoadScreenButtonSprite("Editor\Buttons\2x1")
    \ButtonSprite_2x2       = LoadScreenButtonSprite("Editor\Buttons\2x2")
    
    \IconSprite_Tileset     = LoadScreenButtonIcon("Editor\Icons\Tileset")
    \IconSprite_Object      = LoadScreenButtonIcon("Editor\Icons\Object")
    \IconSprite_Scene       = LoadScreenButtonIcon("Editor\Icons\Scene")
    \IconSprite_Background  = LoadScreenButtonIcon("Editor\Icons\Background")
    \IconSprite_B1          = LoadScreenButtonIcon("Editor\Icons\B1")
    \IconSprite_B2          = LoadScreenButtonIcon("Editor\Icons\B2")
    \IconSprite_B3          = LoadScreenButtonIcon("Editor\Icons\B3")
    \IconSprite_W           = LoadScreenButtonIcon("Editor\Icons\W")
    \IconSprite_G           = LoadScreenButtonIcon("Editor\Icons\G")
    \IconSprite_F           = LoadScreenButtonIcon("Editor\Icons\F")
    \IconSprite_O1          = LoadScreenButtonIcon("Editor\Icons\O1")
    \IconSprite_O2          = LoadScreenButtonIcon("Editor\Icons\O2")
    \IconSprite_O           = LoadScreenButtonIcon("Editor\Icons\O")
    \IconSprite_O3          = LoadScreenButtonIcon("Editor\Icons\O3")
    \IconSprite_O4          = LoadScreenButtonIcon("Editor\Icons\O4")
    \IconSprite_SingleLayer = LoadScreenButtonIcon("Editor\Icons\SingleLayer")
    \IconSprite_Adapt       = LoadScreenButtonIcon("Editor\Icons\Adapt")
    \IconSprite_Adjust      = LoadScreenButtonIcon("Editor\Icons\Adjust")
    \IconSprite_Clear       = LoadScreenButtonIcon("Editor\Icons\Clear")
    \IconSprite_Grass       = LoadScreenButtonIcon("Editor\Icons\Grass")
    \IconSprite_StoryEditor = LoadScreenButtonIcon("Editor\Icons\StoryEditor")
    \IconSprite_Properties  = LoadScreenButtonIcon("Editor\Icons\Properties")
    \IconSprite_Challange   = LoadScreenButtonIcon("Editor\Icons\Challange")
    \IconSprite_Save        = LoadScreenButtonIcon("Editor\Icons\Save")
    \IconSprite_Load        = LoadScreenButtonIcon("Editor\Icons\Load")
    \IconSprite_Help        = LoadScreenButtonIcon("Editor\Icons\Help")
    
    \BarSprite   = LoadSprite(#PB_Any, "Graphics\Editor\Bar.bmp", #PB_Sprite_Texture)
    \BarSprite3D = CreateSprite3D(#PB_Any, \BarSprite)
    TransparentSpriteColor(\BarSprite, $FF00FF)
    
    \Menu = CreateScreenMenu()
      \Menu_Tileset    = CreateScreenButton(0, #SceneHeight, \ButtonSprite_2x2, \IconSprite_Tileset)
      \Menu_Object     = CreateScreenButton(2, #SceneHeight, \ButtonSprite_2x2, \IconSprite_Object)
      \Menu_Scene      = CreateScreenButton(4, #SceneHeight, \ButtonSprite_2x2, \IconSprite_Scene)
      \Menu_Background = CreateScreenButton(6, #SceneHeight, \ButtonSprite_2x2, \IconSprite_Background)
      
      \Menu_B1 = CreateScreenButton(9 , #SceneHeight, \ButtonSprite_1x1Green, \IconSprite_B1)
      \Menu_B2 = CreateScreenButton(10, #SceneHeight, \ButtonSprite_1x1Green, \IconSprite_B2)
      \Menu_B3 = CreateScreenButton(11, #SceneHeight, \ButtonSprite_1x1Green, \IconSprite_B3)
      \Menu_W  = CreateScreenButton(12, #SceneHeight, \ButtonSprite_1x1Yellow, \IconSprite_W)
      \Menu_G  = CreateScreenButton(13, #SceneHeight, \ButtonSprite_1x1Yellow, \IconSprite_G)
      \Menu_F  = CreateScreenButton(14, #SceneHeight, \ButtonSprite_1x1Red, \IconSprite_F)
      \Menu_O1 = CreateScreenButton(9 , #SceneHeight + 1, \ButtonSprite_1x1Green, \IconSprite_O1)
      \Menu_O2 = CreateScreenButton(10, #SceneHeight + 1, \ButtonSprite_1x1Green, \IconSprite_O2)
      \Menu_O  = CreateScreenButton(11, #SceneHeight + 1, \ButtonSprite_1x1Yellow, \IconSprite_O)
      \Menu_O3 = CreateScreenButton(12, #SceneHeight + 1, \ButtonSprite_1x1Red, \IconSprite_O3)
      \Menu_O4 = CreateScreenButton(13, #SceneHeight + 1, \ButtonSprite_1x1Red, \IconSprite_O4)
      \Menu_SingleLayer = CreateScreenButton(14, #SceneHeight + 1, \ButtonSprite_1x1, \IconSprite_SingleLayer)
      
      \Menu_Adapt  = CreateScreenButton(16, #SceneHeight, \ButtonSprite_1x1, \IconSprite_Adapt)
      \Menu_Adjust = CreateScreenButton(17, #SceneHeight, \ButtonSprite_1x1, \IconSprite_Adjust)
      \Menu_Clear  = CreateScreenButton(16, #SceneHeight + 1, \ButtonSprite_1x1, \IconSprite_Clear)
      \Menu_Grass  = CreateScreenButton(17, #SceneHeight + 1, \ButtonSprite_1x1, \IconSprite_Grass)
      
      \Menu_StoryEditor = CreateScreenButton(21, #SceneHeight, \ButtonSprite_1x1, \IconSprite_StoryEditor)
      \Menu_Properties  = CreateScreenButton(22, #SceneHeight, \ButtonSprite_1x1, \IconSprite_Properties)
      \Menu_Challange   = CreateScreenButton(22, #SceneHeight + 1, \ButtonSprite_1x1, \IconSprite_Challange)
      
      \Menu_Save = CreateScreenButton(23, #SceneHeight, \ButtonSprite_2x1, \IconSprite_Save)
      \Menu_Load = CreateScreenButton(23, #SceneHeight + 1, \ButtonSprite_2x1, \IconSprite_Load)
      \Menu_Help = CreateScreenButton(25, #SceneHeight, \ButtonSprite_2x2, \IconSprite_Help)
    
    \StartSceneSprite = LoadSprite(#PB_Any, "Graphics\Editor\Start.bmp")
    TransparentSpriteColor(\StartSceneSprite, $FF00FF)
    
    ; Allocate lists and arrays:
    \UsedScenes  = CreateQueue(@FreeScene())
    \Selection   = CreateSelection()
    
    ; Configure start scene:
    \SceneX       = #SceneMap_Width  / 2
    \SceneY       = #SceneMap_Height / 2
    \CurrentScene = CreateScene(\SceneX, \SceneY)
    CreateTable()
    
    ; Set start position:
    \StartScene = \CurrentScene
    \StartX     = #SceneWidth  / 2
    \StartY     = #SceneHeight / 2
    
    \LeftClickAllowed = #True
    
    \Challange\Folder = "Temp\"
    \Challange\Name   = "Temp"
    \Challange\Author = "Unknow"
  ;}
  
  Repeat
    ExamineEvents()
    
    RenderEditor()
    
    ;{ Events
      Select \Event
        Case #WM_LBUTTONUP
          ; Das setzen von Tiles und Objekten erlauben:
          \LeftClickAllowed = #True
        Case #WM_LBUTTONDBLCLK
          ; Objekt konfigurieren (falls mglich):
          If \Layer => #BackgroundObject1
            *Object = GetObject(\CurrentScene, \Layer, \TileX, \TileY)
            If *Object
              _Call(*Object\Registered\Entry\Callback, #Null, #Object_EditorConfigure, *Object)
            EndIf
          EndIf
        Case #WM_KEYDOWN
          Select \wParam
            Case #VK_A, #VK_NUMPAD4
              If \CurrentTile
                SelectTile(\CurrentTile\Tileset, RegTileX(\CurrentTile) - 1, RegTileY(\CurrentTile))
              EndIf
            Case #VK_S, #VK_NUMPAD5
              If \CurrentTile
                SelectTile(\CurrentTile\Tileset, RegTileX(\CurrentTile), RegTileY(\CurrentTile) + 1)
              EndIf
            Case #VK_D, #VK_NUMPAD6
              If \CurrentTile
                SelectTile(\CurrentTile\Tileset, RegTileX(\CurrentTile) + 1, RegTileY(\CurrentTile))
              EndIf
            Case #VK_W, #VK_NUMPAD8
              If \CurrentTile
                SelectTile(\CurrentTile\Tileset, RegTileX(\CurrentTile), RegTileY(\CurrentTile) - 1)
              EndIf
            Case #VK_LEFT
              If \SceneX > 0
                ChangeScene(\OverSceneX - 1, \OverSceneY, #True)
              EndIf
            Case #VK_RIGHT
              If \SceneX < #SceneMap_Width - 1
                ChangeScene(\OverSceneX + 1, \OverSceneY, #True)
              EndIf
            Case #VK_UP
              If \SceneY > 0
                ChangeScene(\OverSceneX, \OverSceneY - 1, #True)
              EndIf
            Case #VK_DOWN
              If \SceneY < #SceneMap_Height - 1
                ChangeScene(\OverSceneX, \OverSceneY + 1, #True)
              EndIf
            Case #VK_Q
              \StartScene = \CurrentScene
              \StartX     = \TileX
              \StartY     = \TileY
            Case #VK_C
              ClearQueue(\Selection\Queue)
              SelectionMode(\Selection, @Selection_Callback_Copy(), "Select tiles to copy")
              If Not IsSelectionInSingleScene(\Selection)
                MessageRequester("Warning", "Selection is not in a single scene!")
                ClearQueue(\Selection\Queue)
              EndIf
            Case #VK_L
              \AutoLayerChange = #True - \AutoLayerChange
          EndSelect
        Case #WM_CLOSE
          ; Editor schlieen:
          Break
      EndSelect
      
      ExamineScreenGadgets(\Event)
      
      If \OverY < #SceneHeight
        ExamineSceneEvents()
      ElseIf \OverY < #SceneHeight + 2
        ExamineInterfaceEvents()
      Else
        ExamineTilesetEvents()
      EndIf
    ;}
    
    If WindowMouseX(#ScreenWindow) < 5 And WindowMouseY(#ScreenWindow) => #EditorScreenHeight/2 - #ToolbarHeight/2 And WindowMouseY(#ScreenWindow) < #EditorScreenHeight/2 + #ToolbarHeight/2
      OpenTilesetToolbar()
    EndIf
    
    OptimizeFPS()
  Until KeyPressedDown(#VK_ESCAPE)
  
  CloseGMetricaScreen()
  
  EndWith
EndProcedure

; IDE Options = PureBasic 4.51 (Windows - x86)
; CursorPosition = 227
; FirstLine = 203
; Folding = --
; EnableXP