;**************************************************************
;     Object Viewer
;**************************************************************
;   Copyright (c) 2010 Josef Sniatecki
;
; This program is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License
; As published by the Free Software Foundation; either version
; 3 of the License, Or (at your option) any later version.
;
; This program is distributed in the hope that it will be
; useful, but WITHOUT ANY WARRANTY; without even the implied
; warranty of MERCHANTABILITY Or FITNESS For A PARTICULAR
; PURPOSE. See the GNU General Public License For more details.
;
; You should have received a copy of the GNU General Public
; License along With this program;
; If Not, see "http://www.gnu.org/licenses/".
;**************************************************************



#ObjectViewer_Image_Width  = 216
#ObjectViewer_Image_Height = 192



Global *ObjectViewer_Object.ObjectEntry
Global ObjectViewer_Image.i, ObjectViewer_ObjectImage.i
Global ObjectViewer_BackgroundColor.i = $FFFFFF



Procedure.i UpdateObjectViewerObjects(*Parent.Entry, Level.i = 0)
  Protected *Item.QueueItem, *Entry.Entry, *Object.ObjectEntry, Index.i
  
  *Item = *Parent\Entries\First
  While *Item
    *Entry = *Item\Memory
    
    AddGadgetItem(#OV_Objects, -1, *Entry\Name, 0, Level)
    Index = CountGadgetItems(#OV_Objects) - 1
    SetGadgetItemData(#OV_Objects, Index, *Entry)
    
    If *Entry\Type = #Entry_Folder
      UpdateObjectViewerObjects(*Entry, Level + 1)
    Else
      *Object = *Entry\Memory
      If *Object\Registered
        SetGadgetItemColor(#OV_Objects, Index, #PB_Gadget_BackColor, $C0C0C0)
      EndIf
    EndIf
    
    _Move(*Item, N)
  Wend
EndProcedure
Procedure.i UpdateObjectViewerImage()
  StartDrawing(ImageOutput(ObjectViewer_Image))
    Box(0, 0, #ObjectViewer_Image_Width, #ObjectViewer_Image_Height, ObjectViewer_BackgroundColor)
    If *ObjectViewer_Object
      DrawingMode(#PB_2DDrawing_AlphaBlend)
      DrawImage(ImageID(ObjectViewer_ObjectImage), #ObjectViewer_Image_Width/2 - ImageWidth(ObjectViewer_ObjectImage)/2, #ObjectViewer_Image_Height/2 - ImageHeight(ObjectViewer_ObjectImage)/2)
    EndIf
  StopDrawing()
  
  SetGadgetState(#OV_Image, ImageID(ObjectViewer_Image))
EndProcedure
Procedure.i SelectObjectViewerObject(*Object.ObjectEntry)
  Protected Image.i, ViewImage.i, *ViewRegion.ViewRegion
  
  *ObjectViewer_Object = *Object
  
  If *Object
    Image       = LoadImage(#PB_Any, ObjectViewSpriteFilename(*Object))
    *ViewRegion = _Call(*Object\Callback, #Null, #Object_ViewRegion, #Null)
    If *ViewRegion
      ObjectViewer_ObjectImage = GrabImage(Image, #PB_Any, *ViewRegion\X, *ViewRegion\Y, *ViewRegion\Width, *ViewRegion\Height)
    Else
      ObjectViewer_ObjectImage = CopyImage(Image, #PB_Any)
    EndIf
    FreeImage(Image)
    
    If ObjectDescription(*Object)
      DisableGadget(#OV_Info, #False)
    Else
      DisableGadget(#OV_Info, #True)
    EndIf
    DisableGadget(#OV_OK, #False)
  Else
    DisableGadget(#OV_Info, #True)
    DisableGadget(#OV_OK, #True)
  EndIf
  
  UpdateObjectViewerImage()
EndProcedure
Procedure.i OpenObjectViewerPath()
  Protected Index.i, Count.i, Level.i, OtherLevel.i, *Entry.Entry
  
  If *ObjectViewer_Object
    *Entry = *ObjectViewer_Object\Entry
    
    Count = CountGadgetItems(#OV_Objects) - 1
    For Index = 0 To Count
      If GetGadgetItemData(#OV_Objects, Index) = *Entry
        Level = GetGadgetItemAttribute(#OV_Objects, Index, #PB_Tree_SubLevel)
        SetGadgetItemState(#OV_Objects, Index, #PB_Tree_Selected)
        While Level
          Index - 1
          OtherLevel = GetGadgetItemAttribute(#OV_Objects, Index, #PB_Tree_SubLevel)
          If OtherLevel < Level
            SetGadgetItemState(#OV_Objects, Index, #PB_Tree_Expanded)
            Level = OtherLevel
          EndIf
        Wend
        ProcedureReturn
      EndIf
    Next
  EndIf
EndProcedure
Procedure.i OpenObjectViewer()
  Protected Event.i, Window.i, Gadget.i, Type.i
  Protected *Entry.Entry, Index.i, Info.s, Color.i
  
  If Not ObjectViewer_Image
    ObjectViewer_Image = CreateImage(#PB_Any, #ObjectViewer_Image_Width, #ObjectViewer_Image_Height, 24)
  EndIf
  
  OpenWindow(#OVWindow, #PB_Ignore, #PB_Ignore, 500, 270, "Objects", #PB_Window_SystemMenu|#PB_Window_ScreenCentered|#PB_Window_TitleBar)
    Frame3DGadget(#OV_Frame, 10, 2, 240, 220, "Object")
    TreeGadget(#OV_Objects, 260, 10, 230, 210)
    ButtonGadget(#OV_OK, 260, 230, 70, 30, "OK")
    ButtonGadget(#OV_Info, 180, 230, 70, 30, "Info")
    ImageGadget(#OV_Image, 20, 20, #ObjectViewer_Image_Width, #ObjectViewer_Image_Height, #Null)
    ButtonGadget(#OV_Cancel, 420, 230, 70, 30, "Cancel")
  
  UpdateObjectViewerObjects(*ObjectFolder)
  If Editor\CurrentObject
    SelectObjectViewerObject(Editor\CurrentObject\Entry)
    OpenObjectViewerPath()
  Else
    SelectObjectViewerObject(#Null)
  EndIf
  
  Repeat
    Event  = WaitWindowEvent()
    Window = EventWindow()
    Gadget = EventGadget()
    Type   = EventType()
    
    If Window = #OVWindow
      Select Event
        Case #PB_Event_Gadget
          Select Gadget
            Case #OV_Cancel
              CloseObjectViewer()
              ProcedureReturn #False
            Case #OV_Objects
              If Type = #PB_EventType_LeftClick
                Index = GetGadgetState(#OV_Objects)
                If Index <> #None
                  *Entry = GetGadgetItemData(#OV_Objects, Index)
                  If *Entry\Type = #Entry_File
                    SelectObjectViewerObject(*Entry\Memory)
                  EndIf
                Else
                  SelectObjectViewerObject(#Null)
                EndIf
              ElseIf Type = #PB_EventType_LeftDoubleClick
                Editor\LeftClickAllowed = #False
                Break
              EndIf
            Case #OV_Image
              If Type = #PB_EventType_LeftClick
                Editor\LeftClickAllowed = #False
                Break
              ElseIf Type = #PB_EventType_RightClick
                Color = ColorRequester(ObjectViewer_BackgroundColor)
                If Color <> #None
                  ObjectViewer_BackgroundColor = Color
                  UpdateObjectViewerImage()
                EndIf
              EndIf
            Case #OV_Info
              If *ObjectViewer_Object
                MessageRequester(ObjectName(*ObjectViewer_Object), ObjectDescription(*ObjectViewer_Object))
              EndIf
            Case #OV_OK
              Break
          EndSelect
        Case #PB_Event_CloseWindow
          CloseObjectViewer()
          ProcedureReturn #False
      EndSelect
    EndIf
  ForEver
  
  If *ObjectViewer_Object
    SelectObject(*ObjectViewer_Object)
  EndIf
  CloseObjectViewer()
  
  ProcedureReturn #True
EndProcedure
Procedure.i CloseObjectViewer()
  FreeImage(ObjectViewer_Image)
  ObjectViewer_Image = #Null
  
  CloseWindow(#OVWindow)
EndProcedure
; IDE Options = PureBasic 4.51 (Windows - x86)
; CursorPosition = 21
; Folding = --
; EnableXP