;**************************************************************
;     Save & Load
;**************************************************************
;   Copyright (c) 2010 Josef Sniatecki
;
; This program is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License
; As published by the Free Software Foundation; either version
; 3 of the License, Or (at your option) any later version.
;
; This program is distributed in the hope that it will be
; useful, but WITHOUT ANY WARRANTY; without even the implied
; warranty of MERCHANTABILITY Or FITNESS For A PARTICULAR
; PURPOSE. See the GNU General Public License For more details.
;
; You should have received a copy of the GNU General Public
; License along With this program;
; If Not, see "http://www.gnu.org/licenses/".
;**************************************************************



IncludeFile "Editor\Save.pb"
Procedure.i SaveEditorChallange()
  Protected String.s, *Challange.Challange
  
  With Editor
  
  OpenSavingScreen()
  SaveTable()
  
  ;{ Check challange infos
    ; Zuerst mssen die Daten des Challange-Headers geprft werden,
    ; damit das Speichern berhaupt mglich ist:
    UseBlock
      If Not \Challange\Folder
        String = "You have not selected a folder for this challange."
      ElseIf Not \Challange\Name
        String = "Please give this challange a name."
      ElseIf Not \Challange\Author
        String = "You have not typed the author's name!"
      Else
        Break
      EndIf
      
      MessageRequester("Warning", String, #MB_ICONWARNING)
      ProcedureReturn #False
    EndBlock
  ;}
  
  ;{ Pass elements
    PassTilesets()
    PassObjects()
    PassBackgrounds()
    PassEvents()
    PassEffects()
    PassActs()
    PassScenes()
  ;}
  
  ;{ Challange header
    *Challange = GetChallange(\Challange\Folder + \Challange\Name)
    If Not *Challange
      *Challange = CreateChallange(\Challange\Folder, \Challange\Name)
      If Not *Challange
        ProcedureReturn #False
      EndIf
    EndIf
    
    IO_OutputFile = CreateFile(#PB_Any, "Challanges\" + \Challange\Folder + \Challange\Name + "\Header.bin")
    If Not IO_OutputFile
      MessageRequester("Error", "Can not create " + #Q + "Challanges\" + \Challange\Folder + \Challange\Name + "\Header.bin" + #Q + ".", #MB_ICONERROR)
      ProcedureReturn #False
    EndIf
    
    WriteNTString(IO_OutputFile, \Challange\Name)
    WriteNTString(IO_OutputFile, \Challange\Author)
    WriteNTString(IO_OutputFile, \Challange\Comment)
    WriteInteger(IO_OutputFile, #False) ;TODO "Locked"
    
    If \Challange\Password
      WriteNTString(IO_OutputFile, DESFingerprint(\Challange\Password, "GMP"))
    Else
      WriteCharacter(IO_OutputFile, #Null)
    EndIf
    
    CloseFile(IO_OutputFile)
  ;}
  
  ;{ Challange settings
    IO_OutputFile = CreateFile(#PB_Any, "Challanges\" + \Challange\Folder + \Challange\Name + "\Settings.bin")
    If Not IO_OutputFile
      MessageRequester("Error", "Can not create " + #Q + "Challanges\" + \Challange\Folder + \Challange\Name + "\Settings.bin" + #Q + ".", #MB_ICONERROR)
      ProcedureReturn #False
    EndIf
    
    WriteInteger(IO_OutputFile, \Challange\Lives)
    WriteInteger(IO_OutputFile, \Challange\MaxEnergy)
    WriteInteger(IO_OutputFile, \Challange\Energy)
    
    WriteInteger(IO_OutputFile, \Challange\CanDoubleJump)
    WriteInteger(IO_OutputFile, \Challange\CanClimb)
    WriteInteger(IO_OutputFile, \Challange\CanLumiFlight)
    WriteInteger(IO_OutputFile, \Challange\CanFly)
    WriteInteger(IO_OutputFile, \Challange\CanShoot)
    WriteInteger(IO_OutputFile, \Challange\CanLoadShot)
    
    CloseFile(IO_OutputFile)
  ;}
  
  ;{ Create map
    IO_OutputFile = CreateFile(#PB_Any, "Challanges\" + \Challange\Folder + \Challange\Name + "\UnpackedMap.bin")
    If Not IO_OutputFile
      MessageRequester("Error", "Can not create map: " + #Q + "Challanges\" + \Challange\Folder + \Challange\Name + "\UnpakedMap.bin" + #Q, #MB_ICONERROR)
      ProcedureReturn #False
    EndIf
    
    ExamineChecksums(*Challange)
    WriteChecksums(IO_OutputFile, *Challange)
  ;}
  
  ;{ Save
    WriteTilesets()
    WriteBanks()
    WriteBackgrounds()
    WriteEvents()
    WriteEffects()
    WriteScenes()
    WriteActs()
  ;}
  
  CloseFile(IO_OutputFile)
  
  ;{ Pack
    If Not PackMap(*Challange)
      ProcedureReturn #False
    EndIf
  ;}
  
  CloseSavingScreen()
  
  ProcedureReturn #True
  
  EndWith
EndProcedure

IncludeFile "Editor\Load.pb"
Procedure.i LoadEditorChallange(Filename.s)
  With Editor
  
  Protected *Challange.Challange, String.s
  
  OpenLoadingScreen()
  
  ClearAll()
  
  ;{ Challange header
    *Challange = GetChallange(SStr(Filename, 1))
    If Not *Challange
      MessageRequester("Error", "Challange not found: " + Filename, #MB_ICONERROR)
      ProcedureReturn #False
    EndIf
    
    IO_InputFile = ReadFile(#PB_Any, "Challanges\" + Filename + "Header.bin")
    If Not IO_InputFile
      MessageRequester("Error", "Can not load " + #Q + "Challanges\" + Filename + "Header.bin" + #Q + ".", #MB_ICONERROR)
      ProcedureReturn #False
    EndIf
    
    \Challange\Name     = ReadNTString(IO_InputFile)
    \Challange\Folder   = SStr(Filename, Len(\Challange\Name) + 1) ;"+1" wegen Schrgstrich in "Filename"
    
    \Challange\Author   = ReadNTString(IO_InputFile)
    \Challange\Comment  = ReadNTString(IO_InputFile)
    \Challange\Password = ReadNTString(IO_InputFile)
    
    If \Challange\Password
      Repeat
        String = InputRequester("Password", "Enter your password please:", "")
        If DESFingerprint(String, "GMP") = \Challange\Password
          \Challange\Password = String
          Break
        Else
          Select MessageRequester("Error", "Wrong password!" + #N + "Retry?", #PB_MessageRequester_YesNo)
            Case #PB_MessageRequester_No
              CloseFile(IO_InputFile)
              ProcedureReturn #False
          EndSelect
        EndIf
      ForEver
    EndIf
    
    CloseFile(IO_InputFile)
  ;}
  
  ;{ Challange settings
    IO_InputFile = ReadFile(#PB_Any, "Challanges\" + Filename + "Settings.bin")
    If Not IO_InputFile
      MessageRequester("Error", "Can not load settings:  " + #Q + "Challanges\" + Filename + "Settings.bin" + #Q + ".", #MB_ICONERROR)
      ProcedureReturn #False
    EndIf
    
    \Challange\Lives     = ReadInteger(IO_InputFile)
    \Challange\MaxEnergy = ReadInteger(IO_InputFile)
    \Challange\Energy    = ReadInteger(IO_InputFile)
    
    \Challange\CanDoubleJump = ReadInteger(IO_InputFile)
    \Challange\CanClimb      = ReadInteger(IO_InputFile)
    \Challange\CanLumiFlight = ReadInteger(IO_InputFile)
    \Challange\CanFly        = ReadInteger(IO_InputFile)
    \Challange\CanShoot      = ReadInteger(IO_InputFile)
    \Challange\CanLoadShot   = ReadInteger(IO_InputFile)
    
    CloseFile(IO_InputFile)
  ;}
  
  ;{ Unpack map
    If Not UnpackMap(*Challange)
      ProcedureReturn #False
    EndIf
  
    IO_InputFile = ReadFile(#PB_Any, "Challanges\" + Filename + "UnpackedMap.bin")
    If Not IO_InputFile
      MessageRequester("Error", "Can not load unpacked map: " + #Q + "Challanges\" + Filename + "UnpackedMap.bin" + #Q + ".")
      ProcedureReturn #False
    EndIf
    
    PassChecksums(IO_InputFile)
  ;}
  
  ;{ Read
    ReadTilesets()
    ReadBanks()
    ReadBackgrounds()
    ReadEvents()
    ReadEffects()
    ReadScenes()
    ReadActs()
    
    InitializeScenes()
    InitializeObjects()
    InitializeActs()
  ;}
  
  CloseFile(IO_InputFile)
  
  CreateTable()
  CloseLoadingScreen()
  
  ProcedureReturn #True
  
  EndWith
EndProcedure
; IDE Options = PureBasic 4.50 (Windows - x86)
; CursorPosition = 239
; FirstLine = 132
; Folding = F0-
; EnableXP