;**************************************************************
;     Tileset Viewer
;**************************************************************
;   Copyright (c) 2010 Josef Sniatecki
;
; This program is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License
; As published by the Free Software Foundation; either version
; 3 of the License, Or (at your option) any later version.
;
; This program is distributed in the hope that it will be
; useful, but WITHOUT ANY WARRANTY; without even the implied
; warranty of MERCHANTABILITY Or FITNESS For A PARTICULAR
; PURPOSE. See the GNU General Public License For more details.
;
; You should have received a copy of the GNU General Public
; License along With this program;
; If Not, see "http://www.gnu.org/licenses/".
;**************************************************************



#TilesetViewer_Image_Width  = 360
#TilesetViewer_Image_Height = 192

#TilesetViewer_ColorDifference = 20



Global TilesetViewer_BackgroundColor.i
Global TilesetViewer_BackgroundImage.i
Global TilesetViewer_TilesetImage.i
Global TilesetViewer_Image.i

Global *TilesetViewer_Tileset.TilesetEntry
Global TilesetViewer_OverX.i, TilesetViewer_OverY.i



Procedure.i UpdateTilesetViewerTilesets(*Parent.Entry, Level.i = 0)
  Protected *Item.QueueItem, *Entry.Entry, *Tileset.TilesetEntry
  
  *Item = *Parent\Entries\First
  While *Item
    *Entry = *Item\Memory
    
    AddGadgetItem(#TV_Tilesets, -1, *Entry\Name, 0, Level)
    SetGadgetItemData(#TV_Tilesets, CountGadgetItems(#TV_Tilesets) - 1, *Entry)
    
    If *Entry\Type = #Entry_Folder
      UpdateTilesetViewerTilesets(*Entry, Level + 1)
    Else
      *Tileset = *Entry\Memory
      If *Tileset\Registered
        SetGadgetItemColor(#TV_Tilesets, CountGadgetItems(#TV_Tilesets) - 1, #PB_Gadget_BackColor, $C0C0C0)
      EndIf
    EndIf
    
    _Move(*Item, N)
  Wend
EndProcedure
Procedure.i DrawTilesetViewerBackground()
  Protected Color1.i, Color2.i, X.i, Y.i
  
  Color1 = TilesetViewer_BackgroundColor
  If (Red(Color1) + Green(Color1) + Blue(Color1)) / 3.0 < (255 / 2.0)
    Color2 = RGBA(Max(Red(Color1) + #TilesetViewer_ColorDifference, 255), Max(Green(Color1) + #TilesetViewer_ColorDifference, 255), Max(Blue(Color1) + #TilesetViewer_ColorDifference, 255), 255)
  Else
    Color2 = RGBA(Min(Red(Color1) - #TilesetViewer_ColorDifference, 0), Min(Green(Color1) - #TilesetViewer_ColorDifference, 0), Min(Blue(Color1) - #TilesetViewer_ColorDifference, 0), 255)
  EndIf
  
  StartDrawing(ImageOutput(TilesetViewer_BackgroundImage))
    For Y = 0 To #TilesetViewer_Image_Height / #FieldSize - 1
      For X = 0 To #TilesetViewer_Image_Width / #FieldSize - 1
        If (X + Y) % 2
          Box(X * #FieldSize, Y * #FieldSize, #FieldSize, #FieldSize, Color2)
        Else
          Box(X * #FieldSize, Y * #FieldSize, #FieldSize, #FieldSize, Color1)
        EndIf
      Next
    Next
  StopDrawing()
EndProcedure
Procedure.i UpdateTilesetViewerImage()
  StartDrawing(ImageOutput(TilesetViewer_Image))
    DrawImage(ImageID(TilesetViewer_BackgroundImage), 0, 0)
    If *TilesetViewer_Tileset
      DrawingMode(#PB_2DDrawing_AlphaBlend)
      DrawImage(ImageID(TilesetViewer_TilesetImage), 0, 0)
      If TilesetViewer_OverX <> #None And TilesetViewer_OverY <> #None
        DrawingMode(#PB_2DDrawing_Outlined)
        Box(TilesetViewer_OverX * #FieldSize, TilesetViewer_OverY * #FieldSize, #FieldSize, #FieldSize, $000000)
        DrawingMode(#PB_2DDrawing_AlphaBlend)
        Box(TilesetViewer_OverX * #FieldSize + 1, TilesetViewer_OverY * #FieldSize + 1, #FieldSize - 2, #FieldSize - 2, RGBA(255, 255, 255, 50))
      EndIf
    EndIf
  StopDrawing()
  
  SetGadgetState(#TV_Image, ImageID(TilesetViewer_Image))
EndProcedure
Procedure.i SelectTilesetViewerTileset(*Tileset.TilesetEntry)
  *TilesetViewer_Tileset = *Tileset
  
  If TilesetViewer_TilesetImage
    FreeImage(TilesetViewer_TilesetImage)
  EndIf
  
  If *Tileset
    TilesetViewer_TilesetImage = LoadImage(#PB_Any, TilesetFilename(*Tileset))
    SetGadgetText(#TV_Frame, "Tileset - " + *Tileset\Entry\Name)
  Else
    TilesetViewer_TilesetImage = #Null
    SetGadgetText(#TV_Frame, "Tileset")
  EndIf
  
  UpdateTilesetViewerImage()
EndProcedure
Procedure.i CloseTilesetViewer()
  FreeImage(TilesetViewer_Image)
  FreeImage(TilesetViewer_BackgroundImage)
  
  CloseWindow(#TVWindow)
EndProcedure
Procedure.i OpenTilesetViewer()
  Protected Event.i, Window.i, Gadget.i, Type.i, OverX.i, OverY.i
  Protected *Entry.Entry, Index.i, Color.i
  
  TilesetViewer_Image           = CreateImage(#PB_Any, #TilesetViewer_Image_Width, #TilesetViewer_Image_Height, 24)
  TilesetViewer_BackgroundImage = CreateImage(#PB_Any, #TilesetViewer_Image_Width, #TilesetViewer_Image_Height, 32)
  TilesetViewer_BackgroundColor = $552055
  DrawTilesetViewerBackground()
  
  OpenWindow(#TVWindow, #PB_Ignore, #PB_Ignore, 640, 230, "Tilesets", #PB_Window_SystemMenu|#PB_Window_ScreenCentered|#PB_Window_TitleBar, WindowID(#ScreenWindow))
    ImageGadget(#TV_Image, 20, 20, 360, 192, #Null)
    Frame3DGadget(#TV_Frame, 10, 0, 380, 225, "Tileset - Description")
    TreeGadget(#TV_Tilesets, 400, 10, 230, 170, #PB_Tree_AlwaysShowSelection)
    ButtonGadget(#TV_OK, 400, 190, 70, 30, "OK")
    ButtonGadget(#TV_Cancel, 580, 190, 50, 30, "Cancel")
  
  If Editor\CurrentTile
    SelectTilesetViewerTileset(Editor\CurrentTile\Tileset\Entry)
  Else
    SelectTilesetViewerTileset(#Null)
  EndIf
  UpdateTilesetViewerTilesets(*TilesetFolder)
  
  Repeat
    Event  = WaitWindowEvent()
    Window = EventWindow()
    Gadget = EventGadget()
    Type   = EventType()
    
    If Window = #TVWindow
      Select Event
        Case #PB_Event_Gadget
          Select Gadget
            Case #TV_Tilesets
              If Type = #PB_EventType_LeftClick
                Index = GetGadgetState(#TV_Tilesets)
                If Index <> #None
                  *Entry = GetGadgetItemData(#TV_Tilesets, Index)
                  If *Entry\Type = #Entry_File
                    SelectTilesetViewerTileset(*Entry\Memory)
                  EndIf
                Else
                  SelectTilesetViewerTileset(#Null)
                EndIf
              ElseIf Type = #PB_EventType_LeftDoubleClick
                Break
              EndIf
            Case #TV_Image
              If Type = #PB_EventType_LeftClick
                If *TilesetViewer_Tileset
                  Editor\LeftClickAllowed = #False
                  Break
                Else
                  CloseTilesetViewer()
                  ProcedureReturn #False
                EndIf
              ElseIf Type = #PB_EventType_RightClick
                Color = ColorRequester(TilesetViewer_BackgroundColor)
                If Color <> #None
                  TilesetViewer_BackgroundColor = Color
                  DrawTilesetViewerBackground()
                  UpdateTilesetViewerImage()
                EndIf
              EndIf
            Case #TV_OK
              Break
            Case #TV_Cancel
              CloseWindow(#TVWindow)
              ProcedureReturn #False
          EndSelect
        Case #WM_MOUSEMOVE
          If *TilesetViewer_Tileset
            OverX = WindowMouseX(#TVWindow) - GadgetX(#TV_Image) : If OverX < 0 : OverX - #FieldSize : EndIf
            OverX / #FieldSize
            OverY = WindowMouseY(#TVWindow) - GadgetY(#TV_Image) : If OverY < 0 : OverY - #FieldSize : EndIf
            OverY / #FieldSize
            
            If OverX => 0 And OverX < ImageWidth(TilesetViewer_TilesetImage) / #FieldSize And OverY => 0 And OverY < ImageHeight(TilesetViewer_TilesetImage) / #FieldSize
              If OverX <> TilesetViewer_OverX Or OverY <> TilesetViewer_OverY
                TilesetViewer_OverX = OverX
                TilesetViewer_OverY = OverY
                UpdateTilesetViewerImage()
              EndIf
            Else
              TilesetViewer_OverX = #None
              TilesetViewer_OverY = #None
              UpdateTilesetViewerImage()
            EndIf
          EndIf
        Case #PB_Event_CloseWindow
          CloseTilesetViewer()
          ProcedureReturn #False
      EndSelect
    EndIf
  ForEver
  
  If *TilesetViewer_Tileset
    If TilesetViewer_OverX <> #None And TilesetViewer_OverY <> #None
      SelectTile(LoadRegTileset(*TilesetViewer_Tileset), TilesetViewer_OverX, TilesetViewer_OverY)
    Else
      SelectTile(LoadRegTileset(*TilesetViewer_Tileset), 0, 0)
    EndIf
  EndIf
  
  CloseTilesetViewer()
  ProcedureReturn #True
EndProcedure
; IDE Options = PureBasic 4.50 (Windows - x86)
; CursorPosition = 157
; FirstLine = 146
; Folding = --
; EnableXP