Procedure.i GrassGenerator(Layer.i, *Tileset.RegisteredTileset, TileY.i)
  Protected SceneWidth.i  = #SceneWidth * Editor\CurrentScene\Width
  Protected SceneHeight.i = #SceneHeight * Editor\CurrentScene\Height
  
  Protected X.i, Y.i
  
  For Y = 0 To SceneHeight - 2
    For X = 0 To SceneWidth - 1
      If Not GetTile(Editor\CurrentScene, #Tile, X, Y) And Not GetTile(Editor\CurrentScene, Layer, X, Y) And GetTile(Editor\CurrentScene, #Tile, X, Y + 1)
        PutTile(Layer, LoadRegTile(Random(*Tileset\Width - 1), TileY, *Tileset), X, Y)
      EndIf
    Next
  Next
EndProcedure



Macro Adjuster_Add(X, Y, State)
  Combination | ((State) << ((Y) * 3 + (2 - X)))
EndMacro
Macro Adjuster_Replace(X, Y)
  ReplaceX = X
  ReplaceY = Y
EndMacro
Macro Adjuster_Combi(A, B, C)
  %C#B#A
EndMacro

Procedure.i Adjuster(Layer.i, *CenterTile.RegisteredTile)
  ; Der Adjuster sorgt dafr, dass alle Tiles die dem "*CenterTile"
  ; gleichen, passend ersetzt werden. Somit knnen z.B. die schwarzen Rnder
  ; und Grasbden automatisch generiert werden. Um eine Ersetzung korrekt
  ; durchzufhren, wird ein Bit-Array verwendet.
  
  Protected SceneWidth.i  = #SceneWidth * Editor\CurrentScene\Width
  Protected SceneHeight.i = #SceneHeight * Editor\CurrentScene\Height
  
  Protected TileX.i, TileY.i, *Tileset.RegisteredTileset
  Protected OutOfRange.i, DoReplace.i, ReplaceX.i, ReplaceY.i, Combination.i
  Protected X.i, Y.i, *Tile.Tile, *ToReplace.RegisteredTile
  
  If Not *CenterTile
    ProcedureReturn #False
  EndIf
  
  *Tileset = *CenterTile\Tileset
  TileX    = RegTileX(*CenterTile)
  TileY    = RegTileY(*CenterTile)
  
  If TileX < 1 Or TileX > *Tileset\Width - 2 Or TileY < 1 Or TileY > *Tileset\Height - 2
    MessageRequester("Error", "The 3x3-box do not fit into the actual tileset.", #MB_ICONERROR)
    ProcedureReturn #False
  ElseIf TileX > *Tileset\Width - 3 Or TileY > *Tileset\Height - 3
    OutOfRange = #True
  EndIf
   
  For Y = 0 To SceneHeight - 1
    For X = 0 To SceneWidth - 1
      *Tile = GetField(Editor\CurrentScene, Layer, X, Y)
      If *Tile And *Tile\Registered = *CenterTile
        Combination = Adjuster_Combi(000,010,000)
        
        ; Horinzonatle und vertikale Nachbar-Tiles ermitteln:
        If X > 0
          Adjuster_Add(0, 1, Is(GetTile(Editor\CurrentScene, Layer, X - 1, Y)))
        Else
          Adjuster_Add(0, 1, #True)
        EndIf
        If X < SceneWidth - 1
          Adjuster_Add(2, 1, Is(GetTile(Editor\CurrentScene, Layer, X + 1, Y)))
        Else
          Adjuster_Add(2, 1, #True)
        EndIf
        If Y > 0
          Adjuster_Add(1, 0, Is(GetTile(Editor\CurrentScene, Layer, X, Y - 1)))
        Else
          Adjuster_Add(1, 0, #True)
        EndIf
        If Y < SceneHeight - 1
          Adjuster_Add(1, 2, Is(GetTile(Editor\CurrentScene, Layer, X, Y + 1)))
        Else
          Adjuster_Add(1, 2, #True)
        EndIf
        ; Tiles, die diagonal zum jeweiligen Tile liegen, werden
        ; nicht beachtet, da sie fr die Angleichung nicht relevant sind.
        
        ; Nun werden alle mglichen Kombinationen abgefragt:
        DoReplace = #True
        Select Combination
          Case Adjuster_Combi(010,111,010) : DoReplace = #False       ;berall verbunden...
          Case Adjuster_Combi(010,011,010) : Adjuster_Replace(-1,  0) ;Rechts, Oben und Unten
          Case Adjuster_Combi(010,110,010) : Adjuster_Replace( 1,  0) ;Links, Oben und Unten
          Case Adjuster_Combi(000,111,010) : Adjuster_Replace( 0, -1) ;Links, Rechts und Unten
          Case Adjuster_Combi(010,111,000) : Adjuster_Replace( 0,  1) ;Links, Rechts und Oben
          Case Adjuster_Combi(000,011,010) : Adjuster_Replace(-1, -1) ;Rechts und Unten
          Case Adjuster_Combi(000,110,010) : Adjuster_Replace( 1, -1) ;Links und Unten
          Case Adjuster_Combi(010,011,000) : Adjuster_Replace(-1,  1) ;Rechts und Oben
          Case Adjuster_Combi(010,110,000) : Adjuster_Replace( 1,  1) ;Links und Oben
          Case Adjuster_Combi(000,111,000) : Adjuster_Replace( 0,  2) ;Links und Rechts
          Case Adjuster_Combi(000,011,000) : Adjuster_Replace(-1,  2) ;Rechts
          Case Adjuster_Combi(000,110,000) : Adjuster_Replace( 1,  2) ;Links
          Case Adjuster_Combi(010,010,010) : Adjuster_Replace( 2,  0) ;Oben und Unten
          Case Adjuster_Combi(010,010,000) : Adjuster_Replace( 2,  1) ;Oben
          Case Adjuster_Combi(000,010,010) : Adjuster_Replace( 2, -1) ;Unten
          Case Adjuster_Combi(000,010,000) : Adjuster_Replace( 2,  2) ;Garnicht verbunden
        EndSelect
            
        ; Tile durch das passende Tile ersetzen:
        If DoReplace And Not ((ReplaceX = 2 Or ReplaceY = 2) And OutOfRange)
          *ToReplace = LoadRegTile(TileX + ReplaceX, TileY + ReplaceY, *Tileset)
          UseRegTile(*ToReplace)
          ReleaseRegTile(*CenterTile)
          *Tile\Registered = *ToReplace
          
          DoReplace = #Null
        EndIf
      EndIf
    Next
  Next
  
  ProcedureReturn #True
EndProcedure



#BitmapGenerator_Zoom = 4

Procedure.i BitmapGenerator()
  ; Dieses Tool erstellt ein Bitmap mit einer Miniautur-Ansicht aller
  ; Szenen. Dadurch kann man z.B. Lsungen weitergeben und kennzeichnen.
  
  Protected Bitmap.i, Image.i, Sprite.i, MinX.i, MinY.i, MaxX.i, MaxY.i
  Protected MapWidth.i, MapHeight.i
  Protected X.i, Y.i, *Scene.Scene, Index.i
  
  Protected Window.i, HilightStartGadget.i, HilightSuperCoinsGadget.i, HilightStarsGadget.i, ShowIndexGadget.i, GenerateGadget.i, CancelGadget.i
  Protected InfoGadget.i
  Protected Event.i, EventWindow.i, EventGadget.i
  
  Protected *SuperCoin1 = GetObjectCallback(GetBank("Coin"), "Coin6")
  Protected *SuperCoin2 = GetObjectCallback(GetBank("Coin"), "ICoin6")
  Protected *SuperCoin3 = GetObjectCallback(GetBank("Coin"), "HitBlock6")
  Protected *Star1      = GetObjectCallback(GetBank("System"), "Genstar")
  Protected *Star2      = GetObjectCallback(GetBank("System"), "Killstar")
  
  Protected NewList *Object.Object()
  Protected StartSprite.i, SuperCoinSprite.i, StarSprite.i, ShowIndex.i, Filename.s
  
  ;{ Einstellungen vornehmen
    Window = OpenWindow(#PB_Any, #PB_Ignore, #PB_Ignore, 200, 140, "Bitmap Generator", #PB_Window_ScreenCentered|#PB_Window_SystemMenu)
      HilightStartGadget      = CheckBoxGadget(#PB_Any, 5, 5 , 150, 20, "Hilight start position")
      HilightSuperCoinsGadget = CheckBoxGadget(#PB_Any, 5, 30, 150, 20, "Hilight super coins")
      HilightStarsGadget      = CheckBoxGadget(#PB_Any, 5, 55, 150, 20, "Hilight gen-/killstars")
      ShowIndexGadget         = CheckBoxGadget(#PB_Any, 5, 80, 150, 20, "Show scene numbers")
      GenerateGadget          = ButtonGadget(#PB_Any, 5, 105, 70, 30, "Generate")
      CancelGadget            = ButtonGadget(#PB_Any, 75, 105, 70, 30, "Cancel")
    
    Repeat
      Event       = WaitWindowEvent()
      EventWindow = EventWindow()
      EventGadget = EventGadget()
      
      If EventWindow <> Window
        Continue
      EndIf
      
      Select Event
        Case #PB_Event_Gadget
          Select EventGadget
            Case GenerateGadget
              Filename = SaveFileRequester("Generate Bitmap", "", "*.png", 0)
              If Filename
                If GetExtensionPart(Filename) <> "png"
                  Filename + ".png"
                EndIf
                Break
              EndIf
            Case CancelGadget
              CloseWindow(Window)
              ProcedureReturn #False
          EndSelect
        Case #PB_Event_CloseWindow
          CloseWindow(Window)
          ProcedureReturn #False
      EndSelect
    ForEver
    
    ; Checkboxen prfen:
    If GetGadgetState(HilightStartGadget) = #PB_CheckBox_Checked
      StartSprite = LoadSprite(#PB_Any, "Graphics\Editor\Bitmap Generator\Start.bmp")
    EndIf
    If GetGadgetState(HilightSuperCoinsGadget) = #PB_CheckBox_Checked
      SuperCoinSprite = LoadSprite(#PB_Any, "Graphics\Editor\Bitmap Generator\SuperCoin.bmp")
    EndIf
    If GetGadgetState(HilightStarsGadget) = #PB_CheckBox_Checked
      StarSprite = LoadSprite(#PB_Any, "Graphics\Editor\Bitmap Generator\Star.bmp")
    EndIf
    If GetGadgetState(ShowIndexGadget) = #PB_CheckBox_Checked
      ShowIndex = #True
    EndIf
    
    CloseWindow(Window)
  ;}
  
  ;{ Generieren
    Window = OpenWindow(#PB_Any, #PB_Ignore, #PB_Ignore, 200, 40, "Generating", #PB_Window_ScreenCentered)
      InfoGadget = TextGadget(#PB_Any, 5, 5, 190, 20, "Create map...")
    
    ; Grenzen der Map ermitteln:
    GetMapBorders(@MinX, @MinY, @MaxX, @MaxY)
    MapWidth  = MaxX - MinX + 1
    MapHeight = MaxY - MinY + 1
    
    ; Bitmap mit den errechneten Gren erstellen:
    Bitmap = CreateImage(#PB_Any, #FieldSize * #SceneWidth * MapWidth / #BitmapGenerator_Zoom, #FieldSize * #SceneHeight * MapHeight / #BitmapGenerator_Zoom, 32)
    
      For Y = MinY To MaxY
        For X = MinX To MaxX
          WindowEvent()
          
          *Scene = GetScene(X, Y)
          SetGadgetText(InfoGadget, "Render scene (" + Str(X) + "|" + Str(Y) + ")...")
          
          If *Scene
            Index + 1
            
            ChangeScene(X, Y, #False, #False)
            RenderScene()
            
            If StartSprite And Editor\StartScene = *Scene
              ; Startposition hervorheben:
              DisplaySprite(StartSprite, Editor\StartX * #FieldSize, Editor\StartY * #FieldSize)
            EndIf
            If SuperCoinSprite
              ; Alle Super-Mnzen hervorheben:
              ClearList(*Object())
              GetObjectsOf(*SuperCoin1, *Scene, #ForegroundObject1, *Object())
              GetObjectsOf(*SuperCoin2, *Scene, #ForegroundObject1, *Object())
              GetObjectsOf(*SuperCoin3, *Scene, #ForegroundObject1, *Object())
              ForEach *Object()
                DisplaySprite(SuperCoinSprite, *Object()\X * #FieldSize, *Object()\Y * #FieldSize)
              Next
            EndIf
            If StarSprite
              ; Alle Gen- und Killstars hervorheben:
              ClearList(*Object())
              GetObjectsOf(*Star1, *Scene, #ForegroundObject1, *Object())
              GetObjectsOf(*Star2, *Scene, #ForegroundObject1, *Object())
              ForEach *Object()
                DisplaySprite(StarSprite, *Object()\X * #FieldSize, *Object()\Y * #FieldSize)
              Next
            EndIf
            
            ; Bildschirminhalt kopieren und speichern:
            Sprite = GrabSprite(#PB_Any, 0, 0, #FieldSize * #SceneWidth, #FieldSize * #SceneHeight)
            SaveSprite(Sprite, "Graphics\Temp.bmp", #PB_ImagePlugin_BMP)
            FreeSprite(Sprite)
            
            ; Gespeicherten Inhalt als Image laden, damit gezeichnet werden kann:
            Image = LoadImage(#PB_Any, "Graphics\Temp.bmp")
            StartDrawing(ImageOutput(Bitmap))
              DrawingMode(#PB_2DDrawing_AlphaBlend)
              DrawImage(ImageID(Image), (X - MinX) * #FieldSize * #SceneWidth / #BitmapGenerator_Zoom, (Y - MinY) * #FieldSize * #SceneHeight / #BitmapGenerator_Zoom, #FieldSize * #SceneWidth / #BitmapGenerator_Zoom, #FieldSize * #SceneHeight / #BitmapGenerator_Zoom)
              DrawingMode(#PB_2DDrawing_Transparent)
              If ShowIndex
                DrawText((X - MinX) * #FieldSize * #SceneWidth / #BitmapGenerator_Zoom + 2, (Y - MinY) * #FieldSize * #SceneHeight / #BitmapGenerator_Zoom + 2, Str(Index), $0000FF)
              EndIf
            StopDrawing()
            FreeImage(Image)
          Else
            ; Transparente Flche zeichnen:
            StartDrawing(ImageOutput(Bitmap))
              DrawingMode(#PB_2DDrawing_AlphaChannel)
              Box((X - MinX) * #FieldSize * #SceneWidth / #BitmapGenerator_Zoom, (Y - MinY) * #FieldSize * #SceneHeight / #BitmapGenerator_Zoom, #FieldSize * #SceneWidth / #BitmapGenerator_Zoom, #FieldSize * #SceneHeight / #BitmapGenerator_Zoom, RGBA(0, 0, 0, 0))
            StopDrawing()
          EndIf
        Next
      Next
    StopDrawing()
  ;}
  
  ; Gesamtes Bitmap speichern und anschlieend freigeben:
  SetGadgetText(InfoGadget, "Finished")
  SaveImage(Bitmap, Filename, #PB_ImagePlugin_PNG)
  FreeImage(Bitmap)
  
  ; Alle Sprites zum Hervorheben wieder lschen:
  If StartSprite
    FreeSprite(StartSprite)
  EndIf
  If SuperCoinSprite
    FreeSprite(SuperCoinSprite)
  EndIf
  If StarSprite
    FreeSprite(StarSprite)
  EndIf
  
  CloseWindow(Window)
  
  ProcedureReturn #True
EndProcedure
; IDE Options = PureBasic 4.51 (Windows - x86)
; CursorPosition = 147
; FirstLine = 122
; Folding = --
; EnableXP