Procedure.i GrassGenerator(Layer.i, *Tileset.RegisteredTileset, TileY.i)
  Protected SceneWidth.i  = #SceneWidth * Editor\CurrentScene\Width
  Protected SceneHeight.i = #SceneHeight * Editor\CurrentScene\Height
  
  Protected X.i, Y.i
  
  For Y = 0 To SceneHeight - 2
    For X = 0 To SceneWidth - 1
      If Not GetTile(Editor\CurrentScene, #Tile, X, Y) And Not GetTile(Editor\CurrentScene, Layer, X, Y) And GetTile(Editor\CurrentScene, #Tile, X, Y + 1)
        PutTile(Layer, LoadRegTile(Random(*Tileset\Width - 1), TileY, *Tileset), X, Y)
      EndIf
    Next
  Next
EndProcedure



Macro Adjuster_Add(X, Y, State)
  Combination | ((State) << ((Y) * 3 + (2 - X)))
EndMacro
Macro Adjuster_Replace(X, Y)
  ReplaceX = X
  ReplaceY = Y
EndMacro
Macro Adjuster_Combi(A, B, C)
  %C#B#A
EndMacro

Procedure.i Adjuster(Layer.i, *CenterTile.RegisteredTile)
  ; Der Adjuster sorgt dafr, dass alle Tiles die dem "*CenterTile"
  ; gleichen, passend ersetzt werden. Somit knnen z.B. die schwarzen Rnder
  ; und Grasbden automatisch generiert werden. Um eine Ersetzung korrekt
  ; durchzufhren, wird ein Bit-Array verwendet.
  
  Protected SceneWidth.i  = #SceneWidth * Editor\CurrentScene\Width
  Protected SceneHeight.i = #SceneHeight * Editor\CurrentScene\Height
  
  Protected TileX.i, TileY.i, *Tileset.RegisteredTileset
  Protected OutOfRange.i, DoReplace.i, ReplaceX.i, ReplaceY.i, Combination.i
  Protected X.i, Y.i, *Tile.Tile, *ToReplace.RegisteredTile
  
  If Not *CenterTile
    ProcedureReturn #False
  EndIf
  
  *Tileset = *CenterTile\Tileset
  TileX    = RegTileX(*CenterTile)
  TileY    = RegTileY(*CenterTile)
  
  If TileX < 1 Or TileX > *Tileset\Width - 2 Or TileY < 1 Or TileY > *Tileset\Height - 2
    MessageRequester("Error", "The 3x3 box do not fit into the actual tileset.", #MB_ICONERROR)
    ProcedureReturn #False
  ElseIf TileX > *Tileset\Width - 3 Or TileY > *Tileset\Height - 3
    OutOfRange = #True
  EndIf
   
  For Y = 0 To SceneHeight - 1
    For X = 0 To SceneWidth - 1
      *Tile = GetField(Editor\CurrentScene, Layer, X, Y)
      If *Tile And *Tile\Registered = *CenterTile
        Combination = Adjuster_Combi(000,010,000)
        
        ; Horinzonatle und vertikale Nachbar-Tiles ermitteln:
        If X > 0
          Adjuster_Add(0, 1, Is(GetTile(Editor\CurrentScene, Layer, X - 1, Y)))
        Else
          Adjuster_Add(0, 1, #True)
        EndIf
        If X < SceneWidth - 1
          Adjuster_Add(2, 1, Is(GetTile(Editor\CurrentScene, Layer, X + 1, Y)))
        Else
          Adjuster_Add(2, 1, #True)
        EndIf
        If Y > 0
          Adjuster_Add(1, 0, Is(GetTile(Editor\CurrentScene, Layer, X, Y - 1)))
        Else
          Adjuster_Add(1, 0, #True)
        EndIf
        If Y < SceneHeight - 1
          Adjuster_Add(1, 2, Is(GetTile(Editor\CurrentScene, Layer, X, Y + 1)))
        Else
          Adjuster_Add(1, 2, #True)
        EndIf
        ; Tiles, die diagonal zum jeweiligen Tile liegen, werden
        ; nicht beachtet, da sie fr die Angleichung nicht relevant sind.
        
        ; Nun werden alle mglichen Kombinationen abgefragt:
        DoReplace = #True
        Select Combination
          Case Adjuster_Combi(010,111,010) : DoReplace = #False       ;berall verbunden...
          Case Adjuster_Combi(010,011,010) : Adjuster_Replace(-1,  0) ;Rechts, Oben und Unten
          Case Adjuster_Combi(010,110,010) : Adjuster_Replace( 1,  0) ;Links, Oben und Unten
          Case Adjuster_Combi(000,111,010) : Adjuster_Replace( 0, -1) ;Links, Rechts und Unten
          Case Adjuster_Combi(010,111,000) : Adjuster_Replace( 0,  1) ;Links, Rechts und Oben
          Case Adjuster_Combi(000,011,010) : Adjuster_Replace(-1, -1) ;Rechts und Unten
          Case Adjuster_Combi(000,110,010) : Adjuster_Replace( 1, -1) ;Links und Unten
          Case Adjuster_Combi(010,011,000) : Adjuster_Replace(-1,  1) ;Rechts und Oben
          Case Adjuster_Combi(010,110,000) : Adjuster_Replace( 1,  1) ;Links und Oben
          Case Adjuster_Combi(000,111,000) : Adjuster_Replace( 0,  2) ;Links und Rechts
          Case Adjuster_Combi(000,011,000) : Adjuster_Replace(-1,  2) ;Rechts
          Case Adjuster_Combi(000,110,000) : Adjuster_Replace( 1,  2) ;Links
          Case Adjuster_Combi(010,010,010) : Adjuster_Replace( 2,  0) ;Oben und Unten
          Case Adjuster_Combi(010,010,000) : Adjuster_Replace( 2,  1) ;Oben
          Case Adjuster_Combi(000,010,010) : Adjuster_Replace( 2, -1) ;Unten
          Case Adjuster_Combi(000,010,000) : Adjuster_Replace( 2,  2) ;Garnicht verbunden
        EndSelect
            
        ; Tile durch das passende Tile ersetzen:
        If DoReplace And Not ((ReplaceX = 2 Or ReplaceY = 2) And OutOfRange)
          *ToReplace = LoadRegTile(TileX + ReplaceX, TileY + ReplaceY, *Tileset)
          UseRegTile(*ToReplace)
          ReleaseRegTile(*CenterTile)
          *Tile\Registered = *ToReplace
          
          DoReplace = #Null
        EndIf
      EndIf
    Next
  Next
  
  ProcedureReturn #True
EndProcedure
; IDE Options = PureBasic 4.51 (Windows - x86)
; CursorPosition = 55
; FirstLine = 67
; Folding = -
; EnableXP