Procedure.i InitializeEventBank(*Bank.EventBank)
  Protected *Memory
  
  *Bank\Library = OpenLibrary(#PB_Any, GetFile("Events\" + *Bank\Name + ".dll"))
  If Not *Bank\Library
    MessageRequester("Error", "Can not load target bank: " + #Q + *Bank\Name + #Q, #MB_ICONERROR)
    ProcedureReturn #False
  EndIf
  ; Funktionen wrappen:
  CallFunction(*Bank\Library, "GM_Init", ?Functions, #None)
  
  *Bank\Entries = CreateQueue()
  ; Ordnerstruktur einlesen:
  *Memory = CallFunction(*Bank\Library, "GM_Build")
  If Not *Memory
    MessageRequester("Warning", "Bank does not contain an target tree: " + #Q + *Bank\Name + #Q, #MB_ICONWARNING)
  Else
    ExtractEventEntries(*Bank, *Memory, *EventFolder)
  EndIf
  
  *Bank\UsedEvents = 0
  
  ProcedureReturn *Bank
EndProcedure
Procedure.i LoadEventBanks(Path.s)
  Protected Directory.i, Name.s, *Bank.EventBank, *Memory
  
  Directory = ExamineDirectory(#PB_Any, Path, "*.dll")
  If Directory
    While NextDirectoryEntry(Directory)
      If DirectoryEntryType(Directory) = #PB_DirectoryEntry_File
        Name = StringField(DirectoryEntryName(Directory), 1, ".")
        If Not GetEventBank(Name)
          *Bank = _New(EventBank)
          AppendQueueItem(*EventBanks, *Bank)
          
          *Bank\Name = Name
          InitializeEventBank(*Bank)
        EndIf
      EndIf
    Wend
    FinishDirectory(Directory)
    ProcedureReturn #True
  Else
    MessageRequester("Error", "Can not find target folder: " + #Q + Path + #Q, #MB_ICONERROR)
    ProcedureReturn #False
  EndIf
EndProcedure
Procedure.i ExtractEventEntries(*Bank.EventBank, *Memory.All, *Parent.Entry)
  Protected *Callback, *Event.EventEntry, *Entry.Entry, String.s
  
  Repeat
    Select *Memory\c
      Case 1 ;File
        *Memory + _Sz(Character)
        String  = PeekS(*Memory)
        *Memory + (Len(String) + 1)*_Sz(Character)
        
        *Event = _New(EventEntry)
        
        *Event\Bank         = *Bank
        *Event\CallbackName = String
        *Event\Callback     = GetFunction(*Bank\Library, "Event_" + String)
        *Event\Entry        = CreateEntryFile(EventName(*Event), *Event, *Parent)
        
        AppendQueueItem(*Bank\Entries, *Event)
      Case 2 ;Folder
        *Memory + _Sz(Character)
        String  = PeekS(*Memory)
        *Memory + (Len(String) + 1)*_Sz(Character)
        *Entry  = GetEntry(*Parent, String)
        If Not *Entry
          *Entry = CreateEntryFolder(String, *Parent)
        EndIf
        *Memory = ExtractEventEntries(*Bank, *Memory, *Entry)
      Case 0 ;End
        ProcedureReturn *Memory + _Sz(Character)
    EndSelect
  ForEver
EndProcedure
Procedure.i GetEventBank(Name.s)
  Protected *Item.QueueItem, *Bank.EventBank
  
  *Item = *EventBanks\First
  While *Item
    *Bank = *Item\Memory
    If *Bank\Name = Name
      ProcedureReturn *Bank
    EndIf
    _Move(*Item, N)
  Wend
  
  ProcedureReturn #Null
EndProcedure
Procedure.i LoadEventBank(Name.s)
  Protected *Item.QueueItem, *Bank.EventBank
  
  *Item = *EventBanks\First
  While *Item
    *Bank = *Item\Memory
    If *Bank\Name = Name
      ProcedureReturn *Bank
    EndIf
    _Move(*Item, N)
  Wend
  
  *Bank = _New(EventBank)
  AppendQueueItem(*EventBanks, *Bank)
  
  *Bank\Name = Name
  ProcedureReturn InitializeEventBank(*Bank)
EndProcedure

Procedure.i GetEventByCallbackName(*Bank.EventBank, Callback.s)
  Protected *Item.QueueItem, *Event.EventEntry
  
  *Item = *Bank\Entries\First
  While *Item
    *Event = *Item\Memory
    If *Event\CallbackName = Callback
      ProcedureReturn *Event
    EndIf
    _Move(*Item, N)
  Wend
  
  ProcedureReturn #Null
EndProcedure
Procedure.s EventName(*Entry.EventEntry)
  ProcedureReturn PeekS(_Call(*Entry\Callback, #Null, #Event_Name, #Null))
EndProcedure
Procedure.s EventDescription(*Entry.EventEntry)
  Protected *Result
  
  *Result = _Call(*Entry\Callback, #Null, #Event_Description, #Null)
  If *Result
    ProcedureReturn PeekS(*Result)
  Else
    ProcedureReturn "No description"
  EndIf
EndProcedure

Procedure.i CreateEvent(*Act.Act, *Entry.EventEntry, Initialize.i = #True)
  Protected *Event.Event
  
  *Event       = _New(Event)
  *Event\Act   = *Act
  *Event\Item  = AppendQueueItem(*Act\Events, *Event)
  
  If *Entry\References = 0
    *Entry\Bank\UsedEvents + 1
  EndIf
  *Entry\References + 1
  
  *Event\Entry      = *Entry
  *Event\Effects    = CreateQueue(@FreeEffect())
  *Event\References = 1
  
  If Initialize
    _Call(*Entry\Callback, #Null, #Event_EditorInitialize, *Event)
  EndIf
  ProcedureReturn *Event
EndProcedure
Procedure.i GetEvent(*Act.Act, Name.s)
  Protected *Item.QueueItem, *Event.Event
  
  If *Act
    *Item = *Act\Events\First
    While *Item
      *Event = *Item\Memory
      If *Event\Entry\Entry\Name = Name
        ProcedureReturn *Event
      EndIf
      _Move(*Item, N)
    Wend
  EndIf
  
  ProcedureReturn #Null
EndProcedure

Procedure.s EventText(*Event.Event)
  ProcedureReturn PeekS(_Call(*Event\Entry\Callback, #Null, #Event_Text, *Event))
EndProcedure
Procedure.i EditEvent(*Event.Event)
  ProcedureReturn _Call(*Event\Entry\Callback, #Null, #Event_Edit, *Event)
EndProcedure
Procedure.i CheckEvent(*Event.Event)
  ProcedureReturn _Call(*Event\Entry\Callback, #Null, #Event_Check, *Event)
EndProcedure

Procedure.i UseEvent(*Event.Event)
  *Event\References + 1
  ProcedureReturn *Event\References
EndProcedure
Procedure.i ReleaseEvent(*Event.Event)
  If *Event\References = 1
    ProcedureReturn RemoveEvent(*Event)
  Else
    *Event\References - 1
    ProcedureReturn #False
  EndIf
EndProcedure
Procedure.i FreeEvent(*Event.Event)
  If *Event\Entry\References = 1
    *Event\Entry\References      = 0
    *Event\Entry\Bank\UsedEvents - 1
  Else
    *Event\Entry\References - 1
  EndIf
  
  FreeQueue(*Event\Effects)
  
  _Call(*Event\Entry\Callback, #Null, #Event_EditorDelete, *Event)
  ProcedureReturn _Free(*Event)
EndProcedure
Procedure.i RemoveEvent(*Event.Event)
  If *Event\References > 1
    MessageRequester("Warning", "This target is used by other effects.", #MB_ICONWARNING)
    ProcedureReturn #False
  Else
    ProcedureReturn FreeQueueItem(*Event\Act\Events, *Event\Item)
  EndIf
EndProcedure


Procedure.i InitializeEffectBank(*Bank.EffectBank)
  Protected *Memory
  
  *Bank\Library = OpenLibrary(#PB_Any, GetFile("Effects\" + *Bank\Name + ".dll"))
  If Not *Bank\Library
    MessageRequester("Error", "Can not load target bank: " + #Q + *Bank\Name + #Q, #MB_ICONERROR)
    ProcedureReturn #False
  EndIf
  ; Funktionen wrappen:
  CallFunction(*Bank\Library, "GM_Init", ?Functions, #None)
  
  *Bank\Entries = CreateQueue()
  ; Ordnerstruktur einlesen:
  *Memory = CallFunction(*Bank\Library, "GM_Build")
  If Not *Memory
    MessageRequester("Warning", "Bank does not contain an target tree: " + #Q + *Bank\Name + #Q, #MB_ICONWARNING)
  Else
    ExtractEffectEntries(*Bank, *Memory, *EffectFolder)
  EndIf
  
  *Bank\UsedEffects = 0
  
  ProcedureReturn *Bank
EndProcedure
Procedure.i LoadEffectBanks(Path.s)
  Protected Directory.i, Name.s, *Bank.EffectBank, *Memory
  
  Directory = ExamineDirectory(#PB_Any, Path, "*.dll")
  If Directory
    While NextDirectoryEntry(Directory)
      If DirectoryEntryType(Directory) = #PB_DirectoryEntry_File
        Name = StringField(DirectoryEntryName(Directory), 1, ".")
        If Not GetEffectBank(Name)
          *Bank = _New(EffectBank)
          AppendQueueItem(*EffectBanks, *Bank)
          
          *Bank\Name = Name
          InitializeEffectBank(*Bank)
        EndIf
      EndIf
    Wend
    FinishDirectory(Directory)
    ProcedureReturn #True
  Else
    MessageRequester("Error", "Can not find effect folder: " + #Q + Path + #Q, #MB_ICONERROR)
    ProcedureReturn #False
  EndIf
EndProcedure
Procedure.i ExtractEffectEntries(*Bank.EffectBank, *Memory.All, *Parent.Entry)
  Protected *Callback, *Effect.EffectEntry, *Entry.Entry, String.s
  
  Repeat
    Select *Memory\c
      Case 1 ;File
        *Memory + _Sz(Character)
        String  = PeekS(*Memory)
        *Memory + (Len(String) + 1) * _Sz(Character)
        
        *Effect = _New(EffectEntry)
        
        *Effect\Bank         = *Bank
        *Effect\CallbackName = String
        *Effect\Callback     = GetFunction(*Bank\Library, "Effect_" + String)
        *Effect\Entry        = CreateEntryFile(EffectName(*Effect), *Effect, *Parent)
        
        AppendQueueItem(*Bank\Entries, *Effect)
      Case 2 ;Folder
        *Memory + _Sz(Character)
        String  = PeekS(*Memory)
        *Memory + (Len(String) + 1) * _Sz(Character)
        *Entry  = GetEntry(*Parent, String)
        If Not *Entry
          *Entry = CreateEntryFolder(String, *Parent)
        EndIf
        *Memory = ExtractEffectEntries(*Bank, *Memory, *Entry)
      Case 0 ;End
        ProcedureReturn *Memory + _Sz(Character)
    EndSelect
  ForEver
EndProcedure
Procedure.i GetEffectBank(Name.s)
  Protected *Item.QueueItem, *Bank.EffectBank
  
  *Item = *EffectBanks\First
  While *Item
    *Bank = *Item\Memory
    If *Bank\Name = Name
      ProcedureReturn *Bank
    EndIf
    _Move(*Item, N)
  Wend
  
  ProcedureReturn #Null
EndProcedure
Procedure.i LoadEffectBank(Name.s)
  Protected *Item.QueueItem, *Bank.EffectBank
  
  *Item = *EffectBanks\First
  While *Item
    *Bank = *Item\Memory
    If *Bank\Name = Name
      ProcedureReturn *Bank
    EndIf
    _Move(*Item, N)
  Wend
  
  *Bank = _New(EffectBank)
  AppendQueueItem(*EffectBanks, *Bank)
  
  *Bank\Name = Name
  ProcedureReturn InitializeEffectBank(*Bank)
EndProcedure

Procedure.i GetEffectByCallbackName(*Bank.EffectBank, Callback.s)
  Protected *Item.QueueItem, *Effect.EffectEntry
  
  *Item = *Bank\Entries\First
  While *Item
    *Effect = *Item\Memory
    If *Effect\CallbackName = Callback
      ProcedureReturn *Effect
    EndIf
    _Move(*Item, N)
  Wend
  
  ProcedureReturn #Null
EndProcedure
Procedure.s EffectName(*Entry.EffectEntry)
  ProcedureReturn PeekS(_Call(*Entry\Callback, #Null, #Effect_Name, #Null))
EndProcedure
Procedure.s EffectDescription(*Entry.EffectEntry)
  Protected *Result
  
  *Result = _Call(*Entry\Callback, #Null, #Effect_Description, #Null)
  If *Result
    ProcedureReturn PeekS(*Result)
  Else
    ProcedureReturn "No description"
  EndIf
EndProcedure

Procedure.i CreateEffect(*Event.Event, *Entry.EffectEntry, Initialize.i = #True)
  Protected *Effect.Effect
  
  *Effect        = _New(Effect)
  *Effect\Event = *Event
  *Effect\Item   = AppendQueueItem(*Event\Effects, *Effect)
  
  If *Entry\References = 0
    *Entry\Bank\UsedEffects + 1
  EndIf
  *Entry\References + 1
  
  *Effect\Entry = *Entry
  
  If Initialize
    _Call(*Entry\Callback, #Null, #Effect_EditorInitialize, *Effect)
  EndIf
  ProcedureReturn *Effect
EndProcedure

Procedure.s EffectText(*Effect.Effect)
  ProcedureReturn PeekS(_Call(*Effect\Entry\Callback, #Null, #Effect_Text, *Effect))
EndProcedure
Procedure.i EditEffect(*Effect.Effect)
  ProcedureReturn _Call(*Effect\Entry\Callback, #Null, #Effect_Edit, *Effect)
EndProcedure
Procedure.i CheckEffect(*Effect.Effect)
  ProcedureReturn _Call(*Effect\Entry\Callback, #Null, #Effect_Check, *Effect)
EndProcedure

Procedure.i FreeEffect(*Effect.Effect)
  If *Effect\Entry\References = 1
    *Effect\Entry\References       = 0
    *Effect\Entry\Bank\UsedEffects - 1
  Else
    *Effect\Entry\References - 1
  EndIf
  
  _Call(*Effect\Entry\Callback, #Null, #Effect_EditorDelete, *Effect)
  ProcedureReturn _Free(*Effect)
EndProcedure
Procedure.i RemoveEffect(*Effect.Effect)
  ProcedureReturn FreeQueueItem(*Effect\Event\Effects, *Effect\Item)
EndProcedure
Procedure.i MoveEffectUp(*Effect.Effect)
  Protected *Item.QueueItem, *PItem.QueueItem
  
  *Item = *Effect\Item
  If *Item\P
    If *Item\N
      *Item\P\N = *Item\N
      *Item\N\P = *Item\P
    Else
      *Item\P\N = #Null
      *Effect\Event\Effects\Last = *Item\P
    EndIf
    *Item\N = *Item\P
    
    If *Item\P\P
      *Item\P\P\N = *Item
      *PItem      = *Item\P
      *Item\P     = *Item\P\P
      *PItem\P    = *Item
    Else
      *Effect\Event\Effects\First = *Item
      *Item\P = #Null
    EndIf
  EndIf
EndProcedure
Procedure.i MoveEffectDown(*Effect.Effect)
  Protected *Item.QueueItem, *NItem.QueueItem
  
  *Item = *Effect\Item
  If *Item\N
    If *Item\P
      *Item\N\P = *Item\P
      *Item\P\N = *Item\N
    Else
      *Item\N\P = #Null
      *Effect\Event\Effects\First = *Item\N
    EndIf
    *Item\P = *Item\N
    
    If *Item\N\N
      *Item\N\N\P = *Item
      *NItem      = *Item\N
      *Item\N     = *Item\N\N
      *NItem\N    = *Item
    Else
      *Effect\Event\Effects\Last = *Item
      *Item\N = #Null
    EndIf
  EndIf
EndProcedure


Procedure.i CreateAct(Name.s = "Unknow", Description.s = "")
  Protected *Item.QueueItem, *Act.Act, Number.i = 2
  Protected NewName.s
  
  ; Nach gleichen Namen prfen:
  NewName = Name
  Repeat
    If Not GetAct(NewName)
      Break
    Else
      NewName = Name + " " + Str(Number)
      Number + 1
    EndIf
  ForEver
  
  *Act      = _New(Act)
  *Act\Item = AppendQueueItem(*Acts, *Act)
  
  *Act\Name        = NewName
  *Act\Description = Description
  *Act\Events     = CreateQueue(@FreeEvent())
  
  *Act\References = 1
  
  ProcedureReturn *Act
EndProcedure
Procedure.i GetAct(Name.s)
  Protected *Item.QueueItem, *Act.Act
  
  *Item = *Acts\First
  While *Item
    *Act = *Item\Memory
    If *Act\Name = Name
      ProcedureReturn *Act
    EndIf
    _Move(*Item, N)
  Wend
  
  ProcedureReturn #Null
EndProcedure
Procedure.s RenameAct(*Act.Act, Name.s)
  Protected NewName.s, Number.i = 2
  
  NewName = Name
  Repeat
    If Not GetAct(NewName)
      Break
    Else
      NewName = Name + " " + Str(Number)
      Number + 1
    EndIf
  ForEver
  
  *Act\Name = NewName
  ProcedureReturn NewName
EndProcedure

Procedure.i UseAct(*Act.Act)
  *Act\References + 1
EndProcedure
Procedure.i ReleaseAct(*Act.Act)
  *Act\References - 1
EndProcedure

Procedure.i FreeAct(*Act.Act)
  FreeQueue(*Act\Events)
  ClearStructure(*Act, Act)
  ProcedureReturn _Free(*Act)
EndProcedure
Procedure.i RemoveAct(*Act.Act)
  If *Act\References = 1
    FreeQueueItem(*Acts, *Act\Item)
    ProcedureReturn #True
  Else
    MessageRequester("Warning", "Scenes, objects or other elements still use this act.")
    ProcedureReturn #False
  EndIf
EndProcedure

; IDE Options = PureBasic 4.51 (Windows - x86)
; CursorPosition = 233
; FirstLine = 27
; Folding = BAgAAAw
; EnableXP