;**************************************************************
;
;     G E O M E T R A
;
;**************************************************************
;   Copyright (c) 2009 Josef Sniatecki
;
; This program is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License
; As published by the Free Software Foundation; either version
; 3 of the License, Or (at your option) any later version.
;
; This program is distributed in the hope that it will be
; useful, but WITHOUT ANY WARRANTY; without even the implied
; warranty of MERCHANTABILITY Or FITNESS For A PARTICULAR
; PURPOSE. See the GNU General Public License For more details.
;
; You should have received a copy of the GNU General Public
; License along With this program;
; If Not, see "http://www.gnu.org/licenses/".
;**************************************************************



EnableExplicit


;******************************************
; Includes
;******************************************

IncludePath "..\..\Includes"
IncludeFile "Standard.pbi"
IncludeFile "Window.pbi"
IncludeFile "Keyboard.pbi"
IncludeFile "Queue.pbi"
IncludeFile "Collision.pbi"



;******************************************
; Constants
;******************************************

#FieldSize = 24

#SceneWidth  = 25
#SceneHeight = 10

#Gravity = 0.9

Enumeration
  #BackgroundTile1
  #BackgroundTile2
  #BackgroundTile3
  #Tile
  #Ground
  #ForegroundTile
  #BackgroundObject
  #ForegroundObject
EndEnumeration

Enumeration 1
  #Object_Frame
  #Object_Load
  #Object_Save
  #Object_Initialize
  #Object_Collision
  #Object_Delete
EndEnumeration
Enumeration 1
  #ObjectDo_Delete
EndEnumeration



;******************************************
; Macros
;******************************************

Macro GetField(Scene, X, Y)
  Scene\Fields + ((Y * #SceneWidth) + X)*_Sz(Field)
EndMacro
Macro CheckFieldCollision(FieldX, FieldY)
  RectangleCollision(Player\X, Player\Y, #FieldSize, #FieldSize, FieldX*#FieldSize, FieldY*#FieldSize, #FieldSize, #FieldSize)
EndMacro



;******************************************
; Structures
;******************************************

Structure Tile
  *Field.Field
  Sprite.i
  X.i
  Y.i
EndStructure
Structure Object
  *Item.QueueItem
  *Callback
  Sprite.i
  
  X.f
  Y.f
  Width.i
  Height.i
  
  Transparency.i
  Invisible.i
  
  *Memory
EndStructure
Structure Field
  *BackgroundTile1.Tile
  *BackgroundTile2.Tile
  *BackgroundTile3.Tile
  *Tile.Tile
  *Ground.Tile
  *ForegroundTile.Tile
  *BackgroundObject.Object
EndStructure
Structure Player
  Sprite.i
  
  X.f
  Y.f
  MoveX.f
  MoveY.f
  
  Speed.f
  Mass.f
  
  Jumpable.i
  Jumptime.i
  Jumpable2.i
  Jumptime2.i
  CanSecondJump.i
  JumpKeyPressed.i
  JumpKeyPressing.i
  
  FieldX.i
  FieldY.i
  *Scene.Scene
  
  Lives.i
  Energy.i
  Coins.i
  Score.i
EndStructure

Structure Challange
  Name.s
  Author.s
  
  Icon.i
  
  Highscore.i
  OwnHighscore.i
EndStructure
Structure CurrentChallange
  *Header.Challange
  
  Energy.i
  Lives.i
  
  *StartScene.Scene
  X.i
  Y.i
EndStructure
Structure Scene
  *BackgroundTiles1
  *BackgroundTiles2
  *BackgroundTiles3
  *Tiles
  *Grounds
  *ForegroundTiles
  *BackgroundObjects.Queue
  *ForegroundObjects.Queue
  
  *Fields
  *Left.Scene
  *Right.Scene
  *Top.Scene
  *Bottom.Scene
EndStructure

Structure Tileset
  Image.i
  Width.i
  Height.i
  *Registered.RegisteredTileset
EndStructure

Structure RegisteredTileset
  Name.s
  *Tileset.Tileset
  *Tiles
  *Item.QueueItem
EndStructure
Structure RegisteredSprite
  Name.s
  Type.i
  Sprite.i
EndStructure



;******************************************
; Declarations
;******************************************

Declare.i LoadChallange(Name.s)
Declare.i LoadTileGroup(File.i, *Scene.Scene, Group.i, Array Tile.i(1))
Declare.i LoadObjectGroup(File.i, *Scene.Scene, Group.i)

Declare.i RegisterTileset(Name.s, *Tileset.Tileset)
Declare.i LoadRegisteredTileset(Name.s)
Declare.i LoadTileset(Name.s)
Declare.i LoadRegisteredTile(*Tileset.Tileset, X.i, Y.i)
Declare.i LoadTile(*Tileset.Tileset, X.i, Y.i)

Declare.i RegisterNamelessSprite(Sprite.i)

Declare.i CreateTile(*Scene.Scene, Group.i, Index.i, Sprite.i, X.i, Y.i)
Declare.i FreeTile(*Tile.Tile)

Declare.i CreateObject(*Scene.Scene, Group.i, *Callback, X.i, Y.i)

Declare.i CreateScene(*Left.Scene, *Right.Scene, *Top.Scene, *Bottom.Scene)
Declare.i DisplayScene_FirstLayer(*Scene.Scene)
Declare.i DisplayScene_SecondLayer(*Scene.Scene)

Declare.i FieldCollision(DeltaX.i, DeltaY.i)
Declare.i PlayerCollision(X.i, Y.i, Direction.i)



;******************************************
; Variables
;******************************************

Global Player.Player

Global *RegisteredTilesets.Queue = CreateQueue()
Global *RegisteredSprites.Queue  = CreateQueue()
Global *RegisteredScenes.Queue   = CreateQueue()


;******************************************
; Procedures
;******************************************

Procedure.i RegisterNamelessSprite(Sprite.i)
  Protected *RegisteredSprite.RegisteredSprite
  
  *RegisteredSprite = _New(RegisteredSprite)
  AppendQueueItem(*RegisteredSprites, *RegisteredSprite)
  
  *RegisteredSprite\Sprite = Sprite
  
  ProcedureReturn *RegisteredSprite
EndProcedure



;******************************************
; Sources
;******************************************

IncludePath "Sources"

IncludeFile "Challange.pb"
IncludeFile "Tiles.pb"
IncludeFile "Objects.pb"
IncludeFile "Collision.pb"
IncludeFile "Scene.pb"



;******************************************
; Main
;******************************************

Macro WriteTile(File, Tileset, Index)
  WriteInteger(File, Tileset)
  WriteInteger(File, Index)
EndMacro
Macro WriteField(File, X, Y, Tile)
  WriteInteger(File, X)
  WriteInteger(File, Y)
  WriteInteger(File, Tile)
EndMacro

CreateFile(0, "Test.bin")
  WriteInteger(0, 1) ;Tileset count
  ; Tilesets:
    WriteNTString(0, "Standard")
  
  WriteInteger(0, 3) ;Tile count
  ; Tiles:
    ; Tile 0 (wall):
    WriteInteger(0, 0)  ;Tileset 1
    WriteInteger(0, 29) ;Index
    
    ; Tile 1 (ground):
    WriteInteger(0, 0)
    WriteInteger(0, 57)
    
    ; Tile 2 (background):
    WriteInteger(0, 0)
    WriteInteger(0, 49)
    
  ; Scenes:
  WriteInteger(0, 2) ;Width
  WriteInteger(0, 1) ;Height
  
  ;Scene (0|0):
    WriteByte(0, 2) ;Type (2 = Start scene)
    
    WriteInteger(0, 9) ;Background tiles 1:
      WriteField(0, #SceneWidth - 5, 8, 2)
      WriteField(0, #SceneWidth - 5, 7, 2)
      WriteField(0, #SceneWidth - 6, 8, 2)
      WriteField(0, #SceneWidth - 6, 7, 2)
      WriteField(0, #SceneWidth - 7, 7, 2)
      WriteField(0, #SceneWidth - 8, 8, 2)
      WriteField(0, #SceneWidth - 8, 7, 2)
      WriteField(0, #SceneWidth - 9, 8, 2)
      WriteField(0, #SceneWidth - 9, 7, 2)
    
    WriteInteger(0, 0) ;Background tiles 2:
    
    WriteInteger(0, 0) ;Background tiles 3:
    
    WriteInteger(0, 23) ;Walls:
      WriteField(0, #SceneWidth - 4, 9, 0)
      WriteField(0, #SceneWidth - 4, 8, 0)
      WriteField(0, #SceneWidth - 4, 7, 0)
      WriteField(0, #SceneWidth - 5, 9, 0)
      WriteField(0, #SceneWidth - 6, 9, 0)
      WriteField(0, #SceneWidth - 7, 9, 0)
      WriteField(0, #SceneWidth - 7, 8, 0)
      WriteField(0, #SceneWidth - 8, 9, 0)
      WriteField(0, #SceneWidth - 9, 9, 0)
      WriteField(0, #SceneWidth - 10, 9, 0)
      WriteField(0, #SceneWidth - 10, 8, 0)
      WriteField(0, #SceneWidth - 10, 7, 0)
      WriteField(0, #SceneWidth - 10, 6, 0)
      WriteField(0, 0, 7, 0)
      WriteField(0, 1, 7, 0)
      WriteField(0, 2, 7, 0)
      WriteField(0, 3, 7, 0)
      WriteField(0, 4, 7, 0)
      WriteField(0, 4, 8, 0)
      WriteField(0, 4, 9, 0)
      WriteField(0, 5, 9, 0)
      WriteField(0, 6, 9, 0)
      WriteField(0, 7, 9, 0)
    
    WriteInteger(0, 14) ;Grounds:
      WriteField(0, #SceneWidth - 1 , 5, 1)
      WriteField(0, #SceneWidth - 2 , 5, 1)
      WriteField(0, #SceneWidth - 3 , 5, 1)
      WriteField(0, #SceneWidth - 4 , 5, 1)
      WriteField(0, #SceneWidth - 7 , 6, 1)
      WriteField(0, #SceneWidth - 8 , 6, 1)
      WriteField(0, #SceneWidth - 9 , 6, 1)
      WriteField(0, #SceneWidth - 12, 5, 1)
      WriteField(0, #SceneWidth - 13, 5, 1)
      WriteField(0, #SceneWidth - 11, 7, 1)
      WriteField(0, #SceneWidth - 12, 7, 1)
      WriteField(0, #SceneWidth - 13, 7, 1)
      WriteField(0, 8, 9, 1)
      WriteField(0, 9, 9, 1)
    
    WriteInteger(0, 0) ;Foreground tiles:
    
    WriteInteger(0, 0) ;Background objects:
    
    WriteInteger(0, 3) ;Foreground objects:
      WriteInteger(0, #SceneWidth - 2)
      WriteInteger(0, 4)
      WriteInteger(0, 0)
      
      WriteInteger(0, #SceneWidth - 13)
      WriteInteger(0, 4)
      WriteInteger(0, 0)
      
      WriteInteger(0, 2)
      WriteInteger(0, 6)
      WriteInteger(0, 1)
    
    WriteInteger(0, #SceneWidth - 8) ;Start X
    WriteInteger(0, 5              ) ;Start Y
  
  ;Scene (1|0)
    WriteByte(0, 1) ;Type (1 = Default scene)
    
    WriteInteger(0, 0) ;Background tiles 1:
    
    WriteInteger(0, 0) ;Background tiles 2:
    
    WriteInteger(0, 0) ;Background tiles 3:
    
    WriteInteger(0, 6) ;Walls:
      WriteField(0, 3, 8, 0)
      WriteField(0, 4, 8, 0)
      WriteField(0, 5, 8, 0)
      WriteField(0, 6, 8, 0)
      WriteField(0, 7, 8, 0)
      WriteField(0, 8, 8, 0)
    
    WriteInteger(0, 9) ;Grounds:
      WriteField(0, 0 , 5, 1)
      WriteField(0, 1 , 5, 1)
      WriteField(0, 2 , 5, 1)
      WriteField(0, 3 , 5, 1)
      WriteField(0, 6 , 6, 1)
      WriteField(0, 7 , 6, 1)
      WriteField(0, 8 , 6, 1)
      WriteField(0, 11, 5, 1)
      WriteField(0, 12, 5, 1)
    
    WriteInteger(0, 0) ;Foreground tiles:
    
    WriteInteger(0, 0) ;Background objects:
    
    WriteInteger(0, 1) ;Foreground objects:
      WriteInteger(0, 7)
      WriteInteger(0, 5)
      WriteInteger(0, 1)
CloseFile(0)
  
  

UsePNGImageDecoder()
UsePNGImageEncoder()

InitSprite()
InitSprite3D()
OpenWindow(0, #PB_Ignore, #PB_Ignore, #FieldSize*#SceneWidth, #FieldSize*#SceneHeight, "Geomix", #PB_Window_ScreenCentered|#PB_Window_SystemMenu)
OpenWindowedScreen(WindowID(0), 0, 0,  #FieldSize*#SceneWidth, #FieldSize*#SceneHeight, 0, 0, 0)



Player\Sprite        = LoadSprite(#PB_Any, "Graphics\Player\Player.bmp")
Player\Jumpable      = #True
Player\CanSecondJump = #True
Player\Speed         = 3
Player\Mass          = 1



LoadChallange("Test.bin")



Define Event.i, *Item.QueueItem, *Object.Object, Collision.i, Result.i

Repeat
  Event = WindowEvent()
  
  StartSpecialFX()
    ClearScreen(#White)
  Start3D()
      DisplayScene_FirstLayer(Player\Scene)
      DisplayTransparentSprite(Player\Sprite, Player\X, Player\Y)
      DisplayScene_SecondLayer(Player\Scene)
  Stop3D()
  StopSpecialFX()
  
  StartDrawing(ScreenOutput())
    DrawingMode(#PB_2DDrawing_Transparent)
    DrawText(0, 0, "Coins: " + Str(Player\Coins), $000000)
  StopDrawing()
  
  FlipBuffers()
  
  
  
  Player\MoveX = 0
  If IsKeyPressing(#VK_LEFT)
    Player\MoveX - Player\Speed
  ElseIf IsKeyPressing(#VK_RIGHT)
    Player\MoveX + Player\Speed
  EndIf
  
  Player\JumpKeyPressed = #False
  If IsKeyPressing(#VK_S)
    If Not Player\JumpKeyPressing
      Player\JumpKeyPressed = #True
      Player\JumpKeyPressing = #True
    EndIf
  ElseIf Player\JumpKeyPressing
    Player\JumpKeyPressing = #False
  EndIf
  
  If Player\Jumpable And Player\MoveY = 0
    If Player\JumpKeyPressed
      Player\Jumpable = #False
      If Player\CanSecondJump
        Player\Jumpable2 = #True
      EndIf
      Player\MoveY    = -6
      Player\Jumptime = 4
    EndIf
  ElseIf Player\Jumptime
    If Player\JumpKeyPressing
      Player\MoveY - 1.5
    EndIf
    Player\Jumptime - 1
  ElseIf Player\Jumpable2
    If Player\JumpKeyPressed
      Player\Jumpable2 = #False
      Player\MoveY     = -6
      Player\Jumptime  = 4
    EndIf
  ElseIf Player\Jumptime2
    If Player\JumpKeyPressing
      Player\MoveY - 1.5
    EndIf
    Player\Jumptime - 1
  EndIf
  
  If IsKeyPressing(#VK_DOWN)
    Player\MoveY + 1
  EndIf
  
  Player\MoveY + #Gravity * Player\Mass
  _Max(Player\MoveY, #Gravity * 8)
  
  Player\X      + Player\MoveX
  Player\Y      + Player\MoveY
  Player\FieldX = (Player\X) / #FieldSize
  Player\FieldY = (Player\Y) / #FieldSize
  
  
  
  ; Collisions:
  If Not FieldCollision(-1, 0)
    FieldCollision(1, 0)
  EndIf
  If Not FieldCollision(0, 1)
    If Not FieldCollision(-1, 1)
      If Not FieldCollision(1, 1)
        If Not FieldCollision(0, -1)
          If Not FieldCollision(-1, -1)
            FieldCollision(1, -1)
          EndIf
        EndIf
      EndIf
    EndIf
  EndIf
  
  *Item = Player\Scene\ForegroundObjects\First
  While *Item
    *Object = *Item\Memory
    If Not *Object\Invisible
      Collision = RectangleCollision(Player\X, Player\Y, #FieldSize, #FieldSize, *Object\X, *Object\Y, *Object\Width, *Object\Height)
      If Collision
        Result = _Call(*Object\Callback, *Object, #Object_Collision, Collision)
        If Result
          Select Result
            Case #ObjectDo_Delete
              *Item = FreeQueueItem(Player\Scene\ForegroundObjects, *Item)
              Continue
          EndSelect
        EndIf
      EndIf
    EndIf
    _Move(*Item, N)
  Wend
  
  
  
  If Player\X < 0
    If Player\Scene\Left
      Player\Scene  = Player\Scene\Left
      Player\X      = (#SceneWidth - 1)*#FieldSize
      Player\FieldX = #SceneWidth - 1
    Else
      Player\X      = 0
      Player\FieldX = 0
    EndIf
  ElseIf Player\X > (#SceneWidth - 1)*#FieldSize
    If Player\Scene\Right
      Player\Scene  = Player\Scene\Right
      Player\X      = 0
      Player\FieldX = 0
    Else
      Player\X     = (#SceneWidth - 1)*#FieldSize
      Player\FieldX = #SceneWidth - 1
    EndIf
  EndIf
  If Player\Y < 0
    If Player\Scene\Top
      Player\Scene  = Player\Scene\Top
      Player\Y      = (#SceneHeight - 1)*#FieldSize
      Player\FieldY = #SceneHeight - 1
    Else
      Player\Y      = 0
      Player\FieldY = 0
    EndIf
  ElseIf Player\Y > (#SceneHeight - 1)*#FieldSize
    If Player\Scene\Bottom
      Player\Scene  = Player\Scene\Bottom
      Player\Y      = 0
      Player\FieldY = 0
    Else
      Player\Y      = (#SceneHeight - 1)*#FieldSize
      Player\FieldY = #SceneHeight - 1
    EndIf
  EndIf
Until Event = #PB_Event_CloseWindow



End



;******************************************
; Data sections
;******************************************

DataSection
  ObjectCallbacks:
    Data.i @Object_Coin()
    Data.i @Object_CoinBox()
  EndObjectCallbacks:
EndDataSection
; IDE Options = PureBasic 4.40 (Windows - x86)
; CursorPosition = 272
; FirstLine = 149
; Folding = IA5
; EnableXP
; Executable = ..\Test.exe