CompilerSelect #Mode
  CompilerCase #Mode_Game ;{
;******************************************
; Game
;******************************************

Procedure.i LoadBank(Name.s)
  Protected *Item.QueueItem, *Bank.Bank
  
  *Item = *Banks\First
  While *Item
    *Bank = *Item\Memory
    If *Bank\Name = Name
      ProcedureReturn *Bank
    EndIf
    _Move(*Item, N)
  Wend
  
  *Bank = _New(Bank)
  AppendQueueItem(*Banks, *Bank)
  
  *Bank\Name    = Name
  *Bank\Library = OpenLibrary(#PB_Any, GetFile("Objects\" + Name + ".dll"))
  
  ; Initilisieren:
  CallFunction(*Bank\Library, "GM_Init", ?Functions, RandomSeed)
  
  ; Geladene Objekte in einer Queue speichern:
  *Bank\Callbacks = CreateQueue(#Queue_SaveMemory)
  
  ProcedureReturn *Bank
EndProcedure
Procedure.i CallBankLoadFunction(*Bank.Bank)
  Protected *Function
  
  *Function = GetFunction(*Bank\Library, "GM_Load")
  If *Function
    CallFunctionFast(*Function)
  EndIf
EndProcedure

Procedure.i LoadObject(*Bank.Bank, Callback.s)
  Protected *Item.QueueItem, *Callback
  
  *Callback = GetFunction(*Bank\Library, "Obj_" + Callback)
  If Not *Callback
    MessageRequester("Error", "Can not find object:" + #N + "bank: " + #T + *Bank\Name + #N + "object: " +#T + Callback, #MB_ICONERROR)
    ProcedureReturn #False
  EndIf
  
  If *Bank\Callbacks
    *Item = *Bank\Callbacks\First
    While *Item
      If *Item\Memory = *Callback
        ProcedureReturn *Callback
      EndIf
      _Move(*Item, N)
    Wend
  Else
    *Bank\Callbacks = CreateQueue(#Queue_SaveMemory)
  EndIf
  
  AppendQueueItem(*Bank\Callbacks, *Callback)
  ProcedureReturn *Callback
EndProcedure
Procedure.i GlobalLoadObject(*Callback)
  _Call(*Callback, #Null, #Object_GlobalLoad, #Null)
EndProcedure
Procedure.i GlobalInitObject(*Callback)
  _Call(*Callback, #Null, #Object_GlobalInitialize, #Null)
EndProcedure
Procedure.i GetObject(*Bank.Bank, Name.s)
  Protected *Callback, *Item.QueueItem
  
  *Callback = GetFunction(*Bank\Library, "Obj_" + Name)
  If Not *Callback
    ProcedureReturn #False
  EndIf
  
  If *Bank\Callbacks
    *Item = *Bank\Callbacks\First
    While *Item
      If *Item\Memory = *Callback
        ProcedureReturn *Callback
      EndIf
      _Move(*Item, N)
    Wend
  EndIf
  
  ProcedureReturn #False
EndProcedure

Procedure.i CreateObject(*Callback, X.f, Y.f, *Scene.Scene, Layer.i, *Parameter = #Null)
  Protected *Object.Object
  
  *Object       = _New(Object)
  *Object\Scene = *Scene
  *Object\Queue = LayerQueue(*Scene, Layer)
  *Object\Item  = AppendQueueItem(*Object\Queue, *Object)
  
  *Object\Callback     = *Callback
  *Object\X            = X
  *Object\Y            = Y
  *Object\Transparency = 255
  
  *Object\References = 1
  
  ProcedureReturn *Object
EndProcedure
Procedure.i FreeObject(*Object.Object)
  *Object\Hidden = 2
  ProcedureReturn #True
EndProcedure
Procedure.i ReleaseObject(*Object.Object)
  Protected *Item.QueueItem
  
  *Item = *Object\Item\N
  FreeQueueItem(*Object\Queue, *Object\Item)
  
  ProcedureReturn *Item
EndProcedure

Procedure.i HideObject(*Object.Object, State.i)
  If *Object\Hidden <> 2
    If *Object\Hidden <> State
      If *Object\Hidden
        _Call(*Object\Callback, *Object, #Object_Unhide, #Null)
      Else
        _Call(*Object\Callback, *Object, #Object_Hide, #Null)
      EndIf
      *Object\Hidden = State
      ProcedureReturn #True
    EndIf
  EndIf
  
  ProcedureReturn #False
EndProcedure

;}
  CompilerCase #Mode_Editor ;{
;******************************************
; Editor
;******************************************

Procedure.i InitializeBank(*Bank.Bank)
  Protected *Memory
  
  *Bank\Library = OpenLibrary(#PB_Any, GetFile("Objects\" + *Bank\Name + ".dll"))
  If Not *Bank\Library
    MessageRequester("Error", "Can not load object bank: " + #Q + *Bank\Name + #Q, #MB_ICONERROR)
    ProcedureReturn #False
  EndIf
  ; Initialisieren (Funktionen wrappen):
  CallFunction(*Bank\Library, "GM_Init", ?Functions, #None)
  
  *Bank\Entries = CreateQueue()
  ; Ordnerstruktur einlesen:
  *Memory = CallFunction(*Bank\Library, "GM_Build")
  If Not *Memory
    MessageRequester("Warning", "Bank does not contain an object tree: " + #Q + *Bank\Name + #Q, #MB_ICONWARNING)
  Else
    ExtractObjectEntries(*Bank, *Memory, *ObjectFolder)
  EndIf
  
  *Bank\RegisteredObjects = CreateQueue(#Queue_SaveMemory)
  
  ProcedureReturn *Bank
EndProcedure
Procedure.i LoadBanks(Path.s)
  Protected Directory.i, Name.s, *Bank.Bank, *Memory
  
  Directory = ExamineDirectory(#PB_Any, Path, "*.dll")
  If Directory
    While NextDirectoryEntry(Directory)
      If DirectoryEntryType(Directory) = #PB_DirectoryEntry_File
        Name = StringField(DirectoryEntryName(Directory), 1, ".")
        If Not GetBank(Name)
          *Bank = _New(Bank)
          AppendQueueItem(*Banks, *Bank)
          
          *Bank\Name = Name
          InitializeBank(*Bank)
        EndIf
      EndIf
    Wend
    FinishDirectory(Directory)
    ProcedureReturn #True
  Else
    MessageRequester("Error", "Can not find folder: " + #Q + Path + #Q, #MB_ICONERROR)
    ProcedureReturn #False
  EndIf
EndProcedure
Procedure.i ExtractObjectEntries(*Bank.Bank, *Memory.All, *Parent.Entry)
  Protected *Callback, *Object.ObjectEntry, *Entry.Entry, String.s
  
  Repeat
    Select *Memory\c
      Case 1 ;File
        *Memory + _Sz(Character)
        String  = PeekS(*Memory)
        *Memory + (Len(String) + 1)*_Sz(Character)
        
        *Object = _New(ObjectEntry)
        
        *Object\Bank         = *Bank
        *Object\CallbackName = String
        *Object\Callback     = GetFunction(*Bank\Library, "Obj_" + String)
        *Object\Entry        = CreateEntryFile(ObjectName(*Object), *Object, *Parent)
        
        AppendQueueItem(*Bank\Entries, *Object\Entry)
      Case 2 ;Folder
        *Memory + _Sz(Character)
        String  = PeekS(*Memory)
        *Memory + (Len(String) + 1)*_Sz(Character)
        *Entry  = GetEntry(*Parent, String)
        If Not *Entry
          *Entry = CreateEntryFolder(String, *Parent)
        EndIf
        *Memory = ExtractObjectEntries(*Bank, *Memory, *Entry)
      Case 0 ;End
        ProcedureReturn *Memory + _Sz(Character)
    EndSelect
  ForEver
EndProcedure

Procedure.i LoadRegObject(*Entry.ObjectEntry)
  Protected *Object.RegisteredObject, *ViewRegion.ViewRegion
  Protected SpriteName.s
  
  If *Entry\Registered
    ProcedureReturn *Entry\Registered
  EndIf
  
  *Object          = _New(RegisteredObject)
  *Object\BankItem = AppendQueueItem(*Entry\Bank\RegisteredObjects, *Object)
  *Object\Entry    = *Entry
  
  SpriteName  = ObjectViewSpriteFilename(*Entry)
  *ViewRegion = _Call(*Entry\Callback, #Null, #Object_ViewRegion, #Null)
  
  If *ViewRegion
    *Object\Sprite = LoadClippedSprite(GetFile(SpriteName), *ViewRegion\X, *ViewRegion\Y, *ViewRegion\Width, *ViewRegion\Height)
  Else
    *Object\Sprite = LoadSprite(#PB_Any, GetFile(SpriteName), #PB_Sprite_AlphaBlending|#PB_Sprite_Texture|#PB_Sprite_Memory)
  EndIf
  If Not *Object\Sprite
    MessageRequester("Error", "Can not load graphic: " + #Q + SpriteName + #Q, #MB_ICONERROR)
    ProcedureReturn #False
  EndIf
  *Object\Sprite3D = CreateSprite3D(#PB_Any, *Object\Sprite)
  
  *Entry\Registered = *Object
  
  ProcedureReturn *Object
EndProcedure
Procedure.i FreeRegObject(*Object.RegisteredObject)
  If *Object\Sprite3D
    FreeSprite3D(*Object\Sprite3D)
    FreeSprite(*Object\Sprite)
  EndIf
  
  *Object\Entry\Registered = #Null
  FreeQueueItem(*Object\Entry\Bank\RegisteredObjects, *Object\BankItem)
  
  _Free(*Object)
  ProcedureReturn #True
EndProcedure

Procedure.i GetObjectByCallbackName(*Bank.Bank, Name.s)
  Protected *Item.QueueItem, *Entry.Entry, *Object.ObjectEntry
  
  Name = PatchObjectCallback(Name)
  
  *Item = *Bank\Entries\First
  While *Item
    *Entry  = *Item\Memory
    *Object = *Entry\Memory
    
    If *Object\CallbackName = Name
      ProcedureReturn *Object
    EndIf
    
    _Move(*Item, N)
  Wend
  
  ProcedureReturn #Null
EndProcedure
Procedure.i GetObjectCallback(*Bank.Bank, Name.s)
  Protected *Item.QueueItem, *Entry.Entry, *Object.ObjectEntry
  
  Name = PatchObjectCallback(Name)
  
  *Item = *Bank\Entries\First
  While *Item
    *Entry  = *Item\Memory
    *Object = *Entry\Memory
    
    If *Object\CallbackName = Name
      ProcedureReturn *Object\Callback
    EndIf
    
    _Move(*Item, N)
  Wend
  
  ProcedureReturn #Null
EndProcedure
Procedure.s ObjectName(*Object.ObjectEntry)
  ProcedureReturn PeekS(_Call(*Object\Callback, #Null, #Object_Name, #Null))
EndProcedure
Procedure.s ObjectViewSpriteFilename(*Object.ObjectEntry)
  ProcedureReturn "Graphics\Objects\" + PeekS(_Call(*Object\Callback, #Null, #Object_ViewSprite, #Null)) + ".png"
EndProcedure
Procedure.s ObjectDescription(*Object.ObjectEntry)
  Protected *Result
  
  *Result = _Call(*Object\Callback, #Null, #Object_Description, #Null)
  If *Result
    ProcedureReturn PeekS(*Result)
  Else
    ProcedureReturn ""
  EndIf
EndProcedure

Procedure.i UseRegObject(*Object.RegisteredObject)
  *Object\References + 1
  ProcedureReturn *Object\References
EndProcedure
Procedure.i ReleaseRegObject(*Object.RegisteredObject)
  If *Object\References = 1
    ProcedureReturn FreeRegObject(*Object)
  Else
    *Object\References - 1
    ProcedureReturn #False
  EndIf
EndProcedure
Procedure.i CheckRegObject(*Object.RegisteredObject)
  If *Object\References = 0
    ProcedureReturn FreeRegObject(*Object)
  Else
    ProcedureReturn #False
  EndIf
EndProcedure

Procedure.i FreeObject(*Object.Object)
  _Call(*Object\Registered\Entry\Callback, #Null, #Object_EditorFree, *Object)
  ReleaseRegObject(*Object\Registered)
  
  ProcedureReturn _Free(*Object)
EndProcedure
Procedure.i RemoveObject(*Scene.Scene, Layer.i, X.i, Y.i)
  Protected *Object.Object
  
  *Object = GetTile(*Scene, Layer, X, Y)
  If *Object
    FreeObject(*Object)
    SetField(*Scene, Layer, X, Y, #Null)
    
    ProcedureReturn #True
  EndIf
  
  ProcedureReturn #False
EndProcedure

Procedure.i GetObject(*Scene.Scene, Layer.i, X.i, Y.i)
  ProcedureReturn GetField(*Scene, Layer, X, Y)
EndProcedure
Procedure.i PutObject(Layer.i, *Registered.RegisteredObject, X.i, Y.i)
  Protected *Object.Object, *Queue.Queue
  
  If _Call(*Registered\Entry\Callback, #Null, #Object_CheckLayer, Layer) = #False
    MessageRequester("Warning", ObjectName(*Registered\Entry) + " can not be placed at the current layer.", #MB_ICONWARNING)
    ProcedureReturn #False
  EndIf
  
  *Object = GetField(Editor\CurrentScene, Layer, X, Y)
  If *Object
    If Editor\Overwrite
      FreeObject(*Object)
    Else
      ProcedureReturn #Null
    EndIf
  EndIf
  
  *Object = _New(Object)
  SetField(Editor\CurrentScene, Layer, X, Y, *Object)
  
  *Object\Scene = Editor\CurrentScene
  *Object\Layer = Layer
  
  *Object\Registered = *Registered
  *Object\X          = X % #SceneWidth
  *Object\Y          = Y % #SceneHeight
  
  UseRegObject(*Registered)
  _Call(*Registered\Entry\Callback, #Null, #Object_EditorInitialize, *Object)
  
  ProcedureReturn *Object
EndProcedure
Procedure.i CopyObject(*Object.Object, Layer.i, X.i, Y.i)
  Protected *NewObject.Object
  
  ; Folgende Daten werden an das neue Objekt bergeben, damit
  ; "*Object\Memory" kopiert werden kann (falls ntig):
  DataSection
    CopyObject:
      CopyObject_Source:
        Data.i #Null
      CopyObject_Destination:
        Data.i #Null
  EndDataSection
  
  *NewObject = PutObject(Layer, *Object\Registered, X, Y)
  
  PokeI(?CopyObject_Source     , *Object)
  PokeI(?CopyObject_Destination, *NewObject)
  
  _Call(*NewObject\Registered\Entry\Callback, #Null, #Object_EditorCopy, ?CopyObject)
  
  ProcedureReturn *NewObject
EndProcedure
Procedure.i GetObjectsOf(*Callback, *Scene.Scene, Layer.i, List *Object.Object())
  Static X.i, Y.i, *SceneObject.Object
  
  For X = 0 To #SceneWidth - 1
    For Y = 0 To #SceneHeight - 1
      *SceneObject = GetField(*Scene, Layer, X, Y)
      If *SceneObject And *SceneObject\Registered\Entry\Callback = *Callback
        AddElement(*Object())
        *Object() = *SceneObject
      EndIf
    Next
  Next
EndProcedure

Procedure.i SelectObject(*Entry.ObjectEntry)
  ChangeObject(LoadRegObject(*Entry))
  Editor\Layer = _Call(*Entry\Callback, #Null, #Object_PreferedLayer, #Null)
EndProcedure
Procedure.i ChangeObject(*Object.RegisteredObject = #Null)
  If *Object <> Editor\CurrentObject
    If Editor\CurrentObject And ReleaseRegObject(Editor\CurrentObject)
      Editor\CurrentObject = #Null
    EndIf
    
    If *Object
      Editor\CurrentObject = *Object
      UseRegObject(*Object)
    EndIf
  EndIf
  
  ProcedureReturn Editor\CurrentObject
EndProcedure
Procedure.i LoadObjectViewSprite(*Object.ObjectEntry)
  Protected Sprite.i, Name.s, *ViewRegion.ViewRegion
  
  Name        = ObjectViewSpriteFilename(*Object)
  *ViewRegion = _Call(*Object\Callback, #Null, #Object_ViewRegion, #Null)
  
  If *ViewRegion
    Sprite = LoadClippedSprite(GetFile(Name), *ViewRegion\X, *ViewRegion\Y, *ViewRegion\Width, *ViewRegion\Height)
  Else
    Sprite = LoadSprite(#PB_Any, GetFile(Name), #PB_Sprite_AlphaBlending|#PB_Sprite_Texture|#PB_Sprite_Memory)
  EndIf
  If Not Sprite
    MessageRequester("Error", "Can not load graphic: " + #Q + Name + #Q, #MB_ICONERROR)
    ProcedureReturn #False
  EndIf
  
  ProcedureReturn Sprite
EndProcedure

;}
CompilerEndSelect

Procedure.i GetBank(Name.s)
  Protected *Item.QueueItem, *Bank.Bank
  
  *Item = *Banks\First
  While *Item
    *Bank = *Item\Memory
    If *Bank\Name = Name
      ProcedureReturn *Bank
    EndIf
    _Move(*Item, N)
  Wend
  
  ProcedureReturn #Null
EndProcedure 

; IDE Options = PureBasic 4.51 (Windows - x86)
; CursorPosition = 153
; FirstLine = 39
; Folding = DgDAAg
; EnableXP