IncludeFile "..\Import.pbi"
IncludeFile "Templates\Walker.pb"



ProcedureDLL.i GM_Build()
  DataSection
    Entries:
      GM_Folder("Animals")
        GM_Folder("Flies")
          GM_File("Firefly1")
          GM_File("Firefly2")
          GM_File("Firefly3")
          GM_End()
        GM_Folder("Bottom")
          GM_File("Stony")
          GM_End()
        GM_Folder("Misc")
          GM_File("WhiteEyes")
          GM_End()
        GM_End()
      GM_End()
  EndDataSection
  
  ProcedureReturn ?Entries
EndProcedure



Procedure.f RandomSgn(Max.i)
  If Random(1)
    ProcedureReturn 1 + Random(Max - 1)
  Else
    ProcedureReturn -(1 + Random(Max - 1))
  EndIf
EndProcedure



;******************************************
; Firefly
;******************************************

#Firefly_Acceleration = 0.1
#Firefly_MaxSpeed     = 0.8

Structure FireflyParameter
  Name.s
  File.s
  Size.i
  Sprite.i
  Sprite3D.i
  
  Initialized.i
EndStructure
Structure FireflyMemory
  MoveX.f
  MoveY.f
  TransparencyAngle.f
  ZoneX.i
  ZoneY.i
EndStructure

Procedure.i FireflyTemplate(*Object.GM_Object, Event.i, *Parameter, *FireflyParameter.FireflyParameter)
  Protected *EditorObject.GM_EditorObject, *Memory.FireflyMemory
  
  If *Object
    *Memory = *Object\Memory
  EndIf
  
  Select Event
    Case #GM_Object_Display
      GM_BlendingMode(5, 2)
      GM_DisplaySprite3D(*FireflyParameter\Sprite3D, *Object\X, *Object\Y, Abs(Sin(*Memory\TransparencyAngle) * 255.0))
      GM_BlendingMode(5, 6)
    Case #GM_Object_Frame
      If Random(150) = 0
        *Memory\MoveX = (#Firefly_MaxSpeed - Random(#Firefly_MaxSpeed * 2))
      EndIf
      If Random(150) = 0
        *Memory\MoveY = (#Firefly_MaxSpeed - Random(#Firefly_MaxSpeed * 2))
      EndIf
      If Random(10) = 0
        *Memory\MoveX + #Firefly_Acceleration * RandomSgn(1)
        If *Memory\MoveX > #Firefly_MaxSpeed
          *Memory\MoveX = #Firefly_MaxSpeed
        ElseIf *Memory\MoveX < -#Firefly_MaxSpeed
          *Memory\MoveX = -#Firefly_MaxSpeed
        EndIf
      EndIf
      If Random(10) = 0
        *Memory\MoveY + #Firefly_Acceleration * RandomSgn(1)
        If *Memory\MoveY > #Firefly_MaxSpeed
          *Memory\MoveY = #Firefly_MaxSpeed
        ElseIf *Memory\MoveY < -#Firefly_MaxSpeed
          *Memory\MoveY = -#Firefly_MaxSpeed
        EndIf
      EndIf
      *Object\X + *Memory\MoveX
      *Object\Y + *Memory\MoveY
      
      *Memory\TransparencyAngle + 0.0001 + Random(200)/1000.0
      If *Memory\TransparencyAngle => 2 * #PI
        *Memory\TransparencyAngle = 0
      EndIf
      
      ; Kollision mit einer Wand beachten:
      If GM_FieldCollision(*Object, *Memory\MoveX, *Memory\MoveY, -1, -1) Or GM_FieldCollision(*Object, *Memory\MoveX, *Memory\MoveY, 1, 1) Or GM_FieldCollision(*Object, *Memory\MoveX, *Memory\MoveY, -1, 1) Or GM_FieldCollision(*Object, *Memory\MoveX, *Memory\MoveY, 1, -1)
        *Memory\MoveX = -*Memory\MoveX
        *Memory\MoveY = -*Memory\MoveY
      Else
        If GM_FieldCollision(*Object, *Memory\MoveX, *Memory\MoveY, -1, 0) Or GM_FieldCollision(*Object, *Memory\MoveX, *Memory\MoveY, 1, 0)
          *Memory\MoveX = -*Memory\MoveX
        EndIf
        If GM_FieldCollision(*Object, *Memory\MoveX, *Memory\MoveY, 0, -1) Or GM_FieldCollision(*Object, *Memory\MoveX, *Memory\MoveY, 0, 1)
          *Memory\MoveY = -*Memory\MoveY
        EndIf
      EndIf
      
      ; Ein Glhwrmchen kann sich nur innerhalb eines 9x9 blocks befinden:
      If *Object\X < (*Memory\ZoneX - 3) * #GM_FieldSize
        *Object\X     = (*Memory\ZoneX - 3) * #GM_FieldSize
        *Memory\MoveX = -*Memory\MoveX
      ElseIf *Object\X + *Object\Width > (*Memory\ZoneX + 3) * #GM_FieldSize
        *Object\X     = (*Memory\ZoneX + 3) * #GM_FieldSize - *Object\Width
        *Memory\MoveX = -*Memory\MoveX
      EndIf
      If *Object\Y < (*Memory\ZoneY - 3) * #GM_FieldSize
        *Object\Y     = (*Memory\ZoneY - 3) * #GM_FieldSize
        *Memory\MoveY = -*Memory\MoveY
      ElseIf *Object\Y + *Object\Height > (*Memory\ZoneY + 3) * #GM_FieldSize
        *Object\Y     = (*Memory\ZoneY + 3) * #GM_FieldSize - *Object\Height
        *Memory\MoveY = -*Memory\MoveY
      EndIf
      
      ; Nach Grenzen der Szene prfen:
      If *Object\X < 0
        *Object\X = 0
        *Memory\MoveX = -*Memory\MoveX
      ElseIf *Object\X + *Object\Width > GM_ScenePixelWidth()
        *Object\X     = GM_ScenePixelWidth() - *Object\Width
        *Memory\MoveX = -*Memory\MoveX
      EndIf
      If *Object\Y < 0
        *Object\Y     = 0
        *Memory\MoveY = -*Memory\MoveY
      ElseIf *Object\Y + *Object\Height > GM_ScenePixelHeight()
        *Object\Y     = GM_ScenePixelHeight() - *Object\Height
        *Memory\MoveY = -*Memory\MoveY
      EndIf
    Case #GM_Object_Delete
      FreeMemory(*Memory)
    
    Case #GM_Object_Load
      GM_LoadPixelPosition(@*Object\X, @*Object\Y)
    Case #GM_Object_Initialize
      *Object\Width  = *FireflyParameter\Size
      *Object\Height = *FireflyParameter\Size
      
      *Memory        = AllocateMemory(SizeOf(FireflyMemory))
      *Object\Memory = *Memory
      *Memory\ZoneX  = *Object\X / #GM_FieldSize
      *Memory\ZoneY  = *Object\Y / #GM_FieldSize
    Case #GM_Object_GlobalLoad
      If Not *FireflyParameter\Sprite
        *FireflyParameter\Sprite   = GM_LoadSprite("Animals\" + *FireflyParameter\File)
        *FireflyParameter\Sprite3D = GM_CreateSprite3D(*FireflyParameter\Sprite)
      EndIf
      
    Case #GM_Object_EditorSave
      *EditorObject = *Parameter
      GM_SaveFieldPosition(*EditorObject\X, *EditorObject\Y)
    Case #GM_Object_EditorLoad
      *EditorObject = *Parameter
      GM_LoadFieldPosition(@*EditorObject\X, @*EditorObject\Y)
      
    Case #GM_Object_ViewSprite
      String = "Animals\" + *FireflyParameter\File + "-View"
      ProcedureReturn @String
    Case #GM_Object_PreferedLayer
      ProcedureReturn #GM_Layer_BackgroundObject2
    Case #GM_Object_CheckLayer
      If *Parameter <> #GM_Layer_StaticObject
        ProcedureReturn #True
      Else
        ProcedureReturn #False
      EndIf
    Case #GM_Object_Name
      ProcedureReturn @*FireflyParameter\Name
  EndSelect
  
  ProcedureReturn #Null
EndProcedure

ProcedureDLL.i Obj_Firefly1(*Object.GM_Object, Event.i, *Parameter)
  Global Firefly1.FireflyParameter
  
  With Firefly1
    If Not \Initialized
      \Name = "White Firefly"
      \File = "Firefly1"
      \Size = 3
      
      \Initialized = #True
    EndIf
  EndWith
  
  ProcedureReturn FireflyTemplate(*Object, Event, *Parameter, @Firefly1)
EndProcedure
ProcedureDLL.i Obj_Firefly2(*Object.GM_Object, Event.i, *Parameter)
  Global Firefly2.FireflyParameter
  
  With Firefly2
    If Not \Initialized
      \Name = "Red Firefly"
      \File = "Firefly2"
      \Size = 3
      
      \Initialized = #True
    EndIf
  EndWith
  
  ProcedureReturn FireflyTemplate(*Object, Event, *Parameter, @Firefly2)
EndProcedure
ProcedureDLL.i Obj_Firefly3(*Object.GM_Object, Event.i, *Parameter)
  Global Firefly3.FireflyParameter
  
  With Firefly3
    If Not \Initialized
      \Name = "Blue Firefly"
      \File = "Firefly3"
      \Size = 3
      
      \Initialized = #True
    EndIf
  EndWith
  
  ProcedureReturn FireflyTemplate(*Object, Event, *Parameter, @Firefly3)
EndProcedure



;******************************************
; White Eyes
;******************************************

Global WhiteEyes_Frameset.i

ProcedureDLL.i Obj_WhiteEyes(*Object.GM_Object, Event.i, *Parameter)
  If Not *Object
    *EditorObject = *Parameter
  EndIf
  
  Select Event
    Case #GM_Object_Display
      GM_DisplayAnimation(*Object\Animation, *Object\X, *Object\Y)
    Case #GM_Object_Frame
      If *Object\Memory = 0
        If GM_PlayAnimation(*Object\Animation)
          If GM_AnimationFrame(*Object\Animation) = 0
            GM_SetAnimationFrame(*Object\Animation, 0)
            *Object\Memory = 30 + Random(300)
          Else
            GM_SetAnimationDuration(*Object\Animation, 2)
          EndIf
        EndIf
      Else
        *Object\Memory - 1
      EndIf
    
    Case #GM_Object_Initialize
      *Object\Width  = #GM_FieldSize
      *Object\Height = #GM_FieldSize
      
      *Object\Animation = GM_CreateAnimation(WhiteEyes_Frameset)
      *Object\Memory    = Random(330)
    Case #GM_Object_Load
      GM_LoadPixelPosition(@*Object\X, @*Object\Y)
    Case #GM_Object_GlobalLoad
      If Not WhiteEyes_Frameset
        WhiteEyes_Frameset = GM_LoadAnimationFrameset("Animals\WhiteEyes", #GM_FieldSize, #GM_FieldSize)
      EndIf
    
    Case #GM_Object_EditorSave
      GM_SaveFieldPosition(*EditorObject\X, *EditorObject\Y)
    Case #GM_Object_EditorLoad
      GM_LoadFieldPosition(@*EditorObject\X, @*EditorObject\Y)
    
    Case #GM_Object_PreferedLayer
      ProcedureReturn #GM_Layer_BackgroundObject2
    Case #GM_Object_CheckLayer
      If *Parameter <> #GM_Layer_StaticObject
        ProcedureReturn #True
      Else
        ProcedureReturn #False
      EndIf
    Case #GM_Object_ViewSprite
      ProcedureReturn @"Animals\WhiteEyes"
    Case #GM_Object_ViewRegion
      ViewRegion\X      = 0
      ViewRegion\Y      = 0
      ViewRegion\Width  = #GM_FieldSize
      ViewRegion\Height = #GM_FieldSize
      ProcedureReturn @ViewRegion
    Case #GM_Object_Name
      ProcedureReturn @"White Eyes"
  EndSelect
  
  ProcedureReturn #Null
EndProcedure



;******************************************
; Stony
;******************************************

Procedure.i Stony_Standing(*Object.GM_Object, *Memory, *Walker.WalkerParameter)
  If Random(80) = 0
    StartWalking(*Object, 0.5)
    GM_SetAnimationFrame(*Object\Animation, 0)
    GM_SetAnimationDuration(*Object\Animation, 2)
  EndIf
EndProcedure
Procedure.i Stony_Walking(*Object.GM_Object, *Memory, *Walker.WalkerParameter)
  If GM_AnimationFrame(*Object\Animation) = 4 And Random(5) = 0
    StopWalking(*Object)
  EndIf
EndProcedure
ProcedureDLL.i Obj_Stony(*Object.GM_Object, Event.i, *Parameter)
  Global Stony.WalkerParameter
  
  With Stony
    If Not \Initialized
      \Name = "Stony"
      \File = "Animals\Stony"
      
      \Width  = 7
      \Height = 9
      
      \Destroyable      = #False
      \StandingCallback = @Stony_Standing()
      \WalkingCallback  = @Stony_Walking()
      \WalkingSpeed     = 2
      \StandingSpeed    = 20
      
      \Initialized = #True
    EndIf
  EndWith
  
  ProcedureReturn WalkerTemplate(*Object, Event, *Parameter, @Stony)
EndProcedure
; IDE Options = PureBasic 4.51 (Windows - x86)
; CursorPosition = 321
; FirstLine = 54
; Folding = B5-
; EnableXP