IncludeFile "..\Import.pbi"
IncludeFile "Templates\Item.pb"



ProcedureDLL.i GM_Build()
  DataSection
    Entries:
      GM_Folder("Items")
        GM_Folder("Keys")
          GM_File("GoldenKey")
          GM_File("SilverKey")
          GM_File("BronzeKey")
          GM_File("RainbowKey")
          GM_End()
        GM_End()
      GM_Folder("Doors")
        GM_Folder("Key")
          GM_File("GoldenKeyDoor")
          GM_File("SilverKeyDoor")
          GM_File("BronzeKeyDoor")
          GM_End()
        GM_Folder("Controllable")
          GM_File("Gate1")
          GM_File("Gate2")
          GM_File("Gate3")
          GM_End()
        GM_End()
      GM_End()
  EndDataSection
  
  ProcedureReturn ?Entries
EndProcedure



Structure KeyDoorParameter
  Name.s
  File.s
  
  Frameset.i
  
  Initialized.i
EndStructure

ProcedureDLL.i KeyDoorTemplate(*Object.GM_Object, Event.i, *Parameter, *KeyDoorParameter.KeyDoorParameter)
  If Not *Object
    *EditorObject = *Parameter
  EndIf
  
  Select Event
    Case #GM_Object_Display
      If *Object\Memory < 2
        GM_DisplayAnimation(*Object\Animation, *Object\X, *Object\Y)
      EndIf
    Case #GM_Object_Frame
      If *Object\Memory = 1
        If GM_PlayAnimation(*Object\Animation)
          If GM_AnimationFrame(*Object\Animation) = 3
            *Object\Memory = 2
          Else
            GM_SetAnimationDuration(*Object\Animation, 4)
          EndIf
        EndIf
      EndIf
    Case #GM_Object_Collision
      If *Object\Memory = 0
        GM_DoWallCollision(*Object, *Parameter)
      EndIf
      
    Case #GM_Object_Initialize
      *Object\Width     = #GM_FieldSize
      *Object\Height    = #GM_FieldSize
      *Object\Animation = GM_CreateAnimation(*KeyDoorParameter\Frameset)
      
      *Object\Memory = 0
    Case #GM_Object_Load
      GM_LoadPixelPosition(@*Object\X, @*Object\Y)
    Case #GM_Object_GlobalLoad
      If Not *KeyDoorParameter\Frameset
        *KeyDoorParameter\Frameset = GM_LoadAnimationFrameset(*KeyDoorParameter\File, #GM_FieldSize, #GM_FieldSize)
      EndIf
    
    Case #GM_Object_EditorSave
      GM_SaveFieldPosition(*EditorObject\X, *EditorObject\Y)
    Case #GM_Object_EditorLoad
      GM_LoadFieldPosition(@*EditorObject\X, @*EditorObject\Y)
    
    Case #GM_Object_PreferedLayer
      ProcedureReturn #GM_Layer_StaticObject
    Case #GM_Object_CheckLayer
      If *Parameter = #GM_Layer_StaticObject
        ProcedureReturn #True
      Else
        ProcedureReturn #False
      EndIf
    Case #GM_Object_ViewSprite
      ProcedureReturn @*KeyDoorParameter\File
    Case #GM_Object_ViewRegion
      ReturnViewRegion(0, 0, 1, 1)
    Case #GM_Object_Name
      ProcedureReturn @*KeyDoorParameter\Name
      
    Case -1
      If *Object\Memory = 0
        *Object\Memory = 1
        GM_SetNextAnimationFrame(*Object\Animation, 1, 4)
        ProcedureReturn #True
      EndIf
  EndSelect
  
  ProcedureReturn #Null
EndProcedure

ProcedureDLL.i Obj_GoldenKeyDoor(*Object.GM_Object, Event.i, *Parameter)
  Global GoldenKeyDoor.KeyDoorParameter
  
  With GoldenKeyDoor
    If Not \Initialized
      \Name = "Golden Key Door"
      \File = "Doors\GoldenKeyDoor"
      
      \Initialized = #True
    EndIf
  EndWith
  
  ProcedureReturn KeyDoorTemplate(*Object, Event, *Parameter, @GoldenKeyDoor)
EndProcedure
ProcedureDLL.i Obj_SilverKeyDoor(*Object.GM_Object, Event.i, *Parameter)
  Global SilverKeyDoor.KeyDoorParameter
  
  With SilverKeyDoor
    If Not \Initialized
      \Name = "Silver Key Door"
      \File = "Doors\SilverKeyDoor"
      
      \Initialized = #True
    EndIf
  EndWith
  
  ProcedureReturn KeyDoorTemplate(*Object, Event, *Parameter, @SilverKeyDoor)
EndProcedure
ProcedureDLL.i Obj_BronzeKeyDoor(*Object.GM_Object, Event.i, *Parameter)
  Global BronzeKeyDoor.KeyDoorParameter
  
  With BronzeKeyDoor
    If Not \Initialized
      \Name = "Bronze Key Door"
      \File = "Doors\BronzeKeyDoor"
      
      \Initialized = #True
    EndIf
  EndWith
  
  ProcedureReturn KeyDoorTemplate(*Object, Event, *Parameter, @BronzeKeyDoor)
EndProcedure

Procedure.i UseKey(*Callback)
  Protected *Object.GM_Object
  
  If *GM_Player\FieldX > 0
    *Object = GM_GetObject(*GM_Player\FieldX - 1, *GM_Player\FieldY)
    If *Object And *Object\Callback = *Callback
      ProcedureReturn CallFunctionFast(*Callback, *Object, -1, #Null)
    EndIf
  EndIf
  If *GM_Player\FieldX < GM_SceneWidth() - 1
    *Object = GM_GetObject(*GM_Player\FieldX + 1, *GM_Player\FieldY)
    If *Object And *Object\Callback = *Callback
      ProcedureReturn CallFunctionFast(*Callback, *Object, -1, #Null)
    EndIf
  EndIf
  If *GM_Player\FieldY > 0
    *Object = GM_GetObject(*GM_Player\FieldX, *GM_Player\FieldY - 1)
    If *Object And *Object\Callback = *Callback
      ProcedureReturn CallFunctionFast(*Callback, *Object, -1, #Null)
    EndIf
  EndIf
  If *GM_Player\FieldY < GM_SceneHeight() - 1
    *Object = GM_GetObject(*GM_Player\FieldX, *GM_Player\FieldY + 1)
    If *Object And *Object\Callback = *Callback
      ProcedureReturn CallFunctionFast(*Callback, *Object, -1, #Null)
    EndIf
  EndIf
  
  ProcedureReturn #Null
EndProcedure

Procedure.i Item_GoldenKey(*Item.GM_Item, Event.i, *Parameter)
  Static *Object.GM_Object
  
  Select Event
    Case #GM_Item_Use
      ProcedureReturn UseKey(@Obj_GoldenKeyDoor())
    Case #GM_Item_IsUseable
      ProcedureReturn #True
    Case #GM_Item_NameSingular
      ProcedureReturn @"Golden Key"
    Case #GM_Item_NamePlural
      ProcedureReturn @"Golden Keys"
  EndSelect
  
  ProcedureReturn #Null
EndProcedure
ProcedureDLL.i Obj_GoldenKey(*Object.GM_Object, Event.i, *Parameter)
  Global GoldenKey.ItemParameter
  
  With GoldenKey
    If Not \Initialized
      \Name        = "Golden Key"
      \File        = "..\Items\Keys\Gold"
      \EnableCount = #False
      
      \ItemName     = "Keys\Gold"
      \ItemCallback = @Item_GoldenKey()
      
      \Width  = 12
      \Height = 12
      
      \Initialized = #True
    EndIf
  EndWith
  
  ProcedureReturn ItemTemplate(*Object, Event, *Parameter, @GoldenKey)
EndProcedure
Procedure.i Item_SilverKey(*Item.GM_Item, Event.i, *Parameter)
  Static *Object.GM_Object
  
  Select Event
    Case #GM_Item_Use
      ProcedureReturn UseKey(@Obj_SilverKeyDoor())
    Case #GM_Item_IsUseable
      ProcedureReturn #True
    Case #GM_Item_NameSingular
      ProcedureReturn @"Silver Key"
    Case #GM_Item_NamePlural
      ProcedureReturn @"Silver Keys"
  EndSelect
  
  ProcedureReturn #Null
EndProcedure
ProcedureDLL.i Obj_SilverKey(*Object.GM_Object, Event.i, *Parameter)
  Global SilverKey.ItemParameter
  
  With SilverKey
    If Not \Initialized
      \Name        = "Silver Key"
      \File        = "..\Items\Keys\Silver"
      \EnableCount = #False
      
      \ItemName     = "Keys\Silver"
      \ItemCallback = @Item_SilverKey()
      
      \Width  = 12
      \Height = 12
      
      \Initialized = #True
    EndIf
  EndWith
  
  ProcedureReturn ItemTemplate(*Object, Event, *Parameter, @SilverKey)
EndProcedure
Procedure.i Item_BronzeKey(*Item.GM_Item, Event.i, *Parameter)
  Static *Object.GM_Object
  
  Select Event
    Case #GM_Item_Use
      ProcedureReturn UseKey(@Obj_BronzeKeyDoor())
    Case #GM_Item_IsUseable
      ProcedureReturn #True
    Case #GM_Item_NameSingular
      ProcedureReturn @"Golden Key"
    Case #GM_Item_NamePlural
      ProcedureReturn @"Golden Keys"
  EndSelect
  
  ProcedureReturn #Null
EndProcedure
ProcedureDLL.i Obj_BronzeKey(*Object.GM_Object, Event.i, *Parameter)
  Global BronzeKey.ItemParameter
  
  With BronzeKey
    If Not \Initialized
      \Name        = "Bronze Key"
      \File        = "..\Items\Keys\Bronze"
      \EnableCount = #False
      
      \ItemName     = "Keys\Bronze"
      \ItemCallback = @Item_BronzeKey()
      
      \Width  = 12
      \Height = 12
      
      \Initialized = #True
    EndIf
  EndWith
  
  ProcedureReturn ItemTemplate(*Object, Event, *Parameter, @BronzeKey)
EndProcedure
Procedure.i Item_RainbowKey(*Item.GM_Item, Event.i, *Parameter)
  Static *Object.GM_Object
  
  Select Event
    Case #GM_Item_Use
      ; Der Regenbogen-Schlssel kann fr jede Tr verwendet werden:
      If UseKey(@Obj_GoldenKeyDoor()) Or UseKey(@Obj_SilverKeyDoor()) Or UseKey(@Obj_BronzeKeyDoor())
        ProcedureReturn #True
      Else
        ProcedureReturn #False
      EndIf
    Case #GM_Item_IsUseable
      ProcedureReturn #True
    Case #GM_Item_NameSingular
      ProcedureReturn @"Rainbow Key"
    Case #GM_Item_NamePlural
      ProcedureReturn @"Rainbow Keys"
  EndSelect
  
  ProcedureReturn #Null
EndProcedure
ProcedureDLL.i Obj_RainbowKey(*Object.GM_Object, Event.i, *Parameter)
  Global RainbowKey.ItemParameter
  
  With RainbowKey
    If Not \Initialized
      \Name        = "Rainbow Key"
      \File        = "..\Items\Keys\Rainbow"
      \EnableCount = #False
      
      \ItemName     = "Keys\Rainbow"
      \ItemCallback = @Item_RainbowKey()
      
      \Width  = 12
      \Height = 12
      
      \Initialized = #True
    EndIf
  EndWith
  
  ProcedureReturn ItemTemplate(*Object, Event, *Parameter, @RainbowKey)
EndProcedure



Enumeration
  #Gate_Action_Closed
  #Gate_Action_Opened
  #Gate_Action_Opening
  #Gate_Action_Closing
EndEnumeration

Structure GateParameter
  File.s
  Name.s
  
  Move.f
  AnimationSpeed.i
  
  Frameset.i
  
  Initialized.i
EndStructure
Structure GateMemory
  OriginY.f
  Action.i
  Message.s
EndStructure

Procedure.i GateTemplate(*Object.GM_Object, Event.i, *Parameter, *GateParameter.GateParameter)
  Static *Memory.GateMemory, Collision.i
  
  If *Object
    *Memory = *Object\Memory
  EndIf
  
  Select Event
    Case #GM_Object_Display
      Select *Memory\Action
        Case #Gate_Action_Closed, #Gate_Action_Opening, #Gate_Action_Closing
          GM_DisplayAnimation(*Object\Animation, *Object\X, *Object\Y)
      EndSelect
    Case #GM_Object_Frame
      Select *Memory\Action
        Case #Gate_Action_Closed
          GM_SetAnimationFrame(*Object\Animation, 0)
        Case #Gate_Action_Opened
          GM_SetAnimationFrame(*Object\Animation, GM_FramesetFrames(*GateParameter\Frameset) - 1)
        Case #Gate_Action_Opening
          If *GateParameter\Move
            *Object\Y - *GateParameter\Move
            If *Object\Y = *Memory\OriginY - #GM_FieldSize
              *Memory\Action = #Gate_Action_Opened
            EndIf
          Else
            If GM_PlayAnimation(*Object\Animation)
              If GM_AnimationFrame(*Object\Animation) = 0
                GM_SetAnimationFrame(*Object\Animation, GM_FramesetFrames(*GateParameter\Frameset) - 1)
                *Memory\Action = #Gate_Action_Opened
              EndIf
              GM_SetAnimationDuration(*Object\Animation, *GateParameter\AnimationSpeed)
            EndIf
          EndIf
        Case #Gate_Action_Closing
          If *GateParameter\Move
            *Object\Y + *GateParameter\Move
            If *Object\Y = *Memory\OriginY
              *Memory\Action = #Gate_Action_Closed
            EndIf
          Else
            If GM_PlayAnimationBack(*Object\Animation)
              If GM_AnimationFrame(*Object\Animation) = GM_FramesetFrames(*GateParameter\Frameset) - 1
                GM_SetAnimationFrame(*Object\Animation, 0)
                *Memory\Action = #Gate_Action_Closed
              EndIf
              GM_SetAnimationDuration(*Object\Animation, *GateParameter\AnimationSpeed)
            EndIf
          EndIf
      EndSelect
    Case #GM_Object_Collision
      If *Memory\Action = #Gate_Action_Closed Or *Memory\Action = #Gate_Action_Closing
        Collision = GM_PlayerCollision(*Object, 0, 0)
        If *GateParameter\Move And Collision = #GM_Collision_Bottom And *GM_Player\Action = #GM_Player_Action_Standing
          ;Spieler wird erdrckt:
          GM_LoseLife()
        Else
          GM_DoWallCollision(*Object, Collision)
          If *Memory\Message
            GM_ShowMessage("Closed", *Memory\Message)
          EndIf
        EndIf
      EndIf
    
    Case #GM_Object_Initialize
      *Object\Width  = #GM_FieldSize
      *Object\Height = #GM_FieldSize
      
      *Object\Animation = GM_CreateAnimation(*GateParameter\Frameset)
      
      *Memory\Action  = #Gate_Action_Closed
      *Memory\OriginY = *Object\Y
    Case #GM_Object_Load
      *Memory        = AllocateMemory(SizeOf(GateMemory))
      *Object\Memory = *Memory
      
      GM_LoadPixelPosition(@*Object\X, @*Object\Y)
      *Memory\Message = GM_LoadText()
    Case #GM_Object_GlobalLoad
      If Not *GateParameter\Frameset
        *GateParameter\Frameset = GM_LoadAnimationFrameset(*GateParameter\File, #GM_FieldSize, #GM_FieldSize)
      EndIf
      
    Case #GM_Object_EditorSave
      *EditorObject = *Parameter
      *Memory       = *EditorObject\Memory
      GM_SaveFieldPosition(*EditorObject\X, *EditorObject\Y)
      GM_SaveText(*Memory\Message)
    Case #GM_Object_EditorLoad
      *EditorObject = *Parameter
      *Memory              = AllocateMemory(SizeOf(GateMemory))
      *EditorObject\Memory = *Memory
      
      GM_LoadFieldPosition(@*EditorObject\X, @*EditorObject\Y)
      *Memory\Message = GM_LoadText()
    Case #GM_Object_EditorInitialize
      *EditorObject = *Parameter
      If Not *EditorObject\Memory
        *EditorObject\Memory = AllocateMemory(SizeOf(GateMemory))
      EndIf
    Case #GM_Object_EditorConfigure
      *EditorObject = *Parameter
      *Memory       = *EditorObject\Memory
      
      *Memory\Message = GM_EditMessage("Info when closed", *Memory\Message)
    Case #GM_Object_EditorFree
      *EditorObject = *Parameter
      ClearStructure(*EditorObject\Memory, GateMemory)
      FreeMemory(*EditorObject\Memory)
    
    Case #GM_Object_CallAction
      Select GM_ObjectActionID(*Parameter)
        Case 0 ;Open
          *Memory\Action = #Gate_Action_Opening
        Case 1 ;Close
          *Memory\Action = #Gate_Action_Closing
      EndSelect
    Case #GM_Object_ActionName
      Select *Parameter
        Case 0
          ProcedureReturn @"Open"
        Case 1
          ProcedureReturn @"Close"
      EndSelect
    Case #GM_Object_ActionCount
      ProcedureReturn 2
    
    Case #GM_Object_PreferedLayer
      ProcedureReturn #GM_Layer_BackgroundObject2
    Case #GM_Object_CheckLayer
      If *Parameter = #GM_Layer_BackgroundObject2
        ProcedureReturn #True
      Else
        ProcedureReturn #False
      EndIf
    Case #GM_Object_ViewSprite
      ProcedureReturn @*GateParameter\File
    Case #GM_Object_ViewRegion
      ReturnViewRegion(0, 0, 1, 1)
    Case #GM_Object_Name
      ProcedureReturn @*GateParameter\Name
  EndSelect
  
  ProcedureReturn #Null
EndProcedure

ProcedureDLL.i Obj_Gate1(*Object.GM_Object, Event.i, *Parameter)
  Global Gate1.GateParameter
  
  With Gate1
    If Not \Initialized
      \Name = "Gate"
      \File = "Doors\Gate"
      
      \Move = 1
      
      \Initialized = #True
    EndIf
  EndWith
  
  ProcedureReturn GateTemplate(*Object, Event, *Parameter, @Gate1)
EndProcedure
ProcedureDLL.i Obj_Gate2(*Object.GM_Object, Event.i, *Parameter)
  Global Gate2.GateParameter
  
  With Gate2
    If Not \Initialized
      \Name = "Stone Gate"
      \File = "Doors\StoneGate"
      
      \Move = 0.25
      
      \Initialized = #True
    EndIf
  EndWith
  
  ProcedureReturn GateTemplate(*Object, Event, *Parameter, @Gate2)
EndProcedure
ProcedureDLL.i Obj_Gate3(*Object.GM_Object, Event.i, *Parameter)
  Global Gate3.GateParameter
  
  With Gate3
    If Not \Initialized
      \Name = "Metal Gate"
      \File = "Doors\MetalGate"
      
      \Move = 2
      
      \Initialized = #True
    EndIf
  EndWith
  
  ProcedureReturn GateTemplate(*Object, Event, *Parameter, @Gate3)
EndProcedure
; IDE Options = PureBasic 4.51 (Windows - x86)
; CursorPosition = 521
; FirstLine = 65
; Folding = BAA-
; EnableXP