;**************************************************************
;     Objects\Liquid
;**************************************************************
;   Copyright (c) 2010 Josef Sniatecki
;
; This program is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License
; As published by the Free Software Foundation; either version
; 3 of the License, Or (at your option) any later version.
;
; This program is distributed in the hope that it will be
; useful, but WITHOUT ANY WARRANTY; without even the implied
; warranty of MERCHANTABILITY Or FITNESS For A PARTICULAR
; PURPOSE. See the GNU General Public License For more details.
;
; You should have received a copy of the GNU General Public
; License along With this program;
; If Not, see "http://www.gnu.org/licenses/".
;**************************************************************



EnableExplicit



IncludeFile "..\Import.pbi"
IncludeFile "Templates\Emitter.pb"



ProcedureDLL.i GM_Build()
  DataSection
    Entries:
      GM_Folder("Liquid")
        GM_Folder("Water")
          GM_File("WS1")
          GM_File("W1")
          GM_File("WS2")
          GM_File("W2")
          GM_File("WS3")
          GM_File("W3")
          GM_File("WS4")
          GM_File("W4")
          GM_End()
        GM_Folder("Waterfalls")
          GM_File("WF1")
          GM_File("WFT1")
          GM_End()
        GM_Folder("Dangerous")
          GM_File("Flood1")
          GM_File("Flood4")
          GM_File("FireBallEmitter")
          GM_End()
        GM_End()
      GM_Folder("Nature")
        GM_File("Rain")
        GM_File("Rainbow")
        GM_End()
      GM_End()
  EndDataSection
  
  ProcedureReturn ?Entries
EndProcedure



;******************************************
; Water
;******************************************

IncludeFile "Liquid\Water.pb"



;******************************************
; Waterfalls
;******************************************

#Waterfall_Duration = 1
#Waterfall_Frames   = 6

Structure WaterfallParameter
  Name.s
  File.s
  Frameset.i
  Initialized.i
EndStructure

Procedure.i WaterfallTemplate(*Object.GM_Object, Event.i, *Parameter, *WaterfallParameter.WaterfallParameter)
  Select Event
    Case #GM_Object_Display
      GM_DisplayFramesetFrame(*WaterfallParameter\Frameset, *Object\Memory / #Waterfall_Duration, 0, *Object\X, *Object\Y)
    Case #GM_Object_Frame
      If *Object\Memory = #Waterfall_Duration * #Waterfall_Frames - 1
        *Object\Memory = 0
      Else
        *Object\Memory + 1
      EndIf
      
    Case #GM_Object_Initialize
      *Object\Width  = #GM_FieldSize
      *Object\Height = #GM_FieldSize
      *Object\Memory = 0
    Case #GM_Object_Load
      GM_LoadPixelPosition(@*Object\X, @*Object\Y)
    Case #GM_Object_GlobalLoad
      If Not *WaterfallParameter\Frameset
        *WaterfallParameter\Frameset = GM_LoadAnimationFrameset("Liquid\" + *WaterfallParameter\File, #GM_FieldSize, #GM_FieldSize)
      EndIf
      
    Case #GM_Object_EditorLoad
      *EditorObject = *Parameter
      GM_LoadFieldPosition(@*EditorObject\X, @*EditorObject\Y)
    Case #GM_Object_EditorSave
      *EditorObject = *Parameter
      GM_SaveFieldPosition(*EditorObject\X, *EditorObject\Y)
    
    Case #GM_Object_Name
      ProcedureReturn @*WaterfallParameter\Name
    Case #GM_Object_ViewSprite
      String = "Liquid\" + *WaterfallParameter\File
      ProcedureReturn @String
    Case #GM_Object_ViewRegion
      ViewRegion\X      = 0
      ViewRegion\Y      = 0
      ViewRegion\Width  = #GM_FieldSize
      ViewRegion\Height = #GM_FieldSize
      ProcedureReturn @ViewRegion
    Case #GM_Object_PreferedLayer
      ProcedureReturn #GM_Layer_BackgroundObject2
    Case #GM_Object_CheckLayer
      If Not *Parameter = #GM_Layer_StaticObject
        ProcedureReturn #True
      Else
        ProcedureReturn #False
      EndIf
  EndSelect
  
  ProcedureReturn #Null
EndProcedure

ProcedureDLL.i Obj_WF1(*Object.GM_Object, Event.i, *Parameter)
  Global WF1.WaterfallParameter
  
  With WF1
    If Not \Initialized
      \Name = "Waterfall (blue)"
      \File = "Fall1"
      
      \Initialized = #True
    EndIf
  EndWith
  
  ProcedureReturn WaterfallTemplate(*Object, Event, *Parameter, @WF1)
EndProcedure



#WaterfallTop_Duration = 5
#WaterfallTop_Frames   = 3

Structure WaterfallTopParameter
  Name.s
  File.s
  Frameset.i
  Initialized.i
EndStructure

Procedure.i WaterfallTopTemplate(*Object.GM_Object, Event.i, *Parameter, *WaterfallTopParameter.WaterfallParameter)
  Select Event
    Case #GM_Object_Display
      GM_DisplayFramesetFrame(*WaterfallTopParameter\Frameset, *Object\Memory / #WaterfallTop_Duration, 0, *Object\X, *Object\Y)
    Case #GM_Object_Frame
      If *Object\Memory = #WaterfallTop_Duration * #WaterfallTop_Frames - 1
        *Object\Memory = 0
      Else
        *Object\Memory + 1
      EndIf
      
    Case #GM_Object_Initialize
      *Object\Width  = #GM_FieldSize
      *Object\Height = #GM_FieldSize
      *Object\Memory = Random(#WaterfallTop_Duration * #WaterfallTop_Frames - 1)
    Case #GM_Object_Load
      GM_LoadPixelPosition(@*Object\X, @*Object\Y)
    Case #GM_Object_GlobalLoad
      If Not *WaterfallTopParameter\Frameset
        *WaterfallTopParameter\Frameset = GM_LoadAnimationFrameset("Liquid\" + *WaterfallTopParameter\File, #GM_FieldSize, #GM_FieldSize)
      EndIf
      
    Case #GM_Object_EditorLoad
      *EditorObject = *Parameter
      GM_LoadFieldPosition(@*EditorObject\X, @*EditorObject\Y)
    Case #GM_Object_EditorSave
      *EditorObject = *Parameter
      GM_SaveFieldPosition(*EditorObject\X, *EditorObject\Y)
    
    Case #GM_Object_Name
      ProcedureReturn @*WaterfallTopParameter\Name
    Case #GM_Object_ViewSprite
      String = "Liquid\" + *WaterfallTopParameter\File
      ProcedureReturn @String
    Case #GM_Object_ViewRegion
      ViewRegion\X      = 0
      ViewRegion\Y      = 0
      ViewRegion\Width  = #GM_FieldSize
      ViewRegion\Height = #GM_FieldSize
      ProcedureReturn @ViewRegion
    Case #GM_Object_PreferedLayer
      ProcedureReturn #GM_Layer_BackgroundObject2
    Case #GM_Object_CheckLayer
      If Not *Parameter = #GM_Layer_StaticObject
        ProcedureReturn #True
      Else
        ProcedureReturn #False
      EndIf
  EndSelect
  
  ProcedureReturn #Null
EndProcedure

ProcedureDLL.i Obj_WFT1(*Object.GM_Object, Event.i, *Parameter)
  Global WFT1.WaterfallParameter
  
  With WFT1
    If Not \Initialized
      \Name = "Waterfall Top (blue)"
      \File = "FallTop1"
      
      \Initialized = #True
    EndIf
  EndWith
  
  ProcedureReturn WaterfallTopTemplate(*Object, Event, *Parameter, @WFT1)
EndProcedure



;******************************************
; Rain & Drops
;******************************************

IncludeFile "Liquid\Rain.pb"



;******************************************
; Fire balls
;******************************************

Global FireBall_Frameset.i

Procedure.i FireBall(*Object.GM_Object, Event.i, *Parameter)
  Select Event
    Case #GM_Object_Display
      GM_DisplayYellowLight(*Object\X + *Object\Width/2, *Object\Y + *Object\Height/2, *Object\Transparency - Random(*Object\Transparency/5.0))
      GM_DisplayAnimation(*Object\Animation, *Object\X, *Object\Y, *Object\Transparency)
    Case #GM_Object_Frame
      If *Object\Memory = 0
        ProcedureReturn #GM_Object_DoDelete
      EndIf
      
      *Object\Y - 0.1 * *Object\Memory
      If GM_FieldCollision(*Object, 0, -0.1 * *Object\Memory, 0, -1, #False)
        GM_EmitTinyExplosion(*Object\X + *Object\Width/2                  , *Object\Y + *Object\Height/2                  )
        GM_EmitTinyExplosion(*Object\X + *Object\Width/2 + 10 - Random(20), *Object\Y + *Object\Height/2 + 10 - Random(20))
        GM_EmitTinyExplosion(*Object\X + *Object\Width/2 + 10 - Random(20), *Object\Y + *Object\Height/2 + 10 - Random(20))
        ProcedureReturn #GM_Object_DoDelete
      EndIf
      
      If GM_PlayAnimation(*Object\Animation)
        GM_SetAnimationDuration(*Object\Animation, 3)
      EndIf
      If *Object\Memory < 10
        *Object\Transparency = 255.0/10.0 * *Object\Memory
      EndIf
      
      *Object\Memory - 1
    Case #GM_Object_Collision
      ; Schaden hinzufgen:
      GM_HitPlayer(4)
      ; Das Fliegen soll unterbrochen werden, damit der
      ; Spieler kurzzeitig am Fallen ist und schnell die
      ; Flugtaste wieder bettigen muss:
      If *GM_Player\Action = #GM_Player_Action_Flying
        GM_ChangePlayerAction(#GM_Player_Action_FlyingTruncated)
      EndIf
    Case #GM_Object_Delete
      GM_FreeAnimation(*Object\Animation)
      
    Case #GM_Object_Initialize
      *Object\Width     = 20
      *Object\Height    = 17
      *Object\Memory    = 50 + Random(30)
      *Object\Animation = GM_CreateAnimation(FireBall_Frameset)
  EndSelect
  
  ProcedureReturn #Null
EndProcedure
Procedure.i FireBallEmitter_Emit(*Object.GM_Object)
  GM_CreateObject(#GM_Layer_ForegroundObject1, @FireBall(), *Object\X + *Object\Width/2 - #GM_FieldSize/2, *Object\Y)
EndProcedure
ProcedureDLL.i Obj_FireBallEmitter(*Object.GM_Object, Event.i, *Parameter)
  Global FireBallEmitter.EmitterParameter
  
  With FireBallEmitter
    If Not \Initialized
      \Name               = "Fireball Emitter"
      \File               = "Liquid\FireBallEmitter"
      \MinDuration        = 180
      \AdditionalDuration = 30
      \EmitCallback       = @FireBallEmitter_Emit()
      
      \Initialized = #True
    EndIf
  EndWith
  
  If Event = #GM_Object_Display
    If EmitterTime(*Object) < 30
      GM_DisplayYellowLight(*Object\X + *Object\Width/2, *Object\Y + *Object\Height/2, 20 + Random(50))
    EndIf
  ElseIf Event = #GM_Object_GlobalLoad
    FireBall_Frameset = GM_LoadAnimationFrameset("Liquid\FireBall", 20, 30)
  EndIf
  
  ProcedureReturn EmitterTemplate(*Object, Event, *Parameter, @FireBallEmitter)
EndProcedure

; IDE Options = PureBasic 4.51 (Windows - x86)
; CursorPosition = 22
; FirstLine = 6
; Folding = l+
; EnableXP