#Pike_Size      = 9
#Pike_OuterSize = #GM_FieldSize - #Pike_Size
#Pike_OuterTop  = 2
#Pike_OuterSide = 3

#Pike_NoPeriod     = 3
#Pike_PeriodLength = 70
#Pike_Periods      = 3

Enumeration
  #Pike_Action_Stopped
  #Pike_Action_Extend
  #Pike_Action_Retract
EndEnumeration

Enumeration
  #Pike_Direction_Left
  #Pike_Direction_Right
  #Pike_Direction_Up
  #Pike_Direction_Down
EndEnumeration



Structure PikeParameter
  Name.s
  File.s
  
  Direction.i
  Period.i
  
  Sprite.i
  Sprite3D.i
  
  Initialized.i
EndStructure
Structure PikeMemory
  Action.i
  Time.i
  StructureUnion
    X.i
    Y.i
  EndStructureUnion
EndStructure



Procedure.i PikeTemplate(*Object.GM_Object, Event.i, *Parameter, *Pike.PikeParameter)
  Static *Memory.PikeMemory
  
  If *Object
    *Memory = *Object\Memory
  EndIf
  
  Select Event
    Case #GM_Object_Display
      GM_DisplaySprite3D(*Pike\Sprite3D, *Object\X, *Object\Y)
    Case #GM_Object_Frame
      Select *Memory\Action
        Case #Pike_Action_Extend
          Select *Pike\Direction
            Case #Pike_Direction_Left
              If *Object\X > *Memory\X - #Pike_Size
                *Object\X - 1
              Else
                *Memory\Action = #Pike_Action_Stopped
              EndIf
            Case #Pike_Direction_Right
              If *Object\X < *Memory\X + #Pike_Size
                *Object\X + 1
              Else
                *Memory\Action = #Pike_Action_Stopped
              EndIf
            Case #Pike_Direction_Up
              If *Object\Y > *Memory\Y - #Pike_Size
                *Object\Y - 1
              Else
                *Memory\Action = #Pike_Action_Stopped
              EndIf
            Case #Pike_Direction_Down
              If *Object\Y < *Memory\Y + #Pike_Size
                *Object\Y + 1
              Else
                *Memory\Action = #Pike_Action_Stopped
              EndIf
          EndSelect
        Case #Pike_Action_Retract
          Select *Pike\Direction
            Case #Pike_Direction_Left
              If *Object\X < *Memory\X
                *Object\X + 1
              Else
                *Memory\Action = #Pike_Action_Stopped
              EndIf
            Case #Pike_Direction_Right
              If *Object\X > *Memory\X
                *Object\X - 1
              Else
                *Memory\Action = #Pike_Action_Stopped
              EndIf
            Case #Pike_Direction_Up
              If *Object\Y < *Memory\Y
                *Object\Y + 1
              Else
                *Memory\Action = #Pike_Action_Stopped
              EndIf
            Case #Pike_Direction_Down
              If *Object\Y > *Memory\Y
                *Object\Y - 1
              Else
                *Memory\Action = #Pike_Action_Stopped
              EndIf
          EndSelect
      EndSelect
        
      If *Pike\Period <> #Pike_NoPeriod
        If *Memory\Time = *Pike\Period * #Pike_PeriodLength
          *Memory\Action = #Pike_Action_Extend
        ElseIf *Memory\Time = (*Pike\Period + 1) * #Pike_PeriodLength
          *Memory\Action = #Pike_Action_Retract
        EndIf
        If *Memory\Time => #Pike_PeriodLength * #Pike_Periods
          *Memory\Time = 0
        Else
          *Memory\Time + 1
        EndIf
      EndIf
    Case #GM_Object_Collision
      GM_LoseLife()
      
    Case #GM_Object_Initialize
      *Memory        = AllocateMemory(SizeOf(PikeMemory))
      *Object\Memory = *Memory
      
      Select *Pike\Direction
        Case #Pike_Direction_Left
          *Object\X      + #Pike_OuterSize
          *Object\Width  = #Pike_Size
          *Object\Height = #GM_FieldSize
          *Memory\X      = *Object\X + #Pike_Size
        Case #Pike_Direction_Right
          *Object\Width  = #Pike_Size
          *Object\Height = #GM_FieldSize
          *Memory\X      = *Object\X - #Pike_Size
        Case #Pike_Direction_Up
          *Object\Y      + #Pike_OuterSize
          *Object\Width  = #GM_FieldSize
          *Object\Height = #Pike_Size
          *Memory\Y      = *Object\Y + #Pike_Size
        Case #Pike_Direction_Down
          *Object\Width  = #GM_FieldSize
          *Object\Height = #Pike_Size
          *Memory\Y      = *Object\Y - #Pike_Size
      EndSelect
      
      *Memory\Action = #Pike_Action_Stopped
      *Memory\Time   = 0
      
      If *Pike\Period <> #Pike_NoPeriod And *Pike\Period <> 0
        Select *Pike\Direction
          Case #Pike_Direction_Left, #Pike_Direction_Right
            *Object\X = *Memory\X
          Case #Pike_Direction_Up, #Pike_Direction_Down
            *Object\Y = *Memory\Y
        EndSelect
      EndIf 
    Case #GM_Object_Load
      GM_LoadPixelPosition(@*Object\X, @*Object\Y)
    Case #GM_Object_GlobalLoad
      If Not *Pike\Sprite
        *Pike\Sprite   = GM_LoadSprite("Obstacles\Pike" + *Pike\File)
        *Pike\Sprite3D = GM_CreateSprite3D(*Pike\Sprite)
      EndIf
      
    Case #GM_Object_CallAction
      Select GM_ObjectActionID(*Parameter)
        Case 0 ;Extend
          *Memory\Action = #Pike_Action_Extend
        Case 1 ;Retract
          *Memory\Action = #Pike_Action_Retract
        Case 2 ;Extended
          *Memory\Action = #Pike_Action_Stopped
          Select *Pike\Direction
            Case #Pike_Direction_Left, #Pike_Direction_Right
              *Object\X = *Memory\X
            Case #Pike_Direction_Up, #Pike_Direction_Down
              *Object\Y = *Memory\Y
          EndSelect
        Case 3 ;Retracted
          *Memory\Action = #Pike_Action_Stopped
          Select *Pike\Direction
            Case #Pike_Direction_Left
              *Object\X = *Memory\X + #Pike_Size
            Case #Pike_Direction_Right
              *Object\X = *Memory\X - #Pike_Size
            Case #Pike_Direction_Up
              *Object\Y = *Memory\Y + #Pike_Size
            Case #Pike_Direction_Down
              *Object\Y = *Memory\Y - #Pike_Size
          EndSelect
      EndSelect
    Case #GM_Object_ActionName
      Select *Parameter
        Case 0
          ProcedureReturn @"Extend"
        Case 1
          ProcedureReturn @"Retract"
        Case 2
          ProcedureReturn @"Set to extended"
        Case 3
          ProcedureReturn @"Set to retracted"
      EndSelect
    Case #GM_Object_ActionCount
      ProcedureReturn 4
    
    Case #GM_Object_EditorSave
      *EditorObject = *Parameter
      GM_SaveFieldPosition(*EditorObject\X, *EditorObject\Y)
    Case #GM_Object_EditorLoad
      *EditorObject = *Parameter
      GM_LoadFieldPosition(@*EditorObject\X, @*EditorObject\Y)
    
    Case #GM_Object_PreferedLayer
      ProcedureReturn #GM_Layer_BackgroundObject2
    Case #GM_Object_CheckLayer
      If *Parameter <> #GM_Layer_StaticObject
        ProcedureReturn #True
      Else
        ProcedureReturn #False
      EndIf
    Case #GM_Object_ViewSprite
      ReturnString("Obstacles\Pike" + *Pike\File + "-View")
    Case #GM_Object_ViewRegion
      ReturnViewRegion(*Pike\Period, 0, 1, 1)
    Case #GM_Object_Name
      If *Pike\Period <> #Pike_NoPeriod
        ReturnString(*Pike\Name + " Pike (" + Str(*Pike\Period + 1) + ")")
      Else
        ReturnString("Controllable " + *Pike\Name + " Pike")
      EndIf
  EndSelect
EndProcedure



ProcedureDLL.i Obj_PikeB1(*Object.GM_Object, Event.i, *Parameter)
  Static PikeB1.PikeParameter
  
  With PikeB1
    If Not \Initialized
      \Name = "Bottom"
      \File = "B"
      
      \Direction = #Pike_Direction_Up
      \Period    = 0
      
      \Initialized = #True
    EndIf
  EndWith
  
  ProcedureReturn PikeTemplate(*Object, Event, *Parameter, @PikeB1)
EndProcedure
ProcedureDLL.i Obj_PikeB2(*Object.GM_Object, Event.i, *Parameter)
  Static PikeB2.PikeParameter
  
  With PikeB2
    If Not \Initialized
      \Name = "Bottom"
      \File = "B"
      
      \Direction = #Pike_Direction_Up
      \Period    = 1
      
      \Initialized = #True
    EndIf
  EndWith
  
  ProcedureReturn PikeTemplate(*Object, Event, *Parameter, @PikeB2)
EndProcedure
ProcedureDLL.i Obj_PikeB3(*Object.GM_Object, Event.i, *Parameter)
  Static PikeB3.PikeParameter
  
  With PikeB3
    If Not \Initialized
      \Name = "Bottom"
      \File = "B"
      
      \Direction = #Pike_Direction_Up
      \Period    = 2
      
      \Initialized = #True
    EndIf
  EndWith
  
  ProcedureReturn PikeTemplate(*Object, Event, *Parameter, @PikeB3)
EndProcedure
ProcedureDLL.i Obj_PikeB4(*Object.GM_Object, Event.i, *Parameter)
  Static PikeB4.PikeParameter
  
  With PikeB4
    If Not \Initialized
      \Name = "Bottom"
      \File = "B"
      
      \Direction = #Pike_Direction_Up
      \Period    = #Pike_NoPeriod
      
      \Initialized = #True
    EndIf
  EndWith
  
  ProcedureReturn PikeTemplate(*Object, Event, *Parameter, @PikeB4)
EndProcedure

ProcedureDLL.i Obj_PikeL1(*Object.GM_Object, Event.i, *Parameter)
  Static PikeL1.PikeParameter
  
  With PikeL1
    If Not \Initialized
      \Name = "Left"
      \File = "L"
      
      \Direction = #Pike_Direction_Right
      \Period    = 0
      
      \Initialized = #True
    EndIf
  EndWith
  
  ProcedureReturn PikeTemplate(*Object, Event, *Parameter, @PikeL1)
EndProcedure
ProcedureDLL.i Obj_PikeL2(*Object.GM_Object, Event.i, *Parameter)
  Static PikeL2.PikeParameter
  
  With PikeL2
    If Not \Initialized
      \Name = "Left"
      \File = "L"
      
      \Direction = #Pike_Direction_Right
      \Period    = 1
      
      \Initialized = #True
    EndIf
  EndWith
  
  ProcedureReturn PikeTemplate(*Object, Event, *Parameter, @PikeL2)
EndProcedure
ProcedureDLL.i Obj_PikeL3(*Object.GM_Object, Event.i, *Parameter)
  Static PikeL3.PikeParameter
  
  With PikeL3
    If Not \Initialized
      \Name = "Left"
      \File = "L"
      
      \Direction = #Pike_Direction_Right
      \Period    = 2
      
      \Initialized = #True
    EndIf
  EndWith
  
  ProcedureReturn PikeTemplate(*Object, Event, *Parameter, @PikeL3)
EndProcedure
ProcedureDLL.i Obj_PikeL4(*Object.GM_Object, Event.i, *Parameter)
  Static PikeL4.PikeParameter
  
  With PikeL4
    If Not \Initialized
      \Name = "Left"
      \File = "L"
      
      \Direction = #Pike_Direction_Right
      \Period    = #Pike_NoPeriod
      
      \Initialized = #True
    EndIf
  EndWith
  
  ProcedureReturn PikeTemplate(*Object, Event, *Parameter, @PikeL4)
EndProcedure

ProcedureDLL.i Obj_PikeR1(*Object.GM_Object, Event.i, *Parameter)
  Static PikeR1.PikeParameter
  
  With PikeR1
    If Not \Initialized
      \Name = "Right"
      \File = "R"
      
      \Direction = #Pike_Direction_Left
      \Period    = 0
      
      \Initialized = #True
    EndIf
  EndWith
  
  ProcedureReturn PikeTemplate(*Object, Event, *Parameter, @PikeR1)
EndProcedure
ProcedureDLL.i Obj_PikeR2(*Object.GM_Object, Event.i, *Parameter)
  Static PikeR2.PikeParameter
  
  With PikeR2
    If Not \Initialized
      \Name = "Right"
      \File = "R"
      
      \Direction = #Pike_Direction_Left
      \Period    = 1
      
      \Initialized = #True
    EndIf
  EndWith
  
  ProcedureReturn PikeTemplate(*Object, Event, *Parameter, @PikeR2)
EndProcedure
ProcedureDLL.i Obj_PikeR3(*Object.GM_Object, Event.i, *Parameter)
  Static PikeR3.PikeParameter
  
  With PikeR3
    If Not \Initialized
      \Name = "Right"
      \File = "R"
      
      \Direction = #Pike_Direction_Left
      \Period    = 2
      
      \Initialized = #True
    EndIf
  EndWith
  
  ProcedureReturn PikeTemplate(*Object, Event, *Parameter, @PikeR3)
EndProcedure
ProcedureDLL.i Obj_PikeR4(*Object.GM_Object, Event.i, *Parameter)
  Static PikeR4.PikeParameter
  
  With PikeR4
    If Not \Initialized
      \Name = "Right"
      \File = "R"
      
      \Direction = #Pike_Direction_Left
      \Period    = #Pike_NoPeriod
      
      \Initialized = #True
    EndIf
  EndWith
  
  ProcedureReturn PikeTemplate(*Object, Event, *Parameter, @PikeR4)
EndProcedure

; IDE Options = PureBasic 4.51 RC 1 (Windows - x86)
; CursorPosition = 192
; FirstLine = 171
; Folding = ---
; EnableXP