Enumeration
  #Walker_Action_Standing
  #Walker_Action_Walking
EndEnumeration
Enumeration
  #Walker_Animation_Standing
  #Walker_Animation_Walking
EndEnumeration
Enumeration
  #Walker_Animation_StandingLeft
  #Walker_Animation_StandingRight
  #Walker_Animation_WalkingLeft
  #Walker_Animation_WalkingRight
EndEnumeration

Structure WalkerParameter
  Name.s
  File.s
  *StandingCallback
  *WalkingCallback
  *CollisionCallback
  *HitCallback
  StandingSpeed.i
  WalkingSpeed.i
  
  Energy.i
  Destroyable.i
  Width.i
  Height.i
  ScoreWhenKilled.i
  
  OuterWidth.i
  OuterHeight.i
  Frameset.i
  
  Initialized.i
EndStructure
Structure WalkerMemory
  Action.i
  Direction.i
  Energy.f
  Speed.f
  *Memory
EndStructure

Procedure.i WalkerTemplate(*Object.GM_Object, Event.i, *Parameter, *Walker.WalkerParameter)
  Static *Memory.WalkerMemory, FieldX.i, FieldY.i
  
  If *Object
    *Memory = *Object\Memory
  Else
    *EditorObject = *Parameter
  EndIf
  
  Select Event
    Case #GM_Object_Display
      GM_DisplayAnimation(*Object\Animation, *Object\X - *Walker\OuterWidth, *Object\Y - *Walker\OuterHeight)
    Case #GM_Object_Frame
      Select *Memory\Action
        Case #Walker_Action_Walking
          *Object\X + *Memory\Speed * *Memory\Direction
          
          FieldX = (*Object\X + *Object\Width/2 - #GM_FieldSize/2 - *Walker\OuterWidth * *Memory\Direction * 2)  / #GM_FieldSize
          FieldY = (*Object\Y + *Object\Height/2 - #GM_FieldSize/2) / #GM_FieldSize
          
          If (Not (GM_GetWall(FieldX + *Memory\Direction, FieldY + 1) Or GM_GetGround(FieldX + *Memory\Direction, FieldY + 1))) Or GM_GetWall(FieldX + *Memory\Direction, FieldY)
            ; Richtung wechseln, wenn kein Boden mehr folgt:
            *Memory\Direction = -*Memory\Direction
            If GM_FramesetRows(*Walker\Frameset) = 4
              Select *Memory\Direction
                Case -1
                  GM_SetAnimationRow(*Object\Animation, #Walker_Animation_WalkingLeft)
                Case 1
                  GM_SetAnimationRow(*Object\Animation, #Walker_Animation_WalkingRight)
              EndSelect
            EndIf
          EndIf
          
          If GM_PlayAnimation(*Object\Animation)
            GM_SetAnimationDuration(*Object\Animation, *Walker\WalkingSpeed)
          EndIf
          
          If *Walker\WalkingCallback
            CallFunctionFast(*Walker\WalkingCallback, *Object, *Memory\Memory, *Walker)
          EndIf
        Case #Walker_Action_Standing
          If GM_PlayAnimation(*Object\Animation)
            GM_SetAnimationDuration(*Object\Animation, *Walker\StandingSpeed)
          EndIf
          
          If *Walker\StandingCallback
            CallFunctionFast(*Walker\StandingCallback, *Object, *Memory\Memory, *Walker)
          EndIf
      EndSelect
    Case #GM_Object_Collision
      If *Walker\CollisionCallback
        ProcedureReturn CallFunctionFast(*Walker\CollisionCallback, *Object, *Memory\Memory, *Walker)
      ElseIf *Memory\Energy
        GM_HitPlayer(*Memory\Energy * 2)
      EndIf
    Case #GM_Object_Hit
      *Shot = *Parameter
      If *Walker\HitCallback
        ProcedureReturn CallFunctionFast(*Walker\HitCallback, *Object, *Memory\Memory, *Parameter, *Walker)
      ElseIf *Walker\Destroyable
        GM_Damage(@*Memory\Energy, @*Shot\Power)
        If *Memory\Energy <= 0
          If *Walker\ScoreWhenKilled
            GM_AddScoreTM(*Walker\ScoreWhenKilled, *Object)
          EndIf
          GM_EmitGhost(*Object\X + *Object\Width/2, *Object\Y + *Object\Height/2)
          ProcedureReturn #GM_Object_DoDelete
        EndIf
      EndIf
      ProcedureReturn #GM_Object_DoCollision
    
    Case #GM_Object_Initialize
      *Object\Width     = *Walker\Width
      *Object\Height    = *Walker\Height
      *Object\Animation = GM_CreateAnimation(*Walker\Frameset)
      
      *Memory        = AllocateMemory(SizeOf(WalkerMemory))
      *Object\Memory = *Memory
      
      *Memory\Action = #Walker_Action_Standing
      *Memory\Energy = *Walker\Energy
    Case #GM_Object_Load
      GM_LoadPixelPosition(@*Object\X, @*Object\Y)
      *Object\X + *Walker\OuterWidth
      *Object\Y + *Walker\OuterHeight
    Case #GM_Object_GlobalLoad
      *Walker\OuterWidth  = (#GM_FieldSize - *Walker\Width) / 2
      *Walker\OuterHeight = #GM_FieldSize - *Walker\Height
      
      If Not *Walker\Frameset
        *Walker\Frameset = GM_LoadAnimationFrameset(*Walker\File, #GM_FieldSize, #GM_FieldSize)
      EndIf
    
    Case #GM_Object_EditorSave
      GM_SaveFieldPosition(*EditorObject\X, *EditorObject\Y)
    Case #GM_Object_EditorLoad
      GM_LoadFieldPosition(@*EditorObject\X, @*EditorObject\Y)
    
    Case #GM_Object_PreferedLayer
      ProcedureReturn #GM_Layer_ForegroundObject1
    Case #GM_Object_CheckLayer
      If *Parameter <> #GM_Layer_StaticObject
        ProcedureReturn #True
      Else
        ProcedureReturn #False
      EndIf
    Case #GM_Object_ViewSprite
      ProcedureReturn @*Walker\File
    Case #GM_Object_ViewRegion
      ReturnViewRegion(0, 0, 1, 1)
    Case #GM_Object_Name
      ProcedureReturn @*Walker\Name
  EndSelect
  
  ProcedureReturn #Null
EndProcedure

Procedure.i StartWalking(*Object.GM_Object, Speed.f, Direction.i = 0)
  Protected *Memory.WalkerMemory = *Object\Memory
  
  *Memory\Action = #Walker_Action_Walking
  
  If Direction = 0
    If Random(1)
      *Memory\Direction = 1
    Else
      *Memory\Direction = -1
    EndIf
  Else
    *Memory\Direction = Direction
  EndIf
  
  *Memory\Speed = Speed
  
  If GM_FramesetRows(GM_AnimationFrameset(*Object\Animation)) = 4
    Select *Memory\Direction
      Case -1
        GM_SetAnimationRow(*Object\Animation, #Walker_Animation_WalkingLeft)
      Case 1
        GM_SetAnimationRow(*Object\Animation, #Walker_Animation_WalkingRight)
    EndSelect
  Else
    GM_SetAnimationRow(*Object\Animation, #Walker_Animation_Walking)
  EndIf
EndProcedure
Procedure.i StopWalking(*Object.GM_Object)
  Protected *Memory.WalkerMemory = *Object\Memory
  
  *Memory\Action = #Walker_Action_Standing
  
  If GM_FramesetRows(GM_AnimationFrameset(*Object\Animation)) = 4
    Select *Memory\Direction
      Case -1
        GM_SetAnimationRow(*Object\Animation, #Walker_Animation_StandingLeft)
      Case 1
        GM_SetAnimationRow(*Object\Animation, #Walker_Animation_StandingRight)
    EndSelect
  Else
    GM_SetAnimationRow(*Object\Animation, #Walker_Animation_Standing)
  EndIf
EndProcedure
; IDE Options = PureBasic 4.51 (Windows - x86)
; CursorPosition = 109
; FirstLine = 78
; Folding = -
; EnableXP