;**************************************************************
;     Player
;**************************************************************
;   Copyright (c) 2010 Josef Sniatecki
;
; This program is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License
; As published by the Free Software Foundation; either version
; 3 of the License, Or (at your option) any later version.
;
; This program is distributed in the hope that it will be
; useful, but WITHOUT ANY WARRANTY; without even the implied
; warranty of MERCHANTABILITY Or FITNESS For A PARTICULAR
; PURPOSE. See the GNU General Public License For more details.
;
; You should have received a copy of the GNU General Public
; License along With this program;
; If Not, see "http://www.gnu.org/licenses/".
;**************************************************************



Procedure.i ResetKeys()
  Protected Key.i
  
  ; Buffer leeren:
  GetAsyncKeyState_(#VK_LEFT)
  GetAsyncKeyState_(#VK_RIGHT)
  GetAsyncKeyState_(#VK_UP)
  GetAsyncKeyState_(#VK_DOWN)
  GetAsyncKeyState_(#VK_W)
  GetAsyncKeyState_(#VK_S)
  GetAsyncKeyState_(#VK_A)
  
  ; Zwischenvariablen zurcksetzen:
  For Key = 0 To #Player_KeyCount - 1
    Player\KeyPressed[Key]  = #False
    Player\KeyPressing[Key] = #False
    Player\KeyReleased[Key] = #False
  Next
EndProcedure
Procedure.i ExaminePlayerField()
  With Player
  
  \FieldX = (\X - (#FieldSize - \Width )/2) / #FieldSize
  \FieldY = (\Y - (#FieldSize - \Height)/2) / #FieldSize
  
  EndWith
EndProcedure
Procedure.i ChangePlayerAction(Action.i)
  ; Hier wird die aktuelle Aktion (Zustand) des Spielers gegen
  ; eine Neue gewechselt. "ActionTime" wird hierbei auf "0" gesetzt,
  ; damit das jeweilige Callback der Aktion prfen kann, wie viele
  ; Frames schon innerhalb der jeweiligen Aktion durchgelaufen wurden.
  
  With Player
  
  \Action     = Action
  \ActionTime = 0
  
  Select Action
    Case #Player_Action_Flying, #Player_Action_Clambering, #Player_Action_Climbing
      \HeightOfFall = 0
    Case #Player_Action_Standing
      PlayerLands()
  EndSelect
  
  EndWith
EndProcedure

Procedure.i LocatePlayer(X.i, Y.i, *Scene.Scene = #Null)
  ; Der Spieler wird auf ein angegebenes Feld (X|Y) gesetzt.
  ; Diese Funktion sorgt dafr, dass der Spieler auf dem Boden
  ; und in die Mitte des Feldes gesetzt wird.
  ; Zudem wird dafr gesorgt, dass bestimmte Aktionen, wie
  ; "Fliegen" und "Klettern", deaktiviert werden.
  
  With Player
  
  \FieldX = X
  \FieldY = Y
  
  \X             = X * #FieldSize + #FieldSize/2 - \Width/2
  \Y             = (Y + 1) * #FieldSize - \Height
  \Animation\Set = 0
  
  ChangePlayerAction(#Player_Action_Standing)
  \MoveX = 0
  \MoveY = 0
  
  If *Scene And *Scene <> \Scene
    \PreviousScene = \Scene
    \Scene         = *Scene
    ChangeScene(\PreviousScene, \Scene)
  EndIf
  
  ResetKeys()
  ResetCamera()
  
  EndWith
EndProcedure

Procedure.i EnhanceMultiplierCounter()
  ; In dieser Prozedur wird der Counter fr den
  ; Multiplikator erhht. Der Counter bestimmt, ob
  ; nun der Multiplikator erhht wird, oder nicht.
  
  ; Hochgezhlt wird, sobald gilt: c > 1 + m
  ; ("c" = Counter und "m" = aktueller Multiplikator)
  
  With Player
  
  If \MultiplierCounter => 1 + \Multiplier
    ; Der Multiplikator wird erhht:
    \Multiplier            + 1
    \MultiplierCounter     = 0
    ; Fr ein Wackel-Effekt im HUD wird eine Zeit (frames) festgelegt:
    \MultiplierChangedTime = #Player_MultiplierChangedTime
  Else
    ; Nur der Counter wird hochgezhlt:
    \MultiplierCounter + 1
  EndIf
  ; Die Zeit bis zum Ablauf des Multiplikators wird zurckgesetzt:
  \MultiplierDuration = #Player_MultiplierDuration
  
  If Challange\Script\MultiplierEnhanced
    CallFunctionFast(Challange\Script\MultiplierEnhanced)
  EndIf
  
  EndWith
EndProcedure
Procedure.i SetMultiplier(Multiplier.i)
  With Player
  
  If Multiplier > 0
    \Multiplier         = Multiplier
    \MultiplierCounter  = 0
    \MultiplierDuration = #Player_MultiplierDuration
  EndIf
  
  EndWith
EndProcedure
Procedure.i AddScore(Score.i, Reason.s = "")
  With Player
  
  If Score > 0
    \Score         + Score * \Multiplier
    \AdditiveScore + Score * \Multiplier
    
    EnhanceMultiplierCounter()
    
    CallSpecialEventEffects(*CurrentAct, #SpecialEvent_ScoreAdded)
  EndIf
  
  EndWith
EndProcedure
Procedure.i AddScoreNM(Score.i)
  With Player
  
    If Score > 0
      \Score         + Score
      \AdditiveScore + Score
      
      EnhanceMultiplierCounter()
      
      CallSpecialEventEffects(*CurrentAct, #SpecialEvent_ScoreAdded)
    EndIf
  
  EndWith
EndProcedure
Procedure.i ShowComment(Comment.s, Duration.i = 100)
  With Player
  
  \Comment         = Comment
  \CommentDuration = Duration
  
  EndWith
EndProcedure

Procedure.i LoseLife()
  With Player
  
  If Not \LostLiveTime
    ; Hier wird zwischen Leben und Tod entschieden ;):
    If \Lives = 1
      \LoseGame = #True
      \Lives    = 0
      \Energy   = 0
    Else
      \Lives  - 1
      \Energy = \MaxEnergy
    EndIf
    
    \HitTime                    = 0
    \LostLiveTime               = #Player_LostLiveTime
    \LostLiveAnimation\Duration = #Player_LostLiveAnimationDuration
    \LostLiveAnimation\Frame    = 0
    
    ; Falls sich der Spieler fr Punkte geopfert hat
    ; (was in der Regel als geschummelt gilt) werden diese
    ; Punkte von der eigentlichen Score wieder abgezogen:
    \Score               - \AdditiveScore
    \AdditiveScore       = 0
    \ScoreReason         = ""
    \ScoreReasonDuration = 0
    \NewScore            = 0
    
    \Comment         = ""
    \CommentDuration = 0
    
    \MultiplierCounter     = 0
    \Multiplier            = 1
    \MultiplierDuration    = 0
    \MultiplierChangedTime = 0
    
    \MoveX        = 0
    \MoveY        = 0
    \HeightOfFall = 0
    
    CallSpecialEventEffects(*CurrentAct, #SpecialEvent_LostLive)
  EndIf
  
  EndWith
EndProcedure
Procedure.i HitPlayer(Energy.i)
  With Player
  
  ; Der Spieler verliert nur dann Energie (und ggf. Leben), wenn
  ; nicht gerade schon Energie verloren wurde und kein Schild
  ; aktiv ist.
  
  If (Not \HitTime) And (Not (\LostLiveTime Or \LoseGame)) And (Not \Shield)
    ; Angegebene Energie abziehen:
    \Energy - Energy
    
    If \Energy <= 0
      ; Keine Energie mehr - ein Leben verlieren:
      LoseLife()
    Else
      ; Ansonsten die Zeit festlegen, in der der Spieler nicht
      ; ein erneutes Mal verletzt werden kann:
      \HitTime = #Player_HitTime
    EndIf
  EndIf
  
  EndWith
EndProcedure
Procedure.i PlayerLands()
  With Player
  
  If \Action = #Player_Action_Standing
    If \HeightOfFall > #FieldSize * 6
      ; Nach einem langen Fall Staub-Effekt hinzufgen:
      CreateParticle(Particle\Landed, \X + \Width/2 - 18/2, \Y + \Height - 6, 6)
      If \HeightOfFall > #FieldSize * 10
        ; Bei einer zu harten Landung wird der Spieler geschdigt:
        HitPlayer(2)
        ShowComment("Argg!", 50)
      EndIf
    EndIf
    
    \HeightOfFall  = 0
    \MoveY         = 0
    \PreviousMoveY = 0
  EndIf
  
  EndWith
EndProcedure

Procedure.i PlayerInScene()
  With Player
  
  If \FieldX => 0 And \FieldX <  \Scene\Width * #SceneWidth And \FieldY => 0 And \FieldY < \Scene\Height * #SceneHeight
    ProcedureReturn #True
  Else
    ProcedureReturn #False
  EndIf
  
  EndWith
EndProcedure
Procedure.i IsWall(DX.i, DY.i)
  With Player
  
  Protected FieldX.i, FieldY.i, *Field.Field
  
  FieldX = \FieldX + DX
  FieldY = \FieldY + DY
  
  If FieldX => 0 And FieldX < \Scene\Width * #SceneWidth And FieldY => 0 And FieldY < \Scene\Height * #SceneHeight
    If GetWall(FieldX, FieldY)
      ProcedureReturn #True
    EndIf
  EndIf
  
  ProcedureReturn #False
  
  EndWith
EndProcedure
Procedure.i IsGround(DX.i, DY.i)
  With Player
  
  Protected FieldX.i, FieldY.i, *Field.Field
  
  FieldX = \FieldX + DX
  FieldY = \FieldY + DY
  
  If FieldX => 0 And FieldX < \Scene\Width*#SceneWidth And FieldY => 0 And FieldY < \Scene\Height*#SceneHeight
    If GetWall(FieldX, FieldY) Or GetGround(FieldX, FieldY)
      ProcedureReturn #True
    EndIf
  EndIf
  
  ProcedureReturn #False
  
  EndWith
EndProcedure

Procedure.i NeighbourObjectQualifiedForClambering(DX.i, DY.i)
  Protected *Scene.Scene
  
  If PlayerNeighbourScene(DX, DY)
    ProcedureReturn #True
  EndIf
EndProcedure
Procedure.i ObjectQualifiedForClambering()
  With Player
  
  Protected *Object.Object
  
  If PlayerInScene()
    *Object = GetObject(Player\FieldX, Player\FieldY)
    If *Object And _Call(*Object\Callback, *Object, #Object_AllowClambering, #Null)
      ProcedureReturn #True
    EndIf
  Else
    ; Da der Spieler kurz vor dem Wechsel der Szene (durch berschreitung
    ; der Rnder) mit "FieldX" und "FieldY" sich nicht mehr im Koordinatensystem
    ; der aktuellen Szene befindet, muss geprft werden, ob sich ein
    ; Kletterobjekt in der benachbarten Szene befindet:
    If \FieldX < 0                                And NeighbourObjectQualifiedForClambering(-1,  0)
      ProcedureReturn #True
    ElseIf \FieldX > \Scene\Width*#SceneWidth-1   And NeighbourObjectQualifiedForClambering( 1,  0)
      ProcedureReturn #True
    EndIf
    If \FieldY < 0                                And NeighbourObjectQualifiedForClambering( 0, -1)
      ProcedureReturn #True
    ElseIf \FieldY > \Scene\Height*#SceneHeight-1 And NeighbourObjectQualifiedForClambering( 0,  1)
      ProcedureReturn #True
    EndIf
  EndIf
  
  ProcedureReturn #False
  
  EndWith
EndProcedure
Procedure.i NeighbourWallQualifiedForClimbing(Direction.i, DY.i)
  Protected *Scene.Scene
  
  If PlayerNeighbourScene(0, DY)
    ProcedureReturn #True
  EndIf
EndProcedure
Procedure.i WallQualifiedForClimbing(Direction.i)
  With Player
  
  Protected *Object.Object
  
  If PlayerInScene()
    *Object = GetObject(Player\FieldX + Direction, Player\FieldY)
    If Not (*Object And _Call(*Object\Callback, *Object, #Object_ForbidClimbing, #Null))
      ProcedureReturn #True
    EndIf
  Else
    If \FieldY < 0                                  And NeighbourWallQualifiedForClimbing(Direction, -1)
      ProcedureReturn #True
    ElseIf \FieldY > \Scene\Height*#SceneHeight - 1 And NeighbourWallQualifiedForClimbing(Direction,  1)
      ProcedureReturn #True
    EndIf
  EndIf
  
  ProcedureReturn #False
  
  EndWith
EndProcedure

Procedure.i UseGravity()
  With Player
  
  \MoveY + #Gravity * \Mass
  If \MoveY > 0
    If \MoveY > #Player_MaxFallSpeed
      \MoveY = #Player_MaxFallSpeed
    EndIf
    ; Damit die Fallhhe fr "PlayerLands()" bestimmt
    ; werden kann, wird "HeightOfFall" seit dem Hochpunkt
    ; gezhlt:
    \HeightOfFall + \MoveY
  Else
    ; Die Hhe des Falls soll bei Null bleiben, solange
    ; der Spieler noch nicht seinen Hochpunkt im Sprung
    ; erreicht hat.
    \HeightOfFall = 0
  EndIf
  
  EndWith
EndProcedure

IncludeFile "Shot.pb"
IncludeFile "PlayerCore.pb"

Procedure.i DrawHUD(DisplayItems.i = #True)
  Protected Y.i, Index.i
  
  With Player
  
  DisplaySprite(BarSprite, 0, #SceneHeight * #FieldSize)
  
  Y = #SceneHeight * #FieldSize + 1
  
  DisplayText(24, Y             , RSet(Str(\Score - \AdditiveScore), 8, "0"), *StandardCharSet)
  DisplayText(24, Y + #FieldSize, RSet(Str(\Lives), 3, "0"), *StandardCharSet)
  
  If \MultiplierDuration > 0
    If \AdditiveScore
      DisplayText(125, Y, RSet(Str(\AdditiveScore), 5, "0"), *StandardCharSet)
    EndIf
  ElseIf \AdditiveScore
    DisplayText(125, Y, RSet(Str(\AdditiveScore), 5, "0"), *StandardCharSet)
    \AdditiveScore - (1 + \AdditiveScore/15)
    If \AdditiveScore < 0
      \AdditiveScore = 0
    EndIf
    If \NewScore
      \NewScore = 0
    EndIf
  Else
    DisplayText(125, Y, RSet("", 5, "0"), *StandardCharSet)
  EndIf
  
  If \CommentDuration
    DisplayText(Screen\Width/2 + 55, Y, UCase(\Comment), *StandardCharSet)
    \CommentDuration - 1
  EndIf
  
  
  If \Multiplier > 1
    If \MultiplierChangedTime
      ; Ruckeleffekt darstellen:
      If \MultiplierChangedTime % 10 < 5
        DisplayText(#ScreenWidth/2 - (*StandardCharSet\Width * 2 + 1)/2 - 1, Y + 2 - 1, "X" + Str(\Multiplier), *StandardCharSet)
      Else
        DisplayText(#ScreenWidth/2 - (*StandardCharSet\Width * 2 + 1)/2 - 1, Y + 2 + 1, "X" + Str(\Multiplier), *StandardCharSet)
      EndIf
    Else
      If \MultiplierDuration > 50 Or \MultiplierDuration % 19 => 10
        DisplayText(#ScreenWidth/2 - (*StandardCharSet\Width * 2 + 1)/2 - 1, Y + 2, "X" + Str(\Multiplier), *StandardCharSet)
      EndIf
    EndIf
  EndIf
  
  If \HitTime
    If \HitTime % 5 < 3
      Y - 1
    Else
      Y + 1
    EndIf
  EndIf
  
  For Index = 0 To 2
    If \UsingItem[Index]
      DisplayNumber(#ScreenWidth - #InventoryEntry_Width * (3 - Index) + 3, #ScreenHeight + #InventoryEntry_Height - *TinyMessageCharSet\Height - 1, Str(\UsingItem[Index]\Count), *TinyMessageCharSet)
    EndIf
  Next
  
  Start3D()
    For Index = 0 To \Energy - 1
      DisplaySprite3D(\EnergySprite3D, 67 + (Index % 10) * 11, Y + 2 + #FieldSize + 9 * (Index / 10))
    Next
    For Index = (\Energy) To \MaxEnergy - 1
      DisplaySprite3D(\NoEnergySprite3D, 67 + (Index % 10) * 11, Y + 2 + #FieldSize + 9 * (Index / 10))
    Next
    
    If DisplayItems
      For Index = 0 To 2
        If \UsingItem[Index]
          DisplaySprite3D(\UsingItem[Index]\Sprite3D, #ScreenWidth - #InventoryEntry_Width * (3 - Index) + #InventoryEntry_InnerWidth/2 - #FieldSize/2 + 2, #ScreenHeight + #InventoryEntry_InnerHeight/2 - #FieldSize/2)
        EndIf
      Next
    EndIf
  Stop3D()
  
  EndWith
EndProcedure

; IDE Options = PureBasic 4.51 (Windows - x86)
; CursorPosition = 488
; FirstLine = 162
; Folding = PAA9
; Markers = 50
; EnableXP