;**************************************************************
;     Player Core
;**************************************************************
;   Copyright (c) 2010 Josef Sniatecki
;
; This program is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License
; As published by the Free Software Foundation; either version
; 3 of the License, Or (at your option) any later version.
;
; This program is distributed in the hope that it will be
; useful, but WITHOUT ANY WARRANTY; without even the implied
; warranty of MERCHANTABILITY Or FITNESS For A PARTICULAR
; PURPOSE. See the GNU General Public License For more details.
;
; You should have received a copy of the GNU General Public
; License along With this program;
; If Not, see "http://www.gnu.org/licenses/".
;**************************************************************
; Hier befindet sich der Hauptteil des Spielers. In diesem wird
; die Steuerung durch die Tastatur, die Kollisionsabfrage und
; der Punktestand behandelt.
;
; Die strukturierte Variable "Player" beinhaltet alle Daten
; des Spielers. Mehr Informationen zu den Attributen unter
; "Main.pb - Structure Player".



Macro ExamineKey(Indentifier, State)
  ; Das Makro berprft, wie eine Taste gerade bettigt wurde:
  ; - Player\KeyPressed[Key]  : Taste wurde gerade runter gedrckt (KeyDown).
  ; - Player\KeyReleased[Key] : Taste wurde losgelassen (KeyUp).
  ; - Player\KeyPressing[Key] : Taste wird gedrckt gehalten.
  
  Player\KeyPressed[#Player_Key_#Indentifier] = #False
  Player\KeyReleased[#Player_Key_#Indentifier] = #False
  If State
    If Not Player\KeyPressing[#Player_Key_#Indentifier]
      Player\KeyPressed[#Player_Key_#Indentifier]  = #True
      Player\KeyPressing[#Player_Key_#Indentifier] = #True
    EndIf
  Else
    If Player\KeyPressing[#Player_Key_#Indentifier]
      Player\KeyReleased[#Player_Key_#Indentifier] = #True
    EndIf
    Player\KeyPressing[#Player_Key_#Indentifier] = #False
  EndIf
EndMacro

Procedure.i InitializeControl()
  If InitJoystick()
    ControlMode = #ControlMode_Joystick
  Else
    ControlMode = #ControlMode_Keyboard
  EndIf
EndProcedure
Procedure.i ExamineControl()
  Select ControlMode
    Case #ControlMode_Keyboard
      ; Die Tastatur wird fr die Steuerung verwendet:
      ExamineKey(Left     , IsKeyPressing(#VK_LEFT))
      ExamineKey(Right    , IsKeyPressing(#VK_RIGHT))
      ExamineKey(Up       , IsKeyPressing(#VK_UP))
      ExamineKey(Down     , IsKeyPressing(#VK_DOWN))
      ExamineKey(Shoot    , IsKeyPressing(#VK_W))
      ExamineKey(Jump     , IsKeyPressing(#VK_S))
      ExamineKey(Fly      , IsKeyPressing(#VK_A))
      ExamineKey(ItemA    , IsKeyPressing(#VK_Y))
      ExamineKey(ItemB    , IsKeyPressing(#VK_X))
      ExamineKey(ItemC    , IsKeyPressing(#VK_C))
      ExamineKey(Inventory, IsKeyPressing(#VK_Q))
    Case #ControlMode_Joystick
      ; Joystick wird zur Steuerung angewendet:
      If Not ExamineJoystick(0)
        ControlMode = #ControlMode_Keyboard
      Else
        Select JoystickAxisX(0)
          Case -1
            ExamineKey(Left, #True)
            ExamineKey(Right, #False)
          Case  1
            ExamineKey(Left, #False)
            ExamineKey(Right, #True)
          Case  0
            ExamineKey(Left, #False)
            ExamineKey(Right, #False)
        EndSelect
        Select JoystickAxisY(0)
          Case -1
            ExamineKey(Up, #True)
            ExamineKey(Down, #False)
          Case  1
            ExamineKey(Up, #False)
            ExamineKey(Down, #True)
          Case  0
            ExamineKey(Up, #False)
            ExamineKey(Down, #False)
        EndSelect
        ExamineKey(Jump     , JoystickButton(0, 1))
        ExamineKey(Shoot    , JoystickButton(0, 2))
        ExamineKey(Fly      , JoystickButton(0, 3))
        ExamineKey(Inventory, JoystickButton(0, 4))
        ExamineKey(ItemA    , JoystickButton(0, 5))
        ExamineKey(ItemB    , JoystickButton(0, 6))
        ExamineKey(ItemC    , JoystickButton(0, 7))
      EndIf
  EndSelect
EndProcedure

Procedure.i ExaminePlayer()
  With Player
  
  Static SceneChanged.i, Layer.i, *Queue.Queue, *Object.Object, *Shot.Shot, *Field.Field
  Static *Item.QueueItem, *OtherItem.QueueItem
  Static Result.i, X.i, Y.i, Collision.i
  Static DeltaX.i, DeltaY.i
  
  ;{ Lives & Energy
    If \LostLiveTime
      \LostLiveTime - 1
      If Not \LostLiveTime
        If \LoseGame
          ; Das Spiel wurde verloren - Game Over:
          LoseGame()
        ElseIf \LastCheckpointScene
          LocatePlayer(\LastCheckpointX, \LastCheckpointY, \LastCheckpointScene)
        Else
          LocatePlayer(Challange\StartX, Challange\StartY, Challange\StartScene)
        EndIf
      Else
        ; Weitere berprfungen werden ignoriert:
        ProcedureReturn
      EndIf
    ElseIf \HitTime
      \HitTime - 1
    EndIf
  ;}
  
  ;{ Misc
    ; Positionierung durch ein Objekt:
    If \SceneToLocate
      LocatePlayer(\FieldXToLocate, \FieldYToLocate, \SceneToLocate)
      \SceneToLocate = #Null
    EndIf
    
    ; Das Licht fr den LumiFlight langsam erlischen lassen:
    \LumiLight  - 15
    If \LumiLight < 0
      \LumiLight = 0
    EndIf
    
    ; Schutzschild vergehen lassen:
    If \Shield
      \Shield - 1
    EndIf
      
  ;}
  
  ;{ Examine keys
    ; In dieser Sektion werden die Zustnde der Tasten
    ; in Variablen gespeichert. Dadurch ist es mglich, dass
    ; das Abfragen von Tastendrcken gleich bleibt, whrend
    ; die Tastatur, ein Joystick und sogar ein Replay fr
    ; die Steuerung verwendet werden kann.
    
    If Replay = 2
      ; Tastendrcke aus Replay lesen:
      ReplayKeys()
    Else
      ; Tastendrcke aus Tastatur oder Joystick lesen:
      ExamineControl()
      
      If Replay = 1
        ; Steuerung in ein neues Replay speichern:
        RecordKeys()
      EndIf
    EndIf
    
    ; Abfragen, ob Items verwendet werden, oder ob
    ; das Item-Fenster geffnet wird:
    If \KeyPressed[#Player_Key_ItemA]
      UseItem(0)
    ElseIf \KeyPressed[#Player_Key_ItemB]
      UseItem(1)
    ElseIf \KeyPressed[#Player_Key_ItemC]
      UseItem(2)
    ElseIf \KeyPressed[#Player_Key_Inventory]
      OpenInventory()
    EndIf
    
    ; Cheats sind im Test-Modus erlaubt:
    If KeyPressedDown(#VK_F1)
      Cheat()
    EndIf
  ;}
  
  ;{ Examine Act
    If *CurrentAct
      ExamineAct(*CurrentAct)
    EndIf
    ExamineParallelActs()
  ;}
  
  ;{ Control
    ; Die Steuerung ber verschiedene Aktionen, wie z.B. Springen und Laufen, erfolgt
    ; in dieser Quelldatei:
    IncludeFile "Control.pb"
  ;}
  
  ;{ Movement
    ; Koordinaten verndern:
    Select Player\Action 
      Case #Player_Action_Standing, #Player_Action_Falling, #Player_Action_Jump, #Player_Action_DoubleJump
        ; Bei Aktionen, die mit einer Bewegung auf dem Boden verbunden sind, kann z.B.
        ; fr einen Rutsch-Effekt mittels "MoveInfluence" und "MoveFriction" gesorgt werden:
        \MoveX = Lim(\DoMoveX * \MoveInfluence + \MoveX * \MoveFriction, -\Speed, \Speed)
      Default
        ; Fr alle anderen Bewegungen wird kein automatischer Rutsch-Effekt angewendet:
        \MoveX = \DoMoveX
    EndSelect
    
    ; Bewegt sich der Spieler mit der Geschwindigkeit "#Player_SpeedForBlur", dann
    ; wird ein Blur-Effekt hinter dem Spieler erzeugt:
    If UpdateBlur(\Blur) And (\MoveX * \MoveX) + (\MoveY * \MoveY) => #Player_SpeedForBlur * #Player_SpeedForBlur And Not \Action = #Player_Action_Climbing
      AddBlurFrame(\Blur, FramesetSprite(\Animation\Frameset, \Animation\Frame, \Animation\Set), \X, \Y)
    EndIf
    
    ; Den Spieler nun ber "MoveX" und "MoveY" bewegen:
    \X + \MoveX
    \Y + \MoveY
  ;}
  
  ;{ Collision
    ; Fr die Kollisionsabfrage werden nur die benachbarten Felder
    ; des Spielers berprft. Der Alorithmus zur Kollisionsabfrage ist
    ; in "Sources\Collision.pb" zu finden.
    
    If Player\Y => -\Height / 2
      If Player\Action = #Player_Action_Climbing
        If PlayerInScene()
          If Not (FieldCollision(\ClimbSide, 0) Or FieldCollision(\ClimbSide, 1)) ;Existiert keine Wand mehr fr das Klettern?
            ; Nach anderen Klettergelegenheiten (z.B. benachbarte Leitern) suchen:
            If ObjectQualifiedForClambering()
              ChangePlayerAction(#Player_Action_Clambering)
            Else
              ; Keine Klettergelegenheit gefunden (fallen):
              ChangePlayerAction(#Player_Action_Falling)
            EndIf
          EndIf
        EndIf
      ElseIf FieldCollision(-1, 0) Or FieldCollision(1, 0) ;Kollision mit einer seitlichen Wand?
        ; An den Wnden klettern (falls mglich):
        If \CanClimb And Not \Action = #Player_Action_LumiFlight
          If \Action = #Player_Action_Standing And \KeyPressing[#Player_Key_Up]
            If \Animation\Set % 2 = 0 And WallQualifiedForClimbing(1)
              \ClimbSide = #Player_ClimbSide_Right
              \Y         - 2
              \MoveY     = -#Player_ClimbSpeed
              ChangePlayerAction(#Player_Action_Climbing)
            ElseIf \Animation\Set % 2 = 1 And WallQualifiedForClimbing(-1)
              \ClimbSide = #Player_ClimbSide_Left
              \Y         - 2
              \MoveY     = -#Player_ClimbSpeed
              ChangePlayerAction(#Player_Action_Climbing)
            EndIf
          ElseIf \Action <> #Player_Action_Standing
            If \Animation\Set % 2 = 0 And WallQualifiedForClimbing(1) And IsWall(1, 0) And Not IsGround(0, 1)
              \ClimbSide = #Player_ClimbSide_Right
              ChangePlayerAction(#Player_Action_Climbing)
            ElseIf \Animation\Set % 2 = 1 And WallQualifiedForClimbing(-1) And IsWall(-1, 0) And Not IsGround(0, 1)
              \ClimbSide = #Player_ClimbSide_Left
              ChangePlayerAction(#Player_Action_Climbing)
            EndIf
          EndIf
        EndIf
      EndIf
    Else
      ; Befindet sich der Spieler ber der aktuellen Szene, wo sich
      ; keine weitere befindet, dann wird nach einer Kollision mit
      ; unendlich hohen Wnden geprft.
      If \MoveX > 0
        If GetWall(\FieldX + 1, 0) And \X + \Width > (\FieldX + 1) * #FieldSize
          \X     = (\FieldX + 1) * #FieldSize - \Width
          \MoveX = 0
        EndIf
      ElseIf \MoveX < 0
        If GetWall(\FieldX - 1, 0) And \X < \FieldX * #FieldSize
          \X     = \FieldX * #FieldSize
          \MoveX = 0
        EndIf
      EndIf
    EndIf
    
    If Not (FieldCollision(0, 1) Or FieldCollision(-1, 1) Or FieldCollision(1, 1))
      If \Action = #Player_Action_Standing And \ActionTime > 0 And Not (\BottomObject Or \StickedObject)
        ChangePlayerAction(#Player_Action_Falling)
      EndIf
    EndIf
      
    ; Kollision mit einer Decke:
    If Not FieldCollision(0, -1)
      If Not FieldCollision(-1, -1)
        FieldCollision(1, -1)
      EndIf
    EndIf
  
    If \Action = #Player_Action_Clambering
      If Not ObjectQualifiedForClambering()
        ChangePlayerAction(#Player_Action_Falling)
      EndIf
    EndIf
    
    ; Folgende Schleifen gehen alle Objekte der jeweiligen Szene durch, die
    ; sich im BackgroundLayer2 und ForegroundLayer1 befinden:
    \BottomObject  = #Null
    \StickedObject = #Null
    
    *Item = \Scene\BackgroundObjects2\First
    While *Item
      *Object = *Item\Memory
      If Not *Object\Hidden
        If RectangleOverlap(\X, \Y, \Width, \Height, *Object\X, *Object\Y, *Object\Width, *Object\Height)
          Result = _Call(*Object\Callback, *Object, #Object_Collision, #Null)
          If Result = #ObjectDo_Delete
            *Item = ReleaseObject(*Object)
            Continue
          EndIf
        EndIf
      EndIf
      _Move(*Item, N)
    Wend
    
    *Item = \Scene\ForegroundObjects1\First
    While *Item
      *Object = *Item\Memory
      If Not *Object\Hidden
        If RectangleOverlap(\X, \Y, \Width, \Height, *Object\X, *Object\Y, *Object\Width, *Object\Height)
          Result = _Call(*Object\Callback, *Object, #Object_Collision, #Null)
          If Result = #ObjectDo_Delete
            *Item = ReleaseObject(*Object)
            Continue
          EndIf
        EndIf
      EndIf
      _Move(*Item, N)
    Wend
    
    ; Nun wird das 3x3 Kollisionsfeld aktualisert (falls ntig), damit
    ; Kollisionen des nchsten Frames ebenso optimal geprft werden.
    ExaminePlayerField()
  ;}
  
  ;{ Scene changing
    ; In diesem Abschnitt wird geprft, ob sich der Spieler auerhalb der
    ; aktuellen Szene befindet. Hier wird also geprft, ob nun in eine
    ; andere Szene gewechselt werden muss.
    
    SceneChanged = #False
    
    If \X < 0 ;berschreitung des linken Randes
      If \Scene\HorinzontalLoop
        If \X + \Width / 2 < 0
          \X = \Scene\Width * #SceneWidth * #FieldSize - \Width / 2
          ResetCameraX()
        EndIf
      ElseIf PlayerNeighbourScene(-1, 0)
        If \X + \Width / 2 < 0
          \PreviousScene = \Scene
          \Scene         = PlayerNeighbourScene(-1, 0)
          SceneChanged   = #True
          
          \Y        - SceneDeferralY(\PreviousScene, \Scene) * #FieldSize
          Camera\YF - SceneDeferralY(\PreviousScene, \Scene) * #FieldSize
          
          \X + \Scene\Width * #SceneWidth * #FieldSize
          ResetCameraX()
        EndIf
      Else
        \X = 0
      EndIf
    ElseIf \X > Player\Scene\Width * #SceneWidth * #FieldSize - \Width ;berschreitung des rechten Randes
      If \Scene\HorinzontalLoop
        If \X + \Width / 2 > Player\Scene\Width * #SceneWidth * #FieldSize
          \X = 0 - \Width / 2
          ResetCameraX()
        EndIf
      ElseIf PlayerNeighbourScene(1, 0)
        If \X + \Width / 2 > Player\Scene\Width * #SceneWidth * #FieldSize
          \PreviousScene = \Scene
          \Scene         = PlayerNeighbourScene(1, 0)
          SceneChanged   = #True
          
          \Y        - SceneDeferralY(\PreviousScene, \Scene) * #FieldSize
          Camera\YF - SceneDeferralY(\PreviousScene, \Scene) * #FieldSize
          
          \X - \PreviousScene\Width * #SceneWidth * #FieldSize
          ResetCameraX()
        EndIf
      Else
        \X = Player\Scene\Width * #SceneWidth * #FieldSize - \Width
      EndIf
    EndIf
    If \Y < 0 ;berschreitung des oberen Randes
      If \Scene\VerticalLoop
        If \Y + \Height / 2 < 0
          \Y = \Scene\Height * #SceneHeight * #FieldSize - \Height / 2
          ResetCameraY()
        EndIf
      ElseIf PlayerNeighbourScene(0, -1)
        If \Y + \Height / 2 < 0
          \PreviousScene = \Scene
          \Scene         = PlayerNeighbourScene(0, -1)
          SceneChanged   = #True
          
          \X        - SceneDeferralX(\PreviousScene, \Scene) * #FieldSize
          Camera\XF - SceneDeferralX(\PreviousScene, \Scene) * #FieldSize
          
          \Y + \Scene\Height * #SceneHeight * #FieldSize
          ResetCameraY()
        EndIf
      ElseIf \Scene\UnpassableTop Or \Action = #Player_Action_Climbing
        ; Prallt an einer unsichtbaren Decke ab, um ungewnschte
        ; Abkrzungen zu vermeiden:
        \Y     = 0
        \MoveY = 0
      EndIf
    ElseIf \Y > \Scene\Height * #SceneHeight * #FieldSize - \Height ;berschreitung des unteren Randes
      If \Scene\VerticalLoop
        If \Y + \Height / 2 > \Scene\Height * #SceneHeight * #FieldSize
          \Y = 0 - \Height / 2
          ResetCameraY()
        EndIf
      ElseIf PlayerNeighbourScene(0, 1)
        If \Y + \Height / 2 > \Scene\Height * #SceneHeight * #FieldSize
          \PreviousScene = \Scene
          \Scene         = PlayerNeighbourScene(0, 1)
          SceneChanged   = #True
          
          \X        - SceneDeferralX(\PreviousScene, \Scene) * #FieldSize
          Camera\XF - SceneDeferralX(\PreviousScene, \Scene) * #FieldSize
          
          \Y - \PreviousScene\Height * #SceneHeight * #FieldSize
          ResetCameraY()
        EndIf
      ElseIf \Action = #Player_Action_Climbing
        ; Der Spieler soll beim runterklettern gestoppt werden, damit ein unerwarteter
        ; Lebensverlust vermieden wird:
        \Y     = \Scene\Height * #SceneHeight * #FieldSize - \Height
        \MoveY = 0
      Else
        ; Andersfalls wird ein Leben verloren:
        LoseLife()
      EndIf
    EndIf
    
    If SceneChanged = #True
      ChangeScene(\PreviousScene, \Scene)
      ExaminePlayerField()
      
      SceneChanged = #False
    EndIf
  ;}
  
  ;{ Examine shots
    If \ShotDuration
      \ShotDuration - 1
    EndIf
    
    *Item = \Shots\First
    While *Item
      *Shot = *Item\Memory
      
      *Shot\X + *Shot\MoveX
      *Shot\Y + *Shot\MoveY
      
      If ShotCollision(*Shot, 1, 0) Or ShotCollision(*Shot, 0, 0) Or ShotCollision(*Shot, -1, 0)
        EmitShotParticle(*Shot)
        *Item = FreeQueueItem(\Shots, *Item)
        Continue
      ElseIf ShotCollision(*Shot, -1, 1) Or ShotCollision(*Shot, 0, 1) Or ShotCollision(*Shot, 1, 1) Or ShotCollision(*Shot, -1, -1) Or ShotCollision(*Shot, 0, -1) Or ShotCollision(*Shot, 1, -1)
        EmitShotParticle(*Shot)
        *Item = FreeQueueItem(\Shots, *Item)
        Continue
      ElseIf *Shot\X + #ShotSize < 0 Or *Shot\X > \Scene\Width * #ScreenWidth
        *Item = FreeQueueItem(\Shots, *Item)
        Continue
      Else
        UseBlock
          *OtherItem = \Scene\ForegroundObjects1\First
          While *OtherItem
            *Object = *OtherItem\Memory
            If Not *Object\Hidden
              If RectangleOverlap(*Shot\X, *Shot\Y, #ShotSize, #ShotSize, *Object\X, *Object\Y, *Object\Width, *Object\Height)
                Result = _Call(*Object\Callback, *Object, #Object_Hit, *Shot)
                If Result = #ObjectDo_Delete
                  *OtherItem = ReleaseObject(*Object)
                ElseIf Result = #ObjectDo_Collision
                  _Move(*OtherItem, N)
                EndIf
                
                If *Shot\Power < 1.0
                  Break 2
                ElseIf Result
                  Continue
                EndIf
              EndIf
            EndIf
            _Move(*OtherItem, N)
          Wend
          
          *OtherItem = \Scene\BackgroundObjects2\First
          While *OtherItem
            *Object = *OtherItem\Memory
            If Not *Object\Hidden
              If RectangleOverlap(*Shot\X, *Shot\Y, #ShotSize, #ShotSize, *Object\X, *Object\Y, *Object\Width, *Object\Height)
                Result = _Call(*Object\Callback, *Object, #Object_Hit, *Shot)
                If Result = #ObjectDo_Delete
                  *OtherItem = ReleaseObject(*Object)
                ElseIf Result = #ObjectDo_Collision
                  _Move(*OtherItem, N)
                EndIf
                
                If *Shot\Power < 1.0
                  Break 2
                ElseIf Result
                  Continue
                EndIf
              EndIf
            EndIf
            _Move(*OtherItem, N)
          Wend
        EndBlock
        
        If *Shot\Power < 1.0
          EmitShotParticle(*Shot)
          *Item  = FreeQueueItem(\Shots, *Item)
          Result = #False
          Continue
        Else
          If ExamineAnimation(*Shot\Animation)
            *Shot\Animation\Duration = 3
          EndIf
        EndIf
      EndIf 
      
      _Move(*Item, N)
    Wend
  ;}
  
  ;{ Examine score
    If \MultiplierDuration <= 0
      \Multiplier        = 1
      \MultiplierCounter = 0
    Else
      \MultiplierDuration - 1
    EndIf
    
    If \MultiplierChangedTime
      \MultiplierChangedTime - 1
    EndIf
  ;}

  EndWith
EndProcedure
Procedure.i DisplayPlayer()
  With Player
  
  If Not \LostLiveTime
    ; Blur:
    DisplayBlur(\Blur)
    
    ; Spieler befindet sich ber der Szene:
    If \Y + \Height <= Camera\Y
      DisplaySprite3D(\SignSprite3D, Int(\X + \Width/2) - #FieldSize/2 - Camera\X, 2 - Camera\Y)
    Else
      If Not (\HitTime And \HitTime % 5 < 3)
        DisplayAnimation(\Animation, Int(\X) - Camera\X, Int(\Y) - Camera\Y)
      EndIf
    EndIf
    
    ; Ldt gerade eine Leuchtkugel auf:
    If \ShotPower
      If \ShotPower = #Player_MaximalShotPower
        DisplayWhiteLight(\X + \Width / 2, \Y + \Height / 2, 200 + Random(55))
      Else
        DisplayWhiteLight(\X + \Width / 2, \Y + \Height / 2, 255.0 / (#Player_MaximalShotPower - 1) * (\ShotPower - 1))
      EndIf
    EndIf
    
    If \LumiLight
      ; Spieler benutzt den Lumi Flight:
      DisplayWhiteLight(\X + \Width / 2, \Y + \Height / 2, \LumiLight)
    ElseIf \Shield
      ; Spieler besitzt ein Schild:
      If \Shield > 50 Or \Shield % 9 < 5
        Sprite3DBlendingMode(5, 2)
        DisplaySprite3D(ShieldSprite\Sprite3D, \X + \Width/2 - 22/2 - Camera\X, \Y + \Height/2 - 22/2 - Camera\Y, 180 + Sin(\Shield * 0.05) * (255 - 180))
        Sprite3DBlendingMode(5, 6)
      EndIf
    EndIf
  Else
    If \LostLiveTime >= #Player_LostLiveTime - #Player_LostLiveAnimationDuration * Player\LostLiveAnimation\Frameset\FramesPerSet
      DisplayAnimation(\LostLiveAnimation, Int(\X + \Width/2) - #FieldSize/2 - Camera\X, Int(\Y + \Height/2) - #FieldSize/2 - Camera\Y)
      If ExamineAnimation(\LostLiveAnimation)
        \LostLiveAnimation\Duration = #Player_LostLiveAnimationDuration
      EndIf
    EndIf
  EndIf
  
  EndWith
EndProcedure

Procedure.i InitializePlayer()
  With Player
  
  Protected Index.i
  
  \Width  = 12
  \Height = 20
  
  \Animation          = CreateAnimation(LoadAnimationFrameset("Player\Player", \Width, \Height))
  \Animation\Duration = 1
  
  ; Fhigkeiten setzen:
  \CanClimb      = #False
  \CanDoubleJump = #False
  \CanFly        = #False
  \Speed         = 2.0
  \Mass          = 1.0
  \MoveFriction  = 0.0
  \MoveInfluence = 1.0
  \MaxMoveX      = 100.0
  
  ; Energie:
  \Lives     = 3
  \MaxEnergy = 5
  \Energy    = \MaxEnergy
  
  ; Schiesen:
  \ShootMode    = 0
  \ShootByMouse = #Disabled
  For Index = 0 To 2
    \ShotFrameset[Index] = LoadAnimationFrameset("Player\Shot" + Str(Index + 1), 20, 7)
  Next
  \Shots = CreateQueue(@FreeShot())
  
  \LostLiveTime      = 0
  \LostLiveAnimation = CreateAnimation(LoadAnimationFrameset("Player\LostLive", #FieldSize, #FieldSize))
  
  \EnergySprite     = LoadSprite(#PB_Any, "Graphics\Player\Energy.png", #PB_Sprite_Memory|#PB_Sprite_Texture|#PB_Sprite_AlphaBlending)
  \EnergySprite3D   = CreateSprite3D(#PB_Any, \EnergySprite)
  \NoEnergySprite   = LoadSprite(#PB_Any, "Graphics\Player\NoEnergy.png", #PB_Sprite_Memory|#PB_Sprite_Texture|#PB_Sprite_AlphaBlending)
  \NoEnergySprite3D = CreateSprite3D(#PB_Any, \NoEnergySprite)
  
  \SignSprite   = LoadSprite(#PB_Any, "Graphics\Player\Sign.png", #PB_Sprite_Memory|#PB_Sprite_Texture|#PB_Sprite_AlphaBlending)
  \SignSprite3D = CreateSprite3D(#PB_Any, \SignSprite)
  
  \Blur = CreateBlur(9, 1)
  
  EndWith
EndProcedure
Procedure.i LoadPlayerSettings()
  With Player
  
  \Lives     = Challange\Lives
  \MaxEnergy = Challange\MaxEnergy
  \Energy    = Challange\Energy
  
  \CanDoubleJump = Challange\CanDoubleJump
  \CanClimb      = Challange\CanClimb
  \CanLumiFlight = Challange\CanLumiFlight
  \CanFly        = Challange\CanFly
  \CanShoot      = Challange\CanShoot
  \CanLoadShot   = Challange\CanLoadShot
  
  EndWith
EndProcedure

; IDE Options = PureBasic 4.51 (Windows - x86)
; CursorPosition = 238
; FirstLine = 186
; Folding = 90r
; EnableXP