;**************************************************************
;     Scene
;**************************************************************
;   Copyright (c) 2010 Josef Sniatecki
;
; This program is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License
; As published by the Free Software Foundation; either version
; 3 of the License, Or (at your option) any later version.
;
; This program is distributed in the hope that it will be
; useful, but WITHOUT ANY WARRANTY; without even the implied
; warranty of MERCHANTABILITY Or FITNESS For A PARTICULAR
; PURPOSE. See the GNU General Public License For more details.
;
; You should have received a copy of the GNU General Public
; License along With this program;
; If Not, see "http://www.gnu.org/licenses/".
;**************************************************************



CompilerSelect #Mode
  CompilerCase #Mode_Game ;{
;******************************************
; Game
;******************************************

Structure Table
  *Scene.Scene
  
  *BackgroundTiles1
  *BackgroundTiles2
  *BackgroundTiles3
  *Grounds
  *Tiles
  *ForegroundTiles
  *Objects
EndStructure

Global Table.Table

Macro GetWall(X, Y)
  PeekI(Table\Tiles + ((Y) * Table\Scene\Width * #SceneWidth + (X)) * SizeOf(Integer))
EndMacro
Macro GetGround(X, Y)
  PeekI(Table\Grounds + ((Y) * Table\Scene\Width * #SceneWidth + (X)) * SizeOf(Integer))
EndMacro
Macro GetObject(X, Y)
  PeekI(Table\Objects + ((Y) * Table\Scene\Width * #SceneWidth + (X)) * SizeOf(Integer))
EndMacro

Procedure.i CreateScene(Width.i = 1, Height.i = 1)
  Protected *Scene.Scene
  
  *Scene = _New(Scene)
  
  ; Liste fr Vorder- und Hintergrundobjekte erstellen:
  *Scene\BackgroundObjects1 = CreateQueue(@FreeObject())
  *Scene\BackgroundObjects2 = CreateQueue(@FreeObject())
  *Scene\Objects            = CreateQueue(@FreeObject())
  *Scene\ForegroundObjects1 = CreateQueue(@FreeObject())
  *Scene\ForegroundObjects2 = CreateQueue(@FreeObject())
  
  ; Array an Feldern erstellen:
  *Scene\Width  = Width
  *Scene\Height = Height     
  
  ProcedureReturn *Scene
EndProcedure
Procedure.i DisplayScene(DisplayPlayer.i = #True)
  ; Hier wird eine Szene nur angezeigt und nicht abgespielt.
  
  Static *Sprite.Integer, Power.f
  Static *Item.QueueItem, *Tile.Tile, *Object.Object, *Shot.Shot, *Particle.Particle
  Static X.i, Y.i, MinX.i, MinY.i, MaxX.i, MaxY.i, Width.i
  
  ; Beim Rendern werden mehrere Arrays (die Ebenen B1, B2, B3, W, G, F) durchlaufen, die
  ; jeweils Sprites aller Felder der aktuellen Szene speichern. Damit der Durchlauf so
  ; schnell wie mglich ist, werden folgende Grenzen fr die Iteratoren "X" und "Y" mittels
  ; der aktuellen Position der Kamera festgelegt:
  
  MinX = Camera\X / #FieldSize - 1                : _Min(MinX, 0)
  MinY = Camera\Y / #FieldSize - 1                : _Min(MinY, 0)
  MaxX = (Camera\X + #ScreenWidth)  / #FieldSize  : _Max(MaxX, Player\Scene\Width * #SceneWidth - 1)
  MaxY = (Camera\Y + #ScreenHeight) / #FieldSize  : _Max(MaxY, Player\Scene\Height * #SceneHeight - 1)
  
  ; Der Zugriff auf ein Array-Element erfolgt manuell, weshalb die Breite (in Feldern) der
  ; Szene bentigt wird:
  Width = Player\Scene\Width * #SceneWidth
  
  For Y = MinY To MaxY
    For X = MinX To MaxX
      *Sprite = Table\BackgroundTiles1 + (Y * Width + X) * SizeOf(Integer)
      If *Sprite\i
        DisplaySprite3D(*Sprite\i, X * #FieldSize - Camera\X, Y * #FieldSize - Camera\Y)
      EndIf
      *Sprite = Table\BackgroundTiles2 + (Y * Width + X) * SizeOf(Integer)
      If *Sprite\i
        DisplaySprite3D(*Sprite\i, X * #FieldSize - Camera\X, Y * #FieldSize - Camera\Y)
      EndIf
    Next
  Next
  
  *Item = Player\Scene\BackgroundObjects1\First
  While *Item
    *Object = *Item\Memory
    If Not *Object\Hidden
      _Call(*Object\Callback, *Object, #Object_Display, 0)
    EndIf
    _Move(*Item, N)
  Wend
  
  For Y = MinY To MaxY
    For X = MinX To MaxX
      *Sprite = Table\BackgroundTiles3 + (Y * Width + X) * SizeOf(Integer)
      If *Sprite\i
        DisplaySprite3D(*Sprite\i, X * #FieldSize - Camera\X, Y * #FieldSize - Camera\Y)
      EndIf
    Next
  Next
  
  *Item = Player\Scene\BackgroundObjects2\First
  While *Item
    *Object = *Item\Memory
    If Not *Object\Hidden
      _Call(*Object\Callback, *Object, #Object_Display, 0)
    EndIf
    _Move(*Item, N)
  Wend
  
  *Item = Player\Scene\Objects\First
  While *Item
    *Object = *Item\Memory
    If Not *Object\Hidden
      _Call(*Object\Callback, *Object, #Object_Display, 0)
    EndIf
    _Move(*Item, N)
  Wend
  
  For Y = MinY To MaxY
    For X = MinX To MaxX
      *Sprite = Table\Grounds + (Y * Width + X) * SizeOf(Integer)
      If *Sprite\i
        DisplaySprite3D(*Sprite\i, X * #FieldSize - Camera\X, Y * #FieldSize - Camera\Y)
      EndIf
    Next
  Next
  
  If DisplayPlayer
    DisplayPlayer()
  EndIf
  
  *Item = Player\Shots\First
  While *Item
    *Shot = *Item\Memory
    Power = *Shot\Power
    If Power <= #Player_MaximalShotPower
      DisplaySmallWhiteLight(*Shot\X + 8 * Sign(*Shot\MoveX), *Shot\Y + 7/2, (255.0 - 50.0)/(#Player_MaximalShotPower) * Power + Random(50))
    ElseIf Power <= #Player_MaximalShotPower * 2
      DisplaySmallWhiteLight(*Shot\X + 8 * Sign(*Shot\MoveX), *Shot\Y + 7/2, (255.0 - 50.0)/(#Player_MaximalShotPower) * (Power - #Player_MaximalShotPower) + Random(50))
    ElseIf Power <= #Player_MaximalShotPower * 3
      DisplaySmallWhiteLight(*Shot\X + 8 * Sign(*Shot\MoveX), *Shot\Y + 7/2, (255.0 - 50.0)/(#Player_MaximalShotPower) * (Power - #Player_MaximalShotPower * 2) + Random(50))
    EndIf
    DisplayAnimation(*Shot\Animation, *Shot\X - 20/2 - Camera\X, *Shot\Y - Camera\Y)
    _Move(*Item, N)
  Wend
  
  For Y = MinY To MaxY
    For X = MinX To MaxX
      *Sprite = Table\Tiles + (Y * Width + X) * SizeOf(Integer)
      If *Sprite\i
        DisplaySprite3D(*Sprite\i, X * #FieldSize - Camera\X, Y * #FieldSize - Camera\Y)
      EndIf
    Next
  Next
  
  *Item = Player\Scene\ForegroundObjects1\First
  While *Item
    *Object = *Item\Memory
    If Not *Object\Hidden
      _Call(*Object\Callback, *Object, #Object_Display, 0)
    EndIf
    _Move(*Item, N)
  Wend
  
  *Item = *Particles\First
  While *Item
    *Particle = *Item\Memory
    DisplayAnimation(*Particle\Animation, *Particle\X - Camera\X, *Particle\Y - Camera\Y, *Particle\Transparency)
    _Move(*Item, N)
  Wend
  
  For Y = MinY To MaxY
    For X = MinX To MaxX
      *Sprite = Table\ForegroundTiles + (Y * Width + X) * SizeOf(Integer)
      If *Sprite\i
        DisplaySprite3D(*Sprite\i, X * #FieldSize - Camera\X, Y * #FieldSize - Camera\Y)
      EndIf
    Next
  Next
  
  *Item = Player\Scene\ForegroundObjects2\First
  While *Item
    *Object = *Item\Memory
    If Not *Object\Hidden
      _Call(*Object\Callback, *Object, #Object_Display, 0)
    EndIf
    _Move(*Item, N)
  Wend
  
  DisplayTinyMessages(#False)
EndProcedure
Procedure.i PlayAndDisplayScene(DisplayPlayer.i = #True)
  ; Diese Prozedur ruft alle Methoden auf, um eine Szene anzuzeigen. Zugleich werden
  ; alle Objekte aufgerufen.
  
  Static *Sprite.Integer, Result.i, Power.f
  Static *Item.QueueItem, *Tile.Tile, *Object.Object, *Shot.Shot, *Particle.Particle
  Static X.i, Y.i, MinX.i, MinY.i, MaxX.i, MaxY.i, Width.i
  
  ; Beim Rendern werden mehrere Arrays (die Ebenen B1, B2, B3, W, G, F) durchlaufen, die
  ; jeweils Sprites aller Felder der aktuellen Szene speichern. Damit der Durchlauf so
  ; schnell wie mglich ist, werden folgende Grenzen fr die Iteratoren "X" und "Y" mittels
  ; der aktuellen Position der Kamera festgelegt:
  MinX = Camera\X / #FieldSize - 1                : _Min(MinX, 0)
  MinY = Camera\Y / #FieldSize - 1                : _Min(MinY, 0)
  MaxX = (Camera\X + #ScreenWidth)  / #FieldSize  : _Max(MaxX, Player\Scene\Width * #SceneWidth - 1)
  MaxY = (Camera\Y + #ScreenHeight) / #FieldSize  : _Max(MaxY, Player\Scene\Height * #SceneHeight - 1)
  
  ; Der Zugriff auf ein Array-Element erfolgt manuell, weshalb die Breite (in Feldern) der
  ; Szene bentigt wird:
  Width = Player\Scene\Width * #SceneWidth
  
  For Y = MinY To MaxY
    For X = MinX To MaxX
      *Sprite = Table\BackgroundTiles1 + (Y * Width + X) * SizeOf(Integer)
      If *Sprite\i
        DisplaySprite3D(*Sprite\i, X * #FieldSize - Camera\X, Y * #FieldSize - Camera\Y)
      EndIf
      *Sprite = Table\BackgroundTiles2 + (Y * Width + X) * SizeOf(Integer)
      If *Sprite\i
        DisplaySprite3D(*Sprite\i, X * #FieldSize - Camera\X, Y * #FieldSize - Camera\Y)
      EndIf
    Next
  Next
  
  *Item = Player\Scene\BackgroundObjects1\First
  While *Item
    *Object = *Item\Memory
    If Not *Object\Hidden
      _Call(*Object\Callback, *Object, #Object_Display, 0)
      Result = _Call(*Object\Callback, *Object, #Object_Frame, 0)
      If Result
        Select Result
          Case #ObjectDo_Delete
            *Item = ReleaseObject(*Object)
            Continue
        EndSelect
      EndIf
    EndIf
    _Move(*Item, N)
  Wend
  
  For Y = MinY To MaxY
    For X = MinX To MaxX
      *Sprite = Table\BackgroundTiles3 + (Y * Width + X) * SizeOf(Integer)
      If *Sprite\i
        DisplaySprite3D(*Sprite\i, X * #FieldSize - Camera\X, Y * #FieldSize - Camera\Y)
      EndIf
    Next
  Next
  
  *Item = Player\Scene\BackgroundObjects2\First
  While *Item
    *Object = *Item\Memory
    If Not *Object\Hidden
      _Call(*Object\Callback, *Object, #Object_Display, 0)
      Result = _Call(*Object\Callback, *Object, #Object_Frame, 0)
      If Result
        Select Result
          Case #ObjectDo_Delete
            *Item = ReleaseObject(*Object)
            Continue
        EndSelect
      EndIf
    EndIf
    _Move(*Item, N)
  Wend
  
  *Item = Player\Scene\Objects\First
  While *Item
    *Object = *Item\Memory
    If Not *Object\Hidden
      _Call(*Object\Callback, *Object, #Object_Display, 0)
      Result = _Call(*Object\Callback, *Object, #Object_Frame, 0)
      If Result
        Select Result
          Case #ObjectDo_Delete
            *Item = ReleaseObject(*Object)
            Continue
        EndSelect
      EndIf
    EndIf
    _Move(*Item, N)
  Wend
  
  For Y = MinY To MaxY
    For X = MinX To MaxX
      *Sprite = Table\Grounds + (Y * Width + X) * SizeOf(Integer)
      If *Sprite\i
        DisplaySprite3D(*Sprite\i, X * #FieldSize - Camera\X, Y * #FieldSize - Camera\Y)
      EndIf
    Next
  Next
  
  If DisplayPlayer
    DisplayPlayer()
  EndIf
  
  *Item = Player\Shots\First
  While *Item
    *Shot = *Item\Memory
    Power = *Shot\Power
    If Power <= #Player_MaximalShotPower
      DisplaySmallWhiteLight(*Shot\X + 8 * Sign(*Shot\MoveX), *Shot\Y + 7/2, (255.0 - 50.0)/(#Player_MaximalShotPower) * Power + Random(50))
    ElseIf Power <= #Player_MaximalShotPower * 2
      DisplaySmallWhiteLight(*Shot\X + 8 * Sign(*Shot\MoveX), *Shot\Y + 7/2, (255.0 - 50.0)/(#Player_MaximalShotPower) * (Power - #Player_MaximalShotPower) + Random(50))
    ElseIf Power <= #Player_MaximalShotPower * 3
      DisplaySmallWhiteLight(*Shot\X + 8 * Sign(*Shot\MoveX), *Shot\Y + 7/2, (255.0 - 50.0)/(#Player_MaximalShotPower) * (Power - #Player_MaximalShotPower * 2) + Random(50))
    EndIf
    DisplayAnimation(*Shot\Animation, *Shot\X - 20/2 - Camera\X, *Shot\Y - Camera\Y)
    _Move(*Item, N)
  Wend
  
  For Y = MinY To MaxY
    For X = MinX To MaxX
      *Sprite = Table\Tiles + (Y * Width + X) * SizeOf(Integer)
      If *Sprite\i
        DisplaySprite3D(*Sprite\i, X * #FieldSize - Camera\X, Y * #FieldSize - Camera\Y)
      EndIf
    Next
  Next
  
  *Item = Player\Scene\ForegroundObjects1\First
  While *Item
    *Object = *Item\Memory
    If Not *Object\Hidden
      _Call(*Object\Callback, *Object, #Object_Display, 0)
      Result = _Call(*Object\Callback, *Object, #Object_Frame, 0)
      If Result
        Select Result
          Case #ObjectDo_Delete
            *Item = ReleaseObject(*Object)
            Continue
        EndSelect
      EndIf
    EndIf
    _Move(*Item, N)
  Wend
  
  *Item = *Particles\First
  While *Item
    *Particle = *Item\Memory
    DisplayAnimation(*Particle\Animation, *Particle\X - Camera\X, *Particle\Y - Camera\Y, *Particle\Transparency)
    _Move(*Item, N)
  Wend
  
  For Y = MinY To MaxY
    For X = MinX To MaxX
      *Sprite = Table\ForegroundTiles + (Y * Width + X) * SizeOf(Integer)
      If *Sprite\i
        DisplaySprite3D(*Sprite\i, X * #FieldSize - Camera\X, Y * #FieldSize - Camera\Y)
      EndIf
    Next
  Next
  
  *Item = Player\Scene\ForegroundObjects2\First
  While *Item
    *Object = *Item\Memory
    If Not *Object\Hidden
      _Call(*Object\Callback, *Object, #Object_Display, 0)
      Result = _Call(*Object\Callback, *Object, #Object_Frame, 0)
      If Result
        Select Result
          Case #ObjectDo_Delete
            *Item = ReleaseObject(*Object)
            Continue
        EndSelect
      EndIf
    EndIf
    _Move(*Item, N)
  Wend
  
  DisplayTinyMessages()
EndProcedure
Procedure.i PlayScene()
  ; Diese Prozedur ruft alle Methoden auf, um eine Szene anzuzeigen. Zugleich werden
  ; alle Objekte aufgerufen.
  
  Static Result.i
  Static *Item.QueueItem, *Object.Object
  
  *Item = Player\Scene\BackgroundObjects1\First
  While *Item
    *Object = *Item\Memory
    If Not *Object\Hidden
      Result = _Call(*Object\Callback, *Object, #Object_Frame, 0)
      If Result
        Select Result
          Case #ObjectDo_Delete
            *Item = ReleaseObject(*Object)
            Continue
        EndSelect
      EndIf
    EndIf
    _Move(*Item, N)
  Wend
  
  *Item = Player\Scene\BackgroundObjects2\First
  While *Item
    *Object = *Item\Memory
    If Not *Object\Hidden
      Result = _Call(*Object\Callback, *Object, #Object_Frame, 0)
      If Result
        Select Result
          Case #ObjectDo_Delete
            *Item = ReleaseObject(*Object)
            Continue
        EndSelect
      EndIf
    EndIf
    _Move(*Item, N)
  Wend
  
  *Item = Player\Scene\Objects\First
  While *Item
    *Object = *Item\Memory
    If Not *Object\Hidden
      Result = _Call(*Object\Callback, *Object, #Object_Frame, 0)
      If Result
        Select Result
          Case #ObjectDo_Delete
            *Item = ReleaseObject(*Object)
            Continue
        EndSelect
      EndIf
    EndIf
    _Move(*Item, N)
  Wend
  
  *Item = Player\Scene\ForegroundObjects1\First
  While *Item
    *Object = *Item\Memory
    If Not *Object\Hidden
      Result = _Call(*Object\Callback, *Object, #Object_Frame, 0)
      If Result
        Select Result
          Case #ObjectDo_Delete
            *Item = ReleaseObject(*Object)
            Continue
        EndSelect
      EndIf
    EndIf
    _Move(*Item, N)
  Wend
  
  *Item = Player\Scene\ForegroundObjects2\First
  While *Item
    *Object = *Item\Memory
    If Not *Object\Hidden
      Result = _Call(*Object\Callback, *Object, #Object_Frame, 0)
      If Result
        Select Result
          Case #ObjectDo_Delete
            *Item = ReleaseObject(*Object)
            Continue
        EndSelect
      EndIf
    EndIf
    _Move(*Item, N)
  Wend
  
  ExamineTinyMessages()
EndProcedure

Procedure.i GetScene(X.i, Y.i)
  ProcedureReturn PeekI(Challange\Scenes + (Y * (Challange\Width + 1) + X) * SizeOf(Integer))
EndProcedure
Procedure.i PlayerNeighbourScene(DX.i, DY.i)
  Protected X.i, Y.i
  
  X = PlayerSceneX() + DX
  Y = PlayerSceneY() + DY
  
  If X => 0 And X <= Challange\Width And Y => 0 And Y <= Challange\Height
    ProcedureReturn GetScene(X, Y)
  Else
    ProcedureReturn #Null
  EndIf
EndProcedure

Procedure.i PlayerRelativeSceneX()
  Protected X.i

  X = Player\Scene\X + (Player\X + Player\Width / 2 - (#SceneWidth * #FieldSize) / 2) / (#SceneWidth * #FieldSize)
  
  If X < Player\Scene\X
    X = Player\Scene\X
  ElseIf X > Player\Scene\X + Player\Scene\Width - 1
    X = Player\Scene\X + Player\Scene\Width - 1
  EndIf
  
  ProcedureReturn X - Player\Scene\X
EndProcedure
Procedure.i PlayerRelativeSceneY()
  Protected Y.i
  
  Y = Player\Scene\Y + (Player\Y + Player\Height / 2 - (#SceneHeight*#FieldSize) / 2) / (#SceneHeight * #FieldSize)
  
  If Y < Player\Scene\Y
    Y = Player\Scene\Y
  ElseIf Y > Player\Scene\Y + Player\Scene\Height - 1
    Y = Player\Scene\Y + Player\Scene\Height - 1
  EndIf
  
  ProcedureReturn Y - Player\Scene\Y
EndProcedure
Procedure.i PlayerSceneX()
  Protected X.i

  X = Player\Scene\X + (Player\X + Player\Width / 2 - (#SceneWidth * #FieldSize) / 2) / (#SceneWidth * #FieldSize)
  
  If X < Player\Scene\X
    X = Player\Scene\X
  ElseIf X > Player\Scene\X + Player\Scene\Width - 1
    X = Player\Scene\X + Player\Scene\Width - 1
  EndIf
  
  ProcedureReturn X
EndProcedure
Procedure.i PlayerSceneY()
  Protected Y.i
  
  Y = Player\Scene\Y + (Player\Y + Player\Height / 2 - (#SceneHeight * #FieldSize) / 2) / (#SceneHeight * #FieldSize)
  
  If Y < Player\Scene\Y
    Y = Player\Scene\Y
  ElseIf Y > Player\Scene\Y + Player\Scene\Height - 1
    Y = Player\Scene\Y + Player\Scene\Height - 1
  EndIf
  
  ProcedureReturn Y
EndProcedure

Procedure.i SceneDeferralX(*Scene.Scene, *OtherScene.Scene)
  ProcedureReturn (*OtherScene\X - *Scene\X) * #SceneWidth
EndProcedure
Procedure.i SceneDeferralY(*Scene.Scene, *OtherScene.Scene)
  ProcedureReturn (*OtherScene\Y - *Scene\Y) * #SceneHeight
EndProcedure

Procedure.i FreeTable()
  With Table
    \Scene = #Null
    
    FreeMemory(\BackgroundTiles1)
    FreeMemory(\BackgroundTiles2)
    FreeMemory(\BackgroundTiles3)
    FreeMemory(\Tiles)
    FreeMemory(\Grounds)
    FreeMemory(\ForegroundTiles)
    FreeMemory(\Objects)
  EndWith
EndProcedure
Procedure.i CreateTable(*Scene.Scene)
  Protected *Cell.Integer, *Tile.Tile, *Item.QueueItem, *Object.Object
  Protected Width.i, Height.i, X.i, Y.i
  
  Width  = #SceneWidth * *Scene\Width
  Height = #SceneHeight * *Scene\Height
  
  With Table
    \Scene = *Scene
    
    \BackgroundTiles1 = AllocateMemory(Width * Height * SizeOf(Integer))
    \BackgroundTiles2 = AllocateMemory(Width * Height * SizeOf(Integer))
    \BackgroundTiles3 = AllocateMemory(Width * Height * SizeOf(Integer))
    \Tiles            = AllocateMemory(Width * Height * SizeOf(Integer))
    \Grounds          = AllocateMemory(Width * Height * SizeOf(Integer))
    \ForegroundTiles  = AllocateMemory(Width * Height * SizeOf(Integer))
    \Objects          = AllocateMemory(Width * Height * SizeOf(Integer))
    
    If *Scene\BackgroundTiles1
      *Cell = *Scene\BackgroundTiles1
      While *Cell\i
        *Tile = *Cell\i
        PokeI(\BackgroundTiles1 + (*Tile\Y * Width + *Tile\X) * SizeOf(Integer), *Tile\Sprite)
        *Cell + SizeOf(Integer)
      Wend
    EndIf
    If *Scene\BackgroundTiles2
      *Cell = *Scene\BackgroundTiles2
      While *Cell\i
        *Tile = *Cell\i
        PokeI(\BackgroundTiles2 + (*Tile\Y * Width + *Tile\X) * SizeOf(Integer), *Tile\Sprite)
        *Cell + SizeOf(Integer)
      Wend
    EndIf
    If *Scene\BackgroundTiles3
      *Cell = *Scene\BackgroundTiles3
      While *Cell\i
        *Tile = *Cell\i
        PokeI(\BackgroundTiles3 + (*Tile\Y * Width + *Tile\X) * SizeOf(Integer), *Tile\Sprite)
        *Cell + SizeOf(Integer)
      Wend
    EndIf
    If *Scene\Tiles
      *Cell = *Scene\Tiles
      While *Cell\i
        *Tile = *Cell\i
        PokeI(\Tiles + (*Tile\Y * Width + *Tile\X) * SizeOf(Integer), *Tile\Sprite)
        *Cell + SizeOf(Integer)
      Wend
    EndIf
    If *Scene\Grounds
      *Cell = *Scene\Grounds
      While *Cell\i
        *Tile = *Cell\i
        PokeI(\Grounds + (*Tile\Y * Width + *Tile\X) * SizeOf(Integer), *Tile\Sprite)
        *Cell + SizeOf(Integer)
      Wend
    EndIf
    If *Scene\ForegroundTiles
      *Cell = *Scene\ForegroundTiles
      While *Cell\i
        *Tile = *Cell\i
        PokeI(\ForegroundTiles + (*Tile\Y * Width + *Tile\X) * SizeOf(Integer), *Tile\Sprite)
        *Cell + SizeOf(Integer)
      Wend
    EndIf
    
    If *Scene\Objects
      *Item = *Scene\Objects\First
      While *Item
        *Object = *Item\Memory
        X       = *Object\X / #FieldSize
        Y       = *Object\Y / #FieldSize
        PokeI(\Objects + (Y * Width + X) * SizeOf(Integer), *Object)
        _Move(*Item, N)
      Wend
    EndIf
  EndWith
EndProcedure

Procedure.i WakeUpScene(*Scene.Scene)
  Protected *Queue.Queue, *Item.QueueItem, *Object.Object, Layer.i
  
  *Scene + #BackgroundObject1*SizeOf(Integer)
  For Layer = #BackgroundObject1 To #ForegroundObject2
    *Queue = PeekI(*Scene)
    *Item = *Queue\First
    While *Item
      *Object = *Item\Memory
      _Call(*Object\Callback, *Object, #Object_PlayerEnteredScene, #Null)
      _Move(*Item, N)
    Wend
    *Scene + SizeOf(Integer)
  Next
EndProcedure
Procedure.i FreezeScene(*Scene.Scene)
  Protected *Queue.Queue, *Item.QueueItem, *Object.Object, Layer.i
  
  *Scene + #BackgroundObject1*SizeOf(Integer)
  For Layer = #BackgroundObject1 To #ForegroundObject2
    *Queue = PeekI(*Scene)
    *Item = *Queue\First
    While *Item
      *Object = *Item\Memory
      _Call(*Object\Callback, *Object, #Object_PlayerLeftScene, #Null)
      _Move(*Item, N)
    Wend
    *Scene + SizeOf(Integer)
  Next
EndProcedure
Procedure.i ChangeScene(*PreviousScene.Scene, *NextScene.Scene)
  FreezeScene(*PreviousScene)
  FreeTable()
  
  ClearQueue(Player\Shots)
  ClearQueue(*Particles)
  
  CreateTable(*NextScene)
  WakeUpScene(*NextScene)
  
  If *PreviousScene\Act <> *NextScene\Act
    If *PreviousScene\Act
      RemoveParallelAct(*PreviousScene\Act)
    EndIf
    If *NextScene\Act
      AddParallelAct(*NextScene\Act)
    EndIf
  EndIf
  
  If Challange\Script\SceneChanged
    CallFunctionFast(Challange\Script\SceneChanged, *PreviousScene, *NextScene)
  EndIf
EndProcedure

;}
  CompilerCase #Mode_Editor ;{
;******************************************
; Editor
;******************************************

Structure Table
  *BackgroundTiles1
  *BackgroundTiles2
  *BackgroundTiles3
  *Tiles
  *Grounds
  *ForegroundTiles
  *BackgroundObjects1
  *BackgroundObjects2
  *Objects
  *ForegroundObjects1
  *ForegroundObjects2
EndStructure
Structure TableCount
  *BackgroundTile1Count
  *BackgroundTile2Count
  *BackgroundTile3Count
  *TileCount
  *GroundCount
  *ForegroundTileCount
  *BackgroundObject1Count
  *BackgroundObject2Count
  *ObjectCount
  *ForegroundObject1Count
  *ForegroundObject2Count
EndStructure

Global Table.Table
Global TableCount.TableCount

Procedure.i FreeTable()
  With Table
    FreeMemory(\BackgroundTiles1)
    FreeMemory(\BackgroundTiles2)
    FreeMemory(\BackgroundTiles3)
    FreeMemory(\Tiles)
    FreeMemory(\Grounds)
    FreeMemory(\ForegroundTiles)
    FreeMemory(\BackgroundObjects1)
    FreeMemory(\BackgroundObjects2)
    FreeMemory(\Objects)
    FreeMemory(\ForegroundObjects1)
    FreeMemory(\ForegroundObjects2)
    
    \BackgroundTiles1   = #Null
    \BackgroundTiles2   = #Null
    \BackgroundTiles3   = #Null
    \Tiles              = #Null
    \Grounds            = #Null
    \BackgroundObjects1 = #Null
    \BackgroundObjects2 = #Null
    \Objects            = #Null
    \ForegroundObjects1 = #Null
    \ForegroundObjects2 = #Null
  EndWith
  
  With TableCount
    FreeMemory(\BackgroundTile1Count)
    FreeMemory(\BackgroundTile2Count)
    FreeMemory(\BackgroundTile3Count)
    FreeMemory(\TileCount)
    FreeMemory(\GroundCount)
    FreeMemory(\ForegroundTileCount)
    FreeMemory(\BackgroundObject1Count)
    FreeMemory(\BackgroundObject2Count)
    FreeMemory(\ObjectCount)
    FreeMemory(\ForegroundObject1Count)
    FreeMemory(\ForegroundObject2Count)
  EndWith
EndProcedure
Procedure.i CreateTable()
  Protected *TableArray, *Cell.Integer, Layer.i, *Field.Field
  Protected Width.i, Height.i
  Protected DX.i, DY.i, *SubScene.Scene
  
  Width  = #SceneWidth * Editor\CurrentScene\Width
  Height = #SceneHeight * Editor\CurrentScene\Height
  
  With Table
    ; Neue Arrays fr jede Ebene erstellen:
    For Layer = #FirstLayer To #LastLayer
      PokeI(@Table + Layer * SizeOf(Integer), AllocateMemory(Width * Height * SizeOf(Integer)))
      PokeI(@TableCount + Layer * SizeOf(Integer), AllocateMemory(Editor\CurrentScene\Width * Editor\CurrentScene\Height * SizeOf(Integer)))
    Next
    
    ; Alle Tiles und Objekte in die Arrays schreiben:
    For DY = 0 To Editor\CurrentScene\Height - 1
      For DX = 0 To Editor\CurrentScene\Width - 1
        *SubScene = GetScene(Editor\CurrentScene\X + DX, Editor\CurrentScene\Y + DY)
        
        For Layer = #FirstLayer To #LastLayer
          *TableArray = TableArray(Layer)
          *Cell       = LayerArray(*SubScene, Layer)
          
          ; Jedes gesetzte Tile/Objekt einer Szene aufnehmen:
          While *Cell\i
            *Field = *Cell\i
            PokeI(*TableArray + ((*Field\Y + DY * #SceneHeight) * Width + (*Field\X + DX * #SceneWidth)) * SizeOf(Integer), *Field)
            *Cell + SizeOf(Integer)
          Wend
          
          PokeI(PeekI(@TableCount + Layer * SizeOf(Integer)) + (DY * Editor\CurrentScene\Width + DX) * SizeOf(Integer), LayerSize(*SubScene, Layer))
        Next
      Next
    Next
  EndWith
EndProcedure
Procedure.i SaveTable()
  Protected Width.i, Height.i, X.i, Y.i, DX.i, DY.i, Layer.i
  Protected *Cell.Integer, *Field.Field, *Memory
  Protected *TableArray, *SubScene.Scene, *SceneArray
  
  Width  = #SceneWidth * Editor\CurrentScene\Width
  Height = #SceneHeight * Editor\CurrentScene\Height
  
  With Table
    ; Jede Teilszene speichern:
    For DY = 0 To Editor\CurrentScene\Height - 1
      For DX = 0 To Editor\CurrentScene\Width - 1
        *SubScene = GetScene(Editor\CurrentScene\X + DX, Editor\CurrentScene\Y + DY)
        ; Fr jedes Layer:
        For Layer = #BackgroundTile1 To #ForegroundObject2
          *TableArray = PeekI(@Table + Layer * SizeOf(Integer))
          
          *Memory = LayerArray(*SubScene, Layer)
          FreeMemory(*Memory)
          *SceneArray = AllocateMemory((TableCount(*SubScene, Layer) + 1) * SizeOf(Integer))
          PokeI(*SubScene + Layer * SizeOf(Integer), *SceneArray)
          
          *Cell = *SceneArray
          For Y = DY * #SceneHeight To DY * #SceneHeight + #SceneHeight - 1
            For X = DX * #SceneWidth To DX * #SceneWidth + #SceneWidth - 1
              *Field = PeekI(*TableArray + (Y * Width + X) * SizeOf(Integer))
              If *Field
                *Cell\i = *Field
                *Cell + SizeOf(Integer)
              EndIf
            Next
          Next
        Next
      Next
    Next
  EndWith    
EndProcedure

Procedure.i TableArray(Layer.i)
  ProcedureReturn PeekI(@Table + Layer * SizeOf(Integer))
EndProcedure
Procedure.i TableCount(*Scene.Scene, Layer.i)
  ProcedureReturn PeekI(PeekI(@TableCount + Layer * SizeOf(Integer)) + ((*Scene\Y - *Scene\Main\Y) * *Scene\Main\Width + (*Scene\X - *Scene\Main\X)) * SizeOf(Integer))
EndProcedure

Procedure.i GetField(*Scene.Scene, Layer.i, X.i, Y.i)
  Protected *Cell.Integer, *Field.Field
  
  If *Scene = Editor\CurrentScene
    ProcedureReturn PeekI(PeekI(@Table + Layer * SizeOf(Integer)) + (Y * *Scene\Width * #SceneWidth + X) * SizeOf(Integer))
  Else
    *Cell = LayerArray(*Scene, Layer)
    While *Cell\i
      *Field = *Cell\i
      If *Field\X = X And *Field\Y = Y
        ProcedureReturn *Field
      EndIf
      *Cell + SizeOf(Integer)
    Wend
    
    ProcedureReturn #Null
  EndIf
EndProcedure
Procedure.i SetField(*Scene.Scene, Layer.i, X.i, Y.i, *Field.Field)
  Protected *TableField.Integer, *Count.Integer
  
  If *Scene = Editor\CurrentScene
    *TableField = PeekI(@Table + Layer * SizeOf(Integer)) + (Y * *Scene\Width * #SceneWidth + X) * SizeOf(Integer)
    If Not *TableField\i
      If *Field
        X / #SceneWidth
        Y / #SceneHeight
        *Count = PeekI(@TableCount + Layer * SizeOf(Integer)) + (Y * *Scene\Width + X) * SizeOf(Integer)
        *Count\i + 1
      EndIf
    ElseIf Not *Field
      X / #SceneWidth
      Y / #SceneHeight
      *Count = PeekI(@TableCount + Layer * SizeOf(Integer)) + (Y * *Scene\Width + X) * SizeOf(Integer)
      *Count\i - 1
    EndIf
    *TableField\i = *Field
    ProcedureReturn #True
  Else
    ProcedureReturn #False
  EndIf
EndProcedure

Procedure.i LayerArray(*Scene.Scene, Layer.i)
  ProcedureReturn PeekI(*Scene + Layer * SizeOf(Integer))
EndProcedure
Procedure.i LayerSize(*Scene.Scene, Layer.i)
  ProcedureReturn (MemorySize(PeekI(*Scene + SizeOf(Integer) * Layer)) - 1) / SizeOf(Integer)
EndProcedure

Procedure.i CreateScene(X.i, Y.i, *Background.RegisteredBackground = #Null)
  Protected *Scene.Scene
  
  *Scene      = _New(Scene)
  *Scene\Item = AppendQueueItem(Editor\UsedScenes, *Scene)
  
  ; Arrays erstellen:
  *Scene\BackgroundTiles1   = AllocateMemory(SizeOf(Integer))
  *Scene\BackgroundTiles2   = AllocateMemory(SizeOf(Integer))
  *Scene\BackgroundTiles3   = AllocateMemory(SizeOf(Integer))
  *Scene\Tiles              = AllocateMemory(SizeOf(Integer))
  *Scene\Grounds            = AllocateMemory(SizeOf(Integer))
  *Scene\ForegroundTiles    = AllocateMemory(SizeOf(Integer))
  *Scene\BackgroundObjects1 = AllocateMemory(SizeOf(Integer))
  *Scene\BackgroundObjects2 = AllocateMemory(SizeOf(Integer))
  *Scene\Objects            = AllocateMemory(SizeOf(Integer))
  *Scene\ForegroundObjects1 = AllocateMemory(SizeOf(Integer))
  *Scene\ForegroundObjects2 = AllocateMemory(SizeOf(Integer))
  
  ; Hintergrund festlegen, falls einer gegeben ist:
  If *Background
    *Scene\Background = *Background
    UseRegBackground(*Background)
  EndIf
  
  ; Position und Gre (kann nachtrglich gendert werden) speichern:
  *Scene\X      = X
  *Scene\Y      = Y
  *Scene\Width  = 1
  *Scene\Height = 1
  
  ; Kopf-Szene auf sich selbst referenzieren:
  *Scene\Main = *Scene
  
  ; In die SceneMap eintragen:
  PokeI(Editor\SceneMap + (Y * #SceneMap_Width + X) * SizeOf(Integer), *Scene)
  
  ProcedureReturn *Scene
EndProcedure
Procedure.i GetScene(X.i, Y.i)
  ProcedureReturn PeekI(Editor\SceneMap + (Y * #SceneMap_Width + X) * SizeOf(Integer))
EndProcedure
Procedure.i GetMapBorders(*MinX.Integer, *MinY.Integer, *MaxX.Integer, *MaxY.Integer)
  Static First.i, MinX.i, MinY.i, MaxX.i, MaxY.i, *Scene.Scene, *Item.QueueItem
  
  First = #True
  *Item = Editor\UsedScenes\First
  While *Item
    *Scene = *Item\Memory
    
    If First
      MinX = *Scene\X
      MinY = *Scene\Y
      MaxX = *Scene\X
      MaxY = *Scene\Y
      
      First = #False
    Else
      If MinX > *Scene\X
        MinX = *Scene\X
      ElseIf MaxX < *Scene\X
        MaxX = *Scene\X
      EndIf
      If MinY > *Scene\Y
        MinY = *Scene\Y
      ElseIf MaxY < *Scene\Y
        MaxY = *Scene\Y
      EndIf
    EndIf
    
    _Move(*Item, N)
  Wend
  
  *MinX\i = MinX ;Left
  *MinY\i = MinY ;Top
  *MaxX\i = MaxX ;Right
  *MaxY\i = MaxY ;Bottom
EndProcedure

Procedure.i FreeScene(*Scene.Scene)
  Protected *Cell.Integer, Layer.i
  
  For Layer = #BackgroundTile1 To #ForegroundTile
    *Cell = LayerArray(*Scene, Layer)
    While *Cell\i
      FreeTile(*Cell\i)
      *Cell + SizeOf(Integer)
    Wend
    FreeMemory(LayerArray(*Scene, Layer))
  Next
  For Layer = #BackgroundObject1 To #ForegroundObject2
    *Cell = LayerArray(*Scene, Layer)
    While *Cell\i
      FreeObject(*Cell\i)
      *Cell + SizeOf(Integer)
    Wend
    FreeMemory(LayerArray(*Scene, Layer))
  Next
  
  If *Scene\Background
    ReleaseRegBackground(*Scene\Background)
  EndIf
  
  If *Scene\Act
    ReleaseAct(*Scene\Act)
  EndIf
  
  PokeI(Editor\SceneMap + (*Scene\Y * #SceneMap_Width + *Scene\X) * SizeOf(Integer), #Null)
  
  _Free(*Scene)
  
  ProcedureReturn #True
EndProcedure
Procedure.i RemoveScene(X.i, Y.i)
  ; Lscht eine Szene per Hand. "FreeScene" ist
  ; der eigentliche Dekonstruktor fr Szenen, jedoch nicht
  ; geeignet als Direktive fr den Editor.
  
  Protected IndexX.i, IndexY.i
  Protected *Scene.Scene, *OtherScene.Scene
  
  *Scene = GetScene(X, Y)
  
  ; Eintrag in der SceneMap mit "Null" berschreiben:
  PokeI(Editor\SceneMap + (Y * #SceneMap_Width + X) * SizeOf(Integer), #Null)
  
  ; Falls diese Szene gerade im Editor ausgewhlt ist, muss
  ; der Zeiger "CurrentScene" auf "Null" gesetzt werden:
  If *Scene = Editor\CurrentScene
    Editor\CurrentScene = #Null
  EndIf
  
  ; Unterszenen werden ebenso gelscht:
  If *Scene\Width * *Scene\Height > 1
    For IndexX = X To X + *Scene\Width
      For IndexY = Y To Y + *Scene\Height
        If IndexX <> X And IndexY <> Y
          RemoveScene(IndexX, IndexY)
        EndIf
      Next
    Next
  EndIf
  
  ; Zuletzt den eigentlichen Dekonstruktor aufrufen und
  ; die Szene zugleich aus der Liste "UsedScenes" lschen:
  FreeQueueItem(Editor\UsedScenes, *Scene\Item)
  
  ProcedureReturn #True
EndProcedure

Procedure.i ClearScene()
  Protected Layer.i, *TableArray, X.i, Y.i, *Tile.Tile, *Object.Object, Width.i, Height.i
  Protected *TableCell.Integer
  
  Width = Editor\CurrentScene\Width * #SceneWidth
  Height = Editor\CurrentScene\Height * #SceneHeight
  
  For Layer = #BackgroundTile1 To #ForegroundTile
    *TableCell = TableArray(Layer)
    For Y = 0 To Height - 1
      For X = 0 To Width - 1
        If *TableCell\i
          FreeTile(*TableCell\i)
          *TableCell\i = #Null
        EndIf
        *TableCell + SizeOf(Integer)
      Next
    Next
  Next
  
  For Layer = #BackgroundObject1 To #ForegroundObject2
    *TableCell = TableArray(Layer)
    For Y = 0 To Height - 1
      For X = 0 To Width - 1
        If *TableCell\i
          FreeObject(*TableCell\i)
          *TableCell\i = #Null
        EndIf
        *TableCell + SizeOf(Integer)
      Next
    Next
  Next
  
  ; Anzahl zurcksetzen:
  For Y = 0 To Editor\CurrentScene\Height - 1
    For X = 0 To Editor\CurrentScene\Width - 1
      For Layer = #BackgroundTile1 To #ForegroundObject2
        PokeI(PeekI(@TableCount + Layer * SizeOf(Integer)) + (Y * Editor\CurrentScene\Width + X) * SizeOf(Integer), 0)
      Next
    Next
  Next
  
  ; Der Hintergrund wird nicht gelscht!
EndProcedure
Procedure.i AdaptScene()
  ; Als Hilfe kann diese Prozedur alle am Rand liegende Teile der
  ; Nachbar-Szenen in diese Szene passend kopieren, damit man
  ; sofort erkennen kann, wie man eine Wand, oder andere Gruppen
  ; von Tilese fortfhren mchte.
  
  Protected X.i, Y.i, *OtherScene.Scene, *Tile.Tile, *Object.Object, Layer.i
  
  If Editor\CurrentScene\X > 0
    For Y = 0 To #SceneHeight * Editor\CurrentScene\Height - 1
      *OtherScene = GetScene(Editor\CurrentScene\X - 1, Editor\CurrentScene\Y + Y / #SceneHeight)
      If *OtherScene
        For Layer = #FirstTile To #LastTile
          *Tile = GetTile(*OtherScene, Layer, *OtherScene\Width * #SceneWidth - 1, Y % #SceneHeight)
          If *Tile
            PutTile(Layer, *Tile\Registered, 0, Y)
          EndIf
        Next
      EndIf
    Next
  EndIf
  If Editor\CurrentScene\X + Editor\CurrentScene\Width < #SceneMap_Width
    For Y = 0 To #SceneHeight * Editor\CurrentScene\Height - 1
      *OtherScene = GetScene(Editor\CurrentScene\X + Editor\CurrentScene\Width, Editor\CurrentScene\Y + Y / #SceneHeight)
      If *OtherScene
        For Layer = #FirstTile To #LastTile
          *Tile = GetTile(*OtherScene, Layer, 0, Y % #SceneHeight)
          If *Tile
            PutTile(Layer, *Tile\Registered, Editor\CurrentScene\Width * #SceneWidth - 1, Y)
          EndIf
        Next
      EndIf
    Next
  EndIf
  If Editor\CurrentScene\Y > 0
    For X = 0 To #SceneWidth * Editor\CurrentScene\Width - 1
      *OtherScene = GetScene(Editor\CurrentScene\X + X / #SceneWidth, Editor\CurrentScene\Y - 1)
      If *OtherScene
        For Layer = #FirstTile To #LastTile
          *Tile = GetTile(*OtherScene, Layer, X % #SceneWidth, *OtherScene\Height * #SceneHeight - 1)
          If *Tile
            PutTile(Layer, *Tile\Registered, X, 0)
          EndIf
        Next
      EndIf
    Next
  EndIf
  If Editor\CurrentScene\Y + Editor\CurrentScene\Height < #SceneMap_Height
    For X = 0 To #SceneWidth * Editor\CurrentScene\Width - 1
      *OtherScene = GetScene(Editor\CurrentScene\X + X / #SceneWidth, Editor\CurrentScene\Y + Editor\CurrentScene\Height)
      If *OtherScene
        For Layer = #FirstTile To #LastTile
          *Tile = GetTile(*OtherScene, Layer, X % #SceneWidth, 0)
          If *Tile
            PutTile(Layer, *Tile\Registered, X, Editor\CurrentScene\Height * #SceneHeight - 1)
          EndIf
        Next
      EndIf
    Next
  EndIf
EndProcedure
Procedure.i ResizeScene(Width.i, Height.i)
  Protected IndexX.i, IndexY.i, MaxX.i, MaxY.i, *OtherScene.Scene
  Protected X.i, Y.i
  
  X = Editor\CurrentScene\X
  Y = Editor\CurrentScene\Y
  
  If Editor\CurrentScene\Width <> Width Or Editor\CurrentScene\Height <> Height
    SaveTable()
    FreeTable()
    
    If Editor\CurrentScene\Width < Width
      MaxX = Width - 1
    Else
      MaxX = Editor\CurrentScene\Width - 1
    EndIf
    MaxX + X
    If Editor\CurrentScene\Height < Height
      MaxY = Height - 1
    Else
      MaxY = Editor\CurrentScene\Height - 1
    EndIf
    MaxY + Y
    
    For IndexY = Y To MaxY
      For IndexX = X To MaxX
        If IndexX <> X Or IndexY <> Y
          If Not (IndexX => X + Width Or IndexY => Y + Height) And (IndexX => X + Editor\CurrentScene\Width Or IndexY => Y + Editor\CurrentScene\Height)
            ;Szene liegt innerhalb des neu hinzugefgten Bereiches:
            *OtherScene = GetScene(IndexX, IndexY)
            If *OtherScene And *OtherScene <> *OtherScene\Main And *OtherScene\Main <> Editor\CurrentScene
              MessageRequester("Warning", "The defined size (" + Str(Width) + "|" + Str(Height) + ") includes parts of other groups.")
              CreateTable()
              ProcedureReturn #False
            EndIf
          EndIf
        EndIf
      Next
    Next
    
    For IndexY = Y To MaxY
      For IndexX = X To MaxX
        If IndexX <> X Or IndexY <> Y
          If IndexX => X + Width Or IndexY => Y + Height
            ;Szene liegt auerhalb des neuen Bereiches:
            *OtherScene = GetScene(IndexX, IndexY)
            If *OtherScene
              *OtherScene\Main = *OtherScene
              RemoveEmptyScene(IndexX, IndexY)
            EndIf
          ElseIf IndexX => X + Editor\CurrentScene\Width Or IndexY => Y + Editor\CurrentScene\Height
            ;Szene liegt innerhalb des neu definierten Bereiches:
            *OtherScene = GetScene(IndexX, IndexY)
            If Not *OtherScene
              *OtherScene = CreateScene(IndexX, IndexY, Editor\CurrentScene\Background)
            EndIf
            *OtherScene\Main = Editor\CurrentScene
          EndIf
        EndIf
      Next
    Next
    
    Editor\CurrentScene\Width  = Width
    Editor\CurrentScene\Height = Height
    
    ; Table aktualisieren:
    CreateTable()
  EndIf
EndProcedure

Procedure.i ChangeScene(X.i, Y.i, ApplyBackground.i = #False, FreeScene.i = #True)
  Protected *PreviousBackground.BackgroundEntry
  
  With Editor
  
  If \CurrentScene
    If ApplyBackground
      *PreviousBackground = \CurrentScene\Background
      If *PreviousBackground
        UseRegBackground(*PreviousBackground)
      EndIf
    EndIf
    
    SaveTable()
    
    ; Falls die aktuelle Szene leer ist, soll diese gelscht werden:
    If FreeScene And IsSceneEmpty(\CurrentScene)
      RemoveScene(\CurrentScene\X, \CurrentScene\Y)
    EndIf
    
    FreeTable()
  EndIf
  
  ; Neue Szene ermitteln:
  \CurrentScene = GetScene(X, Y)
  If Not \CurrentScene
    \CurrentScene            = CreateScene(X, Y)
    \CurrentScene\Background = *PreviousBackground
    \SceneX = X
    \SceneY = Y
  Else
    \CurrentScene = \CurrentScene\Main
    \SceneX = \CurrentScene\X
    \SceneY = \CurrentScene\Y
  EndIf
  
  ; Neues Table erstellen:
  CreateTable()
  
  ; Kamera an die neue aktuelle Szene setzen:
  \CameraX = (X - \CurrentScene\X) * #SceneWidth  * #FieldSize
  \CameraY = (Y - \CurrentScene\Y) * #SceneHeight * #FieldSize
  
  EndWith
EndProcedure
Procedure.i IsSceneEmpty(*Scene.Scene)
  Protected Layer.i, *Queue.Queue
  
  If *Scene
    If *Scene <> *Scene\Main Or *Scene\Width * *Scene\Height > 1
      ProcedureReturn #False
    EndIf
    
    ; Jede Liste von Tiles und Objekten nach dem Inhalt berprfen:
    For Layer = #FirstLayer To #LastLayer
      If TableCount(*Scene, Layer)
        ProcedureReturn #False
      EndIf
    Next
    
    ProcedureReturn #True
  Else
    ProcedureReturn #False
  EndIf
EndProcedure
Procedure.i RemoveEmptyScene(X.i, Y.i)
  If IsSceneEmpty(GetScene(X, Y))
    ProcedureReturn RemoveScene(X, Y)
  Else
    ProcedureReturn #False
  EndIf
EndProcedure
Procedure.i IsMainSceneEqual(X.i, Y.i, *Main.Scene)
  Protected *Scene.Scene
  
  *Scene = GetScene(X, Y)
  If *Scene And *Scene\Main = *Main
    ProcedureReturn #True
  Else
    ProcedureReturn #False
  EndIf
EndProcedure

;}
CompilerEndSelect
; IDE Options = PureBasic 4.51 (Windows - x86)
; CursorPosition = 483
; FirstLine = 434
; Folding = BHAQAAAw
; EnableXP