;**************************************************************
;     Tile
;**************************************************************
;   Copyright (c) 2010 Josef Sniatecki
;
; This program is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License
; As published by the Free Software Foundation; either version
; 3 of the License, Or (at your option) any later version.
;
; This program is distributed in the hope that it will be
; useful, but WITHOUT ANY WARRANTY; without even the implied
; warranty of MERCHANTABILITY Or FITNESS For A PARTICULAR
; PURPOSE. See the GNU General Public License For more details.
;
; You should have received a copy of the GNU General Public
; License along With this program;
; If Not, see "http://www.gnu.org/licenses/".
;**************************************************************



CompilerSelect #Mode
  CompilerCase #Mode_Game ;{
;******************************************
; Game
;******************************************

Procedure.i LoadTileset(Name.s)
  Protected *Item.QueueItem, *Tileset.Tileset
  
  *Item = *Tilesets\First
  While *Item
    *Tileset = *Item\Memory
    If *Tileset\Name = Name
      ProcedureReturn *Tileset
    EndIf
    _Move(*Item, N)
  Wend
  
  *Tileset = _New(Tileset)
  AppendQueueItem(*Tilesets, *Tileset)
  
  *Tileset\Name = Name
  
  *Tileset\Image = LoadImage(#PB_Any, GetFile("Graphics\Tilesets\" + Name + ".png"))
  If Not *Tileset\Image
    MessageRequester("Error", "Can not find tileset: " + #Q + Name + #Q, #MB_ICONERROR)
    ProcedureReturn #False
  ElseIf ImageWidth(*Tileset\Image) % #FieldSize Or ImageHeight(*Tileset\Image) % #FieldSize
    MessageRequester("Error", "Tileset with invalid dimensions: " + #Q + Name + #Q, #MB_ICONERROR)
    ProcedureReturn #False
  EndIf
  
  *Tileset\Width  = ImageWidth(*Tileset\Image) / #FieldSize
  *Tileset\Height = ImageHeight(*Tileset\Image) / #FieldSize
  *Tileset\Tiles  = _Dim(*Tileset\Width * *Tileset\Height * _Sz(Integer))
  
  ProcedureReturn *Tileset
EndProcedure
Procedure.i GetTileset(Name.s)
  Protected *Item.QueueItem, *Tileset.Tileset
  
  *Item = *Tilesets\First
  While *Item
    *Tileset = *Item\Memory
    If *Tileset\Name = Name
      ProcedureReturn *Tileset
    EndIf
    _Move(*Item, N)
  Wend
  
  ProcedureReturn #Null
EndProcedure

Procedure.i LoadTileByIndex(*Tileset.Tileset, Index.i)
  Protected Tile.i, Image.i
  
  Tile = PeekI(*Tileset\Tiles + Index * _Sz(Integer))
  If Tile
    ProcedureReturn Tile
  Else
    Image = GrabImage(*Tileset\Image, #PB_Any, #FieldSize * (Index % *Tileset\Width), #FieldSize * (Index / *Tileset\Width), #FieldSize, #FieldSize)
    SaveImage(Image, "Graphics\Temp.png", #PB_ImagePlugin_PNG, 0, 32)
    FreeImage(Image)
    ; Nun wird das temporr gespeicherte Tile als Sprite geladen:
    Tile = LoadSprite(#PB_Any, "Graphics\Temp.png", #PB_Sprite_Texture|#PB_Sprite_Memory|#PB_Sprite_AlphaBlending)
    Tile = CreateSprite3D(#PB_Any, Tile)
    
    PokeI(*Tileset\Tiles + Index * _Sz(Integer), Tile)
    ProcedureReturn Tile
  EndIf
EndProcedure
Procedure.i LoadTile(*Tileset.Tileset, X.i, Y.i)
  ProcedureReturn LoadTileByIndex(*Tileset, Y * *Tileset\Width + X)
EndProcedure
Procedure.i CreateTile(Sprite.i, X.i, Y.i)
  Protected *Tile.Tile
  
  *Tile = _New(Tile)
  
  *Tile\Sprite = Sprite
  *Tile\X      = X
  *Tile\Y      = Y
  
  ProcedureReturn *Tile
EndProcedure

;}
  CompilerCase #Mode_Editor ;{
;******************************************
; Editor
;******************************************

Procedure.i LoadTilesets(Path.s, *Parent.Entry)
  Protected *Tileset.TilesetEntry, Directory.i, Name.s
  
  Directory = ExamineDirectory(#PB_Any, Path, "*.*")
  If Directory
    While NextDirectoryEntry(Directory)
      Name = DirectoryEntryName(Directory)
      Select DirectoryEntryType(Directory)
        Case #PB_DirectoryEntry_File
          *Tileset = _New(TilesetEntry)
          AppendQueueItem(*Tilesets, *Tileset)
          
          *Tileset\Entry      = CreateEntryFile(StringField(Name, 1, "."), *Tileset, *Parent)
          *Tileset\Registered = #Null
        Case #PB_DirectoryEntry_Directory
          If Name <> "." And Name <> ".."
            LoadTilesets(Path + Name + "\", CreateEntryFolder(Name, *Parent))
          EndIf
      EndSelect
    Wend
  Else
    MessageRequester("Error", "Can not find folder: " + #Q + Path + #Q, #MB_ICONERROR)
    ProcedureReturn #False
  EndIf
EndProcedure
Procedure.i GetTileset(Name.s)
  Protected *Entry.Entry
  
  *Entry = GetEntryByPath(*TilesetFolder, PatchTilesetPath(Name))
  ProcedureReturn *Entry\Memory
EndProcedure

Procedure.s TilesetPath(*Tileset.TilesetEntry)
  ProcedureReturn SStr(EntryPath(*Tileset\Entry), -Len("Tilesets\"))
EndProcedure
Procedure.s TilesetFilename(*Tileset.TilesetEntry)
  ;HINWEIS: EntryPath(*Tileset) gibt automatisch "Tilesets\" zurck.
  
  ProcedureReturn "Graphics\" + EntryPath(*Tileset\Entry) + ".png"
EndProcedure
Procedure.s TilesetName(*Tileset.TilesetEntry)
  ProcedureReturn *Tileset\Entry\Name
EndProcedure
Procedure.i LoadTilesetSprite(*Tileset.TilesetEntry)
  Protected Sprite.i
  
  Sprite = LoadSprite(#PB_Any, GetFile(TilesetFilename(*Tileset)), #PB_Sprite_Texture|#PB_Sprite_Memory|#PB_Sprite_AlphaBlending)
  If Not Sprite
    MessageRequester("Error", "Can not load tileset: " + #Q + TilesetPath(*Tileset) + #Q, #MB_ICONERROR)
    ProcedureReturn #False
  EndIf
  
  ProcedureReturn Sprite
EndProcedure

Procedure.i LoadRegTileset(*Entry.TilesetEntry)
  Protected *Tileset.RegisteredTileset
  
  If *Entry\Registered
    ProcedureReturn *Entry\Registered
  EndIf
  
  *Tileset          = _New(RegisteredTileset)
  *Tileset\Entry    = *Entry
  *Entry\Registered = *Tileset
  
  *Tileset\Image = LoadImage(#PB_Any, TilesetFilename(*Entry))
  If ImageWidth(*Tileset\Image) % #FieldSize Or ImageHeight(*Tileset\Image) % #FieldSize
    MessageRequester("Error", "Tileset with invalid dimensions: " + TilesetPath(*Entry), #MB_ICONERROR)
    
    FreeImage(*Tileset\Image)
    _Free(*Tileset)
    ProcedureReturn #False
  EndIf
  
  *Tileset\Width  = ImageWidth(*Tileset\Image) / #FieldSize
  *Tileset\Height = ImageHeight(*Tileset\Image) / #FieldSize
  
  *Tileset\Tiles           = _Dim((*Tileset\Width * *Tileset\Height) * _Sz(Integer))
  *Tileset\RegisteredTiles = CreateQueue(#Queue_SaveMemory)
  
  ProcedureReturn *Tileset
EndProcedure
Procedure.i FreeRegTileset(*Tileset.RegisteredTileset)
  FreeImage(*Tileset\Image)
  _Free(*Tileset\Tiles)
  FreeQueue(*Tileset\RegisteredTiles)
  *Tileset\Entry\Registered = #Null
  
  ProcedureReturn _Free(*Tileset)
EndProcedure

Procedure.i LoadRegTile(X.i, Y.i, *Tileset.RegisteredTileset)
  Protected *Tile.RegisteredTile, Index.i, Image.i
  
  Index = Y * *Tileset\Width + X
  *Tile = PeekI(*Tileset\Tiles + Index * _Sz(Integer))
  
  If *Tile
    ProcedureReturn *Tile
  EndIf
  
  *Tile         = _New(RegisteredTile)
  *Tile\Item    = AppendQueueItem(*Tileset\RegisteredTiles, *Tile)
  *Tile\Tileset = *Tileset
  PokeI(*Tileset\Tiles + Index * _Sz(Integer), *Tile)
  
  *Tile\Index = Index
  
  ; Tile laden:
  Image = GrabImage(*Tileset\Image, #PB_Any, #FieldSize * X, #FieldSize * Y, #FieldSize, #FieldSize)
  SaveImage(Image, "Graphics\Temp.png", #PB_ImagePlugin_PNG, 0, 32)
  FreeImage(Image)
  ; Nun wird das temporr gespeicherte Tile als Sprite geladen:
  *Tile\Sprite   = LoadSprite(#PB_Any, "Graphics\Temp.png", #PB_Sprite_Texture|#PB_Sprite_Memory|#PB_Sprite_AlphaBlending)
  *Tile\Sprite3D = CreateSprite3D(#PB_Any, *Tile\Sprite)
  
  *Tile\LastLayer = #None
  
  ProcedureReturn *Tile
EndProcedure
Procedure.i FreeRegTile(*Tile.RegisteredTile)
  FreeSprite3D(*Tile\Sprite3D)
  FreeSprite(*Tile\Sprite)
  FreeQueueItem(*Tile\Tileset\RegisteredTiles, *Tile\Item)
  PokeI(*Tile\Tileset\Tiles + *Tile\Index * _Sz(Integer), #Null)
  
  If *Tile\Tileset\RegisteredTiles\Count = 0
    FreeRegTileset(*Tile\Tileset)
  EndIf
  
  ProcedureReturn _Free(*Tile)
EndProcedure

Procedure.i GetRegTile(X.i, Y.i, *Tileset.RegisteredTileset)
  ProcedureReturn PeekI(*Tileset\Tiles + (Y * *Tileset\Width + X) * _Sz(Integer))
EndProcedure
Procedure.i RegTileX(*Tile.RegisteredTile)
  ProcedureReturn *Tile\Index % *Tile\Tileset\Width
EndProcedure
Procedure.i RegTileY(*Tile.RegisteredTile)
  ProcedureReturn *Tile\Index / *Tile\Tileset\Width
EndProcedure

Procedure.i UseRegTile(*Tile.RegisteredTile)
  *Tile\References + 1
  
  ProcedureReturn *Tile\References
EndProcedure
Procedure.i ReleaseRegTile(*Tile.RegisteredTile)
  If *Tile\References = 1
    ProcedureReturn FreeRegTile(*Tile)
  Else
    *Tile\References - 1
    ProcedureReturn #False
  EndIf
EndProcedure

Procedure.i GetTile(*Scene.Scene, Layer.i, X.i, Y.i)
  ProcedureReturn GetField(*Scene, Layer, X, Y)
EndProcedure
Procedure.i PutTile(Layer.i, *Registered.RegisteredTile, X.i, Y.i)
  Protected *Tile.Tile, *Queue.Queue
  
  *Tile = GetField(Editor\CurrentScene, Layer, X, Y)
  If *Tile
    If Editor\Overwrite
      FreeTile(*Tile)
    Else
      ProcedureReturn #Null
    EndIf
  EndIf
  
  *Tile = _New(Tile)
  SetField(Editor\CurrentScene, Layer, X, Y, *Tile)
  
  *Tile\Registered = *Registered
  *Tile\X          = X % #SceneWidth
  *Tile\Y          = Y % #SceneHeight
  
  UseRegTile(*Registered)
  *Registered\LastLayer = Layer
  
  ProcedureReturn *Tile
EndProcedure
Procedure.i FreeTile(*Tile.Tile)
  ReleaseRegTile(*Tile\Registered)
  _Free(*Tile)
  ProcedureReturn #True
EndProcedure
Procedure.i RemoveTile(*Scene.Scene, Layer.i, X.i, Y.i)
  Protected *Tile.Tile
  
  *Tile = GetTile(*Scene, Layer, X, Y)
  If *Tile
    FreeTile(*Tile)
    SetField(*Scene, Layer, X, Y, #Null)
    
    ProcedureReturn #True
  EndIf
  
  ProcedureReturn #False
EndProcedure
Procedure.i CopyTile(*Tile.Tile, Layer.i, X.i, Y.i)
  ProcedureReturn PutTile(Layer, *Tile\Registered, X, Y)
EndProcedure

Procedure.i SelectTile(*Tileset.RegisteredTileset, X.i, Y.i)
  Protected *Tile.RegisteredTile
  
  ; Bei Randberschreitung wird eine neue Position festgelegt:
  If X = -1
    X = *Tileset\Width - 1
  ElseIf X = *Tileset\Width
    X = 0
  EndIf
  If Y = -1
    Y = *Tileset\Height - 1
  ElseIf Y = *Tileset\Height
    Y = 0
  EndIf
  
  ; Tile laden und auswhlen:
  *Tile = LoadRegTile(X, Y, *Tileset)
  ChangeTile(*Tile)
  
  ProcedureReturn *Tile
EndProcedure
Procedure.i ChangeTile(*Tile.RegisteredTile = #Null)
  ; Wechselt das aktuelle Tile im Editor. Hier wird das Sprite
  ; des jeweiligen Tiles geladen, damit im Editor eine Auswahl
  ; im unteren Bereich erscheint.
  
  If *Tile <> Editor\CurrentTile
    If *Tile
      UseRegTile(*Tile)
      If Editor\CurrentTile
        If Editor\CurrentTile\Tileset <> *Tile\Tileset
          FreeSprite3D(Editor\TilesetSprite3D)
          FreeSprite(Editor\TilesetSprite)
          
          Editor\TilesetSprite   = LoadTilesetSprite(*Tile\Tileset\Entry)
          Editor\TilesetSprite3D = CreateSprite3D(#PB_Any, Editor\TilesetSprite)
        EndIf
        ReleaseRegTile(Editor\CurrentTile)
      Else
        Editor\TilesetSprite   = LoadTilesetSprite(*Tile\Tileset\Entry)
        Editor\TilesetSprite3D = CreateSprite3D(#PB_Any, Editor\TilesetSprite)
      EndIf
      
      ; Automatisch in die Ebene wechseln, die zuletzt fr dieses
      ; Tile verwendet wurde:
      If Editor\AutoLayerChange And *Tile\LastLayer <> #None
        Editor\Layer = *Tile\LastLayer
      EndIf
    Else
      If Editor\CurrentTile
        FreeSprite3D(Editor\TilesetSprite3D)
        FreeSprite(Editor\TilesetSprite)
      EndIf
    EndIf
    Editor\CurrentTile = *Tile
  EndIf
EndProcedure

;}
CompilerEndSelect

; IDE Options = PureBasic 4.50 (Windows - x86)
; CursorPosition = 86
; FirstLine = 25
; Folding = JBAAw
; EnableXP