;**************************************************************
;     Tiles
;**************************************************************
;   Copyright (c) 2010 Josef Sniatecki
;
; This program is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License
; As published by the Free Software Foundation; either version
; 3 of the License, Or (at your option) any later version.
;
; This program is distributed in the hope that it will be
; useful, but WITHOUT ANY WARRANTY; without even the implied
; warranty of MERCHANTABILITY Or FITNESS For A PARTICULAR
; PURPOSE. See the GNU General Public License For more details.
;
; You should have received a copy of the GNU General Public
; License along With this program;
; If Not, see "http://www.gnu.org/licenses/".
;**************************************************************



Procedure.i LoadTilesets()
  ; Ldt alle Tilesets. Die Bilddateien werden nur bei
  ; der Auswahl im Editor zwischenzeitig geladen.
  
  Protected Directory.i, *Tileset.Tileset, Image.i
  
  Directory = ExamineDirectory(#PB_Any, "Graphics\Tilesets\", "*.png")
  While NextDirectoryEntry(Directory)
    *Tileset      = _New(Tileset)
    *Tileset\Item = AppendQueueItem(*Tilesets, *Tileset)
    
    *Tileset\Name            = RemoveString(GetFilePart(DirectoryEntryName(Directory)), ".png")
    *Tileset\RegisteredTiles = CreateQueue(@FreeRegisteredTile())
    
    Image = LoadImage(#PB_Any, "Graphics\Tilesets\" + *Tileset\Name + ".png")
    *Tileset\Width  = ImageWidth(Image)/#FieldSize
    *Tileset\Height = ImageHeight(Image)/#FieldSize
    FreeImage(Image)
  Wend
EndProcedure
Procedure.i LoadTilesetImage(*Tileset.Tileset)
  If Not *Tileset\Image
    *Tileset\Image = LoadImage(#PB_Any, GetFile("Graphics\Tilesets\" + *Tileset\Name + ".png"), 32)
  EndIf
  ProcedureReturn *Tileset\Image
EndProcedure
Procedure.i FreeTilesetImage(*Tileset.Tileset)
  If *Tileset\Image And *Tileset\RegisteredTiles\Count = 0
    FreeImage(*Tileset\Image)
    *Tileset\Image = #Null
  EndIf
EndProcedure

Procedure.i GetTileset(Name.s)
  Protected *Item.QueueItem, *Tileset.Tileset
  
  *Item = *Tilesets\First
  While *Item
    *Tileset = *Item\Memory
    If *Tileset\Name = Name
      ProcedureReturn *Tileset
    EndIf
    _Move(*Item, N)
  Wend
  
  ProcedureReturn #Null
EndProcedure

Procedure.i LoadTileset(Name.s)
  Protected *Tileset.Tileset
  
  *Tileset = _New(Tileset)
  
  *Tileset\Name            = Name
  *Tileset\Image           = LoadImage(#PB_Any, GetFile("Graphics\Tilesets\" + Name + ".png"))
  *Tileset\Width           = ImageWidth(*Tileset\Image)/#FieldSize
  *Tileset\Height          = ImageHeight(*Tileset\Image)/#FieldSize
  *Tileset\RegisteredTiles = CreateQueue(@FreeRegisteredTile())
  
  ProcedureReturn *Tileset
EndProcedure
Procedure.i LoadImageOnlyTileset(Name.s)
  Protected *Tileset.Tileset
  
  *Tileset = _New(Tileset)
  
  *Tileset\Image  = LoadImage(#PB_Any, GetFile("Graphics\Tilesets\" + Name + ".png"))
  *Tileset\Width  = ImageWidth(*Tileset\Image)/#FieldSize
  *Tileset\Height = ImageHeight(*Tileset\Image)/#FieldSize
  
  ProcedureReturn *Tileset
EndProcedure
Procedure.i FreeTileset(*Tileset.Tileset)
  If *Tileset\Image
    FreeImage(*Tileset\Image)
  EndIf
  If *Tileset\RegisteredTiles
    FreeQueue(*Tileset\RegisteredTiles)
  EndIf
  ClearStructure(*Tileset, Tileset)
  
  _Free(*Tileset)
  
  ProcedureReturn #True
EndProcedure
Procedure.i RemoveTileset(*Tileset.Tileset)
  Protected *Item.QueueItem
  
  *Item = *Tileset\Item
  
  FreeTileset(*Tileset)
  
  FreeQueueItem(*Tilesets, *Item)
  
  ProcedureReturn #True
EndProcedure

Procedure.i LoadTileSprite(*Tileset.Tileset, Index.i)
  Protected Image.i,  Tile.i
  
  Image = LoadTilesetImage(*Tileset)
  
  ; Schneidet das mittels "index" angegebene Tile aus dem Tileset
  ; aus und speichert dieses:
  Tile = GrabImage(Image, #PB_Any, #FieldSize*(Index % *Tileset\Width), #FieldSize*(Index / *Tileset\Width), #FieldSize, #FieldSize)
  SaveImage(Tile, "Graphics\Temp.png", #PB_ImagePlugin_PNG, 0, 32)
  FreeImage(Tile)
  
  ; Nun wird das temporr gespeicherte Image als Sprite geladen:
  ProcedureReturn LoadSprite(#PB_Any, "Graphics\Temp.png", #PB_Sprite_Texture|#PB_Sprite_Memory|#PB_Sprite_AlphaBlending)
  
  ; Grund fr das Zwischenspeichern ist, dass Images nicht auf Sprites mit
  ; Bercksichtigung des Alpha-Kanals gezeichnet werden knnen und ein
  ; Sprite3D-Objekt nicht geclipped werden kann.
EndProcedure

Procedure.i CreateTile(Sprite.i, X.i, Y.i)
  Protected *Tile.Tile
  
  *Tile = _New(Tile)
  
  *Tile\Sprite = Sprite
  *Tile\X      = X
  *Tile\Y      = Y
  
  ProcedureReturn *Tile
EndProcedure
Procedure.i FreeTile(*Tile.Tile)
  _Free(*Tile)
EndProcedure

Procedure.i LoadRegisteredTile(*Tileset.Tileset, X.i, Y.i)
  Protected *Item.QueueItem, *Tile.RegisteredTile, Index.i
  
  Index = Y * *Tileset\Width + X
  
  ; Wurde das jeweilige Tile schon geladen/registriert:
  *Item = *Tileset\RegisteredTiles\First
  While *Item
    *Tile = *Item\Memory
    If *Tile\Index = Index
      ProcedureReturn *Tile
    EndIf
    _Move(*Item, N)
  Wend
  
  ; Das angegebene Tile wurde noch nicht geladen/registriert:
  *Tile      = _New(RegisteredTile)
  *Tile\Item = AppendQueueItem(*Tileset\RegisteredTiles, *Tile)
  
  *Tile\Tileset = *Tileset
  *Tile\Index   = Index
  
  *Tile\Sprite   = LoadTileSprite(*Tileset, Index)
  *Tile\Sprite3D = CreateSprite3D(#PB_Any, *Tile\Sprite)
  
  ProcedureReturn *Tile
EndProcedure
Procedure.i LoadRegisteredTileByIndex(*Tileset.Tileset, Index.i)
  Protected *Item.QueueItem, *Tile.RegisteredTile
  
  ; Wurde das jeweilige Tile schon geladen/registriert:
  *Item = *Tileset\RegisteredTiles\First
  While *Item
    *Tile = *Item\Memory
    If *Tile\Index = Index
      ProcedureReturn *Tile
    EndIf
    _Move(*Item, N)
  Wend
  
  ; Das angegebene Tile wurde noch nicht geladen/registriert:
  *Tile      = _New(RegisteredTile)
  *Tile\Item = AppendQueueItem(*Tileset\RegisteredTiles, *Tile)
  
  *Tile\Tileset = *Tileset
  *Tile\Index   = Index
  
  *Tile\Sprite   = LoadTileSprite(*Tileset, Index)
  *Tile\Sprite3D = CreateSprite3D(#PB_Any, *Tile\Sprite)
  
  ProcedureReturn *Tile
EndProcedure
Procedure.i FreeRegisteredTile(*Tile.RegisteredTile)
  FreeSprite3D(*Tile\Sprite3D)
  FreeSprite(*Tile\Sprite)
  
  _Free(*Tile)
  
  ProcedureReturn #True
EndProcedure
Procedure.i RemoveRegisteredTile(*Tile.RegisteredTile)
  Protected *Tileset.Tileset, *Item.QueueItem
  
  *Tileset = *Tile\Tileset
  *Item    = *Tile\Item
  
  FreeQueueItem(*Tileset\RegisteredTiles, *Item)
  
  ProcedureReturn #True
EndProcedure

Procedure.i UseTile(*Tileset.Tileset, Index.i)
  ; Benutzt eine Challange ein Tile, so wird dieses durch diese
  ; Prozedur geladen und in einer Liste referenziert, wodurch das
  ; Lschen aller benutzten Tiles mglich ist.
  
  Protected *Tile.UsedTile
  
  *Tile = _New(UsedTile)
  AppendQueueItem(*UsedTiles, *Tile)
  
  *Tile\Sprite   = LoadTileSprite(*Tileset, Index)
  *Tile\Sprite3D = CreateSprite3D(#PB_Any, *Tile\Sprite)
  
  ProcedureReturn *Tile
EndProcedure
Procedure.i FreeUsedTile(*Tile.UsedTile)
  ; Dekonstruktor fr die Liste "*UsedTiles.Queue"
  
  FreeSprite3D(*Tile\Sprite3D)
  FreeSprite(*Tile\Sprite)
  
  _Free(*Tile)
  ProcedureReturn #True
EndProcedure


; IDE Options = PureBasic 4.50 Beta 4 (Windows - x86)
; CursorPosition = 127
; Folding = AEg
; EnableXP